package ch.ige.adr.client;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;

import org.apache.log4j.Logger;

import ch.ige.authz.gui.AuthzJButton;
import ch.ige.authz.gui.AuthzJMenuItem;
import ch.ige.main.client.IGEJToolBar;
import ch.ige.main.client.MainController;
import ch.ige.main.client.MainRootPane;
import ch.ige.main.client.XxMessage;
import ch.ige.util.client.IgeImage;


/**
 * Standard Menues und Buttons
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: SimpleMenuPane.java,v 1.11 2007/05/10 14:16:13 nydegger Exp $
 */
public class SimpleMenuPane extends MainRootPane
{
  // Logging
  private static final Logger LOG = Logger.getLogger( SimpleMenuPane.class );

  // Applikationsmeldungen
  protected XxMessage xxMessage = null;

  // Menu
  protected JMenuBar menuBar = new JMenuBar();
  protected JMenu menFile = new JMenu();
  protected JMenuItem mnuPrint = new AuthzJMenuItem();
  protected JMenuItem mnuExit = new AuthzJMenuItem();

  protected JMenu menGo = new JMenu();
  protected JMenuItem mnuBack = new AuthzJMenuItem();
  protected JMenuItem mnuForward = new AuthzJMenuItem();
  protected JMenuItem mnuHome = new AuthzJMenuItem();

  protected JMenu menHelp = new JMenu();
  protected JMenuItem mnuHelp = new AuthzJMenuItem();
  protected JMenuItem mnuInfo = new AuthzJMenuItem();

  // Toolbar
  protected JToolBar toolBar = new IGEJToolBar();
  protected JButton btnForward = new AuthzJButton();
  protected JButton btnBack = new AuthzJButton();
  protected JButton btnHome = new AuthzJButton();
  protected JButton btnDetail = new AuthzJButton();
  protected JButton btnPrint = new AuthzJButton();
  protected JButton btnHelp = new AuthzJButton();

  // Images
  protected ImageIcon imgHome = IgeImage.imgHome;
  protected ImageIcon imgBack = IgeImage.imgBack;
  protected ImageIcon imgForward = IgeImage.imgForward;
  protected ImageIcon imgDetail = IgeImage.imgDetail;
  protected ImageIcon imgPrint = IgeImage.imgPrint;
  protected ImageIcon imgHelp = IgeImage.imgHelp;
  protected ImageIcon imgIgeLogo = IgeImage.imgIgeLogo;
  protected JLabel statusBar = new JLabel();

  protected BorderLayout borderLayout1 = new BorderLayout();

  protected JLabel bagisLabel = new JLabel();

  /**
   * Konstruktor
   */
  public SimpleMenuPane()
  {
    try
    {
      xxMessage = XxMessage.getInstance();
      jbInit();
    }
    catch ( Exception ex )
    {
      LOG.error( "exception caught", ex );
      JOptionPane.showMessageDialog( this, ex.getMessage(),
                                     "Fehler", JOptionPane.ERROR_MESSAGE );
    }
  }

  /**
   * Konstruktor
   * @param mainController Frame
   * @param mode Modus
   */
  public SimpleMenuPane( MainController mainController, int mode )
  {
    super( mainController, mode );
    try
    {
      xxMessage = XxMessage.getInstance();
      jbInit();
    }
    catch ( Exception ex )
    {
      LOG.error( "exception caught", ex );
      JOptionPane.showMessageDialog( mainController.getWindow(), ex.getMessage(),
                                     "Fehler", JOptionPane.ERROR_MESSAGE );
    }
  }

  /**
   * Komponenten initialisieren
   * @throws Exception
   */
  private void jbInit() throws Exception
  {
    btnHome.setIcon( imgHome );
    btnHome.setToolTipText( "Anfang" );
    btnHome.setEnabled( false );

    btnBack.setIcon( imgBack );
    btnBack.setToolTipText( "Zurück" );
    btnBack.setEnabled( false );
    btnBack.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuBack_actionPerformed( e );
      }
    } );
    btnForward.setIcon( imgForward );
    btnForward.setToolTipText( "Vorwärts" );
    btnForward.setEnabled( false );
    btnForward.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuForward_actionPerformed( e );
      }
    } );

    btnPrint.setIcon( imgPrint );
    btnPrint.setToolTipText( "Drucken" );
    btnPrint.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        btnPrint_actionPerformed( e );
      }
    } );
    btnHelp.setIcon( imgHelp );
    btnHelp.setToolTipText( "Hilfe" );

    bagisLabel.setHorizontalAlignment( SwingConstants.CENTER );
    bagisLabel.setIcon( imgIgeLogo );

    toolBar.add( btnBack );
    toolBar.add( btnForward );
    toolBar.add( btnHome );
    toolBar.addSeparator();
    toolBar.add( btnDetail );
    toolBar.addSeparator();
    toolBar.add( btnPrint );
    toolBar.addSeparator();
    toolBar.add( btnHelp );
    toolBar.setRequestFocusEnabled( false );
    toolBar.setFloatable( false );

    menFile.setText( "Datei" );
    menFile.setMnemonic( 'D' );
    mnuPrint.setText( "Drucken..." );
    mnuPrint.setMnemonic( 'D' );
    mnuPrint.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_P, ActionEvent.CTRL_MASK ) );
    mnuPrint.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuPrint_actionPerformed( e );
      }
    } );
    mnuExit.setText( "Beenden" );
    mnuExit.setMnemonic( 'B' );
    mnuExit.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_F4, ActionEvent.ALT_MASK ) );
    mnuExit.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuExit_actionPerformed( e );
      }
    } );

    menGo.setText( "Verzweigen" );
    menGo.setMnemonic( 'V' );
    mnuBack.setText( "Rückwärts" );
    mnuBack.setMnemonic( 'R' );
    mnuBack.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_R, ActionEvent.CTRL_MASK ) );
    mnuBack.setEnabled( true );
    mnuBack.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuBack_actionPerformed( e );
      }
    } );

    mnuForward.setText( "Vorwärts" );
    mnuForward.setMnemonic( 'V' );
    mnuForward.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_T, ActionEvent.CTRL_MASK ) );
    mnuForward.setEnabled( false );
    mnuForward.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuForward_actionPerformed( e );
      }
    } );

    mnuHome.setText( "Anfang" );
    mnuHome.setEnabled( false );
    mnuHome.setMnemonic( 'A' );
    mnuHome.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_H, ActionEvent.CTRL_MASK ) );
    mnuHome.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuHome_actionPerformed( e );
      }
    } );

    menHelp.setText( "Hilfe" );
    menHelp.setMnemonic( 'H' );
    mnuHelp.setText( "Hilfethemen" );
    mnuHelp.setMnemonic( 'H' );
    mnuHelp.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_F1, 0 ) );
    mnuInfo.setText( "Info" );
    mnuInfo.setMnemonic( 'I' );
    mnuInfo.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuInfo_actionPerformed( e );
      }
    } );

    menFile.add( mnuPrint );
    menFile.addSeparator();
    menFile.add( mnuExit );

    menGo.add( mnuBack );
    menGo.add( mnuForward );
    menGo.add( mnuHome );

    menHelp.add( mnuHelp );
    menHelp.addSeparator();
    menHelp.add( mnuInfo );

    menuBar.add( menFile );
    menuBar.add( menGo );
    menuBar.add( menHelp );

    this.setJMenuBar( menuBar );

    this.getContentPane().setLayout( borderLayout1 );
    this.getContentPane().add( toolBar, BorderLayout.NORTH );
    this.getContentPane().add( statusBar, BorderLayout.SOUTH );
    statusBar.setText( " " );
  }

  /**
   * gibt den Titel zurück
   * @return Titel
   */
  public String getTitle()
  {
    return "Hauptmenü";
  }
}
