package ch.ige.adr.server;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import javax.ejb.EJBException;

import org.apache.log4j.Logger;

import ch.ige.adr.server.dao.AdresseDAO;
import ch.ige.adr.server.dao.BlandDAO;
import ch.ige.adr.server.dao.RegadrDAO;
import ch.ige.adr.vo.AdresseVO;
import ch.ige.adr.vo.RegadrAVO;
import ch.ige.adr.vo.RegadrExistsVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.vo.AdrRegadrNrVO;
import ch.ige.util.vo.RegadrVO;
import ch.ige.util.vo.VersadrVO;
/**
 * Business Object für Register- und Versandadressen ab EP-Band
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: RegadrEpBO.java,v 1.40 2007/07/03 15:20:54 nydegger Exp $
 */

public class RegadrEpBO
{
  private static final Logger LOG = Logger.getLogger(RegadrEpBO.class);
  private AdresseXxIdentNumberLocal xxIdentNumberLocal = null;

  /**
   * Konstruktor
   * @param xxIdentNumberLocal Home Interface
   */
  public RegadrEpBO(AdresseXxIdentNumberLocal xxIdentNumberLocal)
  {
    this.xxIdentNumberLocal = xxIdentNumberLocal;
  }

  /**
   * Kreiert eine Register- und eine Versandadresse.
   * Falls eine Registeradresse mit exakt gleicher regName, RegStrasse,
   * regOrt und blandCd existiert, wird keine neue Registeradresse kreiert,
   * sondern die regadrNr der existierenden Registeradresse zurück gegeben.
   * @param regadrVO Value Object der Registeradresse
   * @return Registeradressnummer
   */
  public int createRegadrEp(RegadrVO regadrVO)
  {
    // Suche nach Registeradressen
    RegadrExistsVO regadrExistsVO = new RegadrExistsVO();
    regadrExistsVO.blandCd = regadrVO.getBlandCd();
    regadrExistsVO.regName = regadrVO.getRegName();
    regadrExistsVO.regStrasse = regadrVO.getRegStrasse();
    regadrExistsVO.regOrt = regadrVO.getRegOrt();
    try
    {
      int regadrNr = new RegadrDAO().selectRegadrNr(regadrExistsVO);
      if (regadrNr == 0)
      {
        // neue Registeradresse kreieren
        //TODO regadrVO.truncateFields() für nächsten Release, wann auch immer
        return new RegadrBO(xxIdentNumberLocal).createRegadr(regadrVO);
      }
      // Registeradressnummer der existierenden Registeradresse zurückgeben
      return regadrNr;
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrEpBO.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Kreiert eine Register- und eine Versandadresse.
   * Falls eine Registeradresse mit exakt gleicher regName, RegStrasse,
   * regOrt und blandCd existiert, wird keine neue Registeradresse kreiert,
   * sondern die regadrNr der existierenden Registeradresse zurück gegeben.
   * Die Versandadresse wird ab der Registeradresse kreiert. Falls eine ähnliche
   * Versandadresse existiert, wird keine neue Versandadresse kreiert, sondern
   * die adrNr der existierenden Registeradresse zurückgegeben.
   * @param regadrVO Value Object der Registeradresse
   * @param sprachCd Sprachcode für Versandadresse
   * @return AdrRegadrNrVO
   */
  public AdrRegadrNrVO createRegadrVersadrEp(RegadrVO regadrVO, String sprachCd)
  {
    // neue Versandadresse ab Registeradresse erstellen
    VersadrVO neueVersadrVO = getVersadrVO(regadrVO, sprachCd);
    
    regadrVO.truncateFields();
    
    // Suche nach Registeradressen
    RegadrExistsVO regadrExistsVO = new RegadrExistsVO();
    regadrExistsVO.blandCd = regadrVO.getBlandCdOrEmptyString().trim();
    regadrExistsVO.regName = regadrVO.getRegNameOrEmptyString().trim();
    regadrExistsVO.regStrasse = regadrVO.getRegStrasseOrEmptyString().trim();
    regadrExistsVO.regOrt = regadrVO.getRegOrtOrEmptyString().trim();

    int existierendeRegadrNr = 0;
    try
    {
      // Existierende Registeradresse suchen
      ArrayList regadrAL = new RegadrDAO().select(regadrExistsVO);
      Iterator iterator = regadrAL.iterator();
      while (iterator.hasNext())
      {
        RegadrAVO regadrAVO = (RegadrAVO)iterator.next();
        if (existierendeRegadrNr == 0)
        {
          // erste gefundene RegadrNr sichern
          existierendeRegadrNr = regadrAVO.getRegadrNr();
        }
        if (regadrAVO.getAdrNr() != 0)
        {
          // Prüfen, ob Versandadresse übereinstimmt.
          AdresseVO adresseVO = new AdresseDAO().select(regadrAVO.getAdrNr());
          // TODO testen
          if (aehnlich(neueVersadrVO, adresseVO))
          {
            // Rückgabe-Objekt aufbereiten
            AdrRegadrNrVO adrRegadrNrVO = new AdrRegadrNrVO();
            adrRegadrNrVO.adrNr = regadrAVO.getAdrNr();
            adrRegadrNrVO.regadrNr = regadrAVO.getRegadrNr();
            return adrRegadrNrVO;
          }
        }
      }
    }
    catch(SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrEpBO.class);
      throw new EJBException(ex);
    }
    // Suche nach Versandadresse
    String adresse = neueVersadrVO.getNameOrEmptyStr() + "." +  neueVersadrVO.getZeile1OrEmptyStr() + "." +
                     neueVersadrVO.getZeile2OrEmptyStr()  + "." + neueVersadrVO.getZeile3OrEmptyStr() + "." +
                     neueVersadrVO.getZeile4OrEmptyStr() + "." + neueVersadrVO.getPlzOrEmptyStr() + "." + neueVersadrVO.getOrtOrEmptyStr();
    adresse = PostOrtNorm.getOrtNorm(adresse);
    ArrayList suchworte = new ArrayList(20);
    try
    {
      StringTokenizer st = new StringTokenizer( adresse, "." );
      for(int i = 0; st.hasMoreTokens()&& i < 20; i++)
      {
        String wort = st.nextToken();
        if (!suchworte.contains(wort))
        {
          // jedes Suchwort nur einmal
          suchworte.add(wort);
        }
      }
    }
    catch(NoSuchElementException ex)
    {
      LOG.error(ex.getMessage());
      throw new EJBException(ex);
    }

    // existierende Versandadressen suchen und vergleichen
    int adrNr = 0;
    try
    {
      ArrayList al;
      if (suchworte.size() > 0)
      {
        // mit Suchworten suchen
        al = new AdresseDAO().select(suchworte);
      }
      else
      {
        // Es sind keine Suchworte vorhanden
        // Exakte Suche mit Name und Ort.
        al = new AdresseDAO().select(neueVersadrVO.getName(), neueVersadrVO.getOrt());
      }
      Iterator iterator = al.iterator();
      while (iterator.hasNext() && adrNr == 0)
      {
        // neue Versandadresse mit existierenden Versandadressen vergleichen
        AdresseVO existierendeAdresseVO = (AdresseVO)iterator.next();
        // TODO hier wird falsche adressnummer zugewiesen von Jeory J anstatt Jeory J.P.
        if (aehnlich(neueVersadrVO, existierendeAdresseVO))
        {
          // existierende ähnliche Versandadresse gefunden
          adrNr = existierendeAdresseVO.getAdrNr();
        }
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrEpBO.class);
      throw new EJBException(ex);
    }

    if (adrNr == 0)
    {
      // keine existierende Versandadresse gefunden
      if (neueVersadrVO.getLandCdOrEmptyStr().equals("CH") || neueVersadrVO.getLandCdOrEmptyStr().equals(AdrConstants.LAND_LI))
      {
        // voller Steuersatz (Inland)
        neueVersadrVO.setTaxClass("1");
      }
      else
      {
        // steuerbefreit (Ausland)
        neueVersadrVO.setTaxClass("0");
      }
      adrNr = new AdresseBO(xxIdentNumberLocal).createVersadr(neueVersadrVO);
    }

    if (existierendeRegadrNr == 0)
    {
      // neue Registeradresse kreieren, weil diese noch nicht besteht
      regadrVO.setAdrNr(adrNr);
      existierendeRegadrNr = new RegadrBO(xxIdentNumberLocal).createRegadr(regadrVO);
    }
    // Rückgabe-Objekt aufbereiten
    AdrRegadrNrVO adrRegadrNrVO = new AdrRegadrNrVO();
    adrRegadrNrVO.adrNr = adrNr;
    adrRegadrNrVO.regadrNr = existierendeRegadrNr;
    return adrRegadrNrVO;
  }

  /**
   * Erstellt aus einer Registeradresse eine Versandadresse
   * @param regadrVO Registeradresse
   * @param sprachCd Sprachcode
   * @return VersadrVO Versandadresse
   */
  protected VersadrVO getVersadrVO(RegadrVO regadrVO, String sprachCd)
  {
    VersadrVO neueVersadrVO = new VersadrVO();
    String[] zeile = new String[]{"", "", "", "", ""};
    String regName = regadrVO.getRegNameOrEmptyString().replace('\n', ' ');
    StringTokenizer st = new StringTokenizer(regName, " ", true);
    StringBuffer sb = new StringBuffer();
    int i = 0;
    while (i < 5 && st.hasMoreTokens())
    {
      String s = st.nextToken();
      if (sb.length() + s.length() <= 40)
      {
        sb.append(s);
      }
      else
      {
        zeile[i] = sb.toString();
        i++;
        sb.setLength(0);
        if (!s.equals(" "))
        {
          sb.append(s);
        }
      }
    }
    if (i < 5)
    {
      zeile[i] = sb.toString();
      i++;
    }
    // Strasse mindestens auf Zeile 2
    if (i < 2)
    {
      i = 2;
    }
    st = new StringTokenizer(regadrVO.getRegStrasseOrEmptyString(), "\n");
    while (i < 5 && st.hasMoreTokens())
    {
      zeile[i] = st.nextToken();
      i++;
    }
    neueVersadrVO.setName(zeile[0]);
    neueVersadrVO.setZeile1(zeile[1]);
    neueVersadrVO.setZeile2(zeile[2]);
    neueVersadrVO.setZeile3(zeile[3]);
    neueVersadrVO.setZeile4(zeile[4]);

    try
    {
      neueVersadrVO.setLandCd(new BlandDAO().selectLandCd(regadrVO.getBlandCd()));
      if (neueVersadrVO.getLandCd() == null)
      {
        String msg = "bland.bland_cd = '" + regadrVO.getBlandCd() + "' nicht gefunden";
        LOG.error(msg);
        throw new EJBException(msg);
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, RegadrEpBO.class);
      throw new EJBException(ex);
    }

    // Falls die Zeichen vor dem ersten " " alles Ziffern sind,
    // werden diese als Postleitzahl übernommen.
    String plz = "";
    String ort = "";
    // line feed entfernen
    String regOrt  = regadrVO.getRegOrtOrEmptyString().replace('\n', ' ');

    /* Extraktion der Postleitzahl
       - Falls in den ersten 10 Zeichen und vor dem ersten Leerzeichen
         eine mindestens 4-stellige Zahl gefunden wird,
         werden die ersten 8 Ziffern ins Attribut 'plz' und
         der Rest ins Attribut 'ort' geschrieben. Die Zeichen vor der Zahl
         werden ignoriert.
       - Andernfalls wird alles ins Attribut 'ort' geschrieben
    */

    char[] charArray = new char[regOrt.length()];
    regOrt.getChars(0, regOrt.length() , charArray, 0);
    int anfang = 0;   // Anfang der PLZ
    // Anfang der PLZ suchen
    while (anfang < charArray.length &&
           anfang < 10 &&
           charArray[anfang] != ' ' &&
           !Character.isDigit(charArray[anfang]))
    {
      anfang++;
    }
    // Ende der PLZ suchen
    int ende = anfang;     // Ende der PLZ + 1
    while (ende < charArray.length &&
           Character.isDigit(charArray[ende]) &&
           ende < 10)
    {
      ende++;
    }

    if (ende - anfang >= 4)
    {
      // PLZ gefunden, Mindestlänge 4 Ziffern
      plz = new String(charArray, anfang, ende - anfang);
      // Leerzeichen überspringen
      while (ende < charArray.length && charArray[ende] == ' ')
      {
        ende++;
      }
      if (ende < charArray.length)
      {
        ort = new String(charArray, ende, charArray.length - ende);
      }
    }
    else
    {
      // keine PLZ gefunden
      ort = new String(charArray);
    }

    neueVersadrVO.setPlz(plz);
    neueVersadrVO.setOrt(ort);
    /** suchbegriff auf 10 Zeichen beschränken */
    if (neueVersadrVO.getNameOrEmptyStr().length() <= 10)
    {
       neueVersadrVO.setSuchbegr(neueVersadrVO.getNameOrEmptyStr().toUpperCase());
    }
    else
    {
        neueVersadrVO.setSuchbegr(neueVersadrVO.getNameOrEmptyStr().toUpperCase().substring(0,10));
    }
    neueVersadrVO.setSprachCd(sprachCd);
    return neueVersadrVO;
  }

  /**
   * Vergleicht die beiden Versandadressen.
   * @param adresse1 erste Versandadresse (VersadrVO)
   * @param adresse2 zweite Versandadresse (AdresseVO)
   * @return true - die beiden Versandadressen sind ähnlich
   */
  public boolean aehnlich(VersadrVO adresse1, AdresseVO adresse2)
  {
    if (adresse2.getSuchbegrOrEmptyStr().equals("**********") || adresse2.getGesperrt() == 1)
    {
      return false;
    }

    if (!adresse1.getLandCdOrEmptyStr().equalsIgnoreCase(adresse2.getLandCdOrEmptyStr()))
    {
      return false;
    }
    String plzNorm1 = PostOrtNorm.getOrtNorm(adresse1.getPlzOrEmptyStr());
    String plzNorm2 = PostOrtNorm.getOrtNorm(adresse2.getPlzOrEmptyStr());
    if (!plzNorm1.equalsIgnoreCase(plzNorm2))
    {
      return false;
    }
    String ortNorm1 = PostOrtNorm.getOrtNorm(adresse1.getOrtOrEmptyStr());
    String ortNorm2 = PostOrtNorm.getOrtNorm(adresse2.getOrtOrEmptyStr());
    if (!ortNorm1.equalsIgnoreCase(ortNorm2))
    {
      return false;
    }

    // In beiden Adressen müssen die gleichen Worte vorkommen.
    // Die Reihenfolge der Worte spielt keine Rolle.
    String adr1 = adresse1.getNameOrEmptyStr() + "." + adresse1.getZeile1OrEmptyStr() + "." +
                  adresse1.getZeile2OrEmptyStr() + "." + adresse1.getZeile3OrEmptyStr() + "." + adresse1.getZeile4OrEmptyStr();
    adr1 = PostOrtNorm.getOrtNorm(adr1);
    String adr2 = adresse2.getNameOrEmptyStr() + "." + adresse2.getZeile1OrEmptyStr() + "." +
                  adresse2.getZeile2OrEmptyStr() + "." + adresse2.getZeile3OrEmptyStr() + "." + adresse2.getZeile4OrEmptyStr();
    adr2 = PostOrtNorm.getOrtNorm(adr2);
 
    int anzahl1 = 0;
    StringTokenizer st = new StringTokenizer(adr1, ".");
    while(st.hasMoreTokens())
    {
      String token = st.nextToken();
      if (adr2.indexOf(token) < 0)
      {
        return false;
      }
      anzahl1++;
    }
    st = new StringTokenizer(adr2, ".");
    int anzahl2 = st.countTokens();
    return anzahl1 == anzahl2;
  }
}
