package ch.ige.adr.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import ch.ige.adr.vo.AdresseSapVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Datenbankzugriff auf die AdresseSap
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: AdresseSapDAO.java,v 1.13 2006/12/11 15:17:28 nydegger Exp $
 */
public class AdresseSapDAO
{
  /**
   * Selektiert eine AdresseSap.
   * @param adrNr Adressnummer
   * @return Value Object der AdresseSap
   * @exception SQLException
   */
  public AdresseSapVO select(int adrNr) throws SQLException
  {
    String query = null;
    AdresseSapVO adresseSapVO = new AdresseSapVO();
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      query = "SELECT adr_nr, cust_group, tax_class, cust_grp1, version " +
              "FROM adressesap WHERE adr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, adrNr);
      rs = ps.executeQuery();
      if(!rs.next())
      {
        return null;
      }
      adresseSapVO.adrNr = rs.getInt("adr_nr");
      adresseSapVO.custGroup = rs.getString("cust_group");
      adresseSapVO.taxClass = rs.getString("tax_class");
      adresseSapVO.custGrp1 = rs.getString("cust_grp1");
      adresseSapVO.version = rs.getInt("version");
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseSapDAO.class, query + " adrNr " + adrNr);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return adresseSapVO;
  }

  /**
   * Kreiert eine AdresseSap.
   * @param adresseSapVO Value Object der AdresseSap
   * @exception SQLException
   */
  public void insert(AdresseSapVO adresseSapVO) throws SQLException
  {
    String query = null;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "INSERT INTO adressesap(adr_nr, cust_group, tax_class, cust_grp1, version)" +
              "VALUES (?, ?, ?, ?, ?)";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, adresseSapVO.adrNr);
      ps.setString(2, adresseSapVO.custGroup);
      ps.setString(3, adresseSapVO.taxClass);
      ps.setString(4, adresseSapVO.custGrp1);
      ps.setInt(5, adresseSapVO.version);
      ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseSapDAO.class, query + adresseSapVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Setzt die AdresseSap auf den Wert des Value Objects.
   * @param adresseSapVO Value Object der AdresseSap
   * @return Anzahl der aktualisierten AdressenSap
   * @exception SQLException
   */
  public int update(AdresseSapVO adresseSapVO) throws SQLException
  {
    String query = "";
    int rowCount;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "UPDATE adressesap SET " +
              "cust_group = ?, " +
              "tax_class = ?, " +
              "cust_grp1 = ?, " +
              "version = ? " +
              "WHERE adr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, adresseSapVO.custGroup);
      ps.setString(2, adresseSapVO.taxClass);
      ps.setString(3, adresseSapVO.custGrp1);
      ps.setInt(4, adresseSapVO.version);
      ps.setInt(5, adresseSapVO.adrNr);
      rowCount = ps.executeUpdate();
      ps.close();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseSapDAO.class, query + adresseSapVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }

  /**
   * Löscht eine AdresseSap.
   * @param adrNr Adressnummer der zu löschenden AdresseSap
   * @return Anzahl der gelöschten AdressenSap
   * @exception SQLException
   */
  public int delete(int adrNr) throws SQLException
  {
    String query = null;
    int rowCount;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "DELETE FROM adressesap WHERE adr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, adrNr);
      rowCount = ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseSapDAO.class, query + " adrNr = " + adrNr);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }
}
