package ch.ige.adr.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import ch.ige.adr.vo.BlandAVO;
import ch.ige.adr.vo.BlandVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für BLAND
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Stephan Madl, Michael Zaugg
 * @version $Id: BlandDAO.java,v 1.15 2006/12/11 16:27:00 nydegger Exp $
 */

public class BlandDAO
{

/**
 * Selektiert alle Attribute eines Datensatzes aus der Tabelle BLAND
 *
 * @param   blandCd
 * @return  BlandVO ValueObject
 *
 * @exception SQLException Unerwarteter SQL Fehler
 */

    public BlandVO select (int blandCd ) throws SQLException
    {
      String                  query = null;
      Connection              con   = null;
      PreparedStatement       ps    = null;
      ResultSet               rs    = null;
      BlandVO                 blvo  = new BlandVO();

      try
      {
        query = "SELECT trim(bland_cd) bland_cd, trim(land_cd) land_cd, " +
                "land_bez_d, land_bez_f, land_bez_i, " +
                " land_bez_e, bland_mma, bland_mmp, version " +
                "FROM bland WHERE bland_cd = ? ";
        con = DBHelper.getConnection ( DBHelper.BAGIS);
        ps  = con.prepareStatement (query);

        ps.setInt ( 1, blandCd );

        rs = ps.executeQuery ();
        if ( !rs.next())
        {
        return null;
        }
        blvo.setBlandCd( rs.getString("bland_cd"));
        blvo.setLandCd(rs.getString("land_cd"));
        blvo.setLandBezD(rs.getString("land_bez_d"));
        blvo.setLandBezF(rs.getString("land_bez_f"));
        blvo.setLandBezI(rs.getString("land_bez_i"));
        blvo.setLandBezE(rs.getString("land_bez_e"));
        blvo.setBlandMma(rs.getInt("bland_mma"));
        blvo.setBlandMmp(rs.getInt("bland_mmp"));
        blvo.setVersion(rs.getInt("version"));
        return blvo;
      }
      catch (SQLException ex)
      {
        LogLevelHelper.log(ex, BlandDAO.class, query + " blandCd = " + blandCd);
        throw ex;
      }
      finally
      {
          DBHelper.close(ps, con);
      }
    }

    /**
    * Fügt einen Datensatz in die Tabelle BLAND ein
    * @param  blvo BlandVO    ValueObject
    * @exception SQLException Unerwarteter SQL Fehler
    */
    public void insert (BlandVO blvo) throws SQLException
    {
      String                query = null;
      Connection            con   = null;
      PreparedStatement     ps    = null;

      try
      {
        query = "INSERT INTO bland ( " +
                "bland_cd, land_cd, land_bez_d, land_bez_f, land_bez_i " +
                " land_bez_e, bland_mma, bland_mmp, version " +
                "VALUES (?,?,?,?,?,?,?,?,? )";
        con = DBHelper.getConnection ( DBHelper.BAGIS);
        ps  = con.prepareStatement (query);
        ps.setString     (1, blvo.getBlandCd() );
        ps.setString     (2, blvo.getLandCd() );
        ps.setString     (3, blvo.getLandBezD() );
        ps.setString     (4, blvo.getLandBezF() );
        ps.setString     (5, blvo.getLandBezI() );
        ps.setString     (6, blvo.getLandBezE() );
        ps.setInt        (7, blvo.getBlandMma() );
        ps.setInt        (8, blvo.getBlandMmp() );
        ps.setInt        (9, blvo.getVersion() );

        ps.executeUpdate();
      }
      catch (SQLException ex)
      {
        LogLevelHelper.log(ex, BlandDAO.class, query + blvo.toString());
        throw ex;
      }
      finally
      {
        DBHelper.close(ps, con);
      }
    }

    /**
    * Aktualisiert einen Datensatz in der Tabelle BLAND
    *
    * @param  blvo BlandVO   ValueObject
    * @return int Anzahl aktualisierter Datensätze
    *
    * @exception SQLException Unerwarteter SQL Fehler
    */

    public int update (BlandVO blvo) throws SQLException
    {
      String                query = null;
      Connection            con   = null;
      PreparedStatement     ps    = null;

      try
      {
        query = "UPDATE bland SET " +
                      " land_cd = ?, land_bez_d = ?, land_bez_f = ?, land_bez _i = ? " +
                      " land_bez_e = ?, bland_mma = ?, bland_mmp = ?, version = ? " +
                "WHERE bland_cd = ? ";
        con = DBHelper.getConnection ( DBHelper.BAGIS);
        ps  = con.prepareStatement (query);
        ps.setString    (1, blvo.getLandCd() );
        ps.setString    (2, blvo.getLandBezD() );
        ps.setString    (3, blvo.getLandBezF() );
        ps.setString    (4, blvo.getLandBezI() );
        ps.setString    (5, blvo.getLandBezE() );
        ps.setInt       (6, blvo.getBlandMma() );
        ps.setInt       (7, blvo.getBlandMmp() );
        ps.setInt       (8, blvo.getVersion() );
        ps.setString    (9, blvo.getBlandCd() );

        return ps.executeUpdate();
      }
      catch (SQLException ex)
      {
        LogLevelHelper.log(ex, BlandDAO.class, query + blvo.toString());
        throw ex;
      }
      finally
      {
        DBHelper.close(ps, con);
      }
    }

    /**
    * Löscht einen Datensatz aus der Tabelle BLAND
    * @param  blandCd
    * @return String Anzahl gelöschter Datensätze
    * @exception SQLException Unerwarteter SQL Fehler
    */
    public int delete (String blandCd) throws SQLException
    {
      String                query = null;
      Connection            con   = null;
      PreparedStatement     ps    = null;

      try
      {
        query = "DELETE FROM bland  " +
                "WHERE bland_cd = ? ";
        con = DBHelper.getConnection ( DBHelper.BAGIS);
        ps  = con.prepareStatement ( query );
        ps.setString (1, blandCd);
        return ps.executeUpdate();
      }
      catch (SQLException ex)
      {
        LogLevelHelper.log(ex, BlandDAO.class, query + " bland_cd = " + blandCd);
        throw ex;
      }
      finally
      {
        DBHelper.close(ps, con);
      }
    }

   /**
   * Liest alle BLänder aus BLAND
   * @return  BlandWorkAVO
   * @exception SQLException
   */
    public ArrayList getAllBland()  throws SQLException
    {
      String                query = null;
      Connection            con   = null;
      PreparedStatement     ps    = null;
      ResultSet             rs    = null;
      ArrayList blandWork = new ArrayList(200);

      try
      {
        query =     "SELECT bland_cd " +
                    "FROM bland " +
                    "ORDER BY 1";
        con = DBHelper.getConnection ( DBHelper.BAGIS);
        con.setTransactionIsolation( Connection.TRANSACTION_READ_UNCOMMITTED );
        ps = con.prepareStatement(query);
        rs = ps.executeQuery();
        while (rs.next())
        {
          blandWork.add(rs.getString("bland_cd").trim());
        }
      }
      catch (SQLException ex)
      {
        LogLevelHelper.log(ex, BlandDAO.class, query);
        throw ex;
      }
      finally
      {
        DBHelper.close(ps, con);
      }
      return blandWork;
    }

  /**
   * Findet alle BLänder, auf welche die Selektionskriterien zutreffen.
   * @param blandCd Ländercode als Selektionskriterium
   * @param landBezD Länderbezeichnungs als Selektionskriterium
   * @return ArrayList mit BlandAVO-Objekten mit den gefundenen BLänder
   * @throws SQLException Fehler
   */
  public ArrayList select(String blandCd, String landBezD) throws SQLException
  {
    // Assertions
    if (blandCd == null) blandCd = "";
    if (landBezD == null) landBezD = "";

    ArrayList al = new ArrayList(300);
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;

    StringBuffer sql = new StringBuffer();

    sql.append("SELECT trim(bland_cd) bland_cd, trim(land_bez_d) land_bez_d FROM bland WHERE 1=1 ");
    if (!blandCd.equals(""))
    {
      sql.append("AND bland_cd LIKE '" + blandCd + "%' ");
    }
    if (!landBezD.equals(""))
    {
      sql.append("AND land_bez_d LIKE '" + landBezD + "%' ");
    }

    try
    {
      con = DBHelper.getConnection(DBHelper.BAGIS);
      stmt =con.createStatement();
      rs = stmt.executeQuery(sql.toString());
      while (rs.next())
      {
        BlandAVO blandAVO = new BlandAVO();
        blandAVO.blandCd = rs.getString("bland_cd");
        blandAVO.landBezD = rs.getString("land_bez_d");
        al.add(blandAVO);
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, BlandDAO.class, sql.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
    return al;
  }

  /**
   * Prüft die Existenz eines BLändercodes.
   * @param blandCd Ländercode, z.B. CH für Schweiz, FR für Frankreich etc.
   * @return true - Ländercode existiert.
   * @throws SQLException Fehler
   */
  public boolean exists(String blandCd) throws SQLException
  {
    String query = null;
    // Assertion
    if (blandCd == null)
    {
      throw new IllegalArgumentException("blandCd darf nicht null sein");
    }
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      query = "SELECT trim(bland_cd) bland_cd FROM bland WHERE bland_cd = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, blandCd);
      rs = ps.executeQuery();
      return rs.next();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, BlandDAO.class, query + " bland_cd = " + blandCd);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Gibt den Landcode aufgrund des BLandcodes zurück.
   * @param blandCd BLandcode, z.B. CH für Schweiz, FR für Frankreich etc.
   * @return Landcode, z.B. CH für Schweiz, F für Frankreich etc.
   * @throws SQLException Fehler
   */
  public String selectLandCd(String blandCd) throws SQLException
  {
    String query = null;
    // Assertion
    if (blandCd == null)
    {
      throw new IllegalArgumentException("blandCd darf nicht null sein");
    }
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      query = "SELECT trim(land_cd) land_cd FROM bland WHERE bland_cd = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, blandCd);
      rs = ps.executeQuery();
      if (!rs.next())
      {
        return null;
      }
      String landCd = rs.getString("land_cd");
      return landCd.trim();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, BlandDAO.class, query + " bland_cd = " + blandCd);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }
}
