package ch.ige.art.client.art;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.apache.log4j.Logger;

import ch.ige.art.client.CustomDlgInfo;
import ch.ige.art.client.DlgPrint;
import ch.ige.art.client.util.CSVFileFilter;
import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.UserMessage;
import ch.ige.art.client.util.renderer.ComboRenderer;
import ch.ige.art.client.util.verifier.RegexDocument;
import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.util.StatistikDelegate;
import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.ArtikelAVO;
import ch.ige.art.vo.ArtikelSelKritVO;
import ch.ige.art.vo.XxDomainVO;
import ch.ige.art.xml.ArtExportListe;
import ch.ige.art.xml.ArtListe;
import ch.ige.art.xml.ArtStammblatt;
import ch.ige.arthelp.HelpUtil;
import ch.ige.authz.gui.GUIAuthorizer;
import ch.ige.main.client.GlassComponent;
import ch.ige.main.client.MainController;
import ch.ige.main.client.MainRootPane;
import ch.ige.main.client.SelectionPane;
import ch.ige.statistikweb.statistikartikel.util.NumberRangeList;
import ch.ige.util.client.LimitedDocument;
import ch.ige.util.client.SwingWorker;
import ch.ige.util.client.TableSorter;

/**
 * <p>Überschrift: Artikel Auswahl</p>
 * <p>Beschreibung: Auswahl der Artikel</p>
 * Copyright Institut für Geistiges Eigentum
 * @author   Anita Rueegsegger
 * @version $Id: ArtAwl.java,v 1.47 2008/07/02 11:00:20 heinzer Exp $
 */
public class ArtAwl extends SelectionPane
{
  // Logging
  private static final Logger LOG = Logger.getLogger( ArtAwl.class );

  private UserMessage userMessage = new UserMessage();

  // Artikeldaten
  List<ArtikelAVO> artikelAVO;

  ArtikelSelKritVO selKritVO; // Selektionskriterien abfüllen

  // Detail-Pane
  private ArtDtl artikelDtl = null;

  // Artikel Tabelle
  private TmArtAuswahl tableModel = new TmArtAuswahl();

  private TableSorter tableSorter = new TableSorter( tableModel, true );

  private JTable tblSel = new JTable( tableSorter );

  //Frame
  private JPanel mainPanel = new JPanel();

  private JPanel pnlSelKrit = new JPanel();

  private JPanel pnlSelTable = new JPanel();

  private JScrollPane scpSelTable = new JScrollPane();

  private BorderLayout lytSelTable = new BorderLayout();

  private GridBagLayout lytSelKrit = new GridBagLayout();

  private JLabel lblArtNr = new JLabel();

  private JLabel lblBezArt = new JLabel();

  private JLabel lblkondartCd = new JLabel();

  private JLabel lblFormartCd = new JLabel();

  private JTextField txfArtNr = new JTextField();

  private JTextField txfBezArt = new JTextField();

  private LimitedDocument ldoBezArt = new LimitedDocument();

  private RegexDocument docArtNr = new RegexDocument();

  private RegexDocument docFormartCd = new RegexDocument();

  private BorderLayout borderLayout1 = new BorderLayout();

  private JComboBox cbxKondartCd = new JComboBox();

  private JLabel lblStatcd3 = new JLabel();

  private JTextField txfFormartCd = new JTextField();

  private JLabel lblRegauszGrp = new JLabel();

  private JLabel lblPublPflicht = new JLabel();

  private JLabel lblErfassProc = new JLabel();

  private JCheckBox ckbSperrCdNot = new JCheckBox();

  private JCheckBox ckbSperrCdYes = new JCheckBox();

  private JComboBox cbxPublPflicht = new JComboBox();

  private JComboBox cbxRegauszGrp = new JComboBox();

  private JComboBox cbxErfassProc = new JComboBox();

  private JComboBox cbxStatcd3 = new JComboBox();

  public JMenuItem mnuExport = new JMenuItem();

  /**
   * Konstruktor mit Parameter für Hauptfenster und Suchmodus
   * @param mainController Hauptfenster, zuständig für Panelsteuerung
   * @param mode Suchmodus<br>
   *    0 - Suche in Auswahl<br>
   *    1 - Suche für Lookup
   */
  public ArtAwl( MainController mainController, int mode )
  {
    super( mainController, mode );
    /** TODO Focus-Handling */
    focusOwner = txfArtNr;
    try
    {
      jbInit();
      initHelp();
      fillComboboxes();
      initStatistik();
      GUIAuthorizer.authorizeFieldsIn( this );
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
    }
  }

  /**
   * gets information about statistic from application server (singleton)
   * is called by frame intialization  - each time user clicking node "Artikel Auswahl" in "Hauptmenü"
   * if statistic is not available an information dialog is shown that statistic is not available and column is hidden 
   */
  private void initStatistik()
  {
    try
    {
      NumberRangeList rangeOfArticlesInUse = StatistikDelegate.getInstance().getRangeOfArticlesInStatistic();
      tableModel.setRangeOfArticlesInStatistic( rangeOfArticlesInUse );
    }
    catch ( Throwable ex )
    {
      LOG.error( ex.getClass() + " " + ex.getMessage() );
      String message = "Statistik Informationen nicht verfügbar";
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );

      TableColumn cm = tblSel.getColumnModel().getColumn( TmArtAuswahl.Column.STATISTIK.ordinal() ); // index of columnmodel
      cm.setMinWidth( 0 );
      cm.setMaxWidth( 0 );
    }
  }

  /**
   * Komponente initialisieren
   * @exception Exception
   */
  void jbInit() throws Exception
  {
    mainPanel.setLayout( borderLayout1 );
    txfArtNr.setMaximumSize( new Dimension( 400, 20 ) );
    txfArtNr.setMinimumSize( new Dimension( 500, 20 ) );
    txfArtNr.setPreferredSize( new Dimension( 500, 20 ) );
    txfArtNr.setRequestFocusEnabled( true );
    txfArtNr.setToolTipText( "Einzelartikel müssen durch Semikola und Artikelbereiche durch Bindestrich getrennt werden, wie z.B.: 1;5-7;10" );

    txfBezArt.setMaximumSize( new Dimension( 500, 20 ) );
    txfBezArt.setMinimumSize( new Dimension( 500, 20 ) );
    txfBezArt.setPreferredSize( new Dimension( 500, 20 ) );
    lblArtNr.setMaximumSize( new Dimension( 140, 16 ) );
    lblArtNr.setMinimumSize( new Dimension( 140, 16 ) );
    lblArtNr.setPreferredSize( new Dimension( 140, 16 ) );
    cbxKondartCd.setBackground( Color.white );
    cbxKondartCd.setEnabled( true );
    cbxKondartCd.setMaximumSize( new Dimension( 500, 20 ) );
    cbxKondartCd.setMinimumSize( new Dimension( 500, 20 ) );
    cbxKondartCd.setPreferredSize( new Dimension( 500, 20 ) );
    lblStatcd3.setMaximumSize( new Dimension( 165, 16 ) );
    lblStatcd3.setPreferredSize( new Dimension( 200, 16 ) );
    lblStatcd3.setToolTipText( "statcd_3" );
    lblStatcd3.setHorizontalAlignment( SwingConstants.RIGHT );
    lblStatcd3.setText( "Unterabschnitt Publikationen:" );
    txfFormartCd.setMaximumSize( new Dimension( 500, 20 ) );
    txfFormartCd.setMinimumSize( new Dimension( 500, 20 ) );
    txfFormartCd.setPreferredSize( new Dimension( 500, 20 ) );
    txfFormartCd.setToolTipText( "Einzelformulare müssen durch Semikola und Formularbereiche durch Bindestrich getrennt werden, wie z.B.: 1;5-7;10" );
    txfFormartCd.setText( "" );
    lblRegauszGrp.setToolTipText( "regausz_grp" );
    lblRegauszGrp.setHorizontalAlignment( SwingConstants.RIGHT );
    lblRegauszGrp.setText( "Registerauszugsgruppe:" );
    lblPublPflicht.setToolTipText( "publ_pflicht" );
    lblPublPflicht.setHorizontalAlignment( SwingConstants.RIGHT );
    lblPublPflicht.setText( "Publikationspflicht:" );
    lblErfassProc.setToolTipText( "erfass_proc" );
    lblErfassProc.setHorizontalAlignment( SwingConstants.RIGHT );
    lblErfassProc.setText( "Erfassungsprozess:" );
    ckbSperrCdNot.setMaximumSize( new Dimension( 110, 24 ) );
    ckbSperrCdNot.setMinimumSize( new Dimension( 110, 24 ) );
    ckbSperrCdNot.setToolTipText( "sperr_cd" );
    ckbSperrCdNot.setHorizontalAlignment( SwingConstants.TRAILING );
    ckbSperrCdNot.setHorizontalTextPosition( SwingConstants.TRAILING );
    ckbSperrCdNot.setSelected( true );
    ckbSperrCdNot.setText( "Artikel nicht gesperrt" );
    lblkondartCd.setRequestFocusEnabled( true );
    lblkondartCd.setToolTipText( "kondart_cd" );
    cbxPublPflicht.setMaximumSize( new Dimension( 500, 20 ) );
    cbxPublPflicht.setMinimumSize( new Dimension( 500, 20 ) );
    cbxPublPflicht.setPreferredSize( new Dimension( 500, 20 ) );
    cbxPublPflicht.setMaximumRowCount( 8 );
    cbxRegauszGrp.setMaximumRowCount( 8 );
    cbxRegauszGrp.setPreferredSize( new Dimension( 500, 20 ) );
    cbxRegauszGrp.setMinimumSize( new Dimension( 500, 20 ) );
    cbxRegauszGrp.setMaximumSize( new Dimension( 500, 20 ) );
    cbxErfassProc.setBackground( Color.white );
    cbxErfassProc.setMaximumSize( new Dimension( 500, 20 ) );
    cbxErfassProc.setMinimumSize( new Dimension( 500, 20 ) );
    cbxErfassProc.setPreferredSize( new Dimension( 500, 20 ) );
    cbxErfassProc.setMaximumRowCount( 8 );
    cbxStatcd3.setMaximumRowCount( 8 );
    cbxStatcd3.setPreferredSize( new Dimension( 500, 20 ) );
    cbxStatcd3.setMinimumSize( new Dimension( 500, 20 ) );
    cbxStatcd3.setMaximumSize( new Dimension( 500, 20 ) );
    ckbSperrCdYes.setText( "Artikel gesperrt" );
    ckbSperrCdYes.setHorizontalTextPosition( SwingConstants.TRAILING );
    ckbSperrCdYes.setSelected( true );
    ckbSperrCdYes.setHorizontalAlignment( SwingConstants.TRAILING );
    ckbSperrCdYes.setMinimumSize( new Dimension( 110, 24 ) );
    ckbSperrCdYes.setToolTipText( "sperr_cd" );
    ckbSperrCdYes.setMaximumSize( new Dimension( 110, 24 ) );
    mainPanel.add( pnlSelTable, BorderLayout.CENTER );
    centerPanel.add( mainPanel, BorderLayout.CENTER );

    pnlSelKrit.setMaximumSize( new Dimension( 2147483647, 160 ) );
    pnlSelKrit.setPreferredSize( new Dimension( 300, 260 ) );
    pnlSelKrit.setMinimumSize( new Dimension( 300, 260 ) );
    pnlSelKrit.setLayout( lytSelKrit );
    pnlSelKrit.setBorder( new TitledBorder( "Selektionskriterien" ) );

    tblSel.setPreferredScrollableViewportSize( new Dimension( 450, 50 ) );
    pnlSelTable.setLayout( lytSelTable );
    pnlSelTable.setBorder( new TitledBorder( "Artikel" ) );
    pnlSelTable.add( scpSelTable, BorderLayout.CENTER );
    mainPanel.add( pnlSelKrit, BorderLayout.NORTH );
    scpSelTable.getViewport().add( tblSel, null );

    btnHelp.setEnabled( true );
    mnuHelp.setEnabled( true );

    mnuExport.setText( "Exportieren..." );
    mnuExport.setMnemonic( 'E' );
    mnuExport.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_E, ActionEvent.CTRL_MASK ) );
    menFile.insert( mnuExport, 4 );
    mnuExport.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuExport_actionPerformed( e );
      }
    } );

    lblArtNr.setText( "Artikel-Nr:" );
    lblArtNr.setHorizontalAlignment( SwingConstants.RIGHT );
    lblArtNr.setToolTipText( "art_nr" );
    lblBezArt.setText( "Artikel-Bezeichnung:" );
    lblBezArt.setHorizontalAlignment( SwingConstants.RIGHT );
    lblBezArt.setToolTipText( "bez_art" );
    lblFormartCd.setText( "Formular:" );
    lblFormartCd.setToolTipText( "formart_cd" );
    lblFormartCd.setHorizontalAlignment( SwingConstants.RIGHT );
    lblkondartCd.setText( "Übergruppe Ereignis:" );
    lblkondartCd.setHorizontalAlignment( SwingConstants.RIGHT );
    txfBezArt.setToolTipText( "Suche mit Platzhalter % möglich" );

    txfArtNr.setDocument( docArtNr );
    txfFormartCd.setDocument( docFormartCd );
    txfBezArt.setDocument( ldoBezArt );

    pnlSelKrit.add( lblArtNr, new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
    pnlSelKrit
        .add( lblBezArt, new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
    pnlSelKrit.add( lblkondartCd, new GridBagConstraints( 0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0,
        0 ) );
    pnlSelKrit.add( lblRegauszGrp, new GridBagConstraints( 0, 5, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0,
        0 ) );
    pnlSelKrit.add( txfFormartCd, new GridBagConstraints( 1, 2, 6, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ), 0,
        0 ) );
    pnlSelKrit.add( cbxKondartCd, new GridBagConstraints( 1, 3, 5, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ), 0,
        0 ) );
    pnlSelKrit
        .add( txfArtNr, new GridBagConstraints( 1, 0, 5, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ), 0, 0 ) );
    pnlSelKrit
        .add( txfBezArt, new GridBagConstraints( 1, 1, 5, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ), 0, 0 ) );
    pnlSelKrit.add( lblFormartCd, new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0,
        0 ) );
    pnlSelKrit.add( cbxRegauszGrp, new GridBagConstraints( 1, 5, 3, 1, 0.0, 0.0, GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
        new Insets( 3, 10, 3, 0 ), 0, 0 ) );
    pnlSelKrit.add( lblErfassProc, new GridBagConstraints( 0, 4, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0,
        0 ) );
    pnlSelKrit.add( lblPublPflicht, new GridBagConstraints( 0, 6, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ),
        0, 0 ) );
    pnlSelKrit.add( cbxErfassProc, new GridBagConstraints( 1, 4, 4, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ),
        0, 0 ) );
    pnlSelKrit.add( cbxPublPflicht, new GridBagConstraints( 1, 6, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ),
        0, 0 ) );
    pnlSelKrit
        .add( lblStatcd3, new GridBagConstraints( 0, 7, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
    pnlSelKrit.add( cbxStatcd3,
        new GridBagConstraints( 1, 7, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ), 0, 0 ) );
    pnlSelKrit.add( ckbSperrCdNot, new GridBagConstraints( 1, 8, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 0, 6, 0, 0 ), 0,
        0 ) );
    pnlSelKrit.add( ckbSperrCdYes, new GridBagConstraints( 2, 8, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 0, 10, 0, 0 ),
        0, 0 ) );

    // Tabelle
    tableSorter.setTableHeader( tblSel.getTableHeader() );
    tblSel.addMouseListener( new MouseAdapter()
    {
      public void mouseClicked( MouseEvent e )
      {
        tblSel_MouseListener( e );
      }
    } );
    tblSel.addFocusListener( new java.awt.event.FocusAdapter()
    {
      public void focusGained( FocusEvent e )
      {
        tblSel_focusGained( e );
      }

      public void focusLost( FocusEvent e )
      {
        tblSel_focusLost( e );
      }
    } );
    tblSel.addKeyListener( new java.awt.event.KeyAdapter()
    {
      public void keyPressed( KeyEvent e )
      {
        tblSel_keyPressed( e );
      }
    } );

    tblSel.setAutoResizeMode( JTable.AUTO_RESIZE_OFF );

    // Spaltenausrichtung setzen
    DefaultTableCellRenderer rendererLeft = new DefaultTableCellRenderer();
    rendererLeft.setHorizontalAlignment( SwingConstants.LEFT );

    tblSel.getColumnModel().getColumn( TmArtAuswahl.Column.ARTIKEL_NR.ordinal() ).setCellRenderer( rendererLeft );
    tblSel.getColumnModel().getColumn( TmArtAuswahl.Column.AUSZUGS_GRUPPE.ordinal() ).setCellRenderer( rendererLeft );

    cbxErfassProc.setBackground( SystemColor.controlLtHighlight );
    cbxKondartCd.setBackground( SystemColor.controlLtHighlight );
    cbxPublPflicht.setBackground( SystemColor.controlLtHighlight );
    cbxRegauszGrp.setBackground( SystemColor.controlLtHighlight );
    cbxStatcd3.setBackground( SystemColor.controlLtHighlight );

    docArtNr.setRegex( "[0-9;\u002D]" ); // Zahlen, Doppelkomma und Bindestrich erlaubt
    docFormartCd.setRegex( "[0-9;\u002D]" ); // Zahlen, Doppelkomma und Bindestrich erlaubt
    ldoBezArt.setMaxLength( 40 );

    // GlassKomponente setzten
    setGlassPane( new GlassComponent() );
    setStateNotFound();

    setColumnWidth();

  } // jbInit

  //********** Öffentliche Standard-Methoden *******************************

  /**
   * Gibt den Titel zurück
   * @return Titel
   */
  public String getTitle()
  {
    return "Artikel Auswahl";
  }

  //********** Ereignisbehandlung zu Menus *******************************

  /**
   * Menu Datensatz erfassen
   * @param e ActionEvent
   */
  protected void mnuNew_actionPerformed( ActionEvent e )
  {
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
    statusBar.setText( "Datensatz erfassen" );

    ArtWizardAbsolute med = new ArtWizardAbsolute( (Frame) mainController.getWindow() );
    String artikelNr = med.newArt();

    if ( artikelNr != null )
    {
      artikelDtl = new ArtDtl( mainController, MainRootPane.DETAIL );
      artikelDtl.setKey( artikelNr );
      artikelDtl.read();
      artikelDtl.setStatusBar( "Datensatz gespeichert." );
      mainController.next( artikelDtl );
    }
    else
    {
      statusBar.setText( "Erfassung wurde abgebrochen" );
    }
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
  }

  /**
   * Menu Datensatz suchen und anzeigen
   * @param e ActionEvent
   */
  protected void mnuFind_actionPerformed( ActionEvent e )
  {
    // Selektionskriterien abfüllen
    final ArtikelSelKritVO selKrit = fillArtikelSelKritVO();

    if ( selKrit.artNr.length() == 0 && selKrit.bezArt.length() == 0 && selKrit.formartCd.length() == 0 && selKrit.kondartCd.length() == 0
        && selKrit.erfassProc == null && selKrit.publPflicht.length() == 0 && selKrit.regauszGrp == null && selKrit.statcd3.length() == 0 )
    {
      JOptionPane.showMessageDialog( null, "Es muss mindestens ein Selektionskriterium abgefüllt sein.", "Information", JOptionPane.INFORMATION_MESSAGE );
      statusBar.setText( " " );
      return;
    }

    // GlassKomponente wird sichtbar gemacht,
    // damit der Benutzer während der Suche keine Eingaben vornehmen kann
    getGlassPane().setVisible( true );

    // Daten lesen
    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        try
        {
          ArtikelDelegate ad = ArtikelDelegate.getInstance();
          artikelAVO = ad.findArtikel( selKrit );
          tableModel.setData( artikelAVO );
        }
        catch ( Exception ex )
        {
          LOG.info( ex.getMessage() );
          String message = userMessage.getUserMessage( ex );
          JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
          statusBar.setText( " " );
          return null;
        }

        return null;
      }

      public void finished()
      {
        if ( tableModel.getRowCount() > 0 )
        {
          setStateFound();
          tblSel.setRowSelectionInterval( 0, 0 );
          tblSel.requestFocus();
          statusBar.setText( "Es wurden " + tableModel.getRowCount() + " Datensätze gelesen !" );
          setColumnWidth();
        }
        else
        {
          setStateNotFound();
          statusBar.setText( "Keine Datensätze gefunden !" );
        }
        getGlassPane().setVisible( false );
      }
    }; // SwingWorker
    worker.start();

  } // mnuFind_actionPerformed

  /**
   * Selektionskritierien abfüllen
   * @param e ActionEvent
   */
  protected ArtikelSelKritVO fillArtikelSelKritVO()
  {
    selKritVO = new ArtikelSelKritVO();
    ComboBoxObject comboBoxObject = null;

    tableModel.clear();
    statusBar.setText( "Bitte warten..." );

    selKritVO.artNr = txfArtNr.getText();
    selKritVO.bezArt = txfBezArt.getText().trim();
    selKritVO.formartCd = txfFormartCd.getText();

    comboBoxObject = (ComboBoxObject) cbxKondartCd.getSelectedItem();
    selKritVO.kondartCd = comboBoxObject.key;

    comboBoxObject = (ComboBoxObject) cbxErfassProc.getSelectedItem();
    selKritVO.erfassProc = ( comboBoxObject.key.equals( "" ) ? null : new Integer( comboBoxObject.key ) );

    comboBoxObject = (ComboBoxObject) cbxPublPflicht.getSelectedItem();
    selKritVO.publPflicht = comboBoxObject.key;

    comboBoxObject = (ComboBoxObject) cbxRegauszGrp.getSelectedItem();
    selKritVO.regauszGrp = ( comboBoxObject.key.equals( "" ) ? null : new Integer( comboBoxObject.key ) );

    comboBoxObject = (ComboBoxObject) cbxStatcd3.getSelectedItem();
    selKritVO.statcd3 = comboBoxObject.key;

    if ( ckbSperrCdNot.isSelected() && !ckbSperrCdYes.isSelected() )
    {
      selKritVO.sperrCd = "0";
    }
    else if ( !ckbSperrCdNot.isSelected() && ckbSperrCdYes.isSelected() )
    {
      selKritVO.sperrCd = "9";
    }

    return selKritVO;
  }

  /**
   * Menu Undo<br>
   * Suchfelder werden gelöscht
   * @param e ActionEvent
   */
  protected void mnuUndo_actionPerformed( ActionEvent e )
  {
    setStateNotFound();
    txfArtNr.requestFocus();
    txfArtNr.setText( "" );
    txfBezArt.setText( "" );
    txfFormartCd.setText( "" );
    cbxKondartCd.setSelectedIndex( 0 );
    cbxErfassProc.setSelectedIndex( 0 );
    cbxPublPflicht.setSelectedIndex( 0 );
    cbxRegauszGrp.setSelectedIndex( 0 );
    cbxStatcd3.setSelectedIndex( 0 );
    ckbSperrCdNot.setSelected( true );
    ckbSperrCdYes.setSelected( true );
    tableModel.clear();
    statusBar.setText( " " );
  }

  /**
   * Menu Verzweigen<br>
   * Artikeldetail wird aufgerufen
   * @param e ActionEvent
   */
  protected void mnuDetail_actionPerformed( ActionEvent e )
  {
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
    statusBar.setText( " " );
    if ( mode == SEARCH )
    {
      setForwardEnabled( true );
      artikelDtl = new ArtDtl( mainController, MainRootPane.DETAIL );
    }
    else if ( mode == SEARCH_LOOKUP )
    {
      artikelDtl = new ArtDtl( mainController, MainRootPane.DETAIL_LOOKUP );
    }

    // Schlüssel abfüllen
    int row = tblSel.getSelectedRows()[0];

    artikelDtl.setKey( (String) tableSorter.getValueAt( row, TmArtAuswahl.Column.ARTIKEL_NR.ordinal() ).toString() );
    artikelDtl.setStatisticRelevant( (Boolean) tableSorter.getValueAt( row, TmArtAuswahl.Column.STATISTIK.ordinal() ) );
    artikelDtl.read();
    mainController.next( artikelDtl );
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
  }

  /**
   * Menu Info<br>
   * Infofeld der Applikation wird angezeigt
   * @param e ActionEvent
   */
  public void mnuInfo_actionPerformed( ActionEvent e )
  {
    new CustomDlgInfo( (JFrame) this.mainController.getWindow(), true ).setVisible( true );
  }

  /**
   * menu export - export selection as csv file
   * @param e Action Event
   */
  public void mnuExport_actionPerformed( ActionEvent e )
  {
    String message = null;
    if ( tableModel.getRowCount() > 0 )
    {
      // Filedialog öffnen
      JFileChooser chooser = new JFileChooser();

      CSVFileFilter filter = new CSVFileFilter();
      chooser.setFileFilter( filter );

      int returnVal = chooser.showSaveDialog( this );
      if ( returnVal == JFileChooser.APPROVE_OPTION )
      {
        File file = chooser.getSelectedFile();
        File csvFile = filter.addExtension( file );

        if ( csvFile.exists() )
        {
          message = "File: " + csvFile.getPath() + " existiert bereits und wird überschrieben";
          int option = JOptionPane.showConfirmDialog( this, message, "File", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE );
          if ( option == JOptionPane.OK_OPTION )
          {
            saveTrefferliste( csvFile );
          }
        }
        else
        {
          saveTrefferliste( csvFile );
        }
      }
    }
    else
    {
      message = "Keine Daten zum Exportieren vorhanden";
      JOptionPane.showMessageDialog( this, message, "Exportieren", JOptionPane.INFORMATION_MESSAGE );
    }
  }

  public void mnuPrint_actionPerformed( ActionEvent e )
  {
    String message = null;

    if ( tableModel.getRowCount() > 0 )
    {
      DlgPrint dlgPrint = new DlgPrint();
      int optionType = dlgPrint.showDialog();

      if ( optionType == DlgPrint.OK_OPTION )
      {
        if ( dlgPrint.isListeSelected() )
        {
          message = "Anzahl auszudruckende Treffer: " + tableModel.getRowCount();
          int option = JOptionPane.showConfirmDialog( this, message, "Druckanzahl", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE );
          if ( option == JOptionPane.OK_OPTION )
          {
            printTrefferliste();
          }
        }
        if ( dlgPrint.isStammblattSelected() )
        {
          message = "Anzahl auszudruckende Stammblätter: " + tableModel.getRowCount();
          int option = JOptionPane.showConfirmDialog( this, message, "Druckanzahl", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE );
          if ( option == JOptionPane.OK_OPTION )
          {
            printStammblatt();
          }
        }
      }
    }
    else
    {
      message = "Keine Daten zum Drucken vorhanden";
      JOptionPane.showMessageDialog( this, message, "Drucken", JOptionPane.INFORMATION_MESSAGE );

    }
  }

  /**
   * Druckt via Jippsclient eine Trefferliste aus,
   * welche die in der Tabelle enthaltenen Daten beinhaltet
   */
  private void saveTrefferliste( final File csvFile )
  {
    // GlassKomponente wird sichtbar gemacht,
    // damit der Benutzer während der Suche keine Eingaben vornehmen kann
    getGlassPane().setVisible( true );

    // Daten lesen
    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        try
        {
          ArtExportListe liste = new ArtExportListe();
          Map map = liste.getKriteriumList( selKritVO );

          liste.save( artikelAVO, map, csvFile );
          statusBar.setText( "Datei exportiert nach: " + csvFile.getPath() );

          openFileInExternalApplication( csvFile );
        }
        catch ( Exception ex )
        {
          LOG.error( ex );
          JOptionPane.showMessageDialog( null, "Die Exportliste konnte nicht gespeichert werden", "Fehlermeldung", JOptionPane.ERROR_MESSAGE );
          statusBar.setText( " " );
          return null;
        }
        return null;
      }

      public void finished()
      {
        getGlassPane().setVisible( false );
      }

    }; // SwingWorker
    worker.start();
  }

  /**
   * Opens a file in an external Application. Careful: This method is windows-dependent.
   * @param file The file to open
   * @throws IOException In case of problems with the file
   */
  private void openFileInExternalApplication( File file )
  {
    String message = "File: " + file.getPath() + " exportiert \nDatei in Excel öffnen?";
    int option = JOptionPane.showConfirmDialog( this, message, "File", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE );
    if ( option == JOptionPane.OK_OPTION )
    {
      try
      {
        // call garbage collector in case of file is locked
        System.gc();

        String[] commands = { "cmd", "/c", "start", "\"DummyTitle\"", file.getAbsolutePath() };
        Runtime.getRuntime().exec( commands );
      }
      catch ( Exception ex )
      {
        LOG.error( ex );
        JOptionPane.showMessageDialog( this, "Die Datei " + file.getPath() + " konnte nicht automatisch geöffnet werden", "Fehlermeldung",
            JOptionPane.ERROR_MESSAGE );
      }
    }

  }

  /**
   * Druckt via Jippsclient eine Trefferliste aus,
   * welche die in der Tabelle enthaltenen Daten beinhaltet
   */
  private void printTrefferliste()
  {
    // GlassKomponente wird sichtbar gemacht,
    // damit der Benutzer während der Suche keine Eingaben vornehmen kann
    getGlassPane().setVisible( true );

    // Daten lesen
    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        try
        {
          ArtListe liste = new ArtListe();
          Map map = liste.getKriteriumList( selKritVO );

          liste.print( artikelAVO, map );
        }
        catch ( Exception ex )
        {
          LOG.error( ex );
          JOptionPane.showMessageDialog( null, "Die Trefferliste konnte nicht ausgedruckt werden", "Fehlermeldung", JOptionPane.ERROR_MESSAGE );
          statusBar.setText( " " );
          return null;
        }
        return null;
      }

      public void finished()
      {
        statusBar.setText( "Es wurden " + tableModel.getRowCount() + " Treffer gedruckt !" );
        getGlassPane().setVisible( false );
      }
    }; // SwingWorker
    worker.start();
  }

  /**
   * Druckt via Jippsclient Stammblatt / Stammblätter aus
   */
  private void printStammblatt()
  {
    // GlassKomponente wird sichtbar gemacht,
    // damit der Benutzer während der Suche keine Eingaben vornehmen kann
    getGlassPane().setVisible( true );

    // Daten lesen
    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        String artNr = null;
        List listArtNr = new ArrayList();

        try
        {
          for ( int i = 0; i < tableModel.getRowCount(); i++ )
          {
            // Schlüssel abfüllen           
            artNr = (String) tableSorter.getValueAt( i, 0 ).toString();
            listArtNr.add( artNr );
          }

          ArtStammblatt artStammblatt = new ArtStammblatt();
          artStammblatt.print( listArtNr );
        }
        catch ( Exception ex )
        {
          LOG.error( ex );
          JOptionPane.showMessageDialog( null, "Die Stammblätter konnten nicht ausgedruckt werden", "Fehlermeldung", JOptionPane.ERROR_MESSAGE );
          statusBar.setText( " " );
          return null;
        }
        return null;
      }

      public void finished()
      {
        statusBar.setText( "Es wurden " + tableModel.getRowCount() + " Stammblätter gedruckt !" );
        getGlassPane().setVisible( false );
      }
    }; // SwingWorker
    worker.start();
  }

  //********** Pane-spezifische Methoden ***********************************

  private void fillComboboxes()
  {
    try
    {

      //ComboBox kondart_cd füllen
      List<XxDomainVO> alDomain = XxDomain.getDomains( XxDomainFactory.KONDART_CD );
      cbxKondartCd.addItem( new ComboBoxObject( "", "" ) ); // leerer Eintrag
      for ( XxDomainVO xxDomainVO: alDomain )
      {
        cbxKondartCd.addItem( new ComboBoxObject( xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
      }

      // Combobox erfass_proc
      cbxErfassProc.setRenderer( new ComboRenderer() );
      cbxErfassProc.addItem( new ComboBoxObject( "", "" ) );
      alDomain = XxDomain.getDomains( XxDomainFactory.ERFASS_PROC );
      for ( XxDomainVO xxDomainVO: alDomain )
      {
        cbxErfassProc.addItem( new ComboBoxObject( Integer.toString( xxDomainVO.domainLnr ), xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
      }

      // Combobox publ_pflicht
      cbxPublPflicht.setRenderer( new ComboRenderer() );
      cbxPublPflicht.addItem( new ComboBoxObject( "", "" ) );
      alDomain = XxDomain.getDomains( XxDomainFactory.PUBL_PFLICHT );
      for ( XxDomainVO xxDomainVO: alDomain )
      {
        cbxPublPflicht.addItem( new ComboBoxObject( Integer.toString( xxDomainVO.domainLnr ), xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
      }
      // Combobox regausz_grp
      cbxRegauszGrp.setRenderer( new ComboRenderer() );
      cbxRegauszGrp.addItem( new ComboBoxObject( "", "" ) );
      alDomain = XxDomain.getDomains( XxDomainFactory.REGAUSZ_GRP );
      for ( XxDomainVO xxDomainVO: alDomain )
      {
        cbxRegauszGrp.addItem( new ComboBoxObject( xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
      }

      // Combobox publ_kap (alle)
      cbxStatcd3.setRenderer( new ComboRenderer() );
      cbxStatcd3.addItem( new ComboBoxObject( "", "" ) );
      alDomain = XxDomain.getDomains( XxDomainFactory.PUBL_KAP );
      for ( XxDomainVO xxDomainVO: alDomain )
      {
        cbxStatcd3.addItem( new ComboBoxObject( xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
      }
    }
    catch ( Exception ex )
    {
      System.out.println( ex );
    }
  }

  /**
   * Behandelt Mausklicks des Benutzers in Auswahltabelle
   * Bei Doppelclick weiterverzweigen
   * @param e MouseEvent
   */
  protected void tblSel_MouseListener( MouseEvent e )
  {
    setStateFound();

    // bei Doppelclick weiterverzweigen
    if ( e.getClickCount() == 2 )
    {
      if ( mode == SEARCH )
      {
        mnuDetail_actionPerformed( null );
      }
      else if ( mode == SEARCH_LOOKUP )
      {
        mnuOk_actionPerformed( null );
      }
    }
  }

  /**
   * Falls Focus nicht mehr in der Tabelle ist, wird der Suchen-Knopf der Default.
   * @param e FocusEvent
   */
  protected void tblSel_focusLost( FocusEvent e )
  {
    if ( !e.isTemporary() )
    {
      setDefaultButton( btnFind );
    }
  }

  /**
   * Falls der Focus wieder auf der Tabelle ist, wird der Detail-Knopf der Default.
   * @param e FocusEvent
   */
  protected void tblSel_focusGained( FocusEvent e )
  {
    if ( mode == SEARCH )
    {
      setDefaultButton( btnDetail );
    }
    else
    {
      setDefaultButton( btnOk );
    }
  }

  /**
   * Enter-Taste in der Tabelle löst den DefaultButton aus.
   * @param e KeyEvent
   */
  protected void tblSel_keyPressed( KeyEvent e )
  {
    if ( e.getKeyCode() == KeyEvent.VK_ENTER )
    {
      getDefaultButton().doClick();
      e.consume();
    }
  }

  /**
   * Setzt Menus und Buttons in den Initialzustand zurück
   * @param e Mousevent
   */
  protected void header_MouseListener( MouseEvent e )
  {
    setStateNotFound();
  }

  /**
   * Popup-Menu anzeigen nur falls GlassPane nicht sichtbar.
   * @param e Auslösendes Ereignis
   */
  protected void showPopupMenu( MouseEvent e )
  {
    if ( !getGlassPane().isVisible() )
    {
      super.showPopupMenu( e );
    }
  }

  /**
   * Setzt die Spaltenbreite
   */
  private void setColumnWidth()
  {
    int anzCol = tblSel.getModel().getColumnCount();

    for ( int i = 0; i < anzCol; i++ )
    {
      TableColumn tc = tblSel.getColumnModel().getColumn( i );
      int tcWidth = this.getPreferredWidthForColumn( tc );
      tc.setMinWidth( tcWidth );
    }
  }

  /**
   * Berechnet die tatsächlich bevorzugte Breite einer Spalte
   * @param col TableColumn Spalten
   * @return int
   */
  public int getPreferredWidthForColumn( TableColumn col )
  {
    int hw = columnHeaderWidth( col );
    int cw = widestCellInColumn( col );
    return hw > cw ? hw : cw;
  }

  /**
   * Berechnet die tatsächlich bevorzugte Breite des Spaltenkopfs
   * @param col TableColumn Spalte
   * @return int bevorzugte Breite des Spaltenkopfs
   */
  private int columnHeaderWidth( TableColumn col )
  {
    JTableHeader header = tblSel.getTableHeader();

    TableCellRenderer defaultHeaderRenderer = null;

    if ( header != null )
    {
      defaultHeaderRenderer = header.getDefaultRenderer();
    }

    Component comp = defaultHeaderRenderer.getTableCellRendererComponent( tblSel, col.getHeaderValue(), false, false, 0, 0 );
    return comp.getPreferredSize().width;
  }

  /**
   * Berechnet die tatsächlich bevorzugte Breite der Spalte anhand der Zellinhalten
   * @param col TableColumn Spalte
   * @return int
   */
  private int widestCellInColumn( TableColumn col )
  {
    int width = 0;
    int maxw = 0;
    int c = col.getModelIndex();

    for ( int r = 0; r < tblSel.getRowCount(); ++r )
    {
      TableCellRenderer renderer = tblSel.getCellRenderer( r, c );
      Component comp = renderer.getTableCellRendererComponent( tblSel, tblSel.getValueAt( r, c ), false, false, r, c );
      width = comp.getPreferredSize().width;
      maxw = width > maxw ? width : maxw;
    }
    return maxw;
  }

  private void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( this.getRootPane(), "intro" );
    HelpUtil.getInstance().enableHelpOnButton( btnHelp, "artikel" );
    HelpUtil.getInstance().enableHelpOnButton( mnuHelp, "intro" );
    HelpUtil.getInstance().enableHelp( mainPanel, "artikel" );
    HelpUtil.getInstance().enableHelp( pnlSelKrit, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( pnlSelTable, "artikel.suchresultat" );
    HelpUtil.getInstance().enableHelp( lblArtNr, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblBezArt, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblErfassProc, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblFormartCd, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblkondartCd, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblPublPflicht, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblRegauszGrp, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblStatcd3, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( txfArtNr, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( txfBezArt, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( txfFormartCd, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( cbxErfassProc, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( cbxKondartCd, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( cbxPublPflicht, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( cbxRegauszGrp, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( cbxStatcd3, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( ckbSperrCdNot, "artikel.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( ckbSperrCdYes, "artikel.selektionskriterien" );
  }

  //********** Authorization ***************************************************

  public static String[] getAuthzSpecifications()
  {
    return new String[] { "*New    = Erstellen/Artikel" };
  }

}
