package ch.ige.art.client.art;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.ArtextVO;

/**
 * <p>Überschrift: Table Model für die Textinformationen</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Texte im Anzeigemodus verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: AtmArtTexte.java,v 1.13 2008/07/02 11:00:20 heinzer Exp $
 */

public class AtmArtTexte extends AbstractTableModel
{
  private final String[] columnNames = new String[] { "Textart", "Sprache", "Text" };

  private ArtextVO[] tableData = new ArtextVO[] {};

  public AtmArtTexte()
  {
  }

  /**
   * Gibt Anzahl Spalten zurück
   * @return Anzahl Spalten
   */
  public int getColumnCount()
  {
    return columnNames.length;
  }

  /**
   * Gibt Anzahl Zeilen zurück
   * @return Anzahl Zeilen
   */
  public int getRowCount()
  {
    return tableData.length;
  }

  /**
   * Gibt Spaltenüberschrift der übergebenen Spalte zurück
   * @param column Spalte deren Spaltenüberschrift angefordert wird
   * @return Spaltenüberschrift
   */
  public String getColumnName( int column )
  {
    return columnNames[column];
  }

  /**
   * Gibt den Wert einer Zelle zurück
   * @param row Zeile
   * @param col Spalte
   * @return Wert der entsprechenden Zelle
   */
  public Object getValueAt( int row, int col )
  {
    switch( col )
    {
      case 0:
        String bezArtTextArtCd = XxDomain.getBez( tableData[row].artTextArtCd, XxDomainFactory.ARTTEXTART_CD );
        return ( tableData[row].artTextArtCd + ", " + bezArtTextArtCd );
      case 1:
        return tableData[row].sprachCd;
      case 2:
        return tableData[row].text;
    }
    return null;
  }

  /**
   * Setzt den Wert einer Zelle
   * @param value Wert der gesetzt wird
   * @param row Zeile
   * @param col Spalte
   */
  public void setValueAt( Object value, int row, int col )
  {
    switch( col )
    {
      case 0:
        int i = value.toString().indexOf( "," );
        tableData[row].artTextArtCd = value.toString().substring( 0, i );
        break;
      case 1:
        tableData[row].sprachCd = value.toString();
        break;
      case 2:
        tableData[row].text = value == null ? "" : value.toString();
        break;
    }
  }

  /**
   * Setzt die Daten ins TableModel
   * Angezeigt werden nur die gültigen Textarten
   * Textarten die nicht mehr gültig sind, werden nicht angezeigt
   * @param data Alle Texte eines Artikels gekapselt in einem ArtextVO
   */
  public void setData( ArtextVO[] data )
  {
    ArrayList alArtextVO = new ArrayList();

    for ( int i = 0; i < data.length; i++ )
    {
      if ( data[i].isValidTextart() )
      {
        alArtextVO.add( data[i] );
      }
    }

    tableData = (ArtextVO[]) alArtextVO.toArray( new ArtextVO[0] );
    fireTableDataChanged();
  }

  /**
   * Daten im TableModel löschen
   */
  public void clear()
  {
    fireTableDataChanged();
  }
}
