package ch.ige.art.client.form;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;

import org.apache.log4j.Logger;

import ch.ige.art.client.CustomDlgInfo;
import ch.ige.art.client.DlgArtikelTexte;
import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.UserMessage;
import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.BformDruckVO;
import ch.ige.art.xml.FormStammblatt;
import ch.ige.arthelp.HelpUtil;
import ch.ige.authz.gui.AuthzJMenuItem;
import ch.ige.authz.gui.GUIAuthorizer;
import ch.ige.main.client.DetailPane;
import ch.ige.main.client.MainController;
import ch.ige.util.client.TableSorterModel;

/**
 * <p>Überschrift: Formular Detail</p>
 * <p>Beschreibung: Detailansicht der Formulardaten</p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Anita Rueegsegger
 * @version $Id: FormDtl.java,v 1.26 2008/07/02 11:00:20 heinzer Exp $
 */

public class FormDtl extends DetailPane
{
  // Logging
  private static final Logger LOG = Logger.getLogger( FormDtl.class );

  private UserMessage userMessage = new UserMessage();

  private DlgArtikelTexte editor = new DlgArtikelTexte();

  // Schlüssel
  private String formartCd = null;

  // Tabellen
  private AtmFormTexte atmFormTexte = null;

  private TableSorterModel sorterTexte = null;

  // Formular-Stammblatt
  private FormStammblatt formStammblatt = null;

  //Frame konstruieren
  private JPanel mainPanel = new JPanel();

  private JPanel pnlFormart = new JPanel();

  private JPanel pnlFormtxt = new JPanel();

  private JScrollPane scpTexte = new JScrollPane();

  private BorderLayout lytBorderMain = new BorderLayout();

  private BorderLayout lytBorderPnlFormart = new BorderLayout();

  private GridBagLayout gridBagLayout1 = new GridBagLayout();

  private JLabel lblDruckSystem = new JLabel();

  private JTextField txfBez = new JTextField();

  private JTextField txfFormartCd = new JTextField();

  private JLabel lblBez = new JLabel();

  private JTextField txfDruckSystem = new JTextField();

  private JLabel lblFormartCd = new JLabel();

  private JTable tblTexte;

  public JMenuItem mnuDuplicate = new AuthzJMenuItem();

  /**
   * parameterloser Konstruktor
   */
  public FormDtl()
  {
    try
    {
      jbInit();
      initHelp();
      GUIAuthorizer.authorizeFieldsIn( this );
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
    }
  }

  /**
   * Konstruktor mit Parameter für Hauptfenster und Suchmodus
   * @param mainController Hauptfenster, zuständig für Panelsteuerung
   * @param mode Suchmodus<br>
   *    0 - Suche in Auswahl<br>
   *    1 - Suche für Lookup
   *
   */
  public FormDtl( MainController mainController, int mode )
  {
    super( mainController, mode );
    try
    {
      jbInit();
      initHelp();
      GUIAuthorizer.authorizeFieldsIn( this );
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
    }
  }

  /**
   * Komponente initialisieren
   * @exception Exception
   */
  private void jbInit() throws Exception
  {
    atmFormTexte = new AtmFormTexte();
    sorterTexte = new TableSorterModel( atmFormTexte );
    tblTexte = new JTable( sorterTexte );

    mainPanel.setLayout( lytBorderMain );
    mainPanel.setBorder( new TitledBorder( "Formular" ) );
    pnlFormtxt.setLayout( gridBagLayout1 );
    pnlFormart.setLayout( lytBorderPnlFormart );
    pnlFormtxt.setMaximumSize( new Dimension( 300, 130 ) );
    pnlFormtxt.setMinimumSize( new Dimension( 300, 130 ) );
    pnlFormtxt.setPreferredSize( new Dimension( 300, 130 ) );
    scpTexte.setBorder( new TitledBorder( "Texte" ) );
    scpTexte.setOpaque( false );

    lblDruckSystem.setToolTipText( "druck_system" );
    lblDruckSystem.setHorizontalAlignment( SwingConstants.RIGHT );
    lblDruckSystem.setText( "Drucksystem:" );
    lblBez.setMaximumSize( new Dimension( 140, 16 ) );
    lblBez.setMinimumSize( new Dimension( 140, 16 ) );
    lblBez.setPreferredSize( new Dimension( 140, 16 ) );
    lblBez.setToolTipText( "bez" );
    lblBez.setHorizontalAlignment( SwingConstants.RIGHT );
    lblBez.setText( "Formular-Bezeichnung:" );
    lblFormartCd.setPreferredSize( new Dimension( 200, 15 ) );
    lblFormartCd.setToolTipText( "formart_cd" );
    lblFormartCd.setHorizontalAlignment( SwingConstants.RIGHT );
    lblFormartCd.setText( "Formular:" );
    txfFormartCd.setMaximumSize( new Dimension( 400, 20 ) );
    txfFormartCd.setMinimumSize( new Dimension( 400, 20 ) );
    txfFormartCd.setPreferredSize( new Dimension( 400, 20 ) );
    txfFormartCd.setEditable( false );
    txfBez.setMaximumSize( new Dimension( 400, 20 ) );
    txfBez.setMinimumSize( new Dimension( 400, 20 ) );
    txfBez.setPreferredSize( new Dimension( 400, 20 ) );
    txfBez.setEditable( false );
    txfDruckSystem.setMaximumSize( new Dimension( 400, 20 ) );
    txfDruckSystem.setMinimumSize( new Dimension( 400, 20 ) );
    txfDruckSystem.setPreferredSize( new Dimension( 400, 20 ) );
    txfDruckSystem.setEditable( false );

    pnlFormtxt.add( lblBez, new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 5, 0, 0 ), 0, 0 ) );
    pnlFormtxt.add( lblDruckSystem, new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 5, 0, 0 ),
        0, 0 ) );
    pnlFormtxt.add( lblFormartCd, new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0,
        0 ) );
    pnlFormtxt.add( txfBez, new GridBagConstraints( 1, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ), 0, 0 ) );
    pnlFormtxt.add( txfFormartCd, new GridBagConstraints( 1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
        new Insets( 3, 10, 3, 0 ), 0, 0 ) );
    pnlFormtxt.add( txfDruckSystem, new GridBagConstraints( 1, 2, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets( 3, 10, 3,
        0 ), 0, 0 ) );

    tblTexte.setPreferredScrollableViewportSize( new Dimension( 450, 50 ) );
    scpTexte.getViewport().add( tblTexte, null );
    pnlFormart.add( scpTexte, BorderLayout.CENTER );

    mainPanel.add( pnlFormart, BorderLayout.CENTER );
    mainPanel.add( pnlFormtxt, BorderLayout.NORTH );
    centerPanel.add( mainPanel, BorderLayout.CENTER );

    //********** Tabellen ******************************************************
    // Spaltenbreite setzten
    this.tblTexte.getColumnModel().getColumn( 0 ).setPreferredWidth( 200 );
    this.tblTexte.getColumnModel().getColumn( 1 ).setPreferredWidth( 40 );
    this.tblTexte.getColumnModel().getColumn( 2 ).setPreferredWidth( 300 );

    // Spaltenausrichtung setzen
    DefaultTableCellRenderer rendererRigth = new DefaultTableCellRenderer();
    rendererRigth.setHorizontalAlignment( SwingConstants.RIGHT );
    DefaultTableCellRenderer rendererCenter = new DefaultTableCellRenderer();
    rendererCenter.setHorizontalAlignment( SwingConstants.CENTER );
    DefaultTableCellRenderer rendererLeft = new DefaultTableCellRenderer();
    rendererLeft.setHorizontalAlignment( SwingConstants.LEFT );
    //tblTexte.getColumnModel().getColumn(0).setCellRenderer(rendererRigth);
    //tblTexte.getColumnModel().getColumn(1).setCellRenderer(rendererRigth);
    //tblTexte.getColumnModel().getColumn(2).setCellRenderer(rendererRigth);

    // Tabellen Listener
    tblTexte.addMouseListener( new java.awt.event.MouseAdapter()
    {
      public void mouseClicked( MouseEvent e )
      {
        tblTexte_mouseClicked( e );
      }
    } );

    //********** Menus *********************************************************

    mnuDuplicate.setText( "Duplizieren" );
    mnuDuplicate.setMnemonic( 'U' );
    menFile.insert( mnuDuplicate, 2 );

    menFile.remove( mnuDelete );
    toolBar.remove( btnDelete );
    btnHelp.setEnabled( true );
    mnuHelp.setEnabled( true );

    //********** Menu-Funktionen*********************************************
    mnuDuplicate.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        mnuDuplicate_actionPerformed( e );
      }
    } );
    tblTexte.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

  } // jbInit

  //********** Öffentliche Standard-Methoden *******************************

  /**
   * Gibt den Titel zurück
   * @return  Titel
   */
  public String getTitle()
  {
    return "Formular Detail";
  }

  /**
   * Setzt den Schlüssel
   * @param formartCd String Schlüssel Artikeldaten
   */
  public void setKey( String formartCd )
  {
    this.formartCd = formartCd;
  }

  /**
   * gibt Text in der Statusbar aus
   * @param text  Text
   */
  public void setStatusBar( String text )
  {
    statusBar.setText( text );
  }

  //********** Lokale Standard-Methoden **********************************

  /**
   * aktualisiert die Daten
   */
  private void refresh()
  {
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
    read();
    statusBar.setText( "Daten gelesen !" );
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
  }

  /**
   * Liest die Daten von der Datenbank
   * Füllt Panel mit den Werten
   */
  public void read()
  {
    // Daten lesen
    BformDruckVO bformDruckVO;
    try
    {
      bformDruckVO = ArtikelDelegate.getInstance().getFormular( formartCd );
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
      statusBar.setText( " " );
      return;
    }
    setData( bformDruckVO );
  }

  /**
   * Füllt Controls mit den Werten des bformDruckVO
   * @param bformDruckVO ValueObject mit Daten eines Formulars
   */
  private void setData( BformDruckVO bformDruckVO )
  {
    txfFormartCd.setText( bformDruckVO.bformartVO.formartCd );
    txfBez.setText( bformDruckVO.bformartVO.bez == null ? "" : bformDruckVO.bformartVO.bez );

    // DruckSystem
    String bezDruckSystem = XxDomain.getBez( bformDruckVO.bformartVO.druckSystem, XxDomainFactory.DRUCK_SYSTEM );
    txfDruckSystem.setText( new ComboBoxObject( bformDruckVO.bformartVO.druckSystem, bezDruckSystem ).toString() );

    // Text-Tabelle
    atmFormTexte.setData( bformDruckVO.bformtxtDruckVO );
  }

  //********** Ereignisbehandlung zu Menus *******************************

  public void mnuInfo_actionPerformed( ActionEvent e )
  {
    new CustomDlgInfo( (JFrame) this.mainController.getWindow(), true ).setVisible( true );
  }

  /**
   * Formular Erstellen
   * @param e ActionEvent
   */
  protected void mnuNew_actionPerformed( ActionEvent e )
  {
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
    statusBar.setText( "Datensatz erfassen" );

    FormWizardMediator med = new FormWizardMediator( (Frame) mainController.getWindow() );
    String formartCode = med.newForm();

    if ( formartCode != null ) // null falls insert abgebrochen
    {
      formartCd = formartCode;
      refresh();
      statusBar.setText( "Datensatz gespeichert" );
    }
    else
    {
      statusBar.setText( "Erfassung wurde abgebrochen" );
    }
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
  }

  /**
   * Formular Bearbeiten
   * @param e ActionEvent
   */
  protected void mnuEdit_actionPerformed( ActionEvent e )
  {
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
    statusBar.setText( "Datensatz bearbeiten" );

    FormWizardMediator med = new FormWizardMediator( (Frame) mainController.getWindow() );
    boolean edit = med.editForm( formartCd );

    if ( edit ) // false wenn Mutation abgebrochen
    {
      refresh();
      statusBar.setText( "Datensatz gespeichert" );
    }
    else
    {
      statusBar.setText( "Mutation wurde abgebrochen" );
    }
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
  }

  /**
   * Formular Duplizieren
   * @param e ActionEvent
   */
  protected void mnuDuplicate_actionPerformed( ActionEvent e )
  {
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
    statusBar.setText( "Datensatz duplizieren" );

    FormWizardMediator med = new FormWizardMediator( (Frame) mainController.getWindow() );
    String rformartCd = med.dupliForm( formartCd );

    if ( rformartCd != null ) // null falls insert abgebrochen
    {
      formartCd = rformartCd;
      refresh();
      statusBar.setText( "Datensatz gespeichert" );
    }
    else
    {
      statusBar.setText( "Duplizieren wurde abgebrochen" );
    }
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
  }

  protected void mnuPrint_actionPerformed( ActionEvent e )
  {
    print( formartCd );
  }

  /**
   * Refresh
   * @param e ActionEvent
   */
  protected void mnuRefresh_actionPerformed( ActionEvent e )
  {
    refresh();
  }

  //********** Pane-spezifische Methoden ***********************************

  protected void tblTexte_mouseClicked( MouseEvent e )
  {
    // bei Doppelclick Text in einem Dialogfenster anzeigen
    if ( e.getClickCount() == 2 )
    {
      editor.showDlgText( (String) atmFormTexte.getValueAt( tblTexte.getSelectedRow(), 2 ) );
    }
  }

  /**
   * Druckt das aktuelle Formular via Jippsclient aus
   * @param formartCd String Formular-Code
   */
  private void print( String formartCd )
  {
    try
    {
      formStammblatt = new FormStammblatt();
      formStammblatt.print( formartCd );
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      JOptionPane.showMessageDialog( null, "Das Stammblatt konnte nicht ausgedruckt werden", "Fehlermeldung", JOptionPane.ERROR_MESSAGE );
      statusBar.setText( " " );
    }
  }

  //********** Authorization ***************************************************

  public static String[] getAuthzSpecifications()
  {
    return new String[] { "*New    = Erstellen/Formular", "*Edit   = MutierenLight/Formular", "*Duplicate = Erstellen/Formular" };
  }

  private void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( this.getRootPane(), "intro" );
    HelpUtil.getInstance().enableHelpOnButton( btnHelp, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelpOnButton( mnuHelp, "intro" );
    HelpUtil.getInstance().enableHelp( mainPanel, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( pnlFormart, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( pnlFormtxt, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( lblFormartCd, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( lblBez, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( lblDruckSystem, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( lblFormartCd, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( txfBez, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( txfFormartCd, "formulare.detaildaten" );
    HelpUtil.getInstance().enableHelp( tblTexte, "formulare.detaildaten" );
  }
}
