package ch.ige.art.client.util;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

import ch.ige.art.client.art.*;
import ch.ige.authz.client.*;
import ch.ige.util.client.*;
import ch.ige.art.xml.*;


/**
 * <p>Überschrift: Navigation Tool Bar</p>
 * <p>Beschreibung: Navigation Tool Bar die über folgende Buttons verfügt
 *  Zeile einfügen, Zeile löschen
 *  Zur ersten Zeile springen, Zur letzten Zeile springen
 *  Zur nächsten Zeile springen, Zur vorangehenden Zeile springen
 * </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: TblNavToolBar.java,v 1.8 2006/12/13 10:08:31 ruegsegger Exp $
 */

public class TblNavToolBar extends JToolBar
    implements ActionListener, MouseListener
{

  // Images
  private ImageIcon imgInsertRow = IgeImage.imgInsertRow;
  private ImageIcon imgDeleteRow = IgeImage.imgDeleteRow;
  private ImageIcon imgFirstRow = IgeImage.imgFirstRow;
  private ImageIcon imgLastRow = IgeImage.imgLastRow;
  private ImageIcon imgNextRow = IgeImage.imgNextRow;
  private ImageIcon imgPriorRow = IgeImage.imgPriorRow;

  // Buttons
  private JButton btnInsert;
  private JButton btnDelete;
  private JButton btnFirst;
  private JButton btnPrior;
  private JButton btnNext;
  private JButton btnLast;

  // Borders
  private Border emptyButtonBorder;
  private Border normalButtonBorder;

  // Tabelle
  private TblArtikelWizard table;

  private AuthorizationProxy authzationProxy = AuthMain.getAuthzProxy();

  public TblNavToolBar(TblArtikelWizard table)
  {
    super();
    this.table = table;
    setFloatable(false);
    initButtons();
  }

  /**
   * Buttons initialisieren und Tooltip setzen
   */
  private void initButtons()
  {
    btnInsert = initButton(imgInsertRow);
    btnDelete = initButton(imgDeleteRow);
    btnFirst = initButton(imgFirstRow);
    btnPrior = initButton(imgPriorRow);
    btnNext = initButton(imgNextRow);
    btnLast = initButton(imgLastRow);

    btnInsert.setToolTipText("Neue Zeile einfügen");
    btnDelete.setToolTipText("Aktuelle Zeile löschen");
    btnFirst.setToolTipText("Zur ersten Zeile springen");
    btnPrior.setToolTipText("Zur vorhergehenden Zeile springen");
    btnNext.setToolTipText("Zur nächsten Zeile springen");
    btnLast.setToolTipText("Zur letzten Zeile springen");
  }

  /**
   * Initialisierungsvorgang pro Button
   * Buttonerstellung, Hinzufügen von Listener, Hinzufügen zu ToolBar
   * @param imageIcon Icon für den Button
   * @return initialisierter Button
   */
  private JButton initButton(ImageIcon imageIcon)
  {
    JButton jButton = new JButton(imageIcon);
    jButton.addActionListener(this);
    jButton.addMouseListener(this);
    jButton.setMargin(new Insets(0, 0, 0, 0));
    jButton.setEnabled(false);
    add(jButton);
    return jButton;
  }

  public void addNotify()
  {
    super.addNotify();

    emptyButtonBorder = new EmptyBorder(btnInsert.getBorder().getBorderInsets(btnInsert));
    normalButtonBorder = UIManager.getBorder("Button.border");

    btnInsert.setBorder(emptyButtonBorder);
    btnDelete.setBorder(emptyButtonBorder);
    btnFirst.setBorder(emptyButtonBorder);
    btnPrior.setBorder(emptyButtonBorder);
    btnNext.setBorder(emptyButtonBorder);
    btnLast.setBorder(emptyButtonBorder);
  }

  /**
   * Buttons werden je nach Tabellenzustand enabled oder disabled
   */
  public void enableButtons()
  {
    TableModel tableModel = table.getModel();   // DefaultTableModel
    int row = tableModel.getRowCount();         // Anzahl Rows
    int selRow = table.getSelectedRow();        // Index selektierte Row
    boolean editAllowed = authzationProxy.accessAllowed("MutierenVoll", "Artikel") &&
                          authzationProxy.accessAllowed("MutierenVoll", "Formular");

    btnInsert.setEnabled(false);
    btnDelete.setEnabled(false);
    btnNext.setEnabled(false);
    btnLast.setEnabled(false);
    btnFirst.setEnabled(false);
    btnPrior.setEnabled(false);

    if (row > 0)                    // Rows vorhanden
    {
      if (selRow != -1)             // Row selektiert
      {

        if (editAllowed) // delete nur erlaubt bei vollrecht
        {
          if (tableModel instanceof DtmArtPreise && tableModel.getValueAt(selRow, 4).equals("I"))
          {
            btnDelete.setEnabled(true);
          }
          else if (!(tableModel instanceof DtmArtPreise))
          {
            btnDelete.setEnabled(true);
          }
        }

        if (selRow != 0)            // selektierte Row nicht erste Zeile
        {
          btnPrior.setEnabled(true);
          btnFirst.setEnabled(true);
        }
        if (selRow != row-1)        // selektierte Row nicht letzte Zeile
        {
          btnNext.setEnabled(true);
          btnLast.setEnabled(true);
        }
      }
      else                          // keine Row selektiert
      {
        btnFirst.setEnabled(true);
        btnLast.setEnabled(true);
      }
    }

    btnInsert.setEnabled(editAllowed); // insert nur erlaubt bei Vollrecht
  }


//** Mouse Events *************************************************************

  public void mouseClicked(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e)
  {
    JButton button = (JButton) e.getSource();

    if (button.isEnabled())
    {
      button.setBorder(normalButtonBorder);
    }
    else
    {
      button.setBorder(emptyButtonBorder);
    }
  }

  public void mouseEntered(MouseEvent e)
  {

    JButton button = (JButton) e.getSource();

    if (button.isEnabled())
    {
      button.setBorder(normalButtonBorder);
    }
  }

  public void mouseExited(MouseEvent e)
  {
    JButton button = (JButton) e.getSource();
    button.setBorder(emptyButtonBorder);
  }


//*** Action Events ***********************************************************

  public void actionPerformed(ActionEvent e)
  {
    JButton button = (JButton) e.getSource();
    if (button == btnInsert)
    {
      btnInsert_actionPerformed(e);
    }
    else if (button == btnDelete)
    {
      btnDelete_actionPerformed(e);
    }
    else if (button == btnFirst)
    {
      btnFirst_actionPerformed(e);
    }
    else if (button == btnPrior)
    {
      btnPrior_actionPerformed(e);
    }
    else if (button == btnNext)
    {
      btnNext_actionPerformed(e);
    }
    else if (button == btnLast)
    {
      btnLast_actionPerformed(e);
    }
  }

  /**
   * Zeile einfügen
   * @param e ActionEvent
   */
  private void btnInsert_actionPerformed(ActionEvent e)
  {
    table.add();
    enableButtons();
  }

  /**
   * Zeile löschen
   * @param e Action Event
   */
  private void btnDelete_actionPerformed(ActionEvent e)
  {
    table.remove();
    enableButtons();
  }

  /**
   * Zur ersten Zeile springen
   * @param e Action Event
   */
  private void btnFirst_actionPerformed(ActionEvent e)
  {
    table.first();
    enableButtons();
  }

  /**
   * Zur nächsten Zeile springen
   * @param e Action Event
   */
  private void btnNext_actionPerformed(ActionEvent e)
  {
    table.next();
    enableButtons();
  }

  /**
   * Zur vorangehenden Zeile springen
   * @param e Action Event
   */
  private void btnPrior_actionPerformed(ActionEvent e)
  {
    table.prior();
    enableButtons();
  }

  /**
   * Zur letzten Zeile springen
   * @param e Action Event
   */
  private void btnLast_actionPerformed(ActionEvent e)
  {
    table.last();
    enableButtons();
  }

  public TblNavToolBar()
  {
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception
  {
    this.setBackground(UIManager.getColor("control"));
  }
}


