package ch.ige.art.server;

import java.sql.SQLException;
import java.util.ArrayList;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.art.server.ArtBO;
import ch.ige.art.server.FormBO;
import ch.ige.art.server.dao.InfoDAO;
import ch.ige.art.server.dao.MaDAO;
import ch.ige.art.server.dao.XxDomainDAO;
import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.ArtikelSelKritVO;
import ch.ige.art.vo.BformDruckVO;
import ch.ige.art.vo.MaVO;
import ch.ige.art.vo.XxDomainSelKritVO;

/**
 * SessionFacade für Artikel
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtikelFacadeBean.java,v 1.38 2006/12/13 13:07:30 ruegsegger Exp $
 */


/**
 * <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *  name="ArtikelFacade"  
 *  description="An EJB named ArtikelFacade"
 *  display-name="ArtikelFacade"
 *  view-type="remote"
 *  jndi-name="ArtikelFacade"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction type="Required"
 * <!-- end-xdoclet-definition --> 
 */
public class ArtikelFacadeBean implements SessionBean
{
  private SessionContext sessionContext;

  private static final Logger LOG = Logger.getLogger( ArtikelFacadeBean.class );

  public void ejbCreate()
  {
  }

  public void ejbRemove()
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void setSessionContext( SessionContext sessionContext )
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Gibt das Value Object des Artikels zurück.
   * @param     artNr          Artikelnummer
   * @return    ArtikelGuiAVO  Value Object der Adresse
   * @exception Exception
   * 
   * @ejb.interface-method 
   */
  public ArtikelGuiAVO getArtikel( String artNr ) throws Exception
  {
    return new ArtBO().getArtikel( artNr );
  }

  /**
   * Kreiert einen Artikel
   * @param artikelGuiAVO Value Object des Artikels
   * @exception Exception
   * 
   * @ejb.interface-method 
   */
  public void createArtikel( ArtikelGuiAVO artikelGuiAVO, String domainMessage )
  {
    new ArtBO().createArtikel( artikelGuiAVO, domainMessage );
  }

  /**
   * Aktualisiert einen Artikel
   * @param artikelGuiAVO Value Object des Artikels
   * @exception Exception
   * 
   * @ejb.interface-method
   */
  public void setArtikel( ArtikelGuiAVO artikelGuiAVO, String domainMessage )
  {
    new ArtBO().setArtikel( artikelGuiAVO, domainMessage );
  }

  /**
   * Findet alle Artikel, auf welche die Selektionskriterien zutreffen.
   *
   * @param artikelSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit ArtikelSucheAVO-Objekten mit den gefundenen Adressen
   * @exception Exception
   * 
   * @ejb.interface-method
   */
  public ArrayList findArtikel( ArtikelSelKritVO artikelSelKritVO ) throws Exception
  {
    return new ArtBO().findArtikel( artikelSelKritVO );
  }

  /**
   * Gibt das Value Object des Formular zurück.
   * @param     formartCd      Formartcode
   * @return    BformDruckVO   Value Object des Formular
   * @exception Exception
   * 
   * @ejb.interface-method
   */
  public BformDruckVO getFormular( String formartCd ) throws Exception
  {
    return new FormBO().getFormular( formartCd );
  }

  /**
   * Kreiert ein Formular
   * @param bformDruckVO  Value Object des Formulars
   * @exception Exception
   * 
   * @ejb.interface-method
   */
  public void createFormular( BformDruckVO bformDruckVO ) throws Exception
  {
    new FormBO().createFormular( bformDruckVO );
  }

  /**
   * Aktualisiert ein Formular
   * @param bformDruckVO Value Object des Formulars
   * @exception Exception
   * 
   * @ejb.interface-method
   */
  public void setFormular( BformDruckVO bformDruckVO ) throws Exception
  {
    new FormBO().setFormular( bformDruckVO );
  }

  /**
   * Findet alle Formulare, auf welche die Selektionskriterien zutreffen.
   *
   * @param formartCd     Formartcode
   * @param bez           Bezeichnung
   * @param druckSystem   Drucksystem
   * @return ArrayList mit ArtikelSucheAVO-Objekten mit den gefundenen Adressen
   * @exception Exception
   * 
   * @ejb.interface-method
   */
  public ArrayList findFormular( String formartCd, String bez, String druckSystem ) throws Exception
  {
    return new FormBO().findFormular( formartCd, bez, druckSystem );
  }

  /**
   * Gibt die Mitarbeiter-Informationen zurück (Bagisuser)
   * @param vmsUsername User aus Systemtabelle
   * @return MaVO Mitarbeiter-Informationen
   * 
   * @ejb.interface-method
   */
  public MaVO getMa( String vmsUsername )
  {
    try
    {
      return new MaDAO().select( vmsUsername );
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method getMaNr error: " + ex.getMessage() );
      throw new EJBException( " | Mitarbeiter kann nicht selektiert werden | " + ex );
    }
  }

  /**
   * Gibt eine ArrayList der gefundenen Domains zurück.
   *
   * @param   xxDomainSelKritVO  VO mit Selektionskriterien
   * @return  ArrayList mit XxDomainVOs mit den gefundenen Domains
   * 
   * @ejb.interface-method
   */
  public ArrayList findXxDomain( XxDomainSelKritVO xxDomainSelKritVO )
  {
    try
    {
      return new XxDomainDAO().select( xxDomainSelKritVO );
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method getXxDomain error: " + ex.getMessage() );
      throw new EJBException( " | Domain Werte können nicht geladen werden | " + ex );
    }
  }

  /**
   * Gibt eine ArrayList aller Domains zurück
   *
   * @return  ArrayList mit XxDomainVOs aller Domains
   * 
   * @ejb.interface-method
   */
  public ArrayList findXxDomain()
  {
    try
    {
      return new XxDomainDAO().select();
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method getXxDomain error: " + ex.getMessage() );
      throw new EJBException( " | Domain Werte können nicht geladen werden | " + ex );
    }
  }

  /**
   * Gibt die Meta-Daten von der Bagis-DB zurück.
   * @return String URL
   * @throws Exception Unerwarteter Fehlersituation
   * 
   * @ejb.interface-method
   */
  public String getBagisDBURL() throws Exception
  {
    return new InfoDAO().getBagisDbURL();
  }
}
