package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import ch.ige.art.vo.ArtikelAVO;
import ch.ige.art.vo.ArtikelSelKritVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt für Artikelsuche es werden nur die nötigen Methoden
 * implementiert.
 *
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtikelSucheDAO.java,v 1.11 2008/06/25 13:44:29 ruegsegger Exp $
 */

public class ArtikelSucheDAO
{

  /**
   * Selektiert alle Artikel, welche die Suchkriterien erfüllen
   *
   * @param  artikelSelKritVO   Value Object mit Selektionskriterien
   * @return ArrayList          ArrayList mit Value Object der gefundenen Artikeln
   * @throws SQLException       Unerwarteter SQL-Fehler
   */
  public ArrayList<ArtikelAVO> select( ArtikelSelKritVO artikelSelKritVO ) throws SQLException
  {
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;
    String query = null;
    ArrayList<ArtikelAVO> al = new ArrayList<ArtikelAVO>();
    String and = "";
    String or = "";

    StringBuffer select = new StringBuffer();
    StringBuffer from = new StringBuffer();
    StringBuffer join = new StringBuffer();
    StringBuffer where = new StringBuffer();
    StringBuffer order = new StringBuffer();

    select.append( "SELECT trim(a.art_nr) as art_nr, " );
    select.append( "to_number(a.art_nr), " );
    select.append( "trim(b.bez_art) as bez_art, " );
    select.append( "trim(a.kz_gratis) as kz_gratis, " );
    select.append( "trim(a.kondart_cd) as kondart_cd, " );
    select.append( "trim(a.sperr_cd) as sperr_cd, " );
    select.append( "trim(a.statcd_1) as statcd_1, " );
    select.append( "trim(a.statcd_2) as statcd_2, " );
    select.append( "trim(a.statcd_3) as statcd_3, " );
    select.append( "trim(c.druck_modus) as druck_modus, " );
    select.append( "trim(c.einz_faktpos) as einz_faktpos, " );
    select.append( "trim(c.formart_cd) as formart_cd, " );
    select.append( "trim(c.kz_einschr) as kz_einschr, " );
    select.append( "trim(c.publ_pflicht) as publ_pflicht, " );
    select.append( "c.regausz_grp, " );
    select.append( "trim(c.frist_art) as frist_art, " );
    select.append( "trim(c.frist_einh) as frist_einh, " );
    select.append( "c.frist_dauer, " );
    select.append( "trim(c.kz_besch) as kz_besch, " );
    select.append( "c.ereig_plaus, " );
    select.append( "c.erfass_proc, " );
    select.append( "trim(c.process_modus) as process_modus, " );
    select.append( "c.process_nr " );

    from.append( "FROM artikel a,  artbez b, bartikel c " );
    where.append( "WHERE a.art_nr = b.art_nr(+) AND a.art_nr = c.art_nr(+) AND " );
    order.append( " ORDER BY 2 " );

    // Suchbegriff Artikel-Nr
    if ( !artikelSelKritVO.artNr.equals( "" ) )
    {
      StringBuffer inBuffer = new StringBuffer();
      StringBuffer betweenBuffer = new StringBuffer();

      this.divideField( artikelSelKritVO.artNr, "a.art_nr", inBuffer, betweenBuffer );

      where.append( "( " );
      if ( inBuffer != null && !inBuffer.equals( "" ) && inBuffer.length() > 0 )
      {
        where.append( "a.art_nr IN ( " );
        where.append( inBuffer );
        where.append( " )" );
        or = " OR ";
        and = " AND ";
      }

      if ( betweenBuffer != null && !betweenBuffer.equals( "" ) && betweenBuffer.length() > 0 )
      {
        where.append( or );
        where.append( betweenBuffer );
        and = " AND ";
        or = "";
      }
      where.append( " )" );
    }

    // Suchbegriff Bezeichnung
    if ( !artikelSelKritVO.bezArt.equals( "" ) )
    {
      where.append( and );
      where.append( "trim(lower(b.bez_art)) " );
      if ( artikelSelKritVO.bezArt.indexOf( '%' ) == -1 )
      {
        where.append( "= " );
      }
      else
      {
        where.append( " LIKE " );
      }
      where.append( "'" );
      where.append( artikelSelKritVO.bezArt.toLowerCase() );
      where.append( "'" );
      and = " AND ";
    }

    // Suchbegriff Ereignisgruppe
    if ( !artikelSelKritVO.kondartCd.equals( "" ) )
    {
      where.append( and );
      where.append( "trim(upper(a.kondart_cd)) = " );
      where.append( "'" );
      where.append( artikelSelKritVO.kondartCd.toUpperCase() );
      where.append( "'" );
      and = " AND ";
    }

    // Suchbegriff Erfassprozess
    if ( artikelSelKritVO.erfassProc != null )
    {
      where.append( and );
      where.append( "c.erfass_proc = " );
      where.append( artikelSelKritVO.erfassProc.intValue() );
      and = " AND ";
    }

    // Suchbegriff Registerauszugsgruppe
    if ( artikelSelKritVO.regauszGrp != null )
    {
      where.append( and );
      where.append( "c.regausz_grp = " );
      where.append( artikelSelKritVO.regauszGrp.intValue() );
      and = " AND ";
    }

    // Suchbegriff Publikationspflicht
    if ( !artikelSelKritVO.publPflicht.equals( "" ) )
    {
      where.append( and );
      where.append( "trim(lower(c.publ_pflicht)) = " );
      where.append( "'" );
      where.append( artikelSelKritVO.publPflicht.toLowerCase() );
      where.append( "'" );
      and = " AND ";
    }

    // Suchbegriff Unterabschnitt Publikation
    if ( !artikelSelKritVO.statcd3.equals( "" ) )
    {
      where.append( and );
      where.append( "trim(upper(a.statcd_3)) " );
      if ( artikelSelKritVO.statcd3.indexOf( '%' ) == -1 )
      {
        where.append( "= " );
      }
      else
      {
        where.append( " LIKE " );
      }
      where.append( "'" );
      where.append( artikelSelKritVO.statcd3.toUpperCase() );
      where.append( "'" );
      and = " AND ";
    }

    // Suchbegriff Artikel Sperrung
    if ( !artikelSelKritVO.sperrCd.equals( "" ) )
    {
      where.append( and );
      where.append( "trim(lower(a.sperr_cd)) = " );
      where.append( "'" );
      where.append( artikelSelKritVO.sperrCd );
      where.append( "'" );
      and = " AND ";
    }

    // formart_cd
    if ( !artikelSelKritVO.formartCd.equals( "" ) )
    {
      StringBuffer inBuffer = new StringBuffer();
      StringBuffer betweenBuffer = new StringBuffer();

      this.divideField( artikelSelKritVO.formartCd, "c.formart_cd", inBuffer, betweenBuffer );

      if ( ( inBuffer != null && !inBuffer.equals( "" ) && inBuffer.length() > 0 )
          || ( betweenBuffer != null && !betweenBuffer.equals( "" ) && betweenBuffer.length() > 0 ) )
      {
        where.append( and );
        where.append( "( " );
        if ( inBuffer != null && !inBuffer.equals( "" ) && inBuffer.length() > 0 )
        {
          where.append( "c.formart_cd IN ( " );
          where.append( inBuffer );
          where.append( " )" );
          or = " OR ";
          and = " AND ";
        }

        if ( betweenBuffer != null && !betweenBuffer.equals( "" ) && betweenBuffer.length() > 0 )
        {
          where.append( or );
          where.append( betweenBuffer );
          and = " AND ";
        }
        where.append( " )" );
      }
    }

    query = select.toString() + from.toString() + join.toString() + ( where.toString().equals( "WHERE " ) ? "" : where.toString() ) + order.toString();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      stmt = con.createStatement();
      rs = stmt.executeQuery( query );
      while ( rs.next() )
      {
        ArtikelAVO artikelAVO = new ArtikelAVO();

        artikelAVO.artNr = rs.getString( "art_nr" );
        artikelAVO.bezArt = rs.getString( "bez_art" );
        artikelAVO.kzGratis = rs.getString( "kz_gratis" );
        artikelAVO.kondartCd = rs.getString( "kondart_cd" );
        artikelAVO.sperrCd = rs.getString( "sperr_cd" );
        artikelAVO.statcd1 = rs.getString( "statcd_1" );
        artikelAVO.statcd2 = rs.getString( "statcd_2" );
        artikelAVO.statcd3 = rs.getString( "statcd_3" );
        artikelAVO.druckModus = rs.getString( "druck_modus" );
        artikelAVO.einzFaktpos = rs.getString( "einz_faktpos" );
        artikelAVO.formartCd = rs.getString( "formart_cd" );
        artikelAVO.kzEinschr = rs.getString( "kz_einschr" );
        artikelAVO.publPflicht = rs.getString( "publ_pflicht" );
        artikelAVO.regauszGrp = rs.getInt( "regausz_grp" );
        artikelAVO.fristArt = rs.getString( "frist_art" );
        artikelAVO.fristEinh = rs.getString( "frist_einh" );
        artikelAVO.fristDauer = new Integer( rs.getInt( "frist_dauer" ) );
        if ( rs.wasNull() )
        {
          artikelAVO.fristDauer = null;
        }
        artikelAVO.kzBesch = rs.getString( "kz_besch" );
        artikelAVO.erfassProc = new Integer( rs.getInt( "erfass_proc" ) );
        if ( rs.wasNull() )
        {
          artikelAVO.erfassProc = null;
        }
        artikelAVO.processModus = rs.getString( "process_modus" );
        artikelAVO.processNr = new Integer( rs.getInt( "process_nr" ) );
        if ( rs.wasNull() )
        {
          artikelAVO.processNr = null;
        }

        al.add( artikelAVO );
      }
      return al;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtikelSucheDAO.class, query + artikelSelKritVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( stmt, con );
    }
  }

  private void divideField( String expression, String field, StringBuffer inBuffer, StringBuffer betweenBuffer )
  {
    if ( !expression.equals( "" ) )
    {
      String separator = "";
      String or = "";

      String[] arField = expression.split( ";" );

      for ( int i = 0; i < arField.length; i++ )
      {
        try
        {
          if ( arField[i].indexOf( "-" ) == -1 ) //kein "-" gefunden IN-Statement anwenden
          {
            Integer.parseInt( arField[i].trim() );
            inBuffer.append( separator );
            inBuffer.append( arField[i].trim() );
            separator = ", ";
          }
          else
          //"-" gefunden BETWEEN-Statement anwenden
          {
            String[] betweenNr = arField[i].split( "-" );

            Integer.parseInt( betweenNr[0].trim() );
            Integer.parseInt( betweenNr[1].trim() );

            // ArtikelNr von UND ArtikelNr Bis wurde eingegeben
            betweenBuffer.append( or );
            betweenBuffer.append( "to_number(" + field + ") BETWEEN " );
            betweenBuffer.append( betweenNr[0].trim() );
            betweenBuffer.append( " AND " );
            betweenBuffer.append( betweenNr[1].trim() );
            or = " OR ";
          }
        }
        catch ( NumberFormatException ex1 )
        {
          System.out.println( ex1 );
          // nichts da ungültige angabe sollte nicht auftreten da überprüfung in gui
        }
      }
    }
  }
}
