package ch.ige.art.server.dao.test;

import java.sql.SQLException;

import ch.ige.art.server.dao.ArtbezDAO;
import ch.ige.art.server.dao.ArtikelDAO;
import ch.ige.art.vo.ArtbezVO;
import ch.ige.art.vo.ArtikelVO;
import ch.ige.art.vo.test.Testdata;

/**
 * Test der Klasse ArtbezDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtbezDAOTest.java,v 1.12 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class ArtbezDAOTest extends DAOTestCase
{
  private ArtikelDAO artikelDAO = new ArtikelDAO();

  private ArtikelVO artikelVO = null;

  private ArtbezDAO artbezDAO = new ArtbezDAO();

  private ArtbezVO artbezVO = null;

  public ArtbezDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    artikelVO = Testdata.createArtikelVO();
    artbezVO = Testdata.createArtBezVO();

    try
    {
      artikelDAO.insert( artikelVO );
    }
    catch ( SQLException ex )
    {
      System.out.println( "set up failed: " + ex );
    }
  }

  @Override
  protected void tearDown()
  {
    try
    {
      artikelDAO.delete( artikelVO.artNr );
    }
    catch ( SQLException ex )
    {
      System.out.println( "tear down failed: " + ex );
    }
  }

  /**
   *  Fügt einen Datensatz in die  Tabelle artbez ein
   */
  public void test1Insert()
  {
    System.out.println( "Test INSERT" );
    ArtbezVO selectVO = null;

    try
    {
      artbezDAO.insert( artbezVO );

      // testen ob Datensatz eingefügt wurde
      selectVO = artbezDAO.select( artbezVO.artNr );

      assertEquals( "1. Test: art_nr: ", artbezVO.artNr, selectVO.artNr );
      assertEquals( "2. Test: sprach_cd: ", "D", selectVO.sprachCd );
      assertEquals( "3. Test: dim: ", "Bagis", selectVO.dim );
      assertEquals( "4. Test: bez_art: ", artbezVO.bezArt, selectVO.bezArt );
      assertEquals( "5. Test erst_dat ", null, selectVO.erstDat );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle artbez
   */
  public void test2Select()
  {
    System.out.println( "Test SELECT " );
    ArtbezVO selectVO = null;

    try
    {
      artbezDAO.insert( artbezVO );
      selectVO = artbezDAO.select( artbezVO.artNr );

      if ( selectVO == null )
      {
        fail( "SELECT: Datensatz existiert nicht" );
      }
      else
      {
        assertEquals( "1. Test: art_nr: ", artbezVO.artNr, selectVO.artNr );
        assertEquals( "2. Test: sprach_cd: ", "D", selectVO.sprachCd );
        assertEquals( "3. Test: dim: ", "Bagis", selectVO.dim );
        assertEquals( "4. Test: bez_art: ", artbezVO.bezArt, selectVO.bezArt );
        assertEquals( "5. Test erst_dat ", null, selectVO.erstDat );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Aktualisiert einen Datensatz der Tabelle artbez
   */
  public void test3Update()
  {
    System.out.println( "Test UPDATE " );
    ArtbezVO updateVO = null;
    ArtbezVO selectVO = null;

    try
    {
      artbezDAO.insert( artbezVO );
      updateVO = artbezDAO.select( artbezVO.artNr );
      if ( updateVO == null )
      {
        fail( "UPDATE: Datensatz existiert nicht" );
      }
      else
      {
        updateVO.bezArt = "JUnit Test after Update";

        int retVal = artbezDAO.update( updateVO );
        assertEquals( "Anzahl Updates", 1, retVal );
        //Datensatz wieder selektieren und testen, ob Änderungen übernommen
        selectVO = artbezDAO.select( artbezVO.artNr );
        assertEquals( "1. Test: bez_art: ", "JUnit Test after Update", selectVO.bezArt );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle artbez
   */
  public void test4Delete()
  {
    System.out.println( "Test DELETE " );

    try
    {
      artbezDAO.insert( artbezVO );

      int retVal = artbezDAO.delete( artbezVO.artNr );
      assertEquals( "1 gelöschter Datensatz: ", 1, retVal );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
