package ch.ige.art.server.dao.test;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.List;

import ch.ige.art.server.dao.ArtikelDAO;
import ch.ige.art.server.dao.ArtpreisDAO;
import ch.ige.art.vo.ArtikelVO;
import ch.ige.art.vo.ArtpreisVO;
import ch.ige.art.vo.test.Testdata;

/**
 * Test der Klasse AltPreisDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtpreisDAOTest.java,v 1.11 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class ArtpreisDAOTest extends DAOTestCase
{
  private ArtikelDAO artikelDAO = new ArtikelDAO();

  private ArtikelVO artikelVO = null;

  private ArtpreisDAO artpreisDAO = new ArtpreisDAO();

  private ArtpreisVO artpreisVO = null;

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyy-MM-dd" );

  public ArtpreisDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    artikelVO = Testdata.createArtikelVO();
    artpreisVO = Testdata.createArtpreisVO();

    try
    {
      artikelDAO.insert( artikelVO );
    }
    catch ( SQLException ex )
    {
      System.out.println( "set up failed: " + ex );
    }
  }

  @Override
  protected void tearDown()
  {
    try
    {
      artikelDAO.delete( artikelVO.artNr );
    }
    catch ( SQLException ex )
    {
      System.out.println( "tear down failed: " + ex );
    }
  }

  /**
   *  Fügt einen Datensatz in die  Tabelle artpreis ein
   */
  public void test01Insert()
  {
    System.out.println( "Test INSERT" );
    ArtpreisVO selectVO = null;

    try
    {
      artpreisDAO.insert( artpreisVO );

      // Testen ob eingefügt
      selectVO = artpreisDAO.select( artpreisVO.artNr, artpreisVO.mengeinh, artpreisVO.artpreisartCd, artpreisVO.gueltigAb );
      if ( selectVO == null )
      {
        fail( "Datensatz existiert nicht" );
      }

      assertEquals( artpreisVO.artNr, selectVO.artNr );
      assertEquals( artpreisVO.mengeinh, selectVO.mengeinh );
      assertEquals( artpreisVO.artpreisartCd, selectVO.artpreisartCd );
      assertEquals( sdf.format( artpreisVO.gueltigAb ), sdf.format( selectVO.gueltigAb ) );
      assertEquals( artpreisVO.preis, selectVO.preis );
      assertEquals( artpreisVO.pro, selectVO.pro );
      assertEquals( artpreisVO.kennzeichen, selectVO.kennzeichen );
      assertEquals( artpreisVO.erstBen, selectVO.erstBen );
      assertEquals( sdf.format( artpreisVO.erstDat ), sdf.format( selectVO.erstDat ) );
      assertEquals( artpreisVO.geaendBen, selectVO.geaendBen );
      assertEquals( artpreisVO.geaendDat, selectVO.geaendDat );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle artpreis
   */
  public void test02Select()
  {
    System.out.println( "Test SELECT " );
    ArtpreisVO selectVO = null;

    try
    {
      artpreisDAO.insert( artpreisVO );
      selectVO = artpreisDAO.select( artpreisVO.artNr, artpreisVO.mengeinh, artpreisVO.artpreisartCd, artpreisVO.gueltigAb );
      if ( selectVO == null )
      {
        fail( "Datensatz existiert nicht" );
      }

      assertEquals( artpreisVO.artNr, selectVO.artNr );
      assertEquals( artpreisVO.mengeinh, selectVO.mengeinh );
      assertEquals( artpreisVO.artpreisartCd, selectVO.artpreisartCd );
      assertEquals( sdf.format( artpreisVO.gueltigAb ), sdf.format( selectVO.gueltigAb ) );
      assertEquals( artpreisVO.preis, selectVO.preis );
      assertEquals( artpreisVO.pro, selectVO.pro );
      assertEquals( artpreisVO.kennzeichen, selectVO.kennzeichen );
      assertEquals( artpreisVO.erstBen, selectVO.erstBen );
      assertEquals( sdf.format( artpreisVO.erstDat ), sdf.format( selectVO.erstDat ) );
      assertEquals( artpreisVO.geaendBen, selectVO.geaendBen );
      assertEquals( artpreisVO.geaendDat, selectVO.geaendDat );

    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert mehrere Datensätze aus der Tabelle artpreis
   */
  public void test03SelectAll()
  {
    System.out.println( "Test SELECT mehrere VO" );
    ArtpreisVO selectVO = null;
    List<ArtpreisVO> selectList = null;
    List<ArtpreisVO> list = Testdata.createListArtpreisVO( artpreisVO.artNr );

    try
    {

      for ( int i = 0; i < list.size(); i++ )
      {
        selectVO = list.get( i );
        artpreisDAO.insert( selectVO );
      }

      selectList = artpreisDAO.select( artpreisVO.artNr );
      selectVO = list.get( 0 );

      assertTrue( "Anzahl = ", selectList.size() == 3 );
      assertEquals( artpreisVO.artNr, selectVO.artNr );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Aktualisiert einen Datensatz
   */
  public void test04Update()
  {
    System.out.println( "Test UPDATE " );
    ArtpreisVO selectVO = null;
    ArtpreisVO updateVO = null;

    try
    {
      artpreisDAO.insert( artpreisVO );
      updateVO = artpreisDAO.select( artpreisVO.artNr, artpreisVO.mengeinh, artpreisVO.artpreisartCd, artpreisVO.gueltigAb );
      if ( updateVO == null )
      {
        fail( "UPDATE: Datensatz existiert nicht" );
      }
      else
      {
        updateVO.preis = ( new BigDecimal( 200.554 ) ).setScale( 2, BigDecimal.ROUND_HALF_UP );
        updateVO.pro = new Integer( 2 );

        int retVal = artpreisDAO.update( updateVO );
        assertEquals( "Anzahl geänderter Datensätze = 1", 1, retVal );

        //prüfen ob Änderung gespeichert
        selectVO = artpreisDAO.select( artpreisVO.artNr, artpreisVO.mengeinh, artpreisVO.artpreisartCd, artpreisVO.gueltigAb );
        assertEquals( updateVO.preis, selectVO.preis );
        assertEquals( updateVO.pro, selectVO.pro );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle artpreis
   */
  public void test05Delete()
  {
    System.out.println( "Test DELETE" );

    try
    {
      artpreisDAO.insert( artpreisVO );
      int retVal = artpreisDAO.delete( artpreisVO.artNr, artpreisVO.mengeinh, artpreisVO.artpreisartCd, artpreisVO.gueltigAb );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
