package ch.ige.art.util;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.vo.ArtikelGuiAVO;

/**
 * Schnittstelle zu SAP
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * @author A. Rueegsegger
 * @version $Id: SAPController.java,v 1.5 2006/12/20 13:29:01 ruegsegger Exp $
 */
public class SAPController
{
  private static final Logger LOG = Logger.getLogger( SAPController.class );

  public SAPController()
  {
  }

  /**
   * Überprüfung auf SAP ob Artikel vorhanden
   * kostenpflichtige Artikel müssen auf SAP vorhanden sein
   * bei gratis Artikel rückfragen ob Speichern
   * @return boolean true falls Überprüfung in Ordnung
   */
  public boolean checkExistenceOnSAP( ArtikelGuiAVO artikelGuiAVO )
  {
    String fehler = null;
    boolean finish = true;

    try
    {
      if ( !ArtikelDelegate.getInstance().existenceCheck( artikelGuiAVO.artNr ) )
      {
        if ( artikelGuiAVO.kzGratis.equals( "0" ) ) // kostenpflichtige Artikel müssen auf SAP vorhanden sein
        {
          fehler = "Der Artikel mit der Artikelnummer " + artikelGuiAVO.artNr +
              " existiert nicht auf SAP!\n\n" +
              "Eine Erfassung/Bearbeitung ist nur möglich sofern der Artikel auf SAP vorhanden ist.\n" +
              "Bitte melden Sie sich beim Userdesk\n";

          JOptionPane.showMessageDialog( null, fehler, "Information",
                                         JOptionPane.INFORMATION_MESSAGE );
          finish = false;
        }
        else // bei gratis Artikel rückfragen ob Speichern
        {
          fehler = "Der Artikel mit der Artikelnummer " + artikelGuiAVO.artNr +
              " existiert nicht auf SAP!\n" +
              "Wollen Sie die Änderungen wirklich abspeichern?\n\n";

          if ( JOptionPane.showConfirmDialog( null, fehler, "Hinweis",
                                              JOptionPane.YES_NO_OPTION,
                                              JOptionPane.QUESTION_MESSAGE ) ==
               JOptionPane.NO_OPTION )
          {
            finish = false;
          }
        }
      }
    }
    catch ( Exception ex )
    {
      LOG.error( ex.getClass() + " " + ex.getMessage() );
      String message = "Der Artikel kann nicht gespeichert werden!\n" +
          "Bei der Artikelüberprüfung auf SAP ist ein Problem aufgetreten\n\n" +
          "Versuchen Sie die Aktion zu einem späteren Zeitpunkt durchzuführen\n" +
          "oder melden Sie sich beim Userdesk";
      JOptionPane.showMessageDialog( null, message, "Fehler", JOptionPane.ERROR_MESSAGE );
      finish = false;
    }
    return finish;
  }

}
