package ch.ige.art.util;

import java.util.*;
import ch.ige.art.vo.*;

/**
 * Beschreibung: Factory-Klasse zur Erstellung der Value-Objects für die Wertebereiche aus der Tabelle XxDomain
 * Bei den Wertebreichen gibt es verschiedene Ausprägungen eines XxDomainVOs, je nachdem in welchem Attribute key und value gespeichert werden
 * Type 1: XxDomainKbezVO Key wird in dem Attribute kbez gespeichert value im Attribute bez
 * Type 2: XxDomainLnrVO Key wird in dem Attribute domainLnr gespeichert value im Attribute kbez
 * Type 3: XxDomainBezVO Key wird in dem Attribute domainLnr gespeichert value im Attribute bez
 * Copyright (c) 2005, Institut für Geistiges Eigentum
 * @author A. Rüegsegger
 * @version $Id: XxDomainFactory.java,v 1.9 2007/12/13 16:50:52 ruegsegger Exp $
 */

public class XxDomainFactory
{
  //kbez, bez
  public static String KONDART_CD     = "kondart_cd";
  public static String FRIST_ART      = "frist_art";
  public static String LAGEINH        = "lageinh";
  public static String DRUCK_SYSTEM   = "druck_system";
  public static String PRIOR_CD       = "prior_cd";
  public static String REGAUSZ_GRP    = "regausz_grp";
  public static String PUBL_KAP_PAT   = "publ_kap_pat";
  public static String PUBL_KAP_CHM   = "publ_kap_chm";
  public static String PUBL_KAP_DES   = "publ_kap_des";
  public static String PUBL_KAP_ESZ   = "publ_kap_esz";
  public static String PREISART_CD    = "preisart_cd";
  public static String ARTTEXTART_CD  = "arttextart_cd";
  public static String SPRACHE        = "sprache";  

  //lnr, kbez
  public static String PROCESS_MODUS  = "process_modus";
  public static String PUBL_PFLICHT   = "publ_pflicht";
  public static String ERFASS_PROC    = "erfass_proc";
  public static String BONUS_CD       = "bonus_cd";
  public static String TEXTART        = "textart";
  public static String STATCD_1       = "statcd_1";
  public static String STATCD_2       = "statcd_2";

  //lnr, bez
  public static String DRUCK_MODUS    = "druck_modus";
  public static String EINZ_FAKTPOS   = "einz_faktpos";
  public static String FRIST_EINH     = "frist_einh";
  public static String KZ_UNTSCHR     = "kz_untschr";
  public static String KZ_METNOT      = "kz_metnot";
  
  //groups
  public static String[] PUBL_KAP = { PUBL_KAP_PAT, PUBL_KAP_CHM, PUBL_KAP_DES, PUBL_KAP_ESZ };  


  private static List listKbez = new ArrayList();
  private static List listLnr = new ArrayList();
  private static List listBez = new ArrayList();

  // Wertebereiche wo key im Attribute kbez und Bezeichnung im Attribute bez enthalten sind
  static
  {
    listKbez.add(KONDART_CD);
    listKbez.add(FRIST_ART);
    listKbez.add(LAGEINH);
    listKbez.add(DRUCK_SYSTEM);
    listKbez.add(PRIOR_CD);
    listKbez.add(REGAUSZ_GRP);
    listKbez.add(PUBL_KAP_PAT);
    listKbez.add(PUBL_KAP_CHM);
    listKbez.add(PUBL_KAP_DES);
    listKbez.add(PUBL_KAP_ESZ);
  }

  // Wertebereiche wo key im Attribute domainLnr und Bezeichnung im Attribute kbez enthalten sind
  static
  {
    listLnr.add(PROCESS_MODUS);
    listLnr.add(PUBL_PFLICHT);
    listLnr.add(ERFASS_PROC);
    listLnr.add(BONUS_CD);
    listLnr.add(STATCD_1);
    listLnr.add(STATCD_2);
  }

  // Wertebereiche wo key im Attribute domainLnr und Bezeichnung im Attribute bez enthalten sind
  static
  {
    listBez.add(DRUCK_MODUS);
    listBez.add(EINZ_FAKTPOS);
    listBez.add(FRIST_EINH);
    listBez.add(KZ_UNTSCHR);
    listBez.add(KZ_METNOT);
  }


  public XxDomainFactory()
  {
  }

  /**
   * Je nach Domäne wird ein ValueObject zurückgegeben
   * @param domain String Domäne
   * @return XxDomainVO ValueObject vom Typ XxDomainVO
   */
  public static XxDomainVO create(String domain)
  {
    if ( listKbez.contains(domain) )
    {
      return new XxDomainKbezVO();
    }
    else if (listLnr.contains(domain))
    {
      return new XxDomainLnrVO();
    }
    else if (listBez.contains(domain))
    {
      return new XxDomainBezVO();
    }
    else
    {
      return new XxDomainVO();
    }
  }

}
