package ch.ige.art.xml;

import java.util.*;

import org.apache.log4j.Logger;


/**
 * <p>Überschrift: Mit dieser Klasse werden die Daten für die Erstellung des XML-Streams zusammengetragen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: Stammblatt.java,v 1.9 2006/12/13 10:08:31 ruegsegger Exp $
 */

abstract public class Stammblatt extends PrintDoc
{
  private static final Logger LOG = Logger.getLogger( Stammblatt.class );


  public Stammblatt() throws Exception
  {
  }

  /**
   * Druck Stammblatt mit dem übergebenen Schlüssel
   * @param key String Schlüssel
   * @throws Exception
   */
  public void print(String key) throws Exception
  {
    List listKey = new ArrayList();
    listKey.add(key);
    print(listKey);
  }


  /**
   * Druckt Stammblätter der übergebenen Schlüssel via Jippsclient
   * @param listKey List Liste von Schlüsselwerten (String)
   * @throws Exception
   */
  public void print( List listKey ) throws Exception
  {
    List list = read(listKey);

    String xmlStream = getXMLString( list );
    LOG.debug( "XML-Stream Stammblatt: \n" + xmlStream );
    sendMessage( xmlStream );
  }

  /**
   * Liest Detaildaten der übergebenen Werten
   * @param list List Liste von Schlüsselwerten
   * @throws Exception
   * @return List Liste mit Detaildaten
   */
  abstract List read( List list ) throws Exception;


  /**
   * Generiert aus den übergebenen Objekten einen XML-Stream
   * @param list List Liste mit Objekten die Detaildaten beinhalten
   * @throws Exception
   * @return String XML-Stream
   */
  abstract protected String getXMLString( List list ) throws Exception;


  /**
   * Neben der numerischen Unicode-Notation gibt es in HTML für häufiger verwendete Sonderzeichen
   * auch so genannte named entities, benannte Zeichen.
   * Alle benannten Zeichen werden mit einem & notiert: &[Name];
   *
   * Der XML-Parser ersetzt alle Sonderzeichen durch die entsprechenden Entities
   * diese Ersetzung der Sonderzeichen wird jedoch in unserem Fall nicht gewünscht
   * und deshalb wieder rückgängig gemacht
   *
   * @param xmlStream String dessen Entites ersetzt werden soll
   * @return String mit den Sonderzeichen anstelle der Entites
   */
  protected String replaceTags(String xmlStream)
  {
    // &lt;br&gt; ersetzen durch <br/>
    xmlStream = xmlStream.replaceAll( "&lt;br&gt;", "<br/>" );

    // <br/><br/> ersetzen durch <p/>-Tags
    xmlStream = xmlStream.replaceAll( "<br/><br/>", "<p/>" );

    // &lt;b&gt; ersetzen durch <b>
    xmlStream = xmlStream.replaceAll( "&lt;b&gt;", "<b>" );

    // &lt;/b&gt; ersetzen durch </b>
    xmlStream = xmlStream.replaceAll( "&lt;/b&gt;", "</b>" );

    return xmlStream;
  }
}

