package ch.ige.epband.clientneu;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.log4j.Logger;

import ch.ige.epband.exception.EpbandDelException;
import ch.ige.epband.exception.EpbandException;
import ch.ige.epband.exception.ProcessingException;
import ch.ige.epband.server.EpbandDelegate;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.StatistikVO;
import ch.ige.main.client.XxMessage;
import ch.ige.util.BuildInfo;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.client.batch.BatchException;
import ch.ige.util.client.batch.BatchReport;
import ch.ige.util.client.batch.BatchReturnCode;
import ch.ige.util.client.batch.NewAbstractBatch;

/**
 * <p>Überschrift:    Europäische Patente (EP) einlesen</p>
 * <p>Beschreibung:   Dies ist ein Batch-Programm.<br>
 * Die OMPI liefert jede Woche die EP's in Form von einem "SGML-File".
 * Die genaue Struktur ist der OMPI-Dokumentation zu entnehmen.
 * (Ursprünglich ein IBM-Tape, daher EBCDIC, Mehrere EP's pro Block usw.)
 * Mir diesem programm werden die EP in die DB geladen. Mit den benötigten
 * Tags werden die einzelnen VO's welche im EpbandVO zusammengefasst sind je
 * Patent aufbereitet. Das EpbandVO wird dann an EpbandBO übergeben, welches
 * dann seinerseits besorgt ist, dass das EP effektiv in die DB geschrieben wird.
 * EpbandVO ist auch besorgt, dass bei einem Delete und nachträglichen Insert
 * von EP's, vorhandene Ereignisse welche nicht gelöscht  werden dürfen,
 * auch wieder in die DB geschrieben werden.</p>
 * <p>Copyright:      Copyright (c) 2002</p>
 * <p>Organisation:   IGE</p>
 * @author            M. Trachsel
 * @version           $Id: Epband.java,v 1.26 2009/01/12 14:21:07 blank_blt Exp $
 */

public class Epband extends NewAbstractBatch
{
  /**
   * The name for this.
   */
  private static final String APPLICATION_NAME = "epband client";

  private String version = BuildInfo.CVS_TAG + " - " + BuildInfo.BUILD_ID;

  private static final Logger LOG = Logger.getLogger( "ch.ige.epband" );

  private EpbandLogFile epbandLogFile = null;

  private String logFile; // arg[0]

  private String zipFileName; // arg[1]

  private static String LOG_FILE = "l";

  private static String ZIP_FILE_NAME = "z";

  private static String LOG_FILE_LONG = "log";

  private static String ZIP_FILE_NAME_LONG = "zip";

  private EpbandDelegate delegate = null;

  private Listen listen;

  private String schutitNr = new String();

  private String gesuchNr = new String();

  private KontrollFileReader kreader;

  private ParseSGML parseSGML;

  private XMLDoc xml;

  private StatistikVO statistikVO = new StatistikVO();

  private int curzipFile = 0;

  private String oldanzzipFile = "";

  private String oldkGesuchNr = "";

  private boolean restart = false;

  // Applikationsmeldungen
  private XxMessage xxMessage = null;

  /**
   * The CLI options to use.
   */
  private static final Options OPTIONS;

  static
  {
    Option opLogFile = new Option( LOG_FILE, LOG_FILE_LONG, true, "The location of the Log File" );
    opLogFile.setType( String.class );
    opLogFile.setRequired( true );

    Option opZipFile = new Option( ZIP_FILE_NAME, ZIP_FILE_NAME_LONG, true, "The location of the Zip File" );
    opZipFile.setType( String.class );
    opZipFile.setRequired( true );

    OPTIONS = new Options();
    OPTIONS.addOption( opLogFile );
    OPTIONS.addOption( opZipFile );
  }

  static
  {
    ClientLogInitalizer.initalizeLogging();
  }

  /**
   * Constructor
   */
  public Epband()
  {
    super( APPLICATION_NAME, OPTIONS );
  }

//------------------------------------------------------------------------------
// Initialisierung
//------------------------------------------------------------------------------

  /***
   * Initialisierung
   * ZipFile und Logfile erstellen
   * Bland-Tabelle und IPC-Tabelle laden
   * Kontrollfile für Restart erstellen
   */
  private void init() throws IOException, Exception
  {
    xxMessage = XxMessage.getInstance();
    delegate = EpbandDelegate.getInstance();

    File file = createZipFile();
    createLogFile( file );

    // Alle BLänder vorladen
    ArrayList<String> blandWork = delegate.getAllBland();

    parseSGML = new ParseSGML( blandWork );

    // Listen vorbereiten
    listen = new Listen();

    // Kontrollfile - Restart überprüfen
    kreader = new KontrollFileReader( zipFileName );
    kreader.readFileEntry();
    oldanzzipFile = kreader.getWhichFi();
    oldkGesuchNr = kreader.getGesuchNr();
    if ( !oldkGesuchNr.trim().equals( "" ) )
    {
      restart = true;
    }
  }

  /**
   * Zipfile überprüfen
   * @return ZipFile
   */
  private File createZipFile() throws IOException
  {
    File file = new File( zipFileName.replace( '\\', '/' ) );

    if ( !file.isFile() || !file.canRead() )
    {
      throw new IOException( "Übergebenes File nicht gefunden oder keine Leserecht" );
    }
    return file;
  }

  /**
   * LogFile erstellen
   * @param file ZipFile
   */
  private void createLogFile( File file )
  {
    // Aus dem Namen des Zip-Files der name für das Log-File bestimmen
    String logFilePath;

    int delim = file.getName().lastIndexOf( "." );
    if ( delim == -1 ) // keine Extension
    {
      logFilePath = logFile + "\\Epband_" + file.getName();
    }
    else
    {
      logFilePath = logFile + "\\Epband_" + file.getName().substring( 0, delim );
    }

    epbandLogFile = EpbandLogFile.getInstance();
    epbandLogFile.setLogFile( logFilePath + ".log" );

    epbandLogFile.write( "" );
    epbandLogFile.write( "" );
    epbandLogFile.write( "EP einlesen    ( Version " + version + " )\r\n" );
    epbandLogFile.write( "Verarbeiten von  : " + zipFileName );
    epbandLogFile.write( "Gestartet am     : " + Util.dateToStringWithTime( new Date() ) );
    epbandLogFile.writePublInfo();
    epbandLogFile.write( "=================================================================" );
  }

//------------------------------------------------------------------------------
// Verarbeitung
//------------------------------------------------------------------------------

  /**
   * ZipFile lesen und verarbeiten. Verarbeitung wird pro ZIP-Eintrag(File) gemacht.
   * Nachfolgende Verarbeitung wird pro File gestartet.
   *
   * @throws IOException
   * @throws EpbandException
   */
  private void processZipFile() throws Exception, EpbandException
  {
    LOG.debug( "process zip file : " + zipFileName );

    ZipEntry entry;
    ZipFile zip = new ZipFile( zipFileName );

    @SuppressWarnings( "unchecked" )
    Enumeration en = zip.entries();
    while ( en.hasMoreElements() )
    {
      entry = (ZipEntry) en.nextElement();

      LOG.debug( "zip file entry found : " + entry.getName() );

      System.out.println( "Abarbeiten von File : " + entry.getName() + " (" + entry.getCompressedSize() + "/" + entry.getSize() + ")" );
      epbandLogFile.write( "" );
      epbandLogFile.write( "Abarbeiten von File : " + entry.getName() );
      epbandLogFile.write( "-----------------------------------------------------" );

      InputStream is = zip.getInputStream( entry );

      // File verarbeiten
      this.doSGML( is );

      is.close();
    }
  }

  /**
   * Das durch die EPO gelieferte File enthält jeweils mehrere<b>
   * Patente. File durchlesen und nach Patent aufsplitten.<br>
   * "<?xml" .... bis /ep-bulletin
   *
   * @param   is  
   * @throws  EpbandException
   */
  private void doSGML( InputStream is ) throws Exception
  {
    EpbandVO epbandVO;
    StringBuffer sb = new StringBuffer();
    char[] ch = new char[8192];

    // wird kein charset angegeben, wird der default genommen (hier cp1252)
    // was bei den sonderzeichen zu probleme führen kann
    InputStreamReader ir = new InputStreamReader( is, "UTF-8" );

    curzipFile++;
    int j;

    while ( ( j = ir.read( ch ) ) > 0 )
    {
      sb.append( ch, 0, j );
    }

    ir.close();

    // -1 if not found
    int start = sb.indexOf( "<?xml" );
    int end = sb.indexOf( "</ep-bulletin>" );
    String xmlString = null;

    while ( start != -1 && end != -1 )
    {
      xmlString = sb.substring( start, end + 14 );

      try
      {
        epbandVO = new EpbandVO();

        xml = new XMLDoc( xmlString );
        parseSGML.setData( epbandVO, xml );

        checkRestart( String.valueOf( curzipFile ), epbandVO.schutitVO.gesuchNr );

        if ( !restart )
        {
          parseSGML.doParseSGML();

          schutitNr = epbandVO.schutitVO.schutitNr;
          gesuchNr = epbandVO.schutitVO.gesuchNr;
          write( epbandVO );
        }
      }
      // Verarbeitungsfehler - Dokument wird übersprungen
      catch ( ProcessingException pex )
      {
        if ( pex.log )
        {
          epbandLogFile.write( "ERROR \t " + pex.getMessage() );
        }
      }

      listen.doListen( xml );

      start = sb.indexOf( "<?xml", end );
      end = sb.indexOf( "</ep-bulletin>", start );
    }

  }

  /**
   * Überprüfen ob es sich um einen Restart handelt und
   * ob Datensatz verarbeitet werden muss
   * @param strcurzipFile
   * @throws IOException
   */
  private void checkRestart( String strcurzipFile, String newGesuchNr ) throws IOException
  {
    // Falls normale Verarbeitung File-Eintrag schreiben
    // Filenummer, Gesuchsnummer, Statistikdaten
    if ( !restart )
    {
      kreader.writeFileEntry( strcurzipFile, newGesuchNr, statistikVO );
    }

    // Falls restart prüfen ob immer noch überlesen werden soll
    // Verarbeitung fortsetzten, sobald aktueller Datensatz mit
    // zuletzt verarbeiteten Datensatz übereinstimmt
    else if ( oldanzzipFile.equals( strcurzipFile ) && oldkGesuchNr.equals( newGesuchNr ) )
    {
      restart = false;
      kreader.readFileEntry();
      statistikVO = kreader.getStatistik();
      epbandLogFile.write( "Letzte verarbeitete Gesuch-Nr: " + newGesuchNr );
      epbandLogFile.write( "Verarbeitung nach Neustart mit Gesuch-Nr: " + newGesuchNr + " fortsetzen" );
      epbandLogFile.write( "" );
    }
  }

  /**
   * Dokument in Datenbank schreiben je nach Typ
   *
   * @param epbandVO Dokumentdaten
   * @throws InterruptedException
   * @throws EpbandException
   */
  private void write( EpbandVO epbandVO ) throws Exception
  {
    // WIPO 30060 schreiben
    if ( epbandVO.wipo )
    {
      try
      {
        statistikVO.anzWipo++;
        delegate.createEreigWipo( epbandVO );
      }
      catch ( ProcessingException pex )
      {
        statistikVO.anzWipofalse++;
        epbandLogFile.write( "WARN  \t Bei PCT " + schutitNr + "/" + gesuchNr + " Ereignis 30060 nicht erstellt" );
        epbandLogFile.write( "WARN  \t " + pex.getMessage() );
      }
    }

    // Zurücknahme 20250 schreiben falls es sich um ein Gesuch handelt.
    // Falls es sich um erteiltes EP handelt - Ereignis 20010 vorhanden.
    // Eintrag ins Logfile. Falls Schutztitel nicht vorhanden, wird nichts unternommen
    else if ( epbandVO.chzuruecknahme )
    {
      try
      {
        statistikVO.anzChzurueck++;
        delegate.createCHzurueck( epbandVO );
      }
      catch ( ProcessingException pex )
      {
        statistikVO.anzChzurueckfalse++;
        epbandLogFile.write( "ERROR \t Bei " + schutitNr + "/" + gesuchNr + " Ereignis 20250 nicht erstellt" );
        epbandLogFile.write( "ERROR \t " + pex.getMessage() );
      }
    }

    // Keineinspruch 20450 schreiben
    else if ( epbandVO.keinEinspruch )
    {
      try
      {
        statistikVO.anzkeinEinsp++;
        delegate.createkeinEinspruch( epbandVO );
      }
      catch ( ProcessingException pex )
      {
        statistikVO.anzkeinEinspfalse++;
        epbandLogFile.write( "ERROR \t Bei EP " + schutitNr + "/" + gesuchNr + " Ereignis 20450 nicht erstellt" );
        epbandLogFile.write( "ERROR \t " + pex.getMessage() );
      }
    }

    // EP-Patent schreiben
    else
    {
      try
      {
        statistikVO.anzcreateEp++;
        delegate.createEp( epbandVO );
      }
      catch ( EpbandDelException delex )
      {
        statistikVO.anzcreateEpfalse++;

        /* Falls es sich beim Löschen um einen logischen Fehler handelt
           wird im xxMessage ein entsprechender Eintrag gefunden.
           Der Fehler wird geloggt und die Verarbeitung fortgesetzt.
           Handelt es sich um einen Systemfehler, wird im xxMessage kein
           entsprechender Eintrag gefunden und die übergebene Meldung wird wieder
           zurückgegebn. In diesm Fall ist ein Programmabbruch erforderlich */
        String message = xxMessage.getMessage( delex );
        if ( message.equals( delex.getMessage() ) )
        {
          throw new EpbandException( message );
        }
        else
        {
          epbandLogFile.write( "ERROR \t Das EP " + schutitNr + "/" + gesuchNr + " konnte nicht neu erstellt werden, " + "und hat daher den alten Zustand" );
          epbandLogFile.write( "ERROR \t " + message );
        }
      }
    }
  }

//------------------------------------------------------------------------------
// FINISH
//------------------------------------------------------------------------------

  /**
   * Abschlussarbeiten
   */
  private void finish() throws Exception
  {
    String publDat = Util.stringToPrintDat( xml.getValueOf( "B405", "date" ) );
    String publWeek = xml.getValueOf( "B405", "bnum" );
    epbandLogFile.writePublInfo( publDat, publWeek );

    epbandLogFile.writeFooter( statistikVO );

    // Listen nun ausgeben
    listen.sendJIPPS();

    // Kontrollfile initialisieren
    // nicht löschen wegen Restart s. epband_laden.bat
    kreader.writeFileEntry( "0", "", statistikVO );
    kreader.close();
    epbandLogFile.close();
  }

//------------------------------------------------------------------------------
//NewAbstractBatch Implemented Methods
//------------------------------------------------------------------------------

  // @see ch.ige.util.client.batch.NewAbstractBatch#getBatchReports()
  @Override
  public BatchReport[] getBatchReports() throws BatchException
  {
    //Not used
    return null;
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getBuildInfo()
  @Override
  protected String getBuildInfo()
  {
    return BuildInfo.BUILD_ID;
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getCvsTag()
  @Override
  protected String getCvsTag()
  {
    return BuildInfo.CVS_TAG;
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getPossibleReturnCodes()
  @Override
  protected BatchReturnCode[] getPossibleReturnCodes()
  {
    return BatchReturnCode.getBatchReturnCodes( Epband.class );
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#checkCommandLine(org.apache.commons.cli.CommandLine, java.util.Map)
  @Override
  protected void checkCommandLine( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws IllegalArgumentException
  {
    if ( parsedCommandLine.get( LOG_FILE ) != null )
    {
      this.logFile = (String) parsedCommandLine.get( LOG_FILE );
    }

    if ( parsedCommandLine.get( ZIP_FILE_NAME ) != null )
    {
      this.zipFileName = (String) parsedCommandLine.get( ZIP_FILE_NAME );
    }
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#run(org.apache.commons.cli.CommandLine, java.util.Map)
  @Override
  protected void run( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws BatchException
  {
    try
    {
      LOG.debug( "before init..." );

      // Initialisierung
      init();

      LOG.debug( "before process..." );

      // ZipFile lesen und Daten verarbeiten
      processZipFile();

      LOG.debug( "before finish..." );

      // Aufräumarbeiten
      finish();

      System.exit( 0 );
    }
    catch ( IOException ioe )
    {
      String message = "Verarbeitung wegen unerwartetem Fehler abgebrochen Zip-File: " + zipFileName + " Exception: \r\n";
      LOG.error( message + ioe );
    }
    catch ( Throwable ex )
    {
      String message = "Verarbeitung wegen unerwartetem Fehler abgebrochen Exception:\n ";
      LOG.error( message + ex );
      LOG.error( "StackTrace: " + ex.getStackTrace() );
    }

    // wird nur im Fehlerfall aufgerufen da bei
    // erfolgreicher Verarbeitung System.exit(0)
    finally
    {
      if ( kreader != null )
      {
        kreader.close(); // Kontrollfile schliessen
      }
      if ( epbandLogFile != null )
      {
        epbandLogFile.close(); // Logfile schliessen
      }
      System.exit( 1 );
    }

  }

//------------------------------------------------------------------------------
//Main
//------------------------------------------------------------------------------
  /**
   * args[0] pfad logfile
   * args[1] Zip-File mit den Patentdaten
   *
   * @param args Argumente
   */
  public static void main( String[] args )
  {
    LOG.info( "Epband is started ..." );
    NewAbstractBatch.start( new Epband(), args );
    LOG.info( "Epband is finished." );
  }

}
