package ch.ige.epband.clientneu;

import java.util.List;

import ch.ige.epband.util.MappingUnicode;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.EreigTxtVO;
import ch.ige.epband.vo.EreignisVO;
import ch.ige.epband.vo.SchutitRegadrVO;
import ch.ige.epband.vo.SchutitVO;
import ch.ige.util.vo.RegadrVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rueegsegger
 * @version $Id: RegAdr.java,v 1.27 2008/05/26 09:02:34 ruegsegger Exp $
 */

public class RegAdr
{
  private EpbandVO epbandVO;

  private List<String> blandWork;

  private XMLDoc xml;

  private EpbandLogFile logFile = EpbandLogFile.getInstance();

  /**
   * Eigener Konstruktor für die Initialisierung der Klassenvariablen.
   * 
   * @param blandWork
   */
  public RegAdr( List<String> blandWork )
  {
    this.blandWork = blandWork;
  }

  /**
   * XML Datei parsen und in Vo speichern
   * 
   * @param xml
   * @param epbandVO
   * @throws Exception
   */
  public void parseSGML( XMLDoc xmlDoc, EpbandVO epVO ) throws Exception
  {
    this.epbandVO = epVO;
    this.xml = xmlDoc;

    // Inhaber
    prepRegadr( SchutitRegadrVO.INHABER, "B711" );

    // Erfinder
    prepRegadr( SchutitRegadrVO.ERFINDER, "B721" );

    // Grantees (Empfaenger)
    prepRegadr( SchutitRegadrVO.INHABER, "B731" );

    // Lizenznehmer
    prepRegadr( SchutitRegadrVO.LIZENZNEHMER, "B791" );
  }

  /**
   * Aufbereiten der Registeradresse
   * 
   * @param adrRolleCd int Art der Adresse
   * @param tag String Endtag der aktuellen Adresse
   */
  private void prepRegadr( int adrRolleCd, String tag ) throws Exception
  {
    boolean addAdr;
    boolean adrTagFound;

    int anz = xml.getNumberOf( tag );
    for ( int i = 1; i <= anz; i++ )
    {
      addAdr = true;
      adrTagFound = false;

      String element = tag + "[" + i + "]";

      RegadrVO regadrVO = new RegadrVO();
      regadrVO.setHerkunft( 1 );
      SchutitRegadrVO schutitRegadrVO = new SchutitRegadrVO();

      // Ereignis wird nur für Lizenznehmer verwendet
      EreignisVO ereignisVO = new EreignisVO();

      schutitRegadrVO.adrRolleCd = adrRolleCd;
      schutitRegadrVO.schutitTyp = SchutitVO.SCHUTIT_TYP_EP;
      schutitRegadrVO.vonEreigIdat = epbandVO.anmeldeIdat;

      if ( epbandVO.schutitVO.registerKz == SchutitVO.REGISTER )
      {
        schutitRegadrVO.regStat = 9;
      }
      else
      {
        schutitRegadrVO.regStat = 8;
      }

      regadrVO.setUpVerzName( MappingUnicode.getInstance().replaceUnicodeValues( xml.getValueOf( element, "syn" ) ) ); // nur
      // 1.
      // syn
      // gültig

      String iid = xml.getValueOf( element, "iid" );
      if ( Util.isnumeric( iid ) != -1 )
      {
        regadrVO.setHerkAdrNr( Util.isnumeric( iid ) );
      }

      regadrVO.setRegName( MappingUnicode.getInstance().replaceUnicodeValues( xml.getValueOf( element, "snm" ) ) );
      adrTagFound = xml.exists( element, "adr" );

      regadrVO.setRegStrasse( MappingUnicode.getInstance().replaceUnicodeValues( xml.getValueOf( element, "adr/str" ) ) );
      regadrVO.setRegOrt( MappingUnicode.getInstance().replaceUnicodeValues( xml.getValueOf( element, "adr/city" ) ) );
      regadrVO.setBlandCd( xml.getValueOf( element, "adr/ctry" ) );

      // Tag bestimmt, ob die Adresse für CH übernommen werden muss
      // geprüft wir nur bei Inhaber, Grantees und Lizenznehmer
      // wobei bei Lizenznehmer der Tag ohne EP ist!

      if ( xml.exists( element, "B716EP" ) || xml.exists( element, "B736EP" ) || xml.exists( element, "B796" ) )
      {
        addAdr = false;

        if ( xml.exists( element, "B716EP" ) )
        {
          List<String> list = xml.getValuesOf( element, "B716EP/ctry" );
          if ( list.size() > 0 )
          {
            addAdr = ( list.contains( "CH" ) || list.contains( "LI" ) );
          }
        }
        else if ( xml.exists( element, "B736EP" ) )
        {
          List<String> list = xml.getValuesOf( element, "B736EP/ctry" );
          if ( list.size() > 0 )
          {
            addAdr = ( list.contains( "CH" ) || list.contains( "LI" ) );
          }
        }
        else if ( xml.exists( element, "B796" ) )
        {
          List<String> list = xml.getValuesOf( element, "B796/ctry" );
          if ( list.size() > 0 )
          {
            addAdr = ( list.contains( "CH" ) || list.contains( "LI" ) );
          }
        }
      }

      if ( xml.exists( element, "B725EP" ) )
      {
        // der/die weiteren/alle Erfinder haben auf Nennung verzichtet
        if ( epbandVO.schutitVO.sprache == 2 ) // Sprache = "FR"
        {
          schutitRegadrVO.regadrNr = 4069999;
        }
        else
        {
          schutitRegadrVO.regadrNr = 4069998;
        }
        schutitRegadrVO.adrRolleCd = SchutitRegadrVO.ERFINDER;
        epbandVO.schutitregAdrAL.add( schutitRegadrVO );
        addAdr = false; // da innerhalb 721 entweder dieser Hinweis oder eine Adresse aber nicht beides
      }

      // Rest aufbereiten
      // addAdr = Adresse für CH bestimmt
      // adrTagFound = es ist eine Adresse und nicht nur eine Bemerkung
      if ( addAdr && adrTagFound )
      {
        if ( regadrVO.getUpVerzNameOrEmptyString().equals( "" ) )
        {
          regadrVO.setUpVerzName( regadrVO.getRegName().replace( '\n', ' ' ) );
        }

        if ( regadrVO.getBlandCdOrEmptyString().equals( "" ) )
        {
          regadrVO.setBlandCd( "CH" );
          String line = "WARN  \t Adresse bei " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " hatte kein Land"
              + "\r\nWARN  \t CH eingesetzt bei " + regadrVO.getRegName();
          logFile.write( line );
        }
        else
        {
          if ( !blandWork.contains( regadrVO.getBlandCd() ) )
          {
            String line = "WARN  \t Ländercode " + regadrVO.getBlandCd() + " in Registeradresse " + epbandVO.schutitVO.schutitNr + "/"
                + epbandVO.schutitVO.gesuchNr + " ungueltig" + "\r\nWARN  \t CH eingesetzt bei " + regadrVO.getRegName();
            logFile.write( line );
            regadrVO.setBlandCd( "CH" );
          }
        }

        if ( adrRolleCd == SchutitRegadrVO.LIZENZNEHMER ) // Lizenznehmer
        {
          ereignisVO.ereigIdat = Util.calculateIdat( xml.getValueOf( element, "date" ) );
          ereignisVO.artNr = EreignisVO.LIZENZEINTRAG; // Lizenzeintrag EP-Gesuch
          Ereignis ereignis = new Ereignis();
          ereignis.prepEreig( ereignisVO, epbandVO );

          // Ereignis nur schreiben, wenn es nicht bereits vorhanden ist.
          for ( EreignisVO eVO: epbandVO.ereigAL )
          {
            if ( eVO.artNr.equals( EreignisVO.LIZENZEINTRAG ) & eVO.ereigIdat == ereignisVO.ereigIdat )
            {
              ereignisVO.ereigIdat = ereignisVO.ereigIdat + 5;
            }
          }
          schutitRegadrVO.vonEreigIdat = ereignisVO.ereigIdat;

          epbandVO.ereigAL.add( ereignisVO );
          // Aufgrund vom Wert im Tag Kind finde ich heraus, ob die Lizenz exklusiv ist oder nicht
          String kindOfLicence = xml.getValueOf( element, "kind" );
          String licText = "";
          if ( epbandVO.schutitVO.sprache == 1 | epbandVO.schutitVO.sprache == 4 )
          {
            if ( kindOfLicence.equals( "E" ) )
            {
              licText = "Ausschliessliche Lizenz";
            }
            else
            {
              licText = "Nicht ausschliessliche Lizenz";
            }
          }
          else if ( epbandVO.schutitVO.sprache == 2 | epbandVO.schutitVO.sprache == 3 )
          {
            if ( kindOfLicence.equals( "E" ) )
            {
              licText = "Licence exclusive";
            }
            else
            {
              licText = "Licence non-exclusive";
            }
          }
          else
          {
            licText = "Sprache noch nicht definiert!";
          }
          EreigTxtVO ereigTxtVO = new EreigTxtVO();
          ereigTxtVO.ereigIdat = ereignisVO.ereigIdat;
          ereigTxtVO.ereigTxt = MappingUnicode.getInstance().replaceUnicodeValues( licText );
          ereigTxtVO.ereigTxtCd = 2;
          epbandVO.ereigTxtAL.add( ereigTxtVO );
        }

        epbandVO.regAdrAL.add( regadrVO );
        epbandVO.schutitregAdrAL.add( schutitRegadrVO );
      }
    }
  }
}
