package ch.ige.epband.clientneu;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rueegsegger
 * @version $Id: Util.java,v 1.17 2008/04/08 19:22:32 ruegsegger Exp $
 */

public class Util
{
  private static EpbandLogFile logFile = EpbandLogFile.getInstance();

  /**
   * Zwischenspeichern der aktuellen schutitNr & gesuchNr
   */
  public static String schutitNr;

  /**
   * gsetzt aus ParseSGML
   */
  public static String gesuchNr;

  private static final SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  private static final SimpleDateFormat PRINT_FORMAT = new SimpleDateFormat( "dd.MM.yyyy" );

  private static final SimpleDateFormat TIMESTAMP_FORMAT = new SimpleDateFormat( "dd.MM.yyyy HH:mm:ss" );

  /**
   * Aus einem String der Form "YYYYMMDD" ein ereig_idat erstellen
   * 
   * @param str String mit Datum
   * @return int mit ereig_idat
   */
  public static int calculateIdat( String str )
  {
    int i = isnumeric( str );

    if ( i == -1 )
    {
      return 0;
    }
    else
    {
      return ( ( i * 100 ) + 5 );
    }
  }

  /**
   * Test ob numerisch
   * 
   * @param str String
   * @return int mit umgewandeltem String, falls nicht numeric wird -1 zurückgegeben, "" ist nicht numerisch!
   */
  public static int isnumeric( String str )
  {
    try
    {
      return Integer.parseInt( str.trim() );
    }
    catch ( NumberFormatException e )
    {
      return -1;
    }
  }

  /**
   * ereigIdat (yyyyMMddzz) in Datum umwandeln
   * 
   * @param ereigIdat int
   * @return Date
   */
  public static Date createDate( int ereigIdat )
  {
    return Util.createDate( Integer.toString( ereigIdat / 100 ) );
  }

  /**
   * String der Form yyyymmdd in Date umwandeln
   * 
   * @param str mit Datum der Form yyyymmdd
   * @return Date
   */
  public static Date createDate( String str )
  {
    Date datum = null;

    if ( str != null && !str.equals( "" ) )
    {
      try
      {
        datum = stringToDate( str ); // Format prüfen
      }
      catch ( ParseException pex )
      {
        String line = "WARN  \t Bei EP " + schutitNr + "/" + gesuchNr + " Ungültiges Datum : " + str + ". Leeres (kein) Datum eingesetzt";
        logFile.write( line );
      }
    }
    return datum;
  }

  /**
   * Datum in String wird in ein Datumsobjekt umgewandelt.
   * 
   * @param str
   * @return Date
   * @throws ParseException
   */
  public static Date stringToDate( String str ) throws ParseException
  {

    sdf.setLenient( false );
    sdf.setTimeZone( java.util.TimeZone.getTimeZone( "Europe/Zurich" ) );
    Date date = sdf.parse( str );
    return date;
  }

  /**
   * Datum in String wird in ein Prioritäts-Datum umgewandelt.
   * 
   * @param dat
   * @return String
   */
  public static String stringToPrintDat( String dat )
  {
    sdf.setLenient( false );
    try
    {
      return PRINT_FORMAT.format( sdf.parse( dat ) );
    }
    catch ( ParseException ex )
    {
      return "";
    }
  }

  /**
   * Date-Time Datum wird in einem String zurückgegeben.
   * 
   * @param date
   * @return String
   */
  public static String dateToStringWithTime( Date date )
  {
    return TIMESTAMP_FORMAT.format( date );
  }

  /**
   * Umbauen der Publikations Nummer PCT (WO-Nummer)
   * 
   * @param iWONr String mit der WO-Nummer
   * @return String mit aufbereiteter Nummer TODO nummer wird ab 01.2002 korrekt geliefert ein Umbau ist nicht mehr
   *         erforderlich die Länge zu überprüfen wäre wahrscheinlich ausreichend
   */
  public static String prepWONr( String iWONr )
  {
    String WONr = iWONr;
    if ( WONr.startsWith( "WO" ) )
    {
      WONr = WONr.substring( 2 );
    }
    // Jahrzahl der Schutit-Nummer muss numerisch sein
    int yy = Util.isnumeric( WONr.substring( 0, 2 ) );
    if ( yy == -1 )
    {
      String line = "ERROR \t Jahrzahlen von WO-Nr " + WONr + " (PCT) nicht numerisch. In : " + schutitNr + "/" + gesuchNr + " Verarbeitung abgebrochen";
      logFile.write( line );

      return "";
    }

    // 7-stellig muss/war die Nummer immer (Jahr 2000 --> 00 als führende Nullen)
    while ( WONr.length() < 7 )
    {
      WONr = "0" + WONr;
    }
    // Ursprünglich Version der Nummer war yynnnnn (7-stellig)
    if ( WONr.length() == 7 )
    {
      WONr = WONr.substring( 0, 2 ) + "0" + WONr.substring( 2 );
    }
    // Version ab 1.7.2002 yy0nnnnn (8-stellig)
    if ( WONr.length() == 8 )
    {
      if ( yy < 30 )
      {
        WONr = "20" + WONr;
      }
      else
      {
        WONr = "19" + WONr;
      }
    }
    // Kein Umbau der WIPO-Nr nötig da schon Version ab 1.1.2002 der Form
    // yyyy0nnnnn
    return WONr;
  }
}
