package ch.ige.epband.clientneu.test;

import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung: Erstellen diverser Inputs (SGML) für Test<br>
 * Es werden wo nötig vollständige SGML erstellt. Die Daten sind "Echt", d.h.<br>
 * sie wurden aus bestehenden Files extrahiert und leich angepasst
 * </p>
 * <p>
 * Copyright: Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * <p>
 * Organisation: IGE
 * </p>
 * 
 * @author M. Trachsel
 * @version $id: $
 */

public class CreateTestpatent
{

  /**
   * Einlesen des XML für ein Gesuch
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createtestgesuch()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML für ein zweites Gesuch
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createtestgesuch1()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch-1.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML für ein erteiltes EP
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createtesteperteilt()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-ep-erteilt.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML für ein erteiltes EP Erteildatum liegt im Delta zwischen letzter
   * Papierpublikation und E-Publikation
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createtesteperteiltDelta()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-ep-erteilt-delta.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML für ein erteiltes EP Erteildatum liegt im Delta zwischen letzter
   * Papierpublikation und E-Publikation
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createtesteperteiltEPubli()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-ep-erteilt-ePubli.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML für ein WIPO Patent
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createwopatent()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-wopat.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML für ein zurückgezogenes Patent
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createchzurueckgesuch()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-chzurueckgesuch.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML für ein Patent ohne Einspruch
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createkeinEinspruch()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-keinEinspruch.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML für eine EP Eintragung
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createEPEintragung()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-epEintragung.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML Publikationssprache englisch [B260]
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createBeschreibung1()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch-beschreibung1.xml";
    return readFile( filename );
  }

  /**
   * Einlesen des XML. Publikationssprache französisch [B260]
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createBeschreibung2()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch-beschreibung2.xml";
    return readFile( filename );
  }

  /**
   * Publikationssprache deutsch [B260], Text mehr als 340 Zeichen
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createBeschreibung3()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch-beschreibung3.xml";
    return readFile( filename );
  }

  /**
   * Publikationssprache weder de, fr noch en [B260]
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createBeschreibung4()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch-beschreibung4.xml";
    return readFile( filename );
  }

  /**
   * Publikationssprache de, deutsche Übersetzung leer, fehlende Schutit-Nr -> keine Verarbeitung
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createBeschreibung5()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch-beschreibung5.xml";
    return readFile( filename );
  }

  /**
   * Publikationssprache de, deutsche Übersetzung leer
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createBeschreibung6()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-patent-beschreibung6.xml";
    return readFile( filename );
  }

  /**
   * ipcr Testfall
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createIpcrClassificationText()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-ipcr-classification-text.xml";
    return readFile( filename );
  }

  /**
   * Prioritäten
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createPrioritaeten()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch-prioritaeten.xml";
    return readFile( filename );
  }

  /**
   * Gesuchs Adressen
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createGesuchAdressen()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch-adressen.xml";
    return readFile( filename );
  }

  /**
   * Listen
   * 
   * @return Inhalt der XML-Datei in einem String
   */
  public static String createListe()
  {
    String filename = "C:\\jp\\projects\\epband\\testfiles\\test-listen.xml";
    return readFile( filename );
  }

  private static String readFile( String filename )
  {
    StringBuffer xmlString = new StringBuffer();
    char[] ch = new char[8192];

    try
    {
      // wird kein charset angegeben, wird der default genommen (hier cp1252)
      // was bei den sonderzeichen zu probleme führen kann
      InputStreamReader ir = new InputStreamReader( new FileInputStream( filename ), "UTF-8" );

      int j;

      while ( ( j = ir.read( ch ) ) > 0 )
      {
        xmlString.append( ch, 0, j );
      }

      ir.close();
    }
    catch ( Exception ex )
    {
      System.out.println( ex );
    }

    return xmlString.toString();
  }

  /**
   * Aufbereiten der benötigten bländer
   * 
   * @return Arraylist der bländer
   */
  public static List<String> createblandWork()
  {
    List<String> al = new ArrayList<String>();
    al.add( "AT" );
    al.add( "AU" );
    al.add( "BE" );
    al.add( "CH" );
    al.add( "CU" );
    al.add( "DE" );
    al.add( "DK" );
    al.add( "ES" );
    al.add( "FR" );
    al.add( "GB" );
    al.add( "GR" );
    al.add( "IT" );
    al.add( "LI" );
    al.add( "LU" );
    al.add( "NL" );
    al.add( "SE" );
    al.add( "US" );
    al.add( "JP" );
    return al;
  }

  /**
   * Aufbereiten der benötigten IPV
   * 
   * @return Arraylist der IPC
   */
  public static List<String> createipcWork()
  {
    List<String> al = new ArrayList<String>();
    al.add( "C07D20962" );
    al.add( "A61K03140" );
    al.add( "C12N01581" );
    al.add( "C12N01516" );
    al.add( "C12N01557" );
    al.add( "C12N01556" );
    al.add( "C12N00116" );
    al.add( "C12N00900" );
    al.add( "C07D20962" );
    al.add( "A61K03140" );
    al.add( "G06F00100" );
    al.add( "G06F00316" );
    al.add( "A61B01739" );
    al.add( "A61B00703" );
    al.add( "A61B00706" );
    al.add( "A01N005720" );
    al.add( "A01N004378" );
    al.add( "A01N004340" );
    al.add( "G02F00113357" );
    return al;
  }
}
