package ch.ige.epband.server;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.log4j.Logger;

import ch.ige.adr.server.AdresseEpFacade;
import ch.ige.adr.server.AdresseEpFacadeHome;
import ch.ige.epband.exception.EpbandDelException;
import ch.ige.epband.exception.ProcessingException;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.IpcVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rüegsegger
 * @version $id: $
 */

/*** <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *    name="EpbandFacade"  
 *    description="An EJB named EpbandFacade"
 *    display-name="EpbandFacade"
 *    view-type="remote"
 *    jndi-name="EpbandFacade"
 *    type="Stateful"
 *    transaction-type="Container"    
 * @ejb.transaction type="Required"
 * @ejb.ejb-ref
 *    ejb-name="EpbandXxIdentNumber"
 *    view-type="local"
 * <!-- end-xdoclet-definition --> 
 */
@SuppressWarnings( "unused" )
public class EpbandFacadeBean implements SessionBean
{
  /**
   *  Defaultwert für die serialVersionUID
   */
  private static final long serialVersionUID = 1L;

  private static final Logger LOG = Logger.getLogger( EpbandFacadeBean.class );

  private SessionContext sessionContext;

  private AdresseEpFacadeHome adresseEpFacadeHome = null;

  private AdresseEpFacade adresseEpFacade = null;

  private EpbandXxIdentNumberLocalHome epbandXxIdentNumeberLocalHome = null;

  private EpbandXxIdentNumberLocal epbandXxIdentNumberLocal = null;

  /**
   * Initialisierung
   */
  public void ejbCreate()
  {
    LOG.debug( "EpbandFacadeBean ejbCreate" );
    try
    {
      // Remote Referenz
      Context context = new InitialContext();
      AdresseEpFacadeHome home = (AdresseEpFacadeHome) context.lookup( AdresseEpFacadeHome.JNDI_NAME );
      adresseEpFacade = home.create();

      // Lokale Referenz
      EpbandXxIdentNumberLocalHome localHome = (EpbandXxIdentNumberLocalHome) context.lookup( EpbandXxIdentNumberLocalHome.COMP_NAME );
      epbandXxIdentNumberLocal = localHome.create();
    }
    catch ( Exception ex )
    {
      LOG.error( "EpbandFacadeBean: Methode ejbCreate Exception:" + ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  public void ejbRemove()
  {
    LOG.debug( "EpbandFacadeBean ejbRemove" );
  }

  public void ejbActivate()
  {
    LOG.debug( "EpbandFacadeBean ejbActivate" );
  }

  public void ejbPassivate()
  {
    LOG.debug( "EpbandFacadeBean ejbPassivate" );
  }

  public void setSessionContext( SessionContext sessionContext )
  {
    LOG.debug( "EpbandFacadeBean setSessionContext" );
    this.sessionContext = sessionContext;
  }

  /**
   * unnötige Methode, dass es kein Warning bei der SessionContext Klassenvariablen mehr gibt.
   * @return SessionContext
   */
  public SessionContext getSessionContext()
  {
    return this.sessionContext;
  }

  /**
   * Kreiert das EP DBTimeoutExcpetion (Retry) wird als ApplicationDBTimeoutException zurückgegeben.
   * ApplicationDBTimeoutException ist eine ApplicationException somit erfolgt nicht wie bei einer SystemException
   * automatisch ein Rollback der Trankaktion. Mit setRollbackOnly() wird ein Flag gesetzt, sodass ein Rollback der
   * Transaktion erfolgt. Bei ApplicationExcpetion wird BeanInstanz(AdresseEpFacade) nicht zerstört, was bei einer
   * SystemException der Fall wäre
   * 
   * @param epbandVO enthält die nötigen daten
   * @throws EpbandDelException für
   * @ejb.interface-method
   */
  public void createEp( EpbandVO epbandVO ) throws EpbandDelException
  {
    new EpbandBO().createEp( epbandVO, adresseEpFacade, epbandXxIdentNumberLocal );
  }

  /**
   * Holt alle BLänder.
   * 
   * @return BlandWork
   * @ejb.interface-method
   */
  public ArrayList<String> getAllBland()
  {
    return new EpbandBO().getAllBland();
  }

  /**
   * Holt alle unicode iso mappings
   * 
   * @return Map
   * @ejb.interface-method
   */
  public HashMap<String, String> getAllMappingUnicode()
  {
    return new EpbandBO().getAllMappingUnicode();
  }

  /**
   * Holt einen IPC-Eintrag aus der Tabelle IPC
   * 
   * @return IpcVO IPC-Eintrag
   * @param ipcSymb IPC-Symbol
   * @param gueltigVon Datum ab wann das IPC-Symbol gültig ist
   * @ejb.interface-method
   */
  public IpcVO getIpc( String ipcSymb, Date gueltigVon )
  {
    return new EpbandBO().getIpc( ipcSymb, gueltigVon );
  }

  /**
   * Holt einen IPC-Eintrag aus der Tabelle IPC
   * 
   * @return IpcVO IPC-Eintrag
   * @param ipcSymb
   * @ejb.interface-method
   */
  public IpcVO getYoungestIpc( String ipcSymb )
  {
    return new EpbandBO().getYoungestIpc( ipcSymb );
  }

  /**
   * Erstellt ein Ereignis wfür WIPO (PCT)
   * 
   * @param epbandVO enthält die nötigen daten
   * @throws ProcessingException
   * @ejb.interface-method 
   */
  public void createEreigWipo( EpbandVO epbandVO ) throws ProcessingException
  {
    new EpbandBO().createEreigWipo( epbandVO );
  }

  /**
   * Ertelt bei EP das Ereignis Zurücknahme
   * 
   * @param epbandVO enthält die nötigen daten
   * @throws ProcessingException
   * @ejb.interface-method
   */
  public void createCHzurueck( EpbandVO epbandVO ) throws ProcessingException
  {
    new EpbandBO().createCHzurueck( epbandVO );
  }

  /**
   * Ertelt bei EP das Ereignis Kein Widerspruch
   * 
   * @param epbandVO enthält die nötigen daten
   * @throws ProcessingException
   * @ejb.interface-method 
   */
  public void createkeinEinspruch( EpbandVO epbandVO ) throws ProcessingException
  {
    new EpbandBO().createkeinEinspruch( epbandVO );
  }
}
