package ch.ige.epband.server;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import ch.ige.util.server.DBHelper;

/**
 * Vergibt den nächsten Primärschlüssel
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author A. Rüegsegger
 * @version $Id: EpbandXxIdentNumberBean.java,v 1.10 2008/03/10 15:57:10 wild Exp $
 */

/*** <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *    name="EpbandXxIdentNumber"  
 *    description="An EJB named EpbandXxIdentNumber"
 *    display-name="EpbandXxIdentNumber"
 *    view-type="local"
 *    jndi-name="EpbandXxIdentNumber"
 *    type="Stateless"
 *    transaction-type="Container"    
 * @ejb.transaction type="Required"
 * <!-- end-xdoclet-definition --> 
 */
public class EpbandXxIdentNumberBean implements SessionBean
{
  /**
   * Defaultwert für die serialVersionUID
   */
  private static final long serialVersionUID = 1L;

  private final int MAX_RETRY = 50;

  private SessionContext sessionContext;

  /**
   * 
   *
   */
  public void ejbCreate()
  {
    /**
     * Nicht benötigte Interface-Methode aus SessionBean Klasse
     */
  }

  public void ejbRemove()
  {
    /**
     * Nicht benötigte Interface-Methode aus SessionBean Klasse
     */
  }

  public void ejbActivate()
  {
    /**
     * Nicht benötigte Interface-Methode aus SessionBean Klasse
     */
  }

  public void ejbPassivate()
  {
    /**
     * Nicht benötigte Interface-Methode aus SessionBean Klasse
     */
  }

  public void setSessionContext( SessionContext sessionContext )
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Diese Methode habe ich erstellt, damit das Warning in der Deklaration der Klassenvariablen nicht mehr kommt
   * 
   * @return SessionContext
   */
  public SessionContext getSessionContext()
  {
    return this.sessionContext;
  }

  /**
   * Vergibt den nächsten Primärschlüssel Diese Funktion muss in einer eigenen Transaktion ablaufen, damit Tabelle nur
   * möglichst kurze Zeit gelockt wird
   * 
   * @param name Name des Primärschlüssels. Konstanten aus XxIdentNumber.
   * @return nächster Primärschlüssel
   * @ejb.interface-method view-type="local"
   */
  public int getNextId( String name )
  {
    Connection con = null;
    PreparedStatement psSelect = null;
    PreparedStatement psUpdate = null;
    ResultSet rs = null;
    String query = null;

    int wert = 0;
    int version = 0;
    int counter = 0;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      do
      {
        psSelect = con.prepareStatement( "SELECT wert, version " + "FROM xx_ident_number " + "WHERE ident = 'bag' AND name = ? AND code = 0" );
        psSelect.setString( 1, name );
        rs = psSelect.executeQuery();

        if ( rs.next() )
        {
          wert = rs.getInt( "wert" );
          version = rs.getInt( "version" );
        }
        else
        {
          throw new EJBException( "Name = " + name + "in Tabelle XX_IDENT_NUMBER nicht gefunden" );
        }

        query = "UPDATE xx_ident_number " + "SET wert = wert + sequenz, " + "version = ? " + "WHERE ident = 'bag' AND name = ? AND code = 0";
        psUpdate = con.prepareStatement( query );
        psUpdate.setInt( 1, version + 1 );
        psUpdate.setString( 2, name );
      } while ( psUpdate.executeUpdate() != 1 && ++counter < MAX_RETRY );

      if ( counter < MAX_RETRY )
      {
        return wert;
      }
      else
      {
        throw new EJBException( "Probleme bei Update xx_ident_number Query: " + query + " version = " + version + ", name = " + name );
      }
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
    finally
    {
      DBHelper.close( psSelect, psUpdate, con );
    }
  }
}
