package ch.ige.epband.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import ch.ige.epband.vo.PrioritaetVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für Priorität
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Anita Rueegsegger, Ueli Ryser
 * @version $Id: PrioritaetDAO.java,v 1.10 2008/03/10 15:57:10 wild Exp $
 */

public class PrioritaetDAO
{

  /**
   * Fügt einen Datensatz in die Tabelle PRIORITAET ein
   * @param vo    ValueObject PrioritaetVO
   * @return int  Number of updated rows
   *
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public int insert( PrioritaetVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO prioritaet(" + "schutit_key, " + "prio_lnr, " + "bland_cd, " + "prio_nr, " + "prio_dat, " + "reg_stat) " + "VALUES( ?,?,?,?,?,? ) ";
      ps = con.prepareStatement( query );
      ps.setInt( 1, vo.schutitKey );
      ps.setInt( 2, vo.prioLnr );
      ps.setString( 3, vo.blandCd );
      ps.setString( 4, vo.prioNr );
      if ( vo.prioDat == null )
      {
        ps.setDate( 5, null );
      }
      else
      {
        ps.setDate( 5, new java.sql.Date( vo.prioDat.getTime() ) );
      }
      ps.setInt( 6, vo.regStat );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, PrioritaetDAO.class, query + vo.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
