package ch.ige.epband.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import ch.ige.epband.vo.SchutitVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für Schutztitel
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Anita Rueegsegger, Ueli Ryser, Stephan Madl, Michael Zaugg
 * @version $Id: SchutitDAO.java,v 1.18 2008/03/10 15:57:10 wild Exp $
 */

public class SchutitDAO
{

  /**
   * Fügt einen Datensatz in die Tabelle SCHUTIT ein
   * @param  vo   ValueObject
   * @return int  Number of inserted Rows
   *
   * @exception SQLException            Unerwarteter SQL Fehler
   */
  public int insert( SchutitVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO schutit(" + "schutit_key, " + "ma_nr, " + "schutit_typ, " + "sprache, " + "gesuch_nr, " + "schutit_nr, " + "register_kz, "
          + "schutz_beginn, " + "schutz_ende, " + "loesch_dat, " + "beschreibung, " + "upp_beschreibung, " + "erfass_ma, " + "erfass_dat, " + "mut_ma, "
          + "mut_dat, " + "erst_ereig_dat, " + "faellig_jahr, " + "version) " + "VALUES ( ?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,? ) ";

      ps = con.prepareStatement( query );
      ps.setInt( 1, vo.schutitKey );
      if ( vo.maNr == null )
      {
        ps.setNull( 2, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 2, vo.maNr.intValue() );
      }
      ps.setInt( 3, vo.schutitTyp );
      ps.setInt( 4, vo.sprache );
      ps.setString( 5, vo.gesuchNr );
      ps.setString( 6, vo.schutitNr );
      ps.setInt( 7, vo.registerKz );
      if ( vo.schutzBegin == null )
      {
        ps.setDate( 8, null );
      }
      else
      {
        ps.setDate( 8, new java.sql.Date( vo.schutzBegin.getTime() ) );
      }
      if ( vo.schutzEnde == null )
      {
        ps.setDate( 9, null );
      }
      else
      {
        ps.setDate( 9, new java.sql.Date( vo.schutzEnde.getTime() ) );
      }
      if ( vo.loeschDat == null )
      {
        ps.setDate( 10, null );
      }
      else
      {
        ps.setDate( 10, new java.sql.Date( vo.loeschDat.getTime() ) );
      }
      ps.setString( 11, vo.beschreibung );
      ps.setString( 12, vo.uppBeschreibung );
      ps.setInt( 13, vo.erfassMa );
      if ( vo.erfassDat == null )
      {
        ps.setDate( 14, null );
      }
      else
      {
        ps.setDate( 14, new java.sql.Date( vo.erfassDat.getTime() ) );
      }
      if ( vo.mutMa == null )
      {
        ps.setNull( 15, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 15, vo.mutMa.intValue() );
      }
      if ( vo.mutDat == null )
      {
        ps.setDate( 16, null );
      }
      else
      {
        ps.setDate( 16, new java.sql.Date( vo.mutDat.getTime() ) );
      }
      if ( vo.erstEreigDat == null )
      {
        ps.setDate( 17, null );
      }
      else
      {
        ps.setDate( 17, new java.sql.Date( vo.erstEreigDat.getTime() ) );
      }

      ps.setInt( 18, vo.faelligJahr );
      ps.setInt( 19, vo.version );

      if ( vo.schutitKey == 10156099 )
      {
        System.out.println( query + vo.toString() );
        System.out.println( new java.sql.Date( vo.schutzBegin.getTime() ).toString() );
      }
      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      System.out.println( query + vo.toString() );
      LogLevelHelper.log( ex, SchutitDAO.class, query + vo.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Sucht den SchutitKey zu einem Schutztitel über SchutitNr
   * @param   schutitNr   String
   * @param   schutitTyp  int
   * @return  int schutitKey
   *
   * @exception SQLException            Unerwarteter SQL Fehler
   */
  public int selectSchutitKeyWithSchutitNr( String schutitNr, int schutitTyp ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    int schutitKey = 0;

    try
    {
      if ( schutitNr != null && !schutitNr.equals( "" ) ) // Gesuche haben nicht unbedigt eine Schutit_Nummer
      {
        con = DBHelper.getConnection( DBHelper.BAGIS );
        query = "SELECT schutit_key " + "FROM schutit " + "WHERE schutit_nr = ? " + "AND schutit_typ = ? ";

        ps = con.prepareStatement( query );
        ps.setString( 1, schutitNr );
        ps.setInt( 2, schutitTyp );
        rs = ps.executeQuery();

        if ( rs.next() )
        {
          schutitKey = rs.getInt( "schutit_key" );
        }
      }
      return schutitKey;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, SchutitDAO.class, query + " schutitNr: " + schutitNr + " schutitTyp: " + schutitTyp );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Sucht den SchutitKey zu einem Schutztitel über GesuchNr
   * @param   gesuchNr    String
   * @param   schutitTyp  int
   * @return  int schutitKey
   *
   * @exception SQLException            Unerwarteter SQL Fehler
   */
  public int selectSchutitKeyWithGesuchNr( String gesuchNr, int schutitTyp ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    int schutitKey = 0;

    try
    {
      if ( gesuchNr != null && !gesuchNr.equals( "" ) ) // kann es eigentlich nicht geben
      {
        con = DBHelper.getConnection( DBHelper.BAGIS );
        query = "SELECT schutit_key " + "FROM schutit " + "WHERE gesuch_nr = ? " + "AND schutit_typ = ? ";
        ps = con.prepareStatement( query );
        ps.setString( 1, gesuchNr );
        ps.setInt( 2, schutitTyp );

        rs = ps.executeQuery();

        if ( rs.next() )
        {
          schutitKey = rs.getInt( "schutit_key" );
        }
      }
      return schutitKey;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, SchutitDAO.class, query + " gesuchNr: " + gesuchNr + " schutitTyp: " + schutitTyp );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Spezialisierter Update eines Schutztitels (EP)
   * ab EP-Band (EPO)
   * @param  vo    ValueObject mit Daten für den Update
   * @return int   Anzahl aktualisierter Datensätze
   *
   * @exception SQLException Unerwarteter SQL Fehler
   */

  public int updateSchutit( SchutitVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    PreparedStatement ps0 = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      ps0 = con.prepareStatement( "SELECT version " + "FROM schutit " + "WHERE schutit_key = ? " );

      ps0.setInt( 1, vo.schutitKey );
      rs = ps0.executeQuery();

      if ( rs.next() )
      {
        vo.version = rs.getInt( "version" );
      }

      query = "UPDATE schutit SET " + "loesch_dat = ?, " + "mut_ma = ?, " + "mut_dat = ?, " + "version = ? " + "WHERE schutit_key = ? ";
      ps = con.prepareStatement( query );
      if ( vo.loeschDat == null )
      {
        ps.setDate( 1, null );
      }
      else
      {
        ps.setDate( 1, new java.sql.Date( vo.loeschDat.getTime() ) );
      }
      if ( vo.mutMa == null )
      {
        ps.setNull( 2, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 2, vo.mutMa.intValue() );
      }
      ps.setDate( 3, new java.sql.Date( System.currentTimeMillis() ) );

      ps.setInt( 4, ++vo.version );
      ps.setInt( 5, vo.schutitKey );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, SchutitDAO.class, query + vo.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps0, ps, con );
    }
  }

  /**
   * Lösche eines EP, wobei bei Nichterfolg False zurückgegeben werden muss
   * @param schutitKey  des zu löschenden EP
   * @return true or false
   * @throws SQLException 
   */
  public int deleteSchutit( int schutitKey ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      // zuerst allfällige Leichen in der Bfaktur löschen...
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "DELETE FROM bfaktur WHERE status_zahlg = '9' and " + "bfakt_id in " + "(select bfakt_id from bfaktpos where schutit_key = ?)";
      ps = con.prepareStatement( query );
      ps.setInt( 1, schutitKey );
      ps.executeUpdate();

      query = "DELETE FROM schutit WHERE schutit_key = ? ";
      ps = con.prepareStatement( query );
      ps.setInt( 1, schutitKey );
      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, SchutitDAO.class, query + schutitKey );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
