package ch.ige.epband.vo;

import java.io.Serializable;
import java.util.Date;

/**
 * Überschrift:   EreignisVO
 *
 * Beschreibung:  EreignisVO ist das VO der Tabelle Ereignis.
 * Copyright:     Copyright (c) 2002
 * Organisation:  IGE
 * @author       Daniel Haller Strahm
 * @version      $Id: EreignisVO.java,v 1.16 2008/03/10 15:57:10 wild Exp $
 */

public class EreignisVO implements Serializable
{
  /**
   *  Defaultwert für die serialVersionUID
   */
  private static final long serialVersionUID = 1L;

  /**
   * 20000
   */
  public static final String ERSTERFASSUNG = "20000";

  /**
   * 20010
   */
  public static final String ERTEILUNG = "20010";

  /**
   * 20140
   */
  public static final String VER_ENGL = "20140";

  /**
   * 20230
   */
  public static final String ZURUECKWEISUNG = "20230";

  /**
   * 20235
   */
  public static final String ZURUECKNAHME = "20235";

  /**
   * 20240
   */
  public static final String FIK_ZURUECKNAHME = "20240";

  /**
   * 20250
   */
  public static final String CH_ZURUECKNAHME = "20250";

  /**
   * 20450
   */
  public static final String KEIN_EINSP = "20450";

  /**
   * 27070
   */
  public static final String LIZENZEINTRAG = "27070";

  /**
   * 30060
   */
  public static final String WIPO = "30060";

  /**
   *0
   */
  public final static int PUBLSTAT_NO_PUBLICATION = 0;

  /**
   * 1
   */
  public final static int PUBLSTAT_WAITLIST = 1;

  /**
   * 2
   */
  public final static int PUBLSTAT_READY_PUBLICATE = 2;

  // Da Ereignisse vor dem Löschen zwischengespeichert werden, ist es erforderlich,
  // dass die unbenutzten Attribute beibehalten werden

  /**
   * Schutit_Key 
   */
  public int schutitKey = 0;

  /**
   * Ereig_idat 
   */
  public int ereigIdat = 0;

  /**
   * Mitarbeiter nummer 
   */
  public int maNr = EpbandVO.MA_NR;

  /**
   * Publ_key 
   */
  public Integer publKey = null; // wird nicht gesetzt aus Band

  /**
   * Ereig_Stat_nr 
   */
  public int ereigStatNr = 0;

  /**
   * iMaNr 
   */
  public Integer iMaNr = null; // wird nicht gesetzt aus Band

  /**
   * art_nr 
   */
  public String artNr = null; // not nullable

  /**
   * adr_nr 
   */
  public Integer adrNr = null; // wird nicht gesetzt aus Band

  /**
   * freistablauf Datum 
   */
  public Date fristablDat = null;

  /**
   * abschluss Datum 
   */
  public Date abschlDat = null; // wird nicht gesetzt aus Band

  /**
   * Wan wurde das Abschlussdatum verändert
   */
  public Date abschlmutDat = null; // wird nicht gesetzt aus Band

  /**
   * Datum der technischen Erfassung 
   */
  public Date erfassDat = null;

  /**
   * Nummer innerhalb des Jahres 
   */
  public Integer pmmblSeite = null; // wird nicht gesetzt aus Band

  /**
   * pmmbl Jahrgang (bzw +pat+ oder modDep 
   */
  public Integer pmmblJahr = null; // wird nicht gesetzt aus Band

  /**
   * kzKorrekt 
   */
  public int kzKorrekt = 0;

  /**
   * reg_Stat 
   */
  public int regStat = 0;

  /**
   * Publikations Datum E-Publikation oder Waitlist Datum 
   */
  public Date publDat = null;

  /**
   * Nachführungsstatus 
   */
  public int publStat = 0;

  /**
   * E-Publikation (1) oder Papierpublikation (0) 
   */
  public int publZiel = 0;

  /**
   * Version der Row auf Tabelle Ereignis 
   */
  public int version = 0;

  /**
   * Gibt den Inhalt des ValueObject EreignisVO zurück
   * @return Inhalt von EreignisVO
   */
  @Override
  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( "  schutitKey       = <" + this.schutitKey + ">\n" );
    buf.append( "  ereigIdat        = <" + this.ereigIdat + ">\n" );
    buf.append( "  maNr             = <" + this.maNr + ">\n" );
    buf.append( "  publKey          = <" + this.publKey + ">\n" );
    buf.append( "  ereigStatNr      = <" + this.ereigStatNr + ">\n" );
    buf.append( "  iMaNr            = <" + this.iMaNr + ">\n" );
    buf.append( "  artNr            = <" + this.artNr + ">\n" );
    buf.append( "  adrNr            = <" + this.adrNr + ">\n" );
    buf.append( "  fristablDat      = <" + this.fristablDat + ">\n" );
    buf.append( "  abschlDat        = <" + this.abschlDat + ">\n" );
    buf.append( "  abschlmutDat     = <" + this.abschlmutDat + ">\n" );
    buf.append( "  erfassDat        = <" + this.erfassDat + ">\n" );
    buf.append( "  pmmblSeite       = <" + this.pmmblSeite + ">\n" );
    buf.append( "  pmmblJahr        = <" + this.pmmblJahr + ">\n" );
    buf.append( "  kzKorrekt        = <" + this.kzKorrekt + ">\n" );
    buf.append( "  regStat          = <" + this.regStat + ">\n" );
    buf.append( "  publDat          = <" + this.publDat + ">\n" );
    buf.append( "  publStat         = <" + this.publStat + ">\n" );
    buf.append( "  publZile         = <" + this.publZiel + ">\n" );
    buf.append( "  version          = <" + this.version + ">\n" );

    return buf.toString();
  }
}
