package ch.ige.epband.vo;

import java.io.Serializable;
import java.util.Date;

/**
 * Überschrift: PatIpcVO Beschreibung: PatIpcVO ist das VO der Tabelle PatIpc. Copyright: Copyright (c) 2002
 * Organisation: IGE
 * 
 * @author Daniel Haller Strahm
 * @version $Id: PatIpcVO.java,v 1.21 2008/05/22 15:53:18 ruegsegger Exp $
 */
public class PatIpcVO implements Serializable
{
  //
  // const
  //
  /**
   *  Defaultwert für die serialVersionUID
   */
  private static final long serialVersionUID = 1L;

  public static final int IPC_CD_H = 2; // Hauptklasse

  public static final int IPC_CD_N = 3; // Nebenklasse

  public static final int IPC_CD_F = 4; // fakultative Klasse, nicht indexiert

  public static final int IPC_CD_I = 5; // Indexierte fakultative Klasse

  public static final String IPC_SYMBOL_F = "F"; // First (Hauptklasse)

  public static final String IPC_SYMBOL_L = "L"; // Last (Nebenklasse)

  public static final String CLASS_LEVEL_ADVANCED = "A"; // Advanced-Modus

  public static final String CLASS_VALUE_INVENTIVE = "I"; // Inventive (erfundene Information)

  public static final String CLASS_VALUE_NON_INVENTIVE = "N"; // Non Inventive (nicht erfundene Information)

  //
  // fields
  //
  public int schutitKey = 0;

  public int patIpcCd = 0;

  public int patIpcLnr = 0;

  public String ipcSymb = null; // not nullable

  public Date ipcGueltigVon = null; // not nullabe

  public String classLevel = CLASS_LEVEL_ADVANCED;

  public String classValue = CLASS_VALUE_INVENTIVE;

  public String geschichte = "B"; // B=Basic (default)

  public String source = "H"; // H=Human (default)

  public String generatingOffice = "CH";

  public Date erfassDat = null; // action-date not nullable

  public int erfassMa = EpbandVO.MA_NR;

  public int version = 0;

  /**
   * Gibt den Inhalt des ValueObject PatIpcVO zurück
   * 
   * @return Inhalt von ParIpcVO
   */
  @Override
  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( "  schutitKey       = <" + this.schutitKey + ">\n" );
    buf.append( "  patIpcCd         = <" + this.patIpcCd + ">\n" );
    buf.append( "  patIpcLnr        = <" + this.patIpcLnr + ">\n" );
    buf.append( "  ipcSymb          = <" + this.ipcSymb + ">\n" );
    buf.append( "  ipcGueltigVon    = <" + this.ipcGueltigVon + ">\n" );
    buf.append( "  classLevel       = <" + this.classLevel + ">\n" );
    buf.append( "  classValue       = <" + this.classValue + ">\n" );
    buf.append( "  geschichte       = <" + this.geschichte + ">\n" );
    buf.append( "  source           = <" + this.source + ">\n" );
    buf.append( "  generatingOffice = <" + this.generatingOffice + ">\n" );
    buf.append( "  erfassDat        = <" + this.erfassDat + ">\n" );
    buf.append( "  erfassMa         = <" + this.erfassMa + ">\n" );
    buf.append( "  version          = <" + this.version + ">\n" );

    return buf.toString();
  }

  /**
   * Gibt einen String aus Symbol und Code zurück damit herausgefunden werden kann, ob eine Kombination 
   * doppelt vorkommt.  
   * @return String
   */
  public String createIdentifierForInsertUniqueness()
  {
    return this.ipcSymb + this.patIpcCd;
  }
}
