package ch.ige.image;

import java.io.StringWriter;
import java.io.Writer;
import java.util.List;

import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataFormatImpl;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.jxpath.JXPathContext;
import org.w3c.dom.Node;

/**
 * Utility for working with javax.imageio metadata.
 * 
 * makes imageio metadata XML trees accessible via xpath.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: IIOMetadataUtil.java,v 1.7 2008/05/20 14:25:56 locher Exp $
 */
public class IIOMetadataUtil
{
  @SuppressWarnings( "unchecked" )
  public static List<Node> queryMetadataNodes( IIOMetadata metadata, String format, String xpathQuery )
  {
    JXPathContext context = JXPathContext.newContext( metadata.getAsTree( format ) );
    return context.selectNodes( xpathQuery );
  }

  public static String queryMetadata( IIOMetadata metadata, String format, String xpathQuery )
  {
    JXPathContext context = JXPathContext.newContext( metadata.getAsTree( format ) );
    return (String) context.getValue( xpathQuery );
  }

  public static void setValue( IIOMetadata metadata, String format, String xpathQuery, String value )
  {
    JXPathContext context = JXPathContext.newContext( metadata.getAsTree( format ) );
    Node target = (Node) context.getValue( xpathQuery );
    target.setNodeValue( value );
  }

  public static String serializeMetadata( IIOMetadata metadata )
  {
    if ( metadata.isStandardMetadataFormatSupported() )
    {
      return serializeMetadata( metadata, IIOMetadataFormatImpl.standardMetadataFormatName );
    }
    return serializeMetadataNative( metadata );
  }

  public static String serializeMetadataNative( IIOMetadata metadata )
  {
    return serializeMetadata( metadata, metadata.getNativeMetadataFormatName() );
  }

  public static String serializeMetadata( IIOMetadata metadata, String metadataFormat )
  {
    StringWriter buffer = new StringWriter();
    serializeMetadata( metadata, metadataFormat, buffer );
    return buffer.toString();
  }

  private static void serializeMetadata( IIOMetadata metadata, String metadataFormat, Writer output )
  {
    serializeDOM( metadata.getAsTree( metadataFormat ), output );
  }

  public static void serializeDOM( Node node, Writer output )
  {
    try
    {
      Transformer t = TransformerFactory.newInstance().newTransformer();
      t.setOutputProperty( OutputKeys.INDENT, "yes" );
      t.setOutputProperty( "{http://xml.apache.org/xslt}indent-amount", "4" );
      t.transform( new DOMSource( node ), new StreamResult( output ) );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
    }
  }

}