package ch.ige.image;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import org.apache.log4j.Logger;

/**
 * Interface to the identify progamm of ImageMagick.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: Identify.java,v 1.1 2008/06/27 13:29:16 locher Exp $
 */
public class Identify
{
  private static final Logger LOG = Logger.getLogger( Identify.class );

  private static final File IDENTIFY_EXECUTABLE = new File( System.getProperty( "imagehandler.identify.executable", "c:/temp/identify.exe" ) );

  private static final ThreadLocal<ProcessBuilder> METADATA_EXTRACTOR_PROCESS_BUILDER = new ThreadLocal<ProcessBuilder>()
  {
    @Override
    protected ProcessBuilder initialValue()
    {
      return new ProcessBuilder();
    }
  };

  public static boolean available()
  {
    return IDENTIFY_EXECUTABLE.exists();
  }

  private static void assertAvailable()
  {
    if ( !available() )
    {
      throw new UnsupportedOperationException( "identify program is not available" );
    }
  }

  public static CharSequence performIdentification( EncodedImage img ) throws IOException
  {
    assertAvailable();
    File tmpFile = File.createTempFile( "identify", null );
    try
    {
      img.write( tmpFile );
      return performIdentification( tmpFile );
    }
    finally
    {
      if ( !tmpFile.delete() )
      {
        LOG.debug( "failed to delete " + tmpFile );
      }
    }
  }

  public static CharSequence performIdentification( File imageFile ) throws IOException
  {
    assertAvailable();
    Process child = METADATA_EXTRACTOR_PROCESS_BUILDER.get().command( IDENTIFY_EXECUTABLE.getAbsolutePath(), "-verbose", imageFile.getAbsolutePath() ).start();
    InputStream in = child.getInputStream();
    String result = IOUtil.toString( in );
    in.close();
    return result;
  }

}
