package ch.ige.image;

import java.awt.Dimension;

/**
 * Utils for resize calculations.
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   galli
 * @version  $Id: ResizeUtil.java,v 1.2 2009/04/14 11:38:04 locher Exp $
 */
public final class ResizeUtil
{
  /**
   * Resizes the {@link Dimension} if height > maximum height. if this is the case, 
   * height = maximum height and width is resized with the width to height ratio.
   * 
   * @param d the actuall {@link Dimension}
   * @param max the maximum {@link Dimension}
   * @return a {@link Dimension} with the same width to heigth ratio, the width is now equal to the given maximum width.
   */
  public static Dimension trimHeight( Dimension d, Dimension max )
  {
    double ratio = d.getWidth() / d.getHeight();
    if ( d.height > max.height )
    {
      d.width = (int) ( ratio * max.height );
      if ( d.width < 1 )
      {
        d.width = 1;
      }
      d.height = max.height;
    }
    return d;
  }

  /**
   * The returend {@link Dimension} has the values:
   * width = maximum width
   * height = maximum width divided through the ratio width/heigth 
   * 
   * @param d the actuall {@link Dimension}
   * @param max the maximum {@link Dimension}
   * @return a {@link Dimension} with the same width to heigth ratio, the width is now equal to the given maximum width.
   */
  public static Dimension stretchWidth( Dimension d, Dimension max )
  {
    double ratio = d.getWidth() / d.getHeight();
    d.width = max.width;
    d.height = (int) ( max.width / ratio );
    if ( d.height < 1 )
    {
      d.height = 1;
    }
    return d;
  }
}
