package ch.ige.image;

import ch.ige.image.format.Format;

/**
 * "Writable" Image Metadata.
 * <p>
 * Note: the image data will <b>not</b> be modified (no reencoding) as a
 * side-effect.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.Image#getWritableMetadata()
 * 
 * @author Michael Locher
 * @version $Id: WritableMetadata.java,v 1.5 2006/12/05 15:57:01 bandschapp Exp $
 */
public interface WritableMetadata extends Metadata
{

  /**
   * Assigns a format to the image.
   * 
   * @param format
   *          the format for the image
   */
  public void setFormat(Format format);

  /**
   * Sets the colorspace of the image.
   * 
   * @param type
   *          the colorspace for the image.
   * @throws MetadataException
   *           if the colorspace can not be set
   */
  public void setColorspaceType(String type) throws MetadataException;

  /**
   * Sets the width of the image.
   * 
   * @param pixels
   *          the width in pixels
   * @throws MetadataException
   *           if the width can not be set
   */
  public void setWidth(int pixels) throws MetadataException;

  /**
   * Sets the height of the image.
   * 
   * @param pixels
   *          the height in pixels
   * @throws MetadataException
   *           if the height can not be set
   */
  public void setHeight(int pixels) throws MetadataException;

  /**
   * Sets the resolution of the image.
   * 
   * @param dpi
   *          the DPI value for the image
   * @throws MetadataException
   *           if the resolution can not be set
   */
  public void setDPI(int dpi) throws MetadataException;

}
