package ch.ige.image.color;

import java.awt.color.ICC_ColorSpace;
import java.awt.color.ICC_Profile;
import java.io.IOException;
import java.net.URL;

/**
 * SMPTE Colorspaces.
 * Society of Motion Picture and Television Engineers.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: SMPTEColorSpace.java,v 1.1 2008/05/21 08:17:33 locher Exp $
 */
public class SMPTEColorSpace
{
  private static ICC_ColorSpace CS_SMPTE_C;

  public static synchronized ICC_ColorSpace getSMPTE_C_ColorSpace()
  {
    if ( CS_SMPTE_C == null )
    {
      CS_SMPTE_C = new ICC_ColorSpace( getSMPTE_C() );
    }
    return CS_SMPTE_C;
  }

  public static ICC_Profile getSMPTE_C()
  {
    URL profile = SMPTEColorSpace.class.getResource( "SMPTE-C.icc" );
    if ( profile != null )
    {
      try
      {
        return ICC_Profile.getInstance( profile.openStream() );
      }
      catch ( IOException e )
      {
        throw new RuntimeException( "SMPTE-C Profile not available" );
      }
    }
    else
    {
      throw new RuntimeException( "SMPTE-C Profile not available" );
    }
  }

}
