package ch.ige.image.filter;

import java.awt.Color;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;

import ch.ige.image.BitmapImage;

/**
 * Low performance pixel visitor. Use for debugging only!
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   locher
 * @version  $Id: AbstractPixelVisitor.java,v 1.1 2009/05/06 09:56:12 locher Exp $
 */
public abstract class AbstractPixelVisitor implements ImageFilter<BitmapImage>
{

  // @see ch.ige.image.filter.ImageFilter#filter(ch.ige.image.BitmapImage)
  public synchronized BitmapImage filter( BitmapImage src )
  {
    visit( src.asBufferedImage() );
    return src;
  }

  protected void visit( BufferedImage img )
  {
    final ColorSpace csXYZ = ColorSpace.getInstance( ColorSpace.CS_sRGB );
    final int xMax = img.getWidth();
    final int yMax = img.getHeight();

    int srgb = 0;
    float[] xyz = null;
    for ( int y = 0; y < yMax; y++ )
    {
      for ( int x = 0; x < xMax; x++ )
      {
        int nextColor = img.getRGB( x, y );
        if ( srgb != nextColor || xyz == null )
        {
          //pixel has the same color as its predecessor
          srgb = nextColor;
          xyz = toOpaqueColor( srgb ).getComponents( csXYZ, xyz );
        }
        visit( x, y, srgb, xyz );
      }
    }
  }

  protected static Color toOpaqueColor( int srgb )
  {
    return new Color( srgb, false );
  }

  protected abstract void visit( int x, int y, int srgb, float[] xyz );

}
