package ch.ige.image.test;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.format.PNGFormat;

/**
 * Test Encoding in PNG.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: PNGEncodingTest.java,v 1.8 2009/03/19 13:26:21 locher Exp $
 */
public class PNGEncodingTest extends AbstractEncodingTester
{

  private static final String TARGET_FORMAT = PNGFormat.IDENTIFIER;

  public PNGEncodingTest(String name)
  {
    super(name);
  }

  @Override
  protected String getTargetType()
  {
    return TARGET_FORMAT;
  }

  public void testDummy() throws Exception
  {
    Image img = this.createDummyImage(300, 300);
    img.getWritableMetadata().setDPI(300);
    EncodedImage encoded = encode(img);
    this.save(encoded, "dummy2png.png");
    postEncoding(img, encoded);
  }

  public void testGIF() throws Exception
  {
    Image img = this.load("circle.gif");
    EncodedImage encoded = encode(img);
    this.save(encoded, "gif2png.png");
    postEncoding(img, encoded);
  }

  public void testPNG() throws Exception
  {
    Image img = this.load("circle.png");
    EncodedImage encoded = encode(img);
    this.save(encoded, "png2png.png");
    postEncoding(img, encoded);
  }

  public void testJPEG() throws Exception
  {
    Image img = this.load("circle.jpg");
    EncodedImage encoded = encode(img);
    this.save(encoded, "jpeg2png.png");
    postEncoding(img, encoded);
  }

  public void testTIFFnoCompress() throws Exception
  {
    Image img = this.load("circle_noCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_no_2png.png");
    postEncoding(img, encoded);
  }

  public void testTIFFdctCompress() throws Exception
  {
    Image img = this.load("circle_dctCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_dct_2png.png");
    postEncoding(img, encoded);
  }

  public void testTIFFlzwCompress() throws Exception
  {
    Image img = this.load("circle_lzwCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_lzw_2png.png");
    postEncoding(img, encoded);
  }

  public void testBMP() throws Exception
  {
    Image img = this.load("circle.bmp");
    EncodedImage encoded = encode(img);
    this.save(encoded, "bmp2png.png");
    postEncoding(img, encoded);
  }

  public void testA6png() throws Exception
  {
    Image img = this.load("a6_300.png");
    EncodedImage encoded = encode(img);
    this.save(encoded, "png_a6_2png.png");
    postEncoding(img, encoded);
  }

  public void testA6tiff() throws Exception
  {
    Image img = this.load("a6_300.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_a6_2png.png");
    postEncoding(img, encoded);
  }

  public void testA6gif() throws Exception
  {
    Image img = this.load("a6_300.gif");
    EncodedImage encoded = encode(img);
    this.save(encoded, "gif_a6_2png.png");
    postEncoding(img, encoded);
  }

}
