package ch.ige.imagehandler;

import java.io.IOException;
import java.net.URL;

import org.xml.sax.SAXException;

import ch.ige.imagehandler.proxy.ProxyingImageHandler;
import ch.ige.imagehandler.rule.RuleException;

/**
 * Factory for ImageHandlers.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: ImageHandlerFactory.java,v 1.6 2008/02/25 14:19:56 lang Exp $
 */
public class ImageHandlerFactory
{

  /**
   * Create an Image Handler which works in the current Java VM.
   * 
   * @param profileURL
   *          the URL of the profile description
   * @return the Imagehandler
   * @see LocalImageHandler
   */
  public static ImageHandler createLocalInstance( URL profileURL ) throws IOException, SAXException, RuleException
  {
    return new LocalImageHandler( profileURL );
  }

  /**
   * Creates an Image Handler which delegates requests to a proxy servlet.
   * 
   * @param profileURL
   *          the URL of the profile description
   * @param imageHandlerProxyURL
   *          the URL of the proxy servlet
   * @return the Imagehandler
   * @see ProxyingImageHandler
   */
  public static ImageHandler createProxiedInstance( URL profileURL, URL imageHandlerProxyURL ) throws IOException, SAXException, RuleException
  {
    return new ProxyingImageHandler( profileURL, imageHandlerProxyURL );
  }

  /**
   * Gets a reference to the ImageHandlerDelegate 
   * @return a reference to the ImageHandlerDelegate
   */
  public static ImageHandlerDelegate getDelegateInstance()
  {
    return ImageHandlerDelegate.getInstance();
  }

}
