package ch.ige.imagehandler.batch;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;

import org.apache.log4j.Logger;

/**
 * Base class for file storage.
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   locher
 * @version  $Id: AbstractFileConsumer.java,v 1.2 2009/04/23 17:39:40 locher Exp $
 */
public abstract class AbstractFileConsumer implements Consumer
{
  private static final Logger LOG = Logger.getLogger( AbstractFileConsumer.class );

  private final File destLocation;

  protected AbstractFileConsumer( File destLocation )
  {
    this.destLocation = destLocation;
    boolean ok = this.destLocation.mkdirs();
    if ( !ok )
    {
      throw new IllegalArgumentException( "target directory could not be created: " + this.destLocation );
    }
  }

  // @see ch.ige.imagehandler.batch.Consumer#notifyNextReady()
  public void notifyNextReady()
  {
    //ignore 
  }

  // @see ch.ige.imagehandler.batch.Consumer#shutdown()
  public void shutdown()
  {
    // do nothing
  }

  protected final File relativeFile( String filename )
  {
    return new File( this.destLocation, filename );
  }

  protected final File relativeFile( String extraDirs, String filename )
  {
    if ( extraDirs == null )
    {
      return this.relativeFile( filename );
    }
    File dir = relativeFile( extraDirs );
    if ( !dir.exists() )
    {
      if ( !dir.mkdirs() )
      {
        LOG.error( "could not create directory: " + dir.getAbsolutePath() );
        dir = this.destLocation;
      }
    }
    if ( !dir.isDirectory() )
    {
      LOG.error( "this is not a directory: " + dir.getAbsolutePath() );
      dir = this.destLocation;
    }
    return new File( dir, filename );
  }

  protected final OutputStreamWriter relativeWriter( String filename )
  {
    return createWriter( relativeFile( filename ) );
  }

  protected OutputStreamWriter createWriter( File target )
  {
    try
    {
      return new OutputStreamWriter( new FileOutputStream( target ), Charset.forName( "UTF-8" ) );
    }
    catch ( FileNotFoundException e )
    {
      throw new IllegalArgumentException( e );
    }
  }

}
