package ch.ige.imagehandler.rule;

import java.awt.color.ColorSpace;

import ch.ige.image.Image;
import ch.ige.image.ImageException;

/**
 * Colors are converted to sRGB or standard grayscale.
 * <p>
 * <em>Example</em>
 * <pre>
 *        &lt;rule class=&quot;StandardizeColorspaceConverter&quot;/&gt;
 * </pre>
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.format.Format#encode(Image, boolean)
 * @author Michael Locher
 * @version $Id: StandardizeColorspaceConverter.java,v 1.6 2009/04/14 11:38:03 locher Exp $
 */
public class StandardizeColorspaceConverter<I extends Image> extends AbstractRule<I> implements Converter<I, Image>
{

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "StandardizeColorspace Converter";
  }

  /* @see ch.ige.imagehandler.rule.Rule#checkParameters() */
  @Override
  public void checkParameters() throws RuleException
  {
  }

  // --------------------------------------------------------------------------------------------
  // Convert

  /* @see ch.ige.imagehandler.rule.Converter#convert(ch.ige.image.Image) */
  public Image convert( I source ) throws RuleException, ImageException
  {
    ColorSpace cs = source.asRenderedImage().getColorModel().getColorSpace();
    int type = cs.getType();
    if ( type == ColorSpace.TYPE_GRAY )
    {
      return source.grayscale();
    }
    else if ( !cs.isCS_sRGB() )
    {
      return source.asRGB();
    }
    return source;
  }

}
