package ch.ige.ipcloadscheme.client.image;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Date;

import org.apache.log4j.Logger;

import ch.ige.ipcloadscheme.client.file.FigureFilter;
import ch.ige.ipcloadscheme.client.file.ImageZipFile;
import ch.ige.ipcloadscheme.server.IpcSchemeDelegate;
import ch.ige.ipcloadscheme.vo.IpcBildVO;

/**
 * Speichert die im Zip-File enthaltenen Bilder (jpg) in die Datenbank
 * Ist unter dem Namen schon ein Bild gespeichert, wird ein update gemacht
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: FiguresProcess.java,v 1.12 2008/07/01 06:27:03 lewis_lee Exp $
 */

public class FiguresProcess
{
  private Date processDate;

  private static final Logger LOG = Logger.getLogger( FiguresProcess.class );

  private boolean error = false;

  private StringBuffer logMsg = new StringBuffer();

  public FiguresProcess( Date processDate )
  {
    this.processDate = processDate;
  }

  /**
    * Gibt an ob sich ein Fehler ereignet hat
    * @return boolean true fehler aufgetreten
    */
  public boolean isErrorOccured()
  {
    return this.error;
  }

  /**
   * Gibt die geloggten Debugmeldungen zurück
   * @return String
   */
  public String getLogMessage()
  {
    return this.logMsg.toString();
  }

  /**
   * Fügt übergebenen String der Fehlermeldung hinzu
   * @param message anzufügende Fehlermeldung
   */
  private void logAndAppendMsg( String message )
  {
    LOG.error( message );
    logMsg.append( "\n" );
    logMsg.append( message );
  }

  public void process()
  {
    LOG.info( "image process started" );

    ImageZipFile imageZipFile = new ImageZipFile( processDate );
    LOG.debug( "process zipfile:  " + imageZipFile.getPath() );

    try
    {
      if ( imageZipFile.exists() )
      {
        String target = imageZipFile.unzip();
        LOG.debug( "unzipped files to directory: " + target );

        File[] f = new File( target ).listFiles( new FigureFilter() );

        for ( int i = 0; i < f.length; i++ )
        {
          insert( f[i] );
        }

        LOG.debug( "delete unzip directory" );
        imageZipFile.deleteUnzippedFiles();

        LOG.debug( "move to directory done" );
        imageZipFile.moveToDoneDirectory();
      }
      else
      {
        error = true;
        logAndAppendMsg( imageZipFile.getPath() + " doesn't exist" );
      }
    }
    catch ( Exception ex )
    {
      error = true;
      logAndAppendMsg( ex.getMessage() );
    }

    LOG.info( "image process done" );
  }

  public void insert( File file )
  {
    IpcBildVO ipcBildVO = new IpcBildVO();

    try
    {
      String[] split = file.getName().split( "\\." );

      if ( split.length == 2 )
      {
        ipcBildVO.bildname = split[0].toLowerCase();
        ipcBildVO.bildext = split[1].toLowerCase();
        ipcBildVO.bild = getImageFile( file );

        LOG.debug( "try to load image into database: " + file.getPath() );
        IpcSchemeDelegate.getInstance().insertImage( ipcBildVO );
      }
    }
    catch ( Exception ex )
    {
      error = true;
      logAndAppendMsg( "insertion failed for image " + file.getName() + " Message: " + ex );
    }
  }

  private byte[] getImageFile( File imageFile ) throws Exception
  {
    InputStream in = new FileInputStream( imageFile );
    ByteArrayOutputStream out = new ByteArrayOutputStream( (int) imageFile.length() );

    int size = 0;
    byte[] buf = new byte[1024];

    while ( ( size = in.read( buf, 0, buf.length ) ) != -1 )
    {
      out.write( buf, 0, size );
    }
    in.close();
    return out.toByteArray();
  }

}
