package ch.ige.ipcloadscheme.client.test;

import java.util.Date;
import java.util.Locale;

import ch.ige.ipcloadscheme.client.SchemeProperties;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link SchemeProperties}.
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: SchemePropertiesTest.java,v 1.6 2008/07/01 06:27:02 lewis_lee Exp $
 */
public class SchemePropertiesTest extends ClientTestCase
{
  /**
   * Some date used for testing.
   */
  private Date testDate;

  /**
   * Creates this.
   * @param name The test's name
   */
  public SchemePropertiesTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();

    this.testDate = DateConverter.stringToDate( "31.03.2008" );
  }

  /**
   * Tests {@link SchemeProperties#toString()}.
   */
  public void testToString()
  {
    try
    {
      String propertiesString = SchemeProperties.getInstance().toString();
      assertTrue( "Checking some property", propertiesString.contains( SchemeProperties.CONFIG ) );
      System.out.println( propertiesString );
    }
    catch ( Exception ex )
    {
      fail( "Can't create properties: " + ex.toString() );
    }
  }

  /**
   * Tests {@link SchemeProperties#getProperty(String)}.
   */
  public void testGetProperty()
  {
    try
    {
      SchemeProperties.getInstance().getProperty( null );
      fail( "Null key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      SchemeProperties.getInstance().getProperty( "" );
      fail( "Empty key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      SchemeProperties.getInstance().getProperty( "gugus" );
      fail( "Unknown key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      SchemeProperties.getInstance().getProperty( SchemeProperties.TEXT_PATHNAME );
      fail( "Language-dependent key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      SchemeProperties.getInstance().getProperty( SchemeProperties.IMAGE_URL );
      fail( "Value with processing date must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    String proxyPort = SchemeProperties.getInstance().getProperty( SchemeProperties.HTTP_PROXY_PORT );
    assertEquals( "Checking property from file", "8080", proxyPort );

    System.setProperty( SchemeProperties.HTTP_PROXY_PORT, "9090" );
    proxyPort = SchemeProperties.getInstance().getProperty( SchemeProperties.HTTP_PROXY_PORT );
    assertEquals( "Checking overridden property", "9090", proxyPort );
  }

  /**
   * Tests {@link SchemeProperties#getProperty(String, java.util.Locale)}.
   */
  public void testGetLanguageProperty()
  {
    try
    {
      SchemeProperties.getInstance().getProperty( SchemeProperties.LANG_FILE, (Locale) null );
      fail( "Null language must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      SchemeProperties.getInstance().getProperty( SchemeProperties.LANG_FILE, Locale.CHINESE );
      fail( "Unused language must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      SchemeProperties.getInstance().getProperty( SchemeProperties.HTTP_PROXY_PORT, Locale.GERMAN );
      fail( "Non-language key must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      SchemeProperties.getInstance().getProperty( SchemeProperties.DOWNLOAD_URL_TEXT, Locale.GERMAN );
      fail( "Value with processing date must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    String textPath = SchemeProperties.getInstance().getProperty( SchemeProperties.LANG_FILE, Locale.ENGLISH );
    assertEquals( "Checking property from file", "c:\\data\\scheme\\ipcr_scheme_en.xml", textPath );

    System.setProperty( SchemeProperties.LANG_FILE + Locale.ENGLISH.getLanguage(), "c:\\temp\\bla.xml" );
    textPath = SchemeProperties.getInstance().getProperty( SchemeProperties.LANG_FILE, Locale.ENGLISH );
    assertEquals( "Checking overridden property", "c:\\temp\\bla.xml", textPath );
  }

  /**
   * Tests {@link SchemeProperties#getProperty(String, Locale, Date)}.
   */
  public void testGetLanguageDateProperty()
  {
    try
    {
      SchemeProperties.getInstance().getProperty( SchemeProperties.DOWNLOAD_URL_TEXT, Locale.GERMAN, null );
      fail( "Null date must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      SchemeProperties.getInstance().getProperty( SchemeProperties.LANG_FILE, Locale.GERMAN, this.testDate );
      fail( "Value without date placeholder must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    String downloadUrl = SchemeProperties.getInstance().getProperty( SchemeProperties.DOWNLOAD_URL_TEXT, Locale.ENGLISH, this.testDate );
    assertEquals( "Checking property from file", "http://www.wipo.int/classifications/ipc/en/download_area/20080331/xml/", downloadUrl );

    try
    {
      System.setProperty( SchemeProperties.DOWNLOAD_URL_TEXT + Locale.ENGLISH.getLanguage(), "http://www.google.com" );
      SchemeProperties.getInstance().getProperty( SchemeProperties.DOWNLOAD_URL_TEXT, Locale.ENGLISH, this.testDate );
      fail( "Overridden value without date placeholder must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    System.setProperty( SchemeProperties.DOWNLOAD_URL_TEXT + Locale.ENGLISH.getLanguage(), "http://www.google.com/yyyyMMdd" );
    downloadUrl = SchemeProperties.getInstance().getProperty( SchemeProperties.DOWNLOAD_URL_TEXT, Locale.ENGLISH, this.testDate );
    assertEquals( "Checking overridden property", "http://www.google.com/20080331", downloadUrl );
  }
}
