package ch.ige.ipcloadscheme.client.transfer;

import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.log4j.Logger;

import ch.ige.ipcloadscheme.client.IPCLoad.FileType;
import ch.ige.ipcloadscheme.client.file.ImageZipFile;
import ch.ige.ipcloadscheme.client.file.TextZipFile;

/**
 * Transferiert die Files vom IBIS-Server (wipo) in ein lokales Verzeichnis
 * Konfigurationseinstellungen sind in einem Property-File abgelegt
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $ Id: $
 */
public class Transfer
{
  private Date processDate = null;

  private Download download = new Download();

  private static final Logger LOG = Logger.getLogger( Download.class );

  private StringBuffer logMsg = new StringBuffer();

  boolean error = false;

  public Transfer( Date processDate )
  {
    this.processDate = processDate;
  }

  /**
   * Gibt an ob sich ein Fehler ereignet hat
   * @return boolean true fehler aufgetreten
   */
  public boolean isErrorOccured()
  {
    return this.error;
  }

  /**
   * Gibt die geloggten Debugmeldungen zurück
   * @return String
   */
  public String getLogMessage()
  {
    return this.logMsg.toString();
  }

  /**
   * Fügt übergebenen String der Fehlermeldung hinzu
   * @param message anzufügende Fehlermeldung
   */
  private void logAndAppendMsg( String message )
  {
    LOG.error( message );
    logMsg.append( "\n" );
    logMsg.append( message );
  }

  public void process( FileType fileType, List<Locale> languages )
  {
    LOG.info( "transfer started" );

    switch( fileType )
    {
      case ALL:
        transferText( languages );
        transferImage();
        break;
      case TEXT:
        transferText( languages );
        break;
      case IMG:
        transferImage();
        break;
    }

    LOG.info( "transfer done" );
  }

  public void transferImage()
  {
    try
    {
      ImageZipFile zipFile = new ImageZipFile( processDate );
      LOG.debug( "zipfile with images to transfer: " + zipFile.getName() );

      download.download( zipFile );
      LOG.debug( "download zipfile with images to : " + zipFile.getPath() );
    }
    catch ( Exception ex )
    {
      error = true;
      logAndAppendMsg( ex.getMessage() );
    }
  }

  public void transferText( List<Locale> languages )
  {
    for ( Locale language: languages )
    {
      try
      {
        if ( !language.equals( Locale.GERMAN ) )
        {
          TextZipFile zipFile = new TextZipFile( processDate, language );
          LOG.debug( "zipfile with text to transfer for language " + language + " :" + zipFile.getName() );

          download.download( zipFile );
          LOG.debug( "download zipfile with text to : " + zipFile.getPath() );
        }
      }
      catch ( Exception ex )
      {
        error = true;
        logAndAppendMsg( ex.getMessage() );
      }
    }
  }
}
