package ch.ige.ipcloadscheme.client.transfer.test;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import ch.ige.ipcloadscheme.client.IPCLoad.FileType;
import ch.ige.ipcloadscheme.client.file.ImageZipFile;
import ch.ige.ipcloadscheme.client.file.TextZipFile;
import ch.ige.ipcloadscheme.client.test.ClientTestCase;
import ch.ige.ipcloadscheme.client.transfer.Transfer;
import ch.ige.util.date.DateConverter;

/**
 * Testcase für Klasse HTTPConnection
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: TestTransfer.java,v 1.9 2008/07/01 06:27:02 lewis_lee Exp $
 */

public class TestTransfer extends ClientTestCase
{
  private Transfer transfer = null;

  private Date processDate;

  private List<Locale> languages = Arrays.asList( Locale.ENGLISH );

  public TestTransfer( String s )
  {
    super( s );
  }

  public void setUp() throws Exception
  {
    processDate = DateConverter.stringToDate( "01.01.2006" );
  }

  public void test01TransferText()
  {
    try
    {
      TextZipFile file = new TextZipFile( processDate, languages.get( 0 ) );
      file.delete();

      transfer = new Transfer( processDate );
      transfer.process( FileType.TEXT, languages );

      assertTrue( file.exists() );

      boolean ok = file.moveToDoneDirectory();
      assertTrue( ok );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }
  }

  public void test02TransferImage()
  {
    try
    {
      ImageZipFile file = new ImageZipFile( processDate );
      file.delete();

      transfer = new Transfer( processDate );
      transfer.process( FileType.IMG, languages );

      assertTrue( file.exists() );

      file.unzip();
      file.deleteUnzippedFiles();
      boolean ok = file.moveToDoneDirectory();
      assertTrue( ok );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }
  }

  public void test03Transfer()
  {
    try
    {
      TextZipFile textfile = new TextZipFile( processDate, languages.get( 0 ) );
      textfile.delete();

      ImageZipFile imagefile = new ImageZipFile( processDate );
      imagefile.delete();

      transfer = new Transfer( processDate );
      transfer.process( FileType.ALL, languages );

      assertTrue( textfile.getName() + "should exist", textfile.exists() );
      assertTrue( imagefile.getName() + "should exist", imagefile.exists() );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }
  }

  public void testTransferGermanText()
  {
    List<Locale> germanLanguages = Arrays.asList( Locale.GERMAN );

    try
    {
      TextZipFile file = new TextZipFile( processDate, germanLanguages.get( 0 ) );
      file.delete();

      transfer = new Transfer( processDate );
      transfer.process( FileType.TEXT, germanLanguages );

      assertTrue( file.getName() + "should not exist", !file.exists() );

      boolean ok = file.moveToDoneDirectory();
      // Can't be moved, since it doesn't exist
      assertTrue( !ok );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }
  }
}
