package ch.ige.ipcloadscheme.server.dao;

import java.io.ByteArrayInputStream;
import java.sql.*;
import ch.ige.ipcloadscheme.vo.IpcBildVO;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für die Tabelle IPC_BILD
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: IpcBildDAO.java,v 1.6 2006/12/19 16:19:52 ruegsegger Exp $
 */
public class IpcBildDAO
{

  public IpcBildDAO()
  {
  }


  /**
   * Fügt einen Datensatz in die Tabelle IPC_BILD ein
   * @param vo IpcBildVO einzufügenden Daten
   * @throws SQLException
   */
  public int insert( IpcBildVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement(
          "INSERT INTO ipc_bild(" +
          "bildname, " +
          "bild, " +
          "bildext) " +
          "VALUES(?,?,?)"
          );

      ps.setString( 1, vo.bildname );

      // image
      if ( vo.bild != null )
      {
        ps.setBinaryStream( 2, new ByteArrayInputStream( vo.bild ), vo.bild.length );
      }
      else
      {
        ps.setNull( 2, java.sql.Types.LONGVARBINARY );
      }

      ps.setString( 3, vo.bildext );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Modifiziert einen Datensatz der Tabelle IPC_BILD
   * @param vo IpcBildVO einzufügenden Daten
   * @throws SQLException
   */
  public int update( IpcBildVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement(
          "UPDATE ipc_bild SET " +
          "bild = ?, " +
          "bildext = ? " +
          "WHERE bildname = ?"
          );

      // image
      if ( vo.bild != null )
      {
        ps.setBinaryStream( 1, new ByteArrayInputStream( vo.bild ), vo.bild.length );
      }
      else
      {
        ps.setNull( 1, java.sql.Types.LONGVARBINARY );
      }

      ps.setString( 2, vo.bildext );
      ps.setString( 3, vo.bildname );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Löscht Daten aus der Tabelle IPC_BILD_KLASSE
   * @param ipcKlasse String PK des zu löschenden Eintrags
   * @throws SQLException
   * @return int Anzahl gelöschte Datensätze
   */
  public int delete( String bildname ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );

      ps = con.prepareStatement( "DELETE FROM ipc_bild WHERE bildname = ?" );

      ps.setString( 1, bildname );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht alle Datensätze aus der Tabelle IPC_BILD welche in der Tabelle IPC_BILD_KLASSE nicht referenziert werden
   * @throws SQLException
   * @return int Anzahl der gelöschten Datensätze
   */
  public int deleteAll() throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );

      ps = con.prepareStatement("DELETE FROM ipc_bild WHERE bildname not in (SELECT bildname FROM ipc_bild_klasse)" );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
    * Überprüft ob ein Datensatz in der Tabelle IPC_BILD vorhanden ist
    * @param bildname String PK
    * @throws SQLException
    * @return boolean true falls Datensatz existiert
    */
   public boolean exists( String bildname ) throws SQLException
   {
     Connection con = null;
     PreparedStatement ps = null;
     ResultSet rs = null;

     try
     {
       con = DBHelper.getConnection( DBHelper.SQLServerIPC );
       ps = con.prepareStatement( "SELECT * FROM ipc_bild WHERE bildname = ?" );
       ps.setString( 1, bildname );
       rs = ps.executeQuery();
       return rs.next();
     }
     finally
     {
       DBHelper.close( ps, con );
     }
   }

}
