package ch.ige.ipcloadscheme.server.dao;

import java.sql.*;
import java.util.*;
import ch.ige.ipcloadscheme.vo.IpcKlasseVO;
import ch.ige.util.server.DBHelper;


/**
 * Data Access Object der Tabelle IPC_KLASSE (SQL-Server)
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: IpcKlasseDAO.java,v 1.13 2006/12/19 16:19:52 ruegsegger Exp $
 */
public class IpcKlasseDAO
{

  public IpcKlasseDAO()
  {
    /** TODO query create table fehlermeldung wegen anzahl zeichen */
    /** TODO was wenn text länger als 18?? zeichen too long */
    /** TODO abklären bei insert was in den anderen spalten einfügen???  */
  }


  /**
   * Insert mittels JDBC-Stapelverarbeitung
   * Alle insert-Statements zusammen werden an den SQL-Server übermittelt in einem Schritt
   * @param list Collection einzufügender IpcKlasseVO
   * @throws SQLException
   * @return int[] insertcount pro Anweisung
   */
  public int[] executeInsert( Collection list ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    IpcKlasseVO ipcVO;

    try
    {

      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement(
          "INSERT INTO ipc_klasse(" +
          "ipc_klasse, " +
          "dot, " +
          "version, " +
          "text_d, " +
          "text_f, " +
          "text_i, " +
          "text_e, " +
          "elternklasse, " +
          "bild) " +
          "VALUES(?,?,?,?,?,?,?,?,?)"
          );

      Iterator iter = list.iterator();
      while ( iter.hasNext() )
      {
        ipcVO = ( IpcKlasseVO ) iter.next();

        ps.setString( 1, ipcVO.ipcKlasse );
        ps.setInt( 2, ipcVO.dot );
        ps.setString( 3, ipcVO.version );
        ps.setString( 4, ipcVO.textD );
        ps.setString( 5, ipcVO.textF );
        ps.setString( 6, ipcVO.textI );
        ps.setString( 7, ipcVO.textE );
        ps.setString( 8, ipcVO.elternklasse );
        ps.setBoolean( 9, ipcVO.bild );

        ps.addBatch();
      }

      return ps.executeBatch();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Insert mittels JDBC-Stapelverarbeitung
   * Alle insert-Statements zusammen werden an den SQL-Server übermittelt in einem Schritt
   * @param list Collection einzufügender IpcKlasseVO
   * @throws SQLException
   * @return int[] insertcount pro Anweisung
   */
  public int[] executeInsert( Map map ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    IpcKlasseVO ipcVO;

    try
    {

      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement(
          "INSERT INTO ipc_klasse(" +
          "ipc_klasse, " +
          "dot, " +
          "version, " +
          "text_d, " +
          "text_f, " +
          "text_i, " +
          "text_e, " +
          "elternklasse, " +
          "bild) " +
          "VALUES(?,?,?,?,?,?,?,?,?)"
          );

      Iterator iter = map.values().iterator();

      while ( iter.hasNext() )
      {
        ipcVO = ( IpcKlasseVO ) iter.next();

        ps.setString( 1, ipcVO.ipcKlasse );
        ps.setInt( 2, ipcVO.dot );
        ps.setString( 3, ipcVO.version );
        ps.setString( 4, ipcVO.textD );
        ps.setString( 5, ipcVO.textF );
        ps.setString( 6, ipcVO.textI );
        ps.setString( 7, ipcVO.textE );
        ps.setString( 8, ipcVO.elternklasse );
        ps.setBoolean( 9, ipcVO.bild );

        ps.addBatch();
      }

      return ps.executeBatch();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Update der Textspalten mittels JDBC-Stapelverarbeitung
   * Alle update-Statements zusammen werden an den SQL-Server übermittelt in einem Schritt
   * @param list Collection zu aktualisierende IpcKlasseVO
   * @throws SQLException
   * @return int[] uodatecount pro Anweisung
   */
  public int[] executeUpdate( Collection list ) throws SQLException
  {
    Connection con = null;
    Statement stmt = null;
    IpcKlasseVO ipcVO;
    StringBuffer update;
    String comma;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      stmt = con.createStatement();

      Iterator iter = list.iterator();
      while ( iter.hasNext() )
      {
        update = new StringBuffer();
        comma = "";

        ipcVO = ( IpcKlasseVO ) iter.next();

        update.append( "UPDATE IPC_KLASSE SET " );

        if ( ipcVO.textD != null && !ipcVO.textD.equals( "" ) )
        {
          update.append( "text_d = " );
          update.append( getSqlString(ipcVO.textD) );
          comma = ", ";
        }
        if ( ipcVO.textE != null && !ipcVO.textE.equals( "" ) )
        {
          update.append( comma );
          update.append( "text_e = " );
          update.append( getSqlString(ipcVO.textE) );
          comma = ", ";
        }
        if ( ipcVO.textF != null && !ipcVO.textF.equals( "" ) )
        {
          update.append( comma );
          update.append( "text_f = " );
          update.append( getSqlString(ipcVO.textF) );
          comma = ", ";
        }
        if (ipcVO.version != null && !ipcVO.version.equals( "" ) )
        {
          update.append( comma );
          update.append( "version = " );
          update.append( getSqlString(ipcVO.version ) );
          comma = ", ";
        }
        if (ipcVO.elternklasse != null && !ipcVO.elternklasse.equals( "" ) )
        {
          update.append( comma );
          update.append( "elternklasse = " );
          update.append( getSqlString(ipcVO.elternklasse ) );
          comma = ", ";
        }
        if ( ipcVO.bild )
        {
          update.append( comma );
          update.append( "bild = 1" );
          comma = ", ";
        }
        update.append( comma );
        update.append( "dot = " );
        update.append( ipcVO.dot );
        comma = ", ";

        update.append( " WHERE ipc_klasse = " );
        update.append( getSqlString(ipcVO.ipcKlasse) );

        stmt.addBatch( update.toString() );
      }

      return stmt.executeBatch();
    }
    finally
    {
      DBHelper.close( stmt, con );
    }
  }



  /**
   * Löscht alle Datensätze aus der Tabelle IPC_KLASSE
   * @throws SQLException
   * @return int Anzahl der gelöschten Datensätze
   */
  public int deleteAll() throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );

      ps = con.prepareStatement( "DELETE FROM IPC_KLASSE" );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle IPC_KLASSE ein
   * @param vo IpcKlasseVO einzufügenden Daten
   * @throws SQLException
   */
  public void insert( IpcKlasseVO ipcVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement(
          "INSERT INTO ipc_klasse(" +
          "ipc_klasse, " +
          "dot, " +
          "version, " +
          "text_d, " +
          "text_f, " +
          "text_i, " +
          "text_e, " +
          "elternklasse, " +
          "bild) " +
          "VALUES(?,?,?,?,?,?,?,?,?)"
          );

      ps.setString( 1, ipcVO.ipcKlasse );
      ps.setInt( 2, ipcVO.dot );
      ps.setString( 3, ipcVO.version );
      ps.setString( 4, ipcVO.textD );
      ps.setString( 5, ipcVO.textF );
      ps.setString( 6, ipcVO.textI );
      ps.setString( 7, ipcVO.textE );
      ps.setString( 8, ipcVO.elternklasse );
      ps.setBoolean( 9, ipcVO.bild );

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Überprüft ob ein Datensatz in der Tabelle IPC_KLASSE vorhanden ist
   * @param ipcKlasse String PK
   * @throws SQLException
   * @return boolean true falls Datensatz existiert
   */
  public boolean exists( String ipcKlasse ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement( "SELECT ipc_klasse FROM ipc_klasse WHERE ipc_klasse = ?" );
      ps.setString( 1, ipcKlasse );
      rs = ps.executeQuery();
      return rs.next();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Löscht einen Datensatz aus der Tabelle IPC_KLASSE
   * @param ipcKlasse String PK des zu löschenden Eintrags
   * @throws SQLException
   * @return int Anzahl gelöschte Datensätze
   */
  public int delete( String ipcKlasse ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );

      ps = con.prepareStatement( "DELETE FROM IPC_KLASSE WHERE ipc_klasse = ?" );

      ps.setString( 1, ipcKlasse );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * da der SQL-Server Apostroph (') als Textdelimitter verwendet,
   * müssen diese im Text ersetzt werden durch doppelte Apostroph
   * @param text String Text
   * @return String aufbereiteter Sql-String
   */
  private String getSqlString( String text )
  {
    StringBuffer buffer = new StringBuffer();

    buffer.append( "'" );
    buffer.append( text.replaceAll( "'", "''" ) );
    buffer.append( "'" );

    return buffer.toString();
  }

}
