package ch.ige.ipcloadvalidity.util;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

/**
 * Handling Validity Properties - Programmeigenschaften anhand Property Datei setzen
 * Pfad der Property Datei wird in der Batch-Prozedur gesetzt
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A.Rueegsegger
 * @version $Id: ValidityProperties.java,v 1.12 2008/07/01 06:37:31 kuenzli Exp $
 */
public class ValidityProperties
{
  private static final Logger LOG = Logger.getLogger( ValidityProperties.class );

  public static final String CONFIG = "ipcloadvalidity.config";

  // Http-Configuration
  public static final String HTTP_PROXY_HOST = "http.proxyHost";

  public static final String HTTP_PROXY_PORT = "http.proxyPort";

  public static final String HTTP_NON_PROXY = "http.nonProxyHosts";

  public static final String HTTP_USER = "http.user";

  public static final String HTTP_PW = "http.password";

  // Mail-Configuration
  public static final String MAIL_SMTP_HOST = "mail.smtp.host";

  public static final String MAIL_SEND_TO = "mail.send.to";

  public static final String MAIL_SEND_CC = "mail.send.cc";

  public static final String MAIL_SEND_FROM = "mail.send.from";

  // Download
  public static final String DOWNLOAD_URL = "download.url";

  // Zipfile
  public static final String ZIPFILE_TARGET = "zipfile.target";

  public static final String UNZIP_DIRECTORY = "unzip.directory";

  // Validity-File
  public static final String VALIDITY_DATE_FROM = "validityfile.date.from";

  public static final String VALIDITY_DATE_TO = "validityfile.date.to";

  public static final String VALIDITY_NAME = "validityfile.name";

  // Validity Logfile
  public static final String LOGFILE_PATH = "logfile.path";

  public static String[] keys = { CONFIG, HTTP_PROXY_HOST, HTTP_NON_PROXY, HTTP_PROXY_PORT, HTTP_USER, HTTP_PW, MAIL_SMTP_HOST, MAIL_SEND_TO, MAIL_SEND_CC,
      MAIL_SEND_FROM, DOWNLOAD_URL, ZIPFILE_TARGET, UNZIP_DIRECTORY, VALIDITY_DATE_FROM, VALIDITY_DATE_TO, VALIDITY_NAME, LOGFILE_PATH };

  private static ValidityProperties INSTANCE = null;

  private Properties validityProps = null;

  private ValidityProperties()
  {

  };

  /**
    * Einlesen einer Programmeigenschaft aus der Property Datei<br>
    * 
    * @param key
    * @return
    */
  public String loadProperty( String key )
  {
    String value = null;
    try
    {
      Properties props = readProperties();
      value = props.getProperty( key );
    }
    catch ( Exception ex )
    {
      LOG.error( "property : " + key + " couldn't be loaded " + ex );
    }

    return value;
  }

  /**
   * Einlesen der Programmeigenschaften aus der Property Datei<br>
   * Setzen der Programmeigenschaften als System-Properties
   * @throws Exception
   */
  public void loadProperties()
  {
    this.validityProps = readProperties();
  }

  /**
   * Einlesen der Programmeigenschaften aus der Property Datei<br>
   * Setzen der Programmeigenschaften als System-Properties
   * @throws Exception
   */
  private Properties readProperties()
  {
    // Property ipcloadvalidity.config wird in Batch-Prozedur gesetzt
    String fileUrl = System.getProperty( CONFIG );
    LOG.info( "load properties from: " + fileUrl );
    Properties props = new Properties();
    try
    {
      URL propsFileURL = new URL( fileUrl );

      props.load( propsFileURL.openStream() );
    }
    catch ( Exception e )
    {
      throw new RuntimeException( "Failed to load properties. Config file url: " + fileUrl, e );
    }
    return props;
  }

  public String getProperties()
  {
    StringBuffer buf = new StringBuffer();
    String val = null;

    if ( this.validityProps != null )
    {
      for ( String key: ValidityProperties.keys )
      {
        val = this.validityProps.getProperty( key );
        buf.append( "  " + key + " = " + val + "\n" );
      }
    }
    else
    {
      buf.append( "null" );
    }
    return buf.toString();
  }

  public static ValidityProperties getInstance()
  {
    if ( INSTANCE == null )
    {
      INSTANCE = new ValidityProperties();
      INSTANCE.loadProperties();
    }
    return INSTANCE;
  }

  /**
   * Returns the value of the passed key. If there is a system property with the same
   * key it's value will be returned instead of the one stored in this object.
   * @param The property key
   * @return The property value
   */
  public String getProperty( String key )
  {
    String retVal = null;
    String systemValue = System.getProperty( key );
    if ( StringUtils.isNotEmpty( systemValue ) && !systemValue.equalsIgnoreCase( "null" ) )
    {
      retVal = systemValue;
    }
    else
    {
      retVal = this.validityProps.getProperty( key );
      if ( StringUtils.isEmpty( retVal ) || retVal.equalsIgnoreCase( "null" ) )
      {
        retVal = null;
      }
    }
    return retVal;
  }
}
