package ch.ige.jipps;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.jipps.dao.AmvDAO;
import ch.ige.jipps.dao.AmvErrorDAO;
import ch.ige.jipps.dao.AmvLaufDAO;
import ch.ige.jipps.dao.BfakturDAO;
import ch.ige.jipps.dao.PotZahlDAO;
import ch.ige.jipps.vo.AmvErrorVO;
import ch.ige.jipps.vo.AmvLaufVO;
import ch.ige.jipps.vo.AmvVO;
import ch.ige.util.LaufNr;

/**
 * AmvTransactionManagerBean: Verarbeitet die Aufrufe auf die verschiedenen Datenbanken.
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Marc Bouquet
 * @version  $Id: AmvTransactionManagerBean.java,v 1.29 2008/02/27 08:26:15 lang Exp $
 * <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *  name="AmvTransactionManager"  
 *  description="AMVTransactionManager"
 *  display-name="AmvTransactionManager"
 *  view-type="remote"
 *  jndi-name="AmvTransactionManager"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction type="RequiresNew"
 * <!-- end-xdoclet-definition --> 
 */
public class AmvTransactionManagerBean implements SessionBean
{
// TODO laj, 17.12.07: AMV2-Code entfernen

  private static final Logger LOG = Logger.getLogger( AmvTransactionManagerBean.class );

  private AmvLaufDAO amvLaufDAO = null;

  private BfakturDAO bfaktDAO = null;

  private PotZahlDAO potZahlDAO = null;

  private AmvDAO amvDAO = null;

  private AmvErrorDAO amvErrorDAO = null;

  private SessionContext sessionContext;

  public void ejbCreate() throws CreateException
  {
  }

  public void ejbRemove()
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void setSessionContext( SessionContext sessionContext )
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Erzeugt ein Value-Object für den Lauf
   * @param typCd String Typ-Code des Reports ( typCd = amv1 oder amv2 )
   * @return AmvLaufVO Value-Object
   * @ejb.interface-method
   */
  public AmvLaufVO createAmvLauf( String typCd )
  {
    if ( amvLaufDAO == null )
    {
      amvLaufDAO = new AmvLaufDAO();
    }
    try
    {
      LOG.debug( "*** Start createAmvLauf für " + typCd + " ..." );
      AmvLaufVO amvLaufVO;

      if ( amvLaufDAO.hasRows() && ( ( amvLaufVO = selectAMVLaufToId( amvLaufDAO.getAmvLaufId( typCd ) ) ) != null ) )
      {
        amvLaufVO.laufNr = LaufNr.getLaufNr( amvLaufVO.laufNr );
      }
      else
      {
        amvLaufVO = new AmvLaufVO();
        amvLaufVO.laufNr = LaufNr.getLaufNr();
      }

      amvLaufVO.verarbStat = 0;
      amvLaufVO.typCd = typCd;
      amvLaufDAO.insert( amvLaufVO );
      // Füllt die neue amvLaufNrId die beim Insert erzeugt wurde in das Value-Object
      amvLaufVO.amvLaufId = amvLaufDAO.selectAmvLaufId( amvLaufVO.laufNr, typCd );
      LOG.debug( "*** AmvLaufVO: " + amvLaufVO.toString() );
      return amvLaufVO;
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "SQL-Fehler im Insert beim AMV-Lauf", sqlex );
      return null;
    }
  }

  /**
   * Gibt das Value-Object des Laufes zurück.
   * @param amvLaufId int übergebene Lauf-Id die wiederholt wird
   * @return AmvLaufVO Value-Object
   * @ejb.interface-method
   */
  public AmvLaufVO selectAMVLaufToId( int amvLaufId )
  {
    if ( amvLaufDAO == null )
    {
      amvLaufDAO = new AmvLaufDAO();
    }
    try
    {
      return amvLaufDAO.selectAMVLaufToId( amvLaufId );
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "SQL-Fehler im SelectAMVLaufToId beim AMV-Lauf", sqlex );
      return null;
    }
  }

  /**
   * Liest die Daten aus der Bagis-Datenbank aus
   * @param amvLaufId int aktuelle AMV-Lauf-Nummer
   * @param typCd String Typ-Code des Reports ( typCd = amv1 oder amv2 )
   * @return List mit allen Daten (inkl. Pot-Zahler) die Verarbeitet werden
   * @ejb.interface-method
   */
  public List<AmvVO> loadBagisDaten( int amvLaufId, String typCd )
  {
    List<AmvVO> amvListIncludingPotZahler = new ArrayList<AmvVO>();
    List<AmvVO> amvList = loadBfaktur( amvLaufId, typCd );

    for ( AmvVO amvVO: amvList )
    {
      amvListIncludingPotZahler.add( amvVO );
      List<Integer> amvPotList = loadBfakturPotZahl( amvVO.bfaktId );
      for ( Integer adrNr: amvPotList )
      {
        AmvVO amvVOPot = (AmvVO) amvVO.clone();
        amvVOPot.adrNr = adrNr;
        amvListIncludingPotZahler.add( amvVOPot );
      }
    }

    // TODO evtl. ist diese Sortierung unnötig; Sortierung wird auch in AmvControl.createAMV() gemacht.
    Collections.sort( amvListIncludingPotZahler );
    return amvListIncludingPotZahler;
  }

  /**
   * Gibt eine Liste zurück mit den benötigten Bfakt-Id's und Schutit-Key's
   * die in Bagis für diesen Lauf bereitgestellt wurden
   * @param amvLaufId int aktuelle AMV-Lauf-Nummer
   * @param typCd String Typ-Code des Reports ( typCd = amv1 oder amv2 )
   * @return List mit Value-Objects
   */
  private List<AmvVO> loadBfaktur( int amvLaufId, String typCd )
  {
    if ( bfaktDAO == null )
    {
      bfaktDAO = new BfakturDAO();
    }
    try
    {
      if ( typCd.equalsIgnoreCase( "amv1" ) )
      {
        LOG.debug( "*** Daten für AMV1 werden geladen ..." );
        return bfaktDAO.selectBFaktIdAMV( 0, 4, amvLaufId );
      }
      else if ( typCd.equalsIgnoreCase( "amv2" ) )
      {
        LOG.debug( "*** Daten für AMV2 werden geladen ..." );
        return bfaktDAO.selectBFaktIdAMV( 0, 5, amvLaufId );
      }
      else
      {
        LOG.error( "Der übergebene Report-Typ " + typCd + " ist ungültig!" );
        return null;
      }
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "Bfaktur-Id's, Schutit-Keys und Adress-Nummern konnten nicht geladen werden", sqlex );
      return null;
    }
  }

  /**
   * Gibt eine Liste zurück mit allen Pot-Zahler-Adress-Ids zu einer Bfakt-Id
   * @param bfaktId int Bfakt-Id
   * @return List mit allen Adress-Nummern zu der übergebenen Bfakt-Id
   */
  private List<Integer> loadBfakturPotZahl( int bfaktId )
  {
    if ( potZahlDAO == null )
    {
      potZahlDAO = new PotZahlDAO();
    }
    try
    {
      return potZahlDAO.select( bfaktId );
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "SQLFehler beim Laden des PotZahlVO", sqlex );
      return null;
    }
  }

  /**
   * Liest die Daten aus der Tabelle AMV, für welche die Verarbeitung wiederholt wird.
   * @param amvLaufId int aktuelle AMV-Lauf-Nummer
   * @return List mit allen Daten (inkl. Pot-Zahler) die Verarbeitet werden
   * @ejb.interface-method
   */
  public List<AmvVO> loadAMVDaten( int amvLaufId )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }
    try
    {
      List<AmvVO> amvList = amvDAO.selectAMVToId( amvLaufId );
      Collections.sort( amvList );
      return amvList;
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "SQLFehler bei loadAMVDaten()", sqlex );
      return null;
    }
  }

  /**
   * Gibt eine Liste mit den Value-Object der AMV-Daten zurück.
   * Diese Methode wird verwendet, wenn ein Lauf wiederholt wird.
   * @param amvLaufId int AMV-Lauf-Nummer für die Wiederverarbeitung
   * @return List Liste mit allen AMV-Daten für die Wiederverarbeitung
   * @ejb.interface-method
   */
  public List<AmvVO> selectAMVToId( int amvLaufId )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }
    try
    {
      return amvDAO.selectAMVToId( amvLaufId );
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "SQL-Fehler in SelectAMVToId", sqlex );
      return null;
    }
  }

  /**
   * Fügt alle Daten die aus Bagis herausgelesen wurde in die JIPPS_AMV-Tabelle ein.
   * Somit ist sichergestellt, dass alle Daten für die Verarbeitung dieses Laufes vorhanden sind
   * @param amvList List mit allen benötigten Daten für diesen Lauf
   * @return int Status
   * @ejb.interface-method
   */
  public boolean insertAmvDaten( List amvList )
  {
    /** TODO Fehlerbehandlung bei Eindeutigkeitsverletzung ev. anpassen */
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }
    try
    {
      if ( amvList != null && amvList.size() > 0 )
      {
        Iterator iter = amvList.iterator();
        int i = 1;
        while ( iter.hasNext() )
        {
          AmvVO amvVO = (AmvVO) iter.next();
          amvVO.verarbStat = 1;
          amvDAO.insert( amvVO );
          i++;
        }
        return true;
      }
      else
      {
        LOG.error( " Die AMV-Liste gibt null zurück. " );
        return false;
      }
    }
    catch ( Exception sqlex )
    {
      LOG.error( "SQLFehler beim Insert der Daten in die AMV-Tabelle", sqlex );
      return false;
    }
  }

  /**
   * Gibt eine List zurück mit allen Bfakt-Id's die in der Tabelle JIPPS_AMV für diesen Lauf enthalten sind.
   * Im der Liste ist eine Bfakt-Id die mehrmals in der Tabelle vorkommt, nur einmal enthalten
   * @param amvLaufId int aktuelle AMV-Lauf-Nummer
   * @return List mit allen Bfakt-Id's die für den Update benötigt werden
   * @ejb.interface-method
   */
  public List selectAmvLaufBfaktIds( int amvLaufId )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }

    try
    {
      return amvDAO.selectBfakturList( amvLaufId );
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "AMV-Liste für Update der Bfakturen konnte nicht erstellt werden", sqlex );
      return null;
    }
  }

  /**
   * 07.09.2004 btm: Work a Round bei Problemen mit Bfaktur
   * Für diese Methode muss die Container-Transaktion auf NotSupported deklariert werden !!!
   * Führt auf der Bagis-DB ein Update aus für die übergebene Bfakt-Id. Kann der Update nicht ausgeführt werden, wird
   * eine Fehlermeldung in die JIPPS_AMV_ERROR-Tabelle geschrieben.
   * @param bfaktId int Bfakt-Id
   * @param statusDruck String Druck-Status auf BAGIS
   * @throws Exception Unerwarteter Fehler
   */
// public void commit( int bfaktId, String statusDruck ) throws Exception
// {
// updateBFakturDruckstatus(bfaktId, statusDruck);
// }
  /**
   * Führt auf der Bagis-DB ein Update aus für die übergebene Bfakt-Id. Kann der Update nicht ausgeführt werden, wird
   * eine Fehlermeldung in die JIPPS_AMV_ERROR-Tabelle geschrieben.
   * @param bfaktId int Bfakt-Id
   * @param statusDruck String Druck-Status auf BAGIS
   * @throws Exception Unerwarteter Fehler
   * @ejb.interface-method
   */
  public void updateBFakturDruckstatus( int bfaktId, String statusDruck ) throws Exception
  {
    if ( bfaktDAO == null )
    {
      bfaktDAO = new BfakturDAO();
    }

    try
    {
      /** TODO: 07.09.2004 / btm: FailureRecovery einbauen damit der Update 50 mal versucht wird ( Dauer ca. 5 min. pro Update im schlechtesten Fall ) */
      bfaktDAO.updateBFakturDruckstatus( bfaktId, statusDruck );
    }
    catch ( Exception ex )
    {
      throw new Exception( "Update des Druckstatus auf der Bfaktur (" + bfaktId + ") konnten nicht ausgeführt werden", ex );
    }
  }

  /**
   * Im Fehlerfall bei einer AMV-Erinnerung wird die Fehlermeldung in die Tabelle JIPPS_AMV_ERROR geschrieben.
   * @param amvErrorVO AmvErrorVO Value-Object
   * @ejb.interface-method
   */
  public void insertErrorMsg( AmvErrorVO amvErrorVO )
  {
    if ( amvErrorDAO == null )
    {
      amvErrorDAO = new AmvErrorDAO();
    }

    try
    {
      amvErrorDAO.insert( amvErrorVO );
    }
    catch ( UnitOfWorkException e )
    {
      LOG.error( "Insert in die AMV-Error-Tabelle mit den Werten (Lauf-Nr.:" + amvErrorVO.amvLaufId + ", Adr-Nr.:" + amvErrorVO.adrNr + ", Schutit-Key: "
          + amvErrorVO.schutitKey + ", Message: " + amvErrorVO.errorMsg + ") konnte nicht ausgeführt werden", e );
    }
  }

  /**
   * Im Fehlerfall bei einer AMV-Erinnerung wird die Fehlermeldung in die Tabelle JIPPS_AMV_ERROR geschrieben.
   * @param amvLaufId int AMV-Lauf-Nummer für die zu löschenden Einträge
   * @ejb.interface-method
   */
  public void deleteErrorMsg( int amvLaufId )
  {
    if ( amvErrorDAO == null )
    {
      amvErrorDAO = new AmvErrorDAO();
    }

    try
    {
      amvErrorDAO.delete( amvLaufId );
    }
    catch ( UnitOfWorkException e )
    {
      LOG.warn( "Die Error-Einträge für die AmvLauf-Nr.: " + amvLaufId + " konnte nicht gelöscht werden", e );
    }
  }

  /**
   * Updated das übergebene Value-Object auf der Tabelle JIPPS_AMV
   * @param amvVO AmvVO Value-Object
   * @ejb.interface-method
   */
  public void updateAMV( AmvVO amvVO )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }

    try
    {
      amvDAO.update( amvVO );
    }
    catch ( UnitOfWorkException e )
    {
      LOG.error( "Update in die AMV-Tabelle mit den Werten (Lauf-Nr.:" + amvVO.amvLaufId + ", Adr-Nr.:" + amvVO.adrNr + " , Bfakt-Id: " + amvVO.bfaktId
          + " , Schutit-Key: " + amvVO.schutitKey + ") konnte nicht ausgeführt werden", e );
    }
  }

  /**
   * Updated die AMV-Daten auf der Tabelle JIPPS_AMV mit der übergebenen AMV-Lauf-Nummer.
   * @param amvLaufId int AMV-Lauf-Nummer für die zu updatenden Einträge
   * @ejb.interface-method
   */
  public void updateAMV( int amvLaufId )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }

    try
    {
      amvDAO.update( amvLaufId );
    }
    catch ( UnitOfWorkException e )
    {
      LOG.warn( "Die AMV-Daten für die AmvLauf-Nr.: " + amvLaufId + " konnte nicht geupdated werden", e );
    }
  }

  /**
   * Aktualisiert auf der Tabelle JIPPS_AMV_LAUF die Lauf-Daten und schliesst den ersten Schritt der Verarbeitung ab
   * @param amvLaufVO AmvLaufVO Value-Object
   * @return int Status
   * @ejb.interface-method
   */
  public boolean updateAmvLauf( AmvLaufVO amvLaufVO )
  {
    if ( amvLaufDAO == null )
    {
      amvLaufDAO = new AmvLaufDAO();
    }
    try
    {
      amvLaufDAO.update( amvLaufVO );
      return true;
    }
    catch ( UnitOfWorkException e )
    {
      LOG.error( "Update auf AMV-Lauf-Id " + amvLaufVO.amvLaufId + " konnte nicht erzeugt werden", e );
      return false;
    }
  }

  /**
   * Erzeugt den entsprechenden Controller für die Erzeugung des Reports
   * @param reportTyp String Reporttyp
   * @param printJobList List printJobList
   * @return byte[] ArryListe mit dem gerenderten Pdf
   * @throws Exception Unerwarteter Fehler
   * @ejb.interface-method
   */
  public byte[] createControl( String reportTyp, List printJobList ) throws Exception
  {
    String className = "ch.ige.jipps." + reportTyp + "Control";

    try
    {
      Class c = Class.forName( className );
      Object o = c.newInstance();
      return ( (ch.ige.jipps.ReportControl) o ).makePdf( printJobList );
    }
    catch ( IllegalAccessException iax )
    {
      throw new Exception( "IllegalAccessException", iax );
    }
    catch ( InstantiationException iex )
    {
      throw new Exception( "InstantiationException", iex );
    }
    catch ( ClassNotFoundException cex )
    {
      throw new Exception( "ClassCastException", cex );
    }
  }

  /**
   * Überprüft ob der gestartete AMV-Lauf erfolgreich abgeschlossen werden kann.
   * @param amvLaufId int aktuelle AMV-Lauf-Nummer
   * @return boolean Status isLaufOk
   * @ejb.interface-method
   */
  public boolean selectCheckVerarbStat( int amvLaufId )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }
    try
    {
      return amvDAO.selectCheckVerarbStat( amvLaufId );
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "Verarbeitungs-Status konnte nicht geprüft werden", sqlex );
      return false;
    }
  }

  /**
   * Mit dieser Methode wird eine Liste zrückgegeben, die alle AMV-Daten enthält die den
   * Verarbeitungs-Status = 3 haben. Diese Methode wird für den Finish trotz fehlerhaften Reports verwendet.
   * @param amvLaufId int aktuelle AMV-Lauf-Nummer
   * @return List Liste mit allen erfolgreichen AMV-Daten
   * @ejb.interface-method
   */
  public List selectVerarbStatOk( int amvLaufId )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }
    try
    {
      return amvDAO.selectVerarbStatOk( amvLaufId );
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "Liste mit allen erfolgreichen AMV-Daten für den Finsih, konnte nicht erstellt werden", sqlex );
      return null;
    }
  }

  /**
   * Mit dieser Methode wird eine Liste zurückgegeben, die alle AMV-Daten enthält die den
   * Verarbeitungs-Status 0, 1 oder 2 haben. Diese Methode werden die fehlerhaften AMV-Daten ein weiteres mal Aufbereitet.
   * Diese Methode wird für den Restart von fehlerhaften Reports verwendet.
   * @param amvLaufId int aktuelle AMV-Lauf-Nummer
   * @return List Liste mit den fehlerhaften AMV-Daten
   * @ejb.interface-method
   */
  public List selectVerarbStatNok( int amvLaufId )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }
    try
    {
      return amvDAO.selectVerarbStatNok( amvLaufId );
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "Liste mit allen fehlerhaften AMV-Daten für den Restart, konnte nicht erstellt werden", sqlex );
      return null;
    }
  }

  /**
   * Insert des Reports als PDF-Datei in die Tabelle JIPPS_AMV.
   * @param amvVO Value-Object mit den End-Werten
   * @ejb.interface-method
   */
  public void insertReportPdf( AmvVO amvVO )
  {
    if ( amvDAO == null )
    {
      amvDAO = new AmvDAO();
    }
    try
    {
      amvDAO.updateReportPdf( amvVO );
    }
    catch ( UnitOfWorkException e )
    {
      LOG.error( "SQL-Fehler beim Einfügen des ReportPdf in die Tabelle JIPPS_AMV", e );
    }
  }
}
