package ch.ige.jipps;

import java.sql.Date;
import java.sql.SQLException;
import java.util.List;

import javax.ejb.EJBException;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.jipps.dao.DruckerDAO;
import ch.ige.jipps.dao.PrintjobDAO;
import ch.ige.jipps.vo.DruckerVO;
import ch.ige.jipps.vo.PrintJobVO;

/**
 * Implementation of {@link ch.ige.jipps.PrintJobStoreServices}.
 * <p>
 * Copyright (c) 2002-2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Johannes Lang
 * @author Marc Bouquet
 * @version $Id: PrintJobStoreServicesImpl.java,v 1.14 2008/05/08 13:49:48 lang Exp $
 */
public class PrintJobStoreServicesImpl implements PrintJobStoreServices
{
  private PrintjobDAO printjobDAO = null;

  private DruckerDAO druckerDAO = null;

  private PrintjobDAO getPrintjobDAO()
  {
    if ( printjobDAO == null )
    {
      printjobDAO = new PrintjobDAO();
    }
    return printjobDAO;
  }

  private DruckerDAO getDruckerDAO()
  {
    if ( druckerDAO == null )
    {
      druckerDAO = new DruckerDAO();
    }
    return druckerDAO;
  }

  // @see ch.ige.jipps.PrintJobStoreServices#insPrintJob(ch.ige.jipps.vo.PrintJobVO)
  public void insPrintJob( PrintJobVO printJobVO ) throws EJBException
  {
    try
    {
      getPrintjobDAO().writePrintjob( printJobVO );
    }
    catch ( SQLException e )
    {
      throw new EJBException( "Got SQLException", e );
    }
  }

  // @see ch.ige.jipps.PrintJobStoreServices#selectPrintJob(int)
  public PrintJobVO selectPrintJob( int printjob_Id ) throws EJBException
  {
    try
    {
      return getPrintjobDAO().readPrintjob( printjob_Id );
    }
    catch ( SQLException e )
    {
      throw new EJBException( "Got SQLException", e );
    }
  }

  // @see ch.ige.jipps.PrintJobStoreServices#updPrintJob(java.util.List, int, int, int)
  public void updPrintJob( List listPrintId, int status, int druckerId ) throws EJBException
  {
    try
    {
      getPrintjobDAO().updatePrintjob( listPrintId, status, druckerId );
    }
    catch ( UnitOfWorkException e )
    {
      throw new EJBException( "Got Exception", e );
    }
  }

  // @see ch.ige.jipps.PrintJobStoreServices#deleteJobs(java.sql.Date)
  public Integer deleteJobs( Date loeschDate )
  {
    return getPrintjobDAO().deletePrintjobs( loeschDate );
  }

  // @see ch.ige.jipps.PrintJobStoreServices#deletePrintJobs(int)
  public void deletePrintJobs( int batchId )
  {
    getPrintjobDAO().deletePrintjobs( batchId );
  }

  // @see ch.ige.jipps.PrintJobStoreServices#deletePrintJob(int, int)
  public void deletePrintJob( int reportId, int batchId )
  {
    getPrintjobDAO().deletePrintjob( reportId, batchId );
  }

  // @see ch.ige.jipps.PrintJobStoreServices#getNextBatchId()
  public int getNextBatchId() throws EJBException
  {
    try
    {
      return getPrintjobDAO().getNextBatchId();
    }
    catch ( SQLException e )
    {
      throw new EJBException( "Got SQLException", e );
    }
  }

  // @see ch.ige.jipps.PrintJobStoreServices#getNextPrintJobId()
  public int getNextPrintJobId() throws EJBException
  {
    try
    {
      return getPrintjobDAO().getNextPrintJobId();
    }
    catch ( SQLException e )
    {
      throw new EJBException( "Got SQLException", e );
    }
  }

  // @see ch.ige.jipps.PrintJobStoreServices#selectPrinter()
  public List<DruckerVO> selectPrinter() throws EJBException
  {
    try
    {
      return getDruckerDAO().getAllPrinters();
    }
    catch ( SQLException e )
    {
      throw new EJBException( "Got SQLException", e );
    }
  }

  /**
   * Gets all print jobs for a given bfaktId.
   * @param bfaktId The unique identifier of the batch run
   * @return A <code>List</code> of print jobs for the batch run
   * @throws EJBException if the print jobs can't be fetched
   * @see ch.ige.jipps.PrintJobStoreServices#getPrintJobsByBfaktId(java.lang.Integer)
   */
  public List<Integer> getPrintJobsByBfaktId( final Integer bfaktId ) throws EJBException
  {
    try
    {
      return getPrintjobDAO().readPrintJobsByBfaktId( bfaktId );
    }
    catch ( SQLException e )
    {
      throw new EJBException( e.getMessage(), e );
    }
  }

  // @see ch.ige.jipps.PrintJobStoreServices#deletePrintJob(int)
  public void deletePrintJob( int printJobId ) throws EJBException
  {
    getPrintjobDAO().deletePrintjob( printJobId );
  }

  // @see ch.ige.jipps.PrintJobStoreServices#deletePrintJobs(java.util.List)
  public void deletePrintJobs( List<Integer> printjobIds ) throws EJBException
  {
    getPrintjobDAO().deletePrintjobs( printjobIds );
  }

}
