package ch.ige.jipps.configuration.test;

import junit.framework.TestCase;
import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.dbutils.dao.DAO;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.util.client.ClientLogInitalizer;

/**
 * Tests {@link ch.ige.jipps.configuration.PathConfiguration}.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author lewis_lee
 * @version $Id: PathConfigurationTest.java,v 1.3 2007/12/18 12:31:37 lewis_lee Exp $
 */
public class PathConfigurationTest extends TestCase
{
  static
  {
    ClientLogInitalizer.initializeLoggingForTests( true, true );
    AbstractDAO.setRunningInJ2EEServer( false );
    System.setProperty( DAO.STANDALONE_CONFIG_PROPERTY_NAME, "true" );
  }

  /**
   * The normal constructor
   * @param args Any arguments
   */
  public PathConfigurationTest( String args )
  {
    super( args );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();

    // Start with a clean state, because these properties may already haven been set by other tests!
    if ( System.getProperty( PropertyConstants.JIPPS_PATH_XSL ) != null )
    {
      System.setProperty( PropertyConstants.JIPPS_PATH_XSL, "" );
    }
    if ( System.getProperty( PropertyConstants.JIPPS_PATH_IMAGE ) != null )
    {
      System.setProperty( PropertyConstants.JIPPS_PATH_IMAGE, "" );
    }
  }

  /**
   * Test method for {@link ch.ige.jipps.configuration.PathConfiguration#getPath(java.lang.String)}.
   */
  public void testGetPath()
  {
    // Get it from the database, no system property set
    String xslPath = PathConfiguration.getPath( PropertyConstants.JIPPS_PATH_XSL );
    assertNotNull( "Checking that path exists", xslPath );
    assertTrue( "Checking that path from DB is a UNIX path", xslPath.startsWith( "/" ) );
    assertTrue( "Checking that path from DB is a UNIX path", xslPath.endsWith( "/" ) );
    assertTrue( "Checking that path is correct", xslPath.indexOf( "xsl" ) > -1 );

    // Get it from the system property, note that this directory doesn't exist,
    // but it's not checked either!
    System.setProperty( PropertyConstants.JIPPS_PATH_XSL, "C:\\gugus\\wqeqwera" );
    xslPath = PathConfiguration.getPath( PropertyConstants.JIPPS_PATH_XSL );
    assertNotNull( "Checking that path exists", xslPath );
    assertEquals( "Checking resulting path", "C:\\gugus\\wqeqwera\\", xslPath );
  }

  /**
   * Test method for {@link ch.ige.jipps.configuration.PathConfiguration#getVerifiedPath(java.lang.String, boolean)}.
   */
  public void testGetVerifiedPath()
  {
    // It's not possible to get the verified path from the database, since it's a UNIX
    // path, and this test may run on Windows.

    try
    {
      String path = "C:\\gugus\\wqeqwera";
      System.setProperty( PropertyConstants.JIPPS_PATH_XSL, path );
      PathConfiguration.getVerifiedPath( PropertyConstants.JIPPS_PATH_XSL, false );
      fail( "Unknown path " + path + " must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      String path = "C:\\jp\\projects\\jipps\\.project";
      System.setProperty( PropertyConstants.JIPPS_PATH_XSL, path );
      PathConfiguration.getVerifiedPath( PropertyConstants.JIPPS_PATH_XSL, false );
      fail( "File " + path + " instead of directory must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    String path = "C:\\jp\\projects\\jipps";
    System.setProperty( PropertyConstants.JIPPS_PATH_XSL, path );
    String verifiedPath = PathConfiguration.getVerifiedPath( PropertyConstants.JIPPS_PATH_XSL, false );
    assertEquals( "Checking verified path", "C:\\jp\\projects\\jipps\\", verifiedPath ); // Added backslash
  }

  /**
   * Test method for
   * {@link ch.ige.jipps.configuration.PathConfiguration#getVerifiedPathAsUrl(java.lang.String, boolean)}.
   */
  public void testGetVerifiedPathAsUrl()
  {
    // The verification is already tested for getVerifiedPath()
    String path = "C:\\jp\\projects\\jipps";
    System.setProperty( PropertyConstants.JIPPS_PATH_XSL, path );
    String verifiedPath = PathConfiguration.getVerifiedPathAsUrl( PropertyConstants.JIPPS_PATH_XSL, false );
    assertEquals( "Checking verified path", "file:///C:/jp/projects/jipps/", verifiedPath ); // Added slash
  }

  /**
   * Resets the System properties to an empy String, so they are ignored during the next test run.
   */
  @Override
  protected void tearDown() throws Exception
  {
    super.tearDown();

    if ( System.getProperty( PropertyConstants.JIPPS_PATH_XSL ) != null )
    {
      System.setProperty( PropertyConstants.JIPPS_PATH_XSL, "" );
    }
    if ( System.getProperty( PropertyConstants.JIPPS_PATH_IMAGE ) != null )
    {
      System.setProperty( PropertyConstants.JIPPS_PATH_IMAGE, "" );
    }
  }
}
