package ch.ige.jipps.currentstate;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.spc.SPC;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.vo.StandDatVO;
import ch.ige.util.constants.ReportTextNumberHandler;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.report.ReportType;

/**
 * Builds the objects for the current state in the SPC report.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: SpcBuilder.java,v 1.14 2007/12/17 17:21:13 lewis_lee Exp $
 */
class SpcBuilder extends BaseBuilder
{
  // See interface
  @Override
  public List<CurrentStateElement> getProtectionStart( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    SPC spc = (SPC) ip;
    List<CurrentStateElement> protectionStarts = new ArrayList<CurrentStateElement>();

    if ( spc.getProtectionStart() != null )
    {
      CurrentStateElement protectionStart = new StandDatVO();
      protectionStart.setReportTextNumber( ReportTextNumberHandler.getReportTextNumberForBeginningOfProtection( spc.getBAGISType() ) );
      addSingleText( protectionStarts, protectionStart, DateFormatter.shortFormat( spc.getProtectionStart(), language ) );
    }

    return protectionStarts;
  }

  // See interface
  @Override
  public List<CurrentStateElement> getProtectionEnd( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language )
  {
    SPC spc = (SPC) ip;
    List<CurrentStateElement> protectionEnds = new ArrayList<CurrentStateElement>();

    if ( spc.getProtectionEnd() != null )
    {
      CurrentStateElement protectionEnd = new StandDatVO();
      protectionEnd.setReportTextNumber( ReportTextNumberHandler.getReportTextNumberForPeriodOfValidity( spc.getBAGISType() ) );
      addSingleText( protectionEnds, protectionEnd, DateFormatter.shortFormat( spc.getProtectionEnd(), language ) );
    }

    return protectionEnds;
  }
}
