package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.BFormTxtDruckVO;
import ch.ige.util.enums.BAGISLanguage;
import ch.ige.util.validation.Check;

/**
 * Accesses the text for printing invoices and letters in the table <code>BFORMTXT_DRUCK</code>.
 * 
 * <p>Copyright (c) 2002-2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: BFormTxtDruckDAO.java,v 1.1 2008/07/23 15:20:09 lewis_lee Exp $
 */
public class BFormTxtDruckDAO extends AbstractBagisDAO
{
  /**
   * The type of value object to return
   */
  private static final Class<BFormTxtDruckVO> VO_TYPE = BFormTxtDruckVO.class;

  /**
   * Selects a bunch of texts
   */
  private static final SQLStatement<BFormTxtDruckVO> SELECT = SQL
      .select( "SELECT TRIM(formart_cd) AS form_type, TRIM(sprach_cd) AS language, TRIM(textart) AS text_type, text " + "FROM bformtxt_druck "
          + "WHERE TRIM(formart_cd) = ? AND TRIM(sprach_cd) = ? " + "ORDER BY textart" );

  /**
   * Gets all texts for a certain form and a given language.
   * @param language The language
   * @param formType The type of form
   * @return The texts found for the form
   * @throws SQLException if the query fails
   */
  public List<BFormTxtDruckVO> select( BAGISLanguage language, Integer formType ) throws SQLException
  {
    Check.notNull( language, "language" );
    Check.notNull( formType, "form type" );

    return selectVOList( 15, VO_TYPE, SELECT, formType.toString(), language.toDB() );
  }
}
