package ch.ige.jipps.dao.test;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.List;
import java.util.Random;

import org.apache.commons.lang.ArrayUtils;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.jipps.dao.AmvDAO;
import ch.ige.jipps.vo.AmvLaufVO;
import ch.ige.jipps.vo.AmvVO;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.jipps.dao.AmvDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: AmvDAOTest.java,v 1.6 2007/06/21 07:48:52 lewis_lee Exp $
 */
public class AmvDAOTest extends BaseDAOWithCRUDTestTemplate<AmvDAO, AmvVO, Integer>
{
  /**
   * The ID of an AMV run that has finished
   */
  private static final int AMV_FINISHED_RUN = 9;

  /**
   * An IP that was processed during {@link #AMV_FINISHED_RUN}
   */
  private static final int FINISHED_IP_KEY = 1100417363;

  /**
   * The ID of an AMV run that hasn't finished yet
   */
  private static final int AMV_UNFINISHED_RUN = 15;

  /**
   * An IP that was processed during {@link #AMV_UNFINISHED_RUN}
   */
  private static final int UNFINISHED_IP_KEY = 1100340158;

  /**
   * The ID of the AMV batch run to which this belongs.
   */
  private Integer amvTestBatchRunId = null;

  /**
   * The only constructor
   * @param name The test's name
   */
  public AmvDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#selectCheckVerarbStat(int)}.
   * @throws Exception if the test fails
   */
  public void testSelectCheckVerarbStat() throws Exception
  {
    boolean isBatchRunDone = getDAO().selectCheckVerarbStat( 99999 );
    assertEquals( "Checking for unknown batch run ID", true, isBatchRunDone );

    //TODO Reactivate as soon as there is interesting data in the DB table again
//    isBatchRunDone = getDAO().selectCheckVerarbStat( AMV_FINISHED_RUN );
//    assertEquals( "Checking for known batch run ID", true, isBatchRunDone );

    //TODO Reactivate as soon as there is interesting data in the DB table again
//    isBatchRunDone = getDAO().selectCheckVerarbStat( AMV_UNFINISHED_RUN );
//    assertEquals( "Checking for known batch run ID", false, isBatchRunDone );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#selectVerarbStatOk(int)}.
   * @throws Exception if the test fails
   */
  public void testSelectVerarbStatOk() throws Exception
  {
    List<AmvVO> amvObjects = getDAO().selectVerarbStatOk( 99999 );
    assertEquals( "Checking for unknown batch run ID", 0, amvObjects.size() );

    amvObjects = getDAO().selectVerarbStatOk( AMV_UNFINISHED_RUN );
    assertEquals( "Checking for unfinished batch run ID", 0, amvObjects.size() );

    //TODO Reactivate as soon as there is interesting data in the DB table again
//    amvObjects = getDAO().selectVerarbStatOk( AMV_FINISHED_RUN );
//    assertEquals( "Checking for finished batch run ID", 50, amvObjects.size() );
//    boolean foundExpected = false;
//    for ( AmvVO amvVO: amvObjects )
//    {
//      if ( amvVO.schutitKey == FINISHED_IP_KEY )
//      {
//        compareVOs( getFinishedAmv(), amvVO );
//        foundExpected = true;
//        break;
//      }
//    }
//    assertTrue( "Checking that the expected AMV was found", foundExpected );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#selectVerarbStatNok(int)}.
   * @throws Exception if the test fails
   */
  public void testSelectVerarbStatNok() throws Exception
  {
    List<AmvVO> amvObjects = getDAO().selectVerarbStatNok( 99999 );
    assertEquals( "Checking for unknown batch run ID", 0, amvObjects.size() );

    amvObjects = getDAO().selectVerarbStatNok( AMV_FINISHED_RUN );
    assertEquals( "Checking for finished batch run ID", 0, amvObjects.size() );

    //TODO Reactivate as soon as there is interesting data in the DB table again
//    amvObjects = getDAO().selectVerarbStatNok( AMV_UNFINISHED_RUN );
//    assertEquals( "Checking for unfinished batch run ID", 9, amvObjects.size() );
//    boolean foundExpected = false;
//    for ( AmvVO amvVO: amvObjects )
//    {
//      if ( amvVO.schutitKey == UNFINISHED_IP_KEY )
//      {
//        compareVOs( getUnfinishedAmv(), amvVO );
//        foundExpected = true;
//        break;
//      }
//    }
//    assertTrue( "Checking that the expected AMV was found", foundExpected );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#selectAMVToId(int)}.
   * @throws Exception if the test fails
   */
  public void testSelectAMVToId() throws Exception
  {
    List<AmvVO> amvObjects = getDAO().selectAMVToId( 99999 );
    assertEquals( "Checking for unknown batch run ID", 0, amvObjects.size() );

    //TODO Reactivate as soon as there is interesting data in the DB table again
//    amvObjects = getDAO().selectAMVToId( AMV_FINISHED_RUN );
//    assertEquals( "Checking for finished batch run ID", 50, amvObjects.size() );
//    for ( AmvVO amvVO: amvObjects )
//    {
//      if ( amvVO.schutitKey == FINISHED_IP_KEY )
//      {
//        compareVOs( getFinishedAmv(), amvVO );
//        break;
//      }
//    }

    //TODO Reactivate as soon as there is interesting data in the DB table again
//    amvObjects = getDAO().selectAMVToId( AMV_UNFINISHED_RUN );
//    assertEquals( "Checking for unfinished batch run ID", 9, amvObjects.size() );
//    for ( AmvVO amvVO: amvObjects )
//    {
//      if ( amvVO.schutitKey == UNFINISHED_IP_KEY )
//      {
//        compareVOs( getUnfinishedAmv(), amvVO );
//        break;
//      }
//    }
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#selectBfakturList(int)}.
   * @throws Exception if the test fails
   */
  public void testSelectBfakturList() throws Exception
  {
    List<AmvVO> amvObjects = getDAO().selectBfakturList( 99999 );
    assertEquals( "Checking for unknown batch run ID", 0, amvObjects.size() );

    //TODO Reactivate as soon as there is interesting data in the DB table again
//    amvObjects = getDAO().selectBfakturList( AMV_FINISHED_RUN );
//    assertEquals( "Checking for finished batch run ID", 50, amvObjects.size() );
//    for ( AmvVO amv: amvObjects )
//    {
//      // These are the columns that are selected
//      assertTrue( "Checking invoice ID", amv.bfaktId != 0 );
//      assertTrue( "Checking address number", amv.adrNr != 0 );
//      assertTrue( "Checking IP key", amv.schutitKey != 0 );
//      if ( amv.schutitKey == FINISHED_IP_KEY )
//      {
//        assertEquals( "Checking invoice ID", 5348155, amv.bfaktId );
//        assertEquals( "Checking address number", 500442, amv.adrNr );
//        assertEquals( "Checking IP key", FINISHED_IP_KEY, amv.schutitKey );
//      }
//    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( AmvVO amv )
  {
    amv.amvLaufId = this.amvTestBatchRunId;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#insert(ch.ige.jipps.vo.AmvVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    this.amvTestBatchRunId = AmvLaufDAOTest.createAndSaveBatchRun();
    defaultTestCreate();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOAfterInsert(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOAfterInsert( AmvVO amv, AmvVO loadedAmv )
  {
    amv.createdDat = loadedAmv.createdDat;
    amv.changedDat = loadedAmv.changedDat;
    amv.printJobFile = loadedAmv.printJobFile;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( AmvVO originalAmv, AmvVO loadedAmv )
  {
    assertEquals( "Checking batch run ID", originalAmv.amvLaufId, loadedAmv.amvLaufId );
    assertEquals( "Checking address number", originalAmv.adrNr, loadedAmv.adrNr );
    assertEquals( "Checking IP key", originalAmv.schutitKey, loadedAmv.schutitKey );
    assertEquals( "Checking invoice ID", originalAmv.bfaktId, loadedAmv.bfaktId );
    assertEquals( "Checking IP number", originalAmv.schutitNr, loadedAmv.schutitNr );
    assertEquals( "Checking processing", originalAmv.verarbStat, loadedAmv.verarbStat );
    assertEquals( "Checking creation date", originalAmv.createdDat, loadedAmv.createdDat );
    assertEquals( "Checking change date", originalAmv.changedDat, loadedAmv.changedDat );
    if ( originalAmv.printJobFile == null )
    {
      assertEquals( "Checking PDF file", originalAmv.printJobFile, loadedAmv.printJobFile );
    }
    else
    {
      assertEquals( "Checking PDF file", originalAmv.printJobFile.length, loadedAmv.printJobFile.length );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( AmvVO amv )
  {
    // The only thing that's updated is the status
    amv.verarbStat = AmvLaufVO.FAULTY_DATA;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#update(ch.ige.jipps.vo.AmvVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    this.amvTestBatchRunId = AmvLaufDAOTest.createAndSaveBatchRun();
    defaultTestUpdate();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOAfterUpdate(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOAfterUpdate( AmvVO amv, AmvVO loadedAmv )
  {
    // Was changed automatically
    amv.changedDat = loadedAmv.changedDat;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#update(int)}.
   * @throws Exception if the test fails
   */
  public void testUpdateBatchRun() throws Exception
  {
    try
    {
      getDAO().update( 99999 );
      fail( "Update of unknown batch run must fail" );
    }
    catch ( UnitOfWorkException e )
    {
      // This is expected
    }

    this.amvTestBatchRunId = AmvLaufDAOTest.createAndSaveBatchRun();
    AmvVO amv = createAndInsert();

    assertTrue( "Checking processing status is set to some unused value", amv.verarbStat != AmvLaufVO.READY_FOR_PROCESSING );

    getDAO().update( amv.amvLaufId );

    AmvVO loadedAmv = read( this.amvTestBatchRunId );
    assertEquals( "Checking processing status", AmvLaufVO.READY_FOR_PROCESSING, loadedAmv.verarbStat );
    assertTrue( "Checking change date", loadedAmv.changedDat.after( amv.createdDat ) );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.AmvDAO#updateReportPdf(ch.ige.jipps.vo.AmvVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdateReportPdf() throws Exception
  {
    this.amvTestBatchRunId = AmvLaufDAOTest.createAndSaveBatchRun();
    AmvVO amv = createAndInsert();

    AmvVO loadedAmv = read( this.amvTestBatchRunId );
    modifyVOAfterInsert( amv, loadedAmv );
    compareVOs( amv, loadedAmv );

    // Now insert a 'PDF file'
    int fileSize = 75518;
    amv.printJobFile = new byte[fileSize];
    new Random().nextBytes( amv.printJobFile );
    getDAO().updateReportPdf( amv );

    loadedAmv = read( this.amvTestBatchRunId );
    compareVOs( amv, loadedAmv );
    assertTrue( "Checking file content", ArrayUtils.isEquals( amv.printJobFile, loadedAmv.printJobFile ) );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<AmvDAO> getDAOClass()
  {
    return AmvDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( AmvVO amv )
  {
    getDAO().insert( amv );
    return amv.getIdentifier();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected AmvVO read( Integer amvBatchRunId ) throws SQLException
  {
    return getDAO().selectAMVToId( amvBatchRunId ).get( 0 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( AmvVO amv )
  {
    getDAO().update( amv );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( Integer amvBatchRunId )
  {
    // Note that this is a cascading delete that also deletes the AmvVOs
    AmvLaufDAOTest.deleteBatchRun( amvBatchRunId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected AmvVO getSerializableVO() throws SQLException
  {
    try
    {
      AmvVO amvVO = new AmvVO();
      amvVO.amvLaufId = 999999;
      amvVO.adrNr = 6717;
      amvVO.schutitKey = 132731994;
      amvVO.bfaktId = 5346414;
      amvVO.schutitNr = "123456";
      amvVO.verarbStat = 37;
      amvVO.printJobFile = new byte[] { 85, 82, 88, 12, 56, -127, 127 };
      amvVO.createdDat = DateConverter.stringToTimestamp( "05.03.2006 10:23:11.000" );
      amvVO.changedDat = DateConverter.stringToTimestamp( "07.12.2013 23:59:59.000" );
      return amvVO;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }

  /**
   * Gets a known AMV from a finished AMV batch run.
   * @return A finished AMV
   * @throws ParseException if the date can't be parsed
   */
  private AmvVO getFinishedAmv() throws ParseException
  {
    AmvVO amvVO = new AmvVO();
    amvVO.amvLaufId = AMV_FINISHED_RUN;
    amvVO.adrNr = 500442;
    amvVO.schutitKey = FINISHED_IP_KEY;
    amvVO.bfaktId = 5348155;
    amvVO.schutitNr = "417363";
    amvVO.verarbStat = AmvLaufVO.PROCESSING_SUCCESSFUL;
    amvVO.createdDat = DateConverter.stringToTimestamp( "03.09.2004 11:47:51.000" );
    amvVO.changedDat = DateConverter.stringToTimestamp( "24.09.2004 13:17:47.554" );
    amvVO.printJobFile = new byte[166958];
    new Random().nextBytes( amvVO.printJobFile );
    return amvVO;
  }

  /**
   * Gets a known AMV from an unfinished AMV batch run.
   * @return An unfinished AMV
   * @throws ParseException if the date can't be parsed
   */
  private AmvVO getUnfinishedAmv() throws ParseException
  {
    AmvVO amvVO = new AmvVO();
    amvVO.amvLaufId = AMV_UNFINISHED_RUN;
    amvVO.adrNr = 501026;
    amvVO.schutitKey = UNFINISHED_IP_KEY;
    amvVO.bfaktId = 5346416;
    amvVO.schutitNr = "340158";
    amvVO.verarbStat = AmvLaufVO.READY_FOR_PROCESSING;
    amvVO.createdDat = DateConverter.stringToTimestamp( "21.09.2004 10:37:00.000" );
    amvVO.changedDat = DateConverter.stringToTimestamp( "21.09.2004 10:37:00.000" );
    amvVO.printJobFile = null;
    return amvVO;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( AmvVO originalAmv, AmvVO serializedAmv )
  {
    boolean isEqual = originalAmv.amvLaufId == serializedAmv.amvLaufId && originalAmv.adrNr == serializedAmv.adrNr
        && originalAmv.schutitKey == serializedAmv.schutitKey && originalAmv.bfaktId == serializedAmv.bfaktId
        && originalAmv.schutitNr.equals( serializedAmv.schutitNr ) && originalAmv.verarbStat == serializedAmv.verarbStat
        && originalAmv.createdDat.equals( serializedAmv.createdDat ) && originalAmv.changedDat.equals( serializedAmv.changedDat );
    if ( isEqual )
    {
      return ArrayUtils.isEquals( originalAmv.printJobFile, serializedAmv.printJobFile );
    }
    else
    {
      return isEqual;
    }
  }

  /**
   * Creates and inserts a trademark renewal.
   * This is normally used by other tests that require a trademark renewal to exist
   * before they can save their objects.
   * @param amvBatchRunId The unique ID of the AMV batch run
   * @return The created trademark renewal
   * @throws SQLException if the trademark renewal can't be created
   */
  public static AmvVO createAndSaveTrademarkRenewal( int amvBatchRunId ) throws SQLException
  {
    AmvDAOTest thisTest = new AmvDAOTest( "whatever" );
    AmvVO trademarkRenewal = thisTest.getSerializableVO();
    trademarkRenewal.amvLaufId = amvBatchRunId;
    thisTest.insert( trademarkRenewal );

    AmvVO loadedTrademarkRenewal = thisTest.read( amvBatchRunId );
    assertEquals( "Checking for batch run ID", amvBatchRunId, loadedTrademarkRenewal.amvLaufId );
    return loadedTrademarkRenewal;
  }
}
