package ch.ige.jipps.dao.test;

import java.util.List;

import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.BFormTxtDruckDAO;
import ch.ige.jipps.vo.BFormTxtDruckVO;
import ch.ige.util.enums.BAGISLanguage;

/**
 * Tests {@link ch.ige.jipps.dao.BFormTxtDruckDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: BFormTxtDruckDAOTest.java,v 1.1 2008/07/23 15:20:15 lewis_lee Exp $
 */
public class BFormTxtDruckDAOTest extends BaseDAOTestTemplate<BFormTxtDruckDAO, BFormTxtDruckVO>
{
  /**
   * The form type to use for the test.
   */
  private static final Integer FORM_TYPE = 800;

  /**
   * The German texts for {@link #FORM_TYPE}.
   */
  private static final String[] GERMAN_TEXTS = { "Markenabteilung", "<b>Widerspruchsverfahren<br>Nr. %%$$Schutit - %%$$Titel<br>%%$$Inhaber<br></b>",
      "Sehr geehrte Damen und Herren", "Mit freundlichen Grüssen<br><br><br><br>%%$$Untersch" };

  /**
   * The Italian texts for {@link #FORM_TYPE}.
   */
  private static final String[] ITALIAN_TEXTS = { "Divisione dei marchi", "<b>Procedura di opposizione<br>N° %%$$Schutit - %%$$Titel<br>%%$$Inhaber<br></b>",
      "Gentile Signora, Egregio Signore,", "Distinti saluti<br><br><br><br>%%$$Untersch" };

  /**
   * The only constructor.
   * @param name The test's name
   */
  public BFormTxtDruckDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<BFormTxtDruckDAO> getDAOClass()
  {
    return BFormTxtDruckDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BFormTxtDruckDAO#select(BAGISLanguage, Integer)}.
   * @throws Exception if the test fails
   */
  public void testSelect() throws Exception
  {
    try
    {
      getDAO().select( BAGISLanguage.DE, null );
      fail( "Null form type must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().select( null, FORM_TYPE );
      fail( "Null language must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    List<BFormTxtDruckVO> texts = getDAO().select( BAGISLanguage.DE, 9999999 );
    assertTrue( "Checking for unknown form type", texts.isEmpty() );

    texts = getDAO().select( BAGISLanguage.EN, FORM_TYPE );
    assertTrue( "Checking for unknown language", texts.isEmpty() );

    // German
    texts = getDAO().select( BAGISLanguage.DE, FORM_TYPE );
    checkTexts( texts, GERMAN_TEXTS, BAGISLanguage.DE );

    // Italian
    texts = getDAO().select( BAGISLanguage.IT, FORM_TYPE );
    checkTexts( texts, ITALIAN_TEXTS, BAGISLanguage.IT );
  }

  /**
   * Checks the texts.
   * @param texts The texts to check
   * @param expectedTexts What the texts should contain
   * @param language The language the texts should contain
   */
  private void checkTexts( List<BFormTxtDruckVO> texts, String[] expectedTexts, BAGISLanguage language )
  {
    assertEquals( "Checking number of texts", expectedTexts.length, texts.size() );
    for ( int i = 0; i < expectedTexts.length; i++ )
    {
      BFormTxtDruckVO text = texts.get( i );
      assertEquals( "Checking text type", i - 1, text.textType - 1 );
      assertEquals( "Checking form type", FORM_TYPE, text.formType );
      assertEquals( "Checking language", language, text.language );
      assertEquals( "Checking text", expectedTexts[i], text.text );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected BFormTxtDruckVO getSerializableVO()
  {
    BFormTxtDruckVO text = new BFormTxtDruckVO();
    text.formType = 234;
    text.textType = 7;
    text.language = BAGISLanguage.FR;
    text.text = "Bonjour, ça va?";
    return text;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( BFormTxtDruckVO originalText, BFormTxtDruckVO serializedText )
  {
    return originalText.formType.equals( serializedText.formType ) && originalText.textType.equals( serializedText.textType )
        && originalText.language.equals( serializedText.language ) && originalText.text.equals( serializedText.text );
  }
}
