package ch.ige.jipps.dao.test;

import java.util.List;
import java.util.Map;

import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.FormTextDAO;
import ch.ige.jipps.vo.FormTextVO;
import ch.ige.util.collection.MapConverter;
import ch.ige.util.enums.BAGISLanguage;

/**
 * Tests {@link ch.ige.jipps.dao.FormTextDAO}. 
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: FormTextDAOTest.java,v 1.1 2008/07/23 14:54:41 lewis_lee Exp $
 */
public class FormTextDAOTest extends BaseDAOTestTemplate<FormTextDAO, FormTextVO>
{
  /**
   * The area where the text is used
   */
  private static final String FORM_AREA = "BA2";

  /**
   * The only constructor
   * @param name The test's name
   */
  public FormTextDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<FormTextDAO> getDAOClass()
  {
    return FormTextDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.FormTextDAO#select(ch.ige.util.enums.BAGISLanguage, String)}.
   * @throws Exception if the test fails
   */
  public void testSelect() throws Exception
  {
    try
    {
      getDAO().select( null, FORM_AREA );
      fail( "Null language must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().select( BAGISLanguage.DE, null );
      fail( "Null area must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().select( BAGISLanguage.DE, "" );
      fail( "Empty area must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    List<FormTextVO> texts = getDAO().select( BAGISLanguage.DE, "whatever" );
    assertTrue( "Checking for unknown form type", texts.isEmpty() );

    texts = getDAO().select( BAGISLanguage.FR, FORM_AREA );
    assertTrue( "Checking for number of texts", texts.size() > 125 );

    // Check some known cases
    Map<Integer, FormTextVO> textMap = MapConverter.toMap( texts, Integer.class, "lineNumber" );

    checkText( textMap, 717, "Office des faillites" );
    checkText( textMap, 600, "Taxes" );
    checkText( textMap, 509, "Usufruitier" );
  }

  /**
   * Checks whether a text exists and is correct.
   * @param textMap The texts to check
   * @param textLine The number of the line to check
   * @param expectedText The text to expect
   */
  private void checkText( Map<Integer, FormTextVO> textMap, Integer textLine, String expectedText )
  {
    assertTrue( "Checking text line " + textLine + " exists ", textMap.containsKey( textLine ) );
    assertEquals( "Checking text line " + textLine, expectedText, textMap.get( textLine ).text );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected FormTextVO getSerializableVO()
  {
    FormTextVO text = new FormTextVO();
    text.lineNumber = 33;
    text.language = BAGISLanguage.IT;
    text.text = "Ciao a tutti!";
    return text;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( FormTextVO originalVO, FormTextVO serializedVO )
  {
    return originalVO.lineNumber.equals( serializedVO.lineNumber ) && originalVO.language.equals( serializedVO.language )
        && originalVO.text.equals( serializedVO.text );
  }
}
