package ch.ige.jipps.dao.test;

import java.sql.SQLException;
import java.text.ParseException;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.jipps.dao.LoeschungLaufDAO;
import ch.ige.jipps.vo.LoeschungLaufVO;
import ch.ige.util.constants.CancellationConstants;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.jipps.dao.LoeschungLaufDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: LoeschungLaufDAOTest.java,v 1.5 2008/05/22 09:23:48 lewis_lee Exp $
 */
public class LoeschungLaufDAOTest extends BaseDAOWithCRUDTestTemplate<LoeschungLaufDAO, LoeschungLaufVO, Integer>
{
  /**
   * A cancellation run ID which exists in the database
   */
  private static final Integer KNOWN_CANCELLATION_RUN_ID = 1048;

  /**
   * A cancellation run ID that's invalid
   */
  private static final Integer INVALID_CANCELLATION_RUN_ID = -1;

  /**
   * A cancellation run ID which doesn't exist in the database
   */
  private static final Integer UNKNOWN_CANCELLATION_RUN_ID = 999999;

  /**
   * The only constructor
   * @param name The test's name
   */
  public LoeschungLaufDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungLaufDAO#select(int)}.
   * @throws Exception if the test fails
   */
  public void testSelect() throws Exception
  {
    defaultTestRead( getSerializableVO(), Integer.valueOf( -1 ), UNKNOWN_CANCELLATION_RUN_ID, KNOWN_CANCELLATION_RUN_ID );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( LoeschungLaufVO cancellationRun )
  {
    // These variables are not set during the insert
    cancellationRun.verarbEnde = null;
    cancellationRun.anzLoeschGen = 0;
    cancellationRun.createdDate = null;
    cancellationRun.changedDate = null;
    cancellationRun.verarbStart = null;
    cancellationRun.verarbEnde = null;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungLaufDAO#insert(ch.ige.jipps.vo.LoeschungLaufVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#createAndInsert()
  @Override
  public LoeschungLaufVO createAndInsert() throws SQLException
  {
    // Used for other tests setting up a cancellation run
    return super.createAndInsert();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOAfterInsert(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOAfterInsert( LoeschungLaufVO cancellationRun, LoeschungLaufVO loadedCancellationRun )
  {
    // These are set automatically during insert
    cancellationRun.verarbStart = loadedCancellationRun.verarbStart;
    cancellationRun.createdDate = loadedCancellationRun.createdDate;
    cancellationRun.changedDate = loadedCancellationRun.changedDate;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( LoeschungLaufVO cancellationRun )
  {
    cancellationRun.jippsStat = 7;
    cancellationRun.anzLoeschGen = 512;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungLaufDAO#update(ch.ige.jipps.vo.LoeschungLaufVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOAfterUpdate(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOAfterUpdate( LoeschungLaufVO cancellationRun, LoeschungLaufVO loadedCancellationRun )
  {
    // These are set automatically during update
    cancellationRun.verarbEnde = loadedCancellationRun.verarbEnde;
    cancellationRun.changedDate = loadedCancellationRun.changedDate;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.LoeschungLaufDAO#delete(int)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( INVALID_CANCELLATION_RUN_ID, UNKNOWN_CANCELLATION_RUN_ID );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( LoeschungLaufVO originalCancellationRun, LoeschungLaufVO loadedCancellationRun )
  {
    assertEquals( "Checking cancellation run ID", originalCancellationRun.loeschungLaufId, loadedCancellationRun.loeschungLaufId );
    assertEquals( "Checking state of the cancellation run", originalCancellationRun.jippsStat, loadedCancellationRun.jippsStat );
    assertEquals( "Checking number of invoices that were read", originalCancellationRun.anzBfaktGel, loadedCancellationRun.anzBfaktGel );
    assertEquals( "Checking number of generated cancellations", originalCancellationRun.anzLoeschGen, loadedCancellationRun.anzLoeschGen );
    assertEquals( "Checking type of cancellation run", originalCancellationRun.druckArt, loadedCancellationRun.druckArt );
    assertEquals( "Checking creation date", originalCancellationRun.createdDate, loadedCancellationRun.createdDate );
    assertEquals( "Checking change date", originalCancellationRun.changedDate, loadedCancellationRun.changedDate );
    assertEquals( "Checking processing start", originalCancellationRun.verarbStart, loadedCancellationRun.verarbStart );
    assertEquals( "Checking processing end", originalCancellationRun.verarbEnde, loadedCancellationRun.verarbEnde );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  protected LoeschungLaufVO getSerializableVO() throws SQLException
  {
    try
    {
      // This is a known cancellation run
      LoeschungLaufVO cancellationRun = new LoeschungLaufVO();
      cancellationRun.loeschungLaufId = KNOWN_CANCELLATION_RUN_ID;
      //TODO Use constant
      cancellationRun.jippsStat = 3;
      cancellationRun.anzBfaktGel = 0;
      cancellationRun.anzLoeschGen = 0;
      cancellationRun.druckArt = CancellationConstants.IP_NOT_PAID_INTEGER;
      cancellationRun.createdDate = DateConverter.stringToTimestamp( "27.03.2008 16:27:05.000" );
      cancellationRun.changedDate = DateConverter.stringToTimestamp( "27.03.2008 16:27:05.000" );
      cancellationRun.verarbStart = DateConverter.stringToTimestamp( "27.03.2008 16:27:05.000" );
      cancellationRun.verarbEnde = DateConverter.stringToTimestamp( "27.03.2008 16:27:05.000" );
      return cancellationRun;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( LoeschungLaufVO originalVO, LoeschungLaufVO serializedVO )
  {
    return originalVO.loeschungLaufId == serializedVO.loeschungLaufId && originalVO.verarbStart.equals( serializedVO.verarbStart )
        && originalVO.verarbEnde.equals( serializedVO.verarbEnde ) && originalVO.jippsStat == serializedVO.jippsStat
        && originalVO.anzBfaktGel == serializedVO.anzBfaktGel && originalVO.anzLoeschGen == serializedVO.anzLoeschGen
        && originalVO.createdDate.equals( serializedVO.createdDate ) && originalVO.changedDate.equals( serializedVO.changedDate )
        && originalVO.druckArt.equals( serializedVO.druckArt );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<LoeschungLaufDAO> getDAOClass()
  {
    return LoeschungLaufDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( LoeschungLaufVO cancellationRun )
  {
    return getDAO().insert( cancellationRun );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object)
  @Override
  protected LoeschungLaufVO read( Integer cancellationRunId ) throws SQLException
  {
    return getDAO().select( cancellationRunId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( LoeschungLaufVO cancellationRun )
  {
    getDAO().update( cancellationRun );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.lang.Object)
  @Override
  protected void delete( Integer cancellationRunId )
  {
    getDAO().delete( cancellationRunId );
  }
}
