package ch.ige.jipps.exception;

/**
 * Creates a message for the GUI from an exception.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author lewis_lee
 * @version $Id: ExceptionMessage.java,v 1.1 2006/12/21 14:38:41 lewis_lee Exp $
 */
public class ExceptionMessage
{
  /**
   * Gets a message from the exception by getting its message or the message from the causing exception(s). The depth of
   * the causing exceptions is 2.
   * 
   * @param e The exception whose message is taken
   * @return An error message usable for a GUI
   */
  public static String getMessage( Exception e )
  {
    String message = extractMessage( e );
    if ( message == null && e.getCause() != null )
    {
      message = extractMessage( e.getCause() );
      if ( message == null && e.getCause().getCause() != null )
      {
        message = extractMessage( e.getCause().getCause() );
      }
    }
    return message;
  }

  /**
   * Gets the message from a <code>Throwable</code> without looking at its cause.
   * 
   * @param t The <code>Throwable</code> whose message is fetched
   * @return The message or <code>null</code> if it can't be fetched
   */
  private static String extractMessage( Throwable t )
  {
    String message = null;
    if ( t != null )
    {
      message = t.getMessage();
      if ( message == null )
      {
        message = t.getLocalizedMessage();
      }
      if ( message == null && t.getCause() == null )
      {
        message = t.toString();
      }
    }
    return message;
  }
}
