package ch.ige.jipps.ftp.web.test;

import java.util.Date;

import junit.framework.TestCase;

import org.apache.commons.io.FilenameUtils;

import ch.ige.ipobjects.time.BAGISTime;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;

public class FTPConfigurationTest extends TestCase
{

  public FTPConfigurationTest( String name )
  {
    super( name );
  }
  
  private static final String TMP_PATH_DOWNLOAD = FilenameUtils.concat( System.getProperty( "java.io.tmpdir" ) , "ftpservlet/downlaod" );
  
  private static final String TMP_PATH_UPLOAD = FilenameUtils.concat( System.getProperty( "java.io.tmpdir" ) , "ftpservlet/uplaod" );
  
  
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    FTPServletConfiguration.addConfigurationEntry(FTPServletConfiguration.TMP_DIR_DOWNLOAD, TMP_PATH_DOWNLOAD);
    FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_UPLOAD, TMP_PATH_UPLOAD);
  }
  
  public void testPathDownloadFailure()
  {
    String path;
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( null, null ) ;
      fail( "Path should not be created because of missing relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "/", "ma" ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "\\", "ma"  ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));

    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "/ige/tmp/all", "ma" ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "/ige/tmp", "/ige/tmp/all" ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "/ige/tmp", "ige/tmp/all" ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch ( Exception e )
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "ige/tmp", "ige/tmp/all" ) ;
      fail( "Path should not be created because username is a path " + path );
    }
    catch ( Exception e )
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "Username" ));
    }
    
    FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_DOWNLOAD, null);
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "downlaod/all", "ma" ) ;
      fail( "Path should not be created because root is null " + path );
    }
    catch ( Exception e )
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "Root" ));
    }

  }

  public void testPathDownload()
  {
    assertEquals( TMP_PATH_DOWNLOAD + "\\ma\\downlaod\\all", FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "downlaod/all", "ma" ) );
    assertEquals(TMP_PATH_DOWNLOAD + "\\ma\\downlaod\\all", FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "downlaod\\all", "ma" ) );
    assertEquals(TMP_PATH_DOWNLOAD + "\\ma\\downlaod\\all\\test", FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "downlaod/all\\test", "ma" ) );

  }
 
  
  public void testPathUploadFailure()
  {
    String path;
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( null, null ) ;
      fail( "Path should not be created because of missing relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "/", "ma" ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "\\", "ma"  ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));

    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "/ige/tmp/all", "ma" ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "/ige/tmp", "/ige/tmp/all" ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch (Exception e)
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "/ige/tmp", "ige/tmp/all" ) ;
      fail( "Path should not be created because of not relative path " + path );
    }
    catch ( Exception e )
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "relative" ));
    }
    
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "ige/tmp", "ige/tmp/all" ) ;
      fail( "Path should not be created because username is a path " + path );
    }
    catch ( Exception e )
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "Username" ));
    }
    
    FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_UPLOAD, null);
    try
    { 
      path = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "uplaod/all", "ma" ) ;
      fail( "Path should not be created because root is null " + path );
    }
    catch ( Exception e )
    {
      assertTrue ( e instanceof IllegalArgumentException);
      assertTrue ( e.getMessage().contains( "Root" ));
    }

  }
  
  public void testPathUpload()
  {
    assertEquals( TMP_PATH_UPLOAD + "\\ma\\uplaod\\all", FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "uplaod/all", "ma" ) );
    assertEquals( TMP_PATH_UPLOAD + "\\ma\\uplaod\\all", FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "uplaod\\all", "ma" ) );
    assertEquals( TMP_PATH_UPLOAD + "\\ma\\uplaod\\all\\test", FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "uplaod/all\\test", "ma" ) );

  }

  public void testLosNumber()
  {
    assertEquals( "Los24", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2006123100 ) ).toString() );
    assertEquals( "Los24", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007010100 ) ).toString() );
    assertEquals( "Los24", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007010900 ) ).toString() );
    assertEquals( "Los01", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007011000 ) ).toString() );
    assertEquals( "Los01", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007011500 ) ).toString() );
    assertEquals( "Los01", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007011600 ) ).toString() );
    assertEquals( "Los01", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007012400 ) ).toString() );
    assertEquals( "Los02", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007012500 ) ).toString() );
    assertEquals( "Los02", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007013100 ) ).toString() );
    assertEquals( "Los02", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007020100 ) ).toString() );
    assertEquals( "Los02", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007020900 ) ).toString() );
    assertEquals( "Los03", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007021000 ) ).toString() );

    assertEquals( "Los03", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007021500 ) ).toString() );
    assertEquals( "Los03", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007022400 ) ).toString() );
    assertEquals( "Los04", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007022500 ) ).toString() );
    assertEquals( "Los04", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007030900 ) ).toString() );
    assertEquals( "Los05", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007031000 ) ).toString() );
    assertEquals( "Los11", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007061500 ) ).toString() );
    assertEquals( "Los11", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007062400 ) ).toString() );
    assertEquals( "Los12", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007062500 ) ).toString() );
    assertEquals( "Los12", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007063000 ) ).toString() );
    assertEquals( "Los12", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007070100 ) ).toString() );
    assertEquals( "Los12", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007070900 ) ).toString() );
    assertEquals( "Los13", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007071000 ) ).toString() );
    assertEquals( "Los13", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007071500 ) ).toString() );
    assertEquals( "Los13", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007072400 ) ).toString() );
    assertEquals( "Los14", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007072500 ) ).toString() );

    assertEquals( "Los18", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007100100 ) ).toString() );
    assertEquals( "Los18", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007100900 ) ).toString() );
    assertEquals( "Los19", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007101000 ) ).toString() );
    assertEquals( "Los19", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007101500 ) ).toString() );
    assertEquals( "Los19", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007102400 ) ).toString() );
    assertEquals( "Los20", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007102500 ) ).toString() );
    assertEquals( "Los20", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007103000 ) ).toString() );
    assertEquals( "Los20", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007110100 ) ).toString() );

    assertEquals( "Los22", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007113000 ) ).toString() );
    assertEquals( "Los22", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007120100 ) ).toString() );
    assertEquals( "Los22", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007120900 ) ).toString() );
    assertEquals( "Los23", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007121000 ) ).toString() );
    assertEquals( "Los23", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007121500 ) ).toString() );
    assertEquals( "Los23", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007122400 ) ).toString() );
    assertEquals( "Los24", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007122500 ) ).toString() );
    assertEquals( "Los24", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007123000 ) ).toString() );
    assertEquals( "Los24", FTPServletConfiguration.computeLosFromEreigiDate( EreigIDat.from( 2007123100 ) ).toString() );

  }

  public void testLosNumberNull()
  {
    try
    {
      FTPServletConfiguration.computeLosFromEreigiDate( null ).toString();
      fail( "should fail with ereigidat null" );
    }
    catch ( Exception e )
    {
      assertTrue( e instanceof IllegalArgumentException );
    }
  }

  public void testResults() throws Exception
  {
    assertEquals( "Sendung_von_" + BAGISTime.formatDateISO8601( new Date() ), FTPServletConfiguration.getDefaultFolderForLocalStorage() );

  }

}
