package ch.ige.jipps.ftp.web.util;

import java.io.IOException;
import java.io.Serializable;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.UnavailableException;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.io.FilenameUtils;
import org.apache.log4j.Logger;

import ch.ige.ipobjects.patent.vo.PatentDocumentVO;
import ch.ige.ipobjects.time.BAGISTime;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.ftp.web.FTPServiceException;
import ch.ige.jipps.ftp.web.FTPServlet;

/**
 * Utility class for the configuration of the FTP Servlet
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   fabrizioli
 * @version  $Id: FTPServletConfiguration.java,v 1.13 2009/02/04 14:45:46 fabrizioli Exp $
 */
public class FTPServletConfiguration
{

  public static final Logger LOG = Logger.getLogger( FTPServlet.class );

  public static final Integer EXPECTED_PATDOC_NUMBER = 32;

  public static final String DEFAULT_PREVIOUS_PAGE = "ftpUpload.jsp";

  public static final String CMD_FILE_RESOURCE = "epaupload_start.bat";

  public static final String DESTINATION_REQUEST_PARAMETER = "destination";

  public static final String SERVERPPATH_REQUEST_PARAMETER = "serverPath";

  public static final String DATASOURCE_FOLDER_REQUEST_PARAMETER = "clientPath";

  public static final String BIBLIOGRAPHIC_DATA_REQUEST_PARAMETER = "biblio";

  public static final String FOLDER_MARK_FOR_SESSION = "sessionMark";

  public static final String BATCH_REQUEST_PARAMETER = "batch";

  public static final String PATDOCIDS_REQUEST_PARAMETER = "patDocIds";

  public static final String PUBL_DATE_REQUEST_PARAMETER = "publDate";

  public static final String PATENT_NUMMER_REQUEST_PARAMETER = "patentNumber";

  public static final String EMPLOYEE_REQUEST_PARAMETER = "ma";

  public static final String SERVICE_REQUEST_PARAMETER = "service";

  public static final Integer PACKET_SIZE = 40;

  public static final String TMP_DIR_UPLOAD = "temp.dir.upload";

  public static final String TMP_DIR_DOWNLOAD = "temp.dir.download";

  public static final String FOLDER_NAME_ROOT = "Sendung_von_";

  public static final String REL_PATH_FOR_XML = "epa.path.xml";

  public static final String REL_PATH_FOR_PDF = "epa.path.pdf";

  public static final String REL_PATH_FOR_XMLBIBLIO = "epa.path.xmlbiblio";

  public static final String REL_PATH_FOR_PDFBIBLIO = "epa.path.pdfbiblio";

  public static final String CODE_FOR_PUBLICATION_DATE = "${publDate}";

  public static final String CODE_FOR_KINDCODE = "${kind}";

  public static final String CODE_FOR_YEAR = "${year}";

  public static final String CODE_FOR_LANG = "${lang}";

  public static final String CODE_FOR_LOS = "${los}";

  public static final String BIBLIO_FOLDER_NAME = "biblio";

  public static final String MARKER_FOR_BIBLIO_FOLDER_NAME = "toupload";

  public static final String FOLDER_SCHABLONE_FOR_XML = "XML-FULL\\" + CODE_FOR_YEAR + "\\" + CODE_FOR_LOS + "\\";

  public static final String FOLDER_SCHABLONE_FOR_XML_BIBLIO = MARKER_FOR_BIBLIO_FOLDER_NAME + "\\" + BIBLIO_FOLDER_NAME + "\\" + CODE_FOR_YEAR + "\\"
      + CODE_FOR_LOS + "\\";

  public static final String FOLDER_SCHABLONE_FOR_PDF = CODE_FOR_PUBLICATION_DATE + "\\" + CODE_FOR_KINDCODE + "\\" + CODE_FOR_LANG + "\\";

  public static final String FOLDER_SCHABLONE_FOR_PDF_BIBLIO = BIBLIO_FOLDER_NAME + "PDF\\" + CODE_FOR_PUBLICATION_DATE + "\\" + CODE_FOR_KINDCODE + "\\"
      + CODE_FOR_LANG + "\\";

  public static final String SHARE_DESTINATION = "SHARE";

  public static final String EPA_DESTINATION = "EPA";

  public static final String FAILURE_FOLDER_UPLOAD = "not-sent";

  public static final String FAILURE_FOLDER_DOWNLOAD = "not-all-read";

  public static final String FOLDER_BATCH_CMD_FILE = "batch_cmd";

  public static final String FAILURE_CODE = "#Fehler#";

  // config of ReuestProcessor
  public static final String SERVICE_EXTRACT = "extract";

  public static final String SERVICE_UPLOAD = "upload";

  public static final String SERVICE_DOWNLOAD = "download";

  public static final String SERVICE_LIST = "list";

  public static final String SERVICE_SYSTEMTEST = "systemtest";

  public static final String SESSION_PARAMETER_RESULT_BEAN = "resultBean";

  public static final String CONTEXT_FTPLOGIN_PARAMTERER_MAP = "ftpLoginParameters";

  public static final String CONTEXT_SHARE_DATA_URL = "shareDataUrl";

  public static final String XSLT_BIBLIO_NAME = "biblio_data";

  private static Map<String, String> serverConfig = new HashMap<String, String>( 2 );

  public static CharSequence computeLosFromEreigiDate( EreigIDat ereig_idat )
  {
    if ( ereig_idat == null || ereig_idat.equals( EreigIDat.NULL ) )
    {
      throw new IllegalArgumentException( "Cannot compute a los number from ereigidat null " + ereig_idat );
    }
    Calendar dateAsCalender = BAGISTime.convertIDatToCalendar( ereig_idat );
    int month = dateAsCalender.get( Calendar.MONTH );
    int day = dateAsCalender.get( Calendar.DAY_OF_MONTH );
    int eval = ( day + ( 31 - 9 ) ) % 31;
    int soll_day = ( eval != 0 ? eval : 31 ); //IF(MOD(A2+31-9;31)<>0;MOD(A2+31-9;31);31); //
    eval = ( soll_day > day ? month - 1 : month );
    int soll_month = ( eval % 12 >= 0 ? eval % 12 : 11 ); //IF(IF(C2>A2;B2-1;B2)<>0;IF(C2>A2;B2-1;B2);12)
    eval = ( soll_month + 1 ) * 2 - Math.round( ( 31f - soll_day ) / 31f );
    int los = eval;
    // commons-lang not part of ige .ear yet
    return ( ( los < 10 ) ? "Los0" + los : "Los" + los );
  }

  public static String formatDateForFolderName( EreigIDat ereig_idat )
  {
    return BAGISTime.formatIDatISO8601( ereig_idat.getCompareTime() ).replace( "-", "" );
  }

  public static String getDefaultFolderForLocalStorage()
  {
    return FOLDER_NAME_ROOT + BAGISTime.todayAsSqlDate();
  }

  /**
   * Get the path for the folder containing the CHST36 xml and image files
   * defined in the <code>PatentDocumentVO</code>. Each patent document is
   * stored in a separate folder within the path specified in this method.
   * <b/> The path for the xml and image files is defined in the configuration
   * file or as a system property. Otherwise the default path is used.
   * 
   * @param patdocVO
   *            the patent document retrieved from BAGIS
   * @return the path where the folder containing the data of the patent
   *         document has to be stored.
   * @throws ServletException 
   */
  public static String getXMLFolderPath( PatentDocumentVO patdocVO, String folderName, String language, boolean biblio ) throws ServletException
  {
    String pathSchablone = null;
    if ( biblio )
    {
      pathSchablone = getConfigurationEntry( REL_PATH_FOR_XML, FOLDER_SCHABLONE_FOR_XML_BIBLIO );
    }
    else
    {
      pathSchablone = getConfigurationEntry( REL_PATH_FOR_XML, FOLDER_SCHABLONE_FOR_XML );
    }
    pathSchablone = FilenameUtils.separatorsToSystem( translateSchablone( pathSchablone, patdocVO, language ) );
    return FilenameUtils.concat( getDataFolderForUpload( folderName ), pathSchablone ); // folder
  }

  public static String translateSchablone( String pathSchablone, PatentDocumentVO patdocVO, String lang )
  {
    String dateForFolder = formatDateForFolderName( patdocVO.getEreig_idat() );
    String translation = pathSchablone.replace( CODE_FOR_PUBLICATION_DATE, dateForFolder );
    translation = translation.replace( CODE_FOR_YEAR, dateForFolder.subSequence( 0, 4 ) );
    translation = translation.replace( CODE_FOR_KINDCODE, patdocVO.getKind_cd().toDB() );
    translation = translation.replace( CODE_FOR_LANG, lang );
    translation = translation.replace( CODE_FOR_LOS, computeLosFromEreigiDate( patdocVO.getEreig_idat() ) );
    return translation;
  }

  /**
   * Get the path for the folder containing pdf for the patent document
   * defined in the <code>PatentDocumentVO</code>. Each patent document pdf
   * is stored in a separate folder within the path specified in this method.
   * <b/> The path for the pds is defined in the configuration file or as a
   * system property. Otherwise the default path is used.
   * 
   * @param patdocVO
   *            the patent document retrieved from BAGIS
   * @return the path where the folder containing the data of the patent
   *         document has to be stored.
   * @throws ServletException 
   */
  public static String getPDFFolderPath( PatentDocumentVO patdocVO, String folderName, String language, boolean biblio ) throws ServletException
  {
    String pathSchablone = null;
    if ( biblio )
    {
      pathSchablone = getConfigurationEntry( REL_PATH_FOR_XML, FOLDER_SCHABLONE_FOR_PDF_BIBLIO );
    }
    else
    {
      pathSchablone = getConfigurationEntry( REL_PATH_FOR_PDF, FOLDER_SCHABLONE_FOR_PDF );
    }

    pathSchablone = FilenameUtils.separatorsToSystem( translateSchablone( pathSchablone, patdocVO, language ) );
    return FilenameUtils.concat( getDataFolderForUpload( folderName ), pathSchablone );
  }

  private static String getDataFolderForUpload( String folderName ) throws ServletException
  {
    return getDataFolder( getConfigurationEntry( TMP_DIR_UPLOAD, null ), folderName );
  }

  private static String getDataFolderForDownload( String folderName ) throws ServletException
  {
    return getDataFolder( getConfigurationEntry( TMP_DIR_DOWNLOAD, null ), folderName );
  }

  private static String getDataFolder( String root, String folderName ) throws ServletException
  {
    if ( root == null )
      throw new UnavailableException( "Not correctly configured: root for data storage path for upload is missing" );
    return FilenameUtils.concat( root, folderName );
  }

  private static synchronized String getConfigurationEntry( String key, String defaultValue )
  {
    String contextEntry = serverConfig.get( key );
    if ( contextEntry == null )
    {
      return defaultValue;
    }
    return contextEntry;
  }

  /**
   * Add a key-value pair to the configuration map. 
   * <p>In the configuration map are stored the data required only to this class. </p>
   */
  public static synchronized void addConfigurationEntry( String key, String value )
  {
    serverConfig.put( key, value );
  }

  /**
   * Get the relative path for storing the data on the share.
   * The data on the share are stored per execution date and not per user, therefore the sub-folder of the "ma" folder is returned when it exists.
   * Otherwise the name of the last folder of the input path is returned.
   * @param folderPath the path where the data are read from
   * @param ma the username
   * @return the relative path on the server where the data have to be stored.
   */
  public static String mapTmpFolderPathToRelShareServerPath( String folderPath, String ma )
  {
    int maIndex = folderPath.indexOf( ma );
    String partialPath = FilenameUtils.getName( folderPath );
    if ( maIndex > -1 && maIndex + ma.length() + 3 < folderPath.length() )
    {
      partialPath = folderPath.substring( maIndex + ma.length() + 1 );
    }
    return partialPath;
  }

  /**
   * Get the absolute path for the temporary directory for the data to upload. 
   * @param folderPath the path where the data are expected without considering the username
   * @param ma the username
   * @return the path where the data are stored considering also the username
   */
  public static String mapRequestFolderPathToTmpDirUpload( String folderPath, String ma )
  {
    return mapRequestFolderPathToTmpDir( folderPath, ma, getConfigurationEntry( TMP_DIR_UPLOAD, null ) );
  }

  /**
   * Get the absolute path for the temporary directory for the data to download. 
   * @param folderPath the path where the data are expected without considering the username
   * @param ma the username
   * @return the path where the data are stored considering also the username
   */
  public static String mapRequestFolderPathToTmpDirDownload( String folderPath, String ma )
  {
    return mapRequestFolderPathToTmpDir( folderPath, ma, getConfigurationEntry( TMP_DIR_DOWNLOAD, null ) );
  }

  private static String mapRequestFolderPathToTmpDir( String folderPath, String ma, String root )
  {
    if ( folderPath == null || folderPath.startsWith( "\\" ) || folderPath.startsWith( "/" ) )
    {
      // concat takes second parameter if second is absolute
      throw new IllegalArgumentException( "Path to concatenate has to be relative" + folderPath );
    }
    if ( ma == null || ma.contains( "\\" ) || ma.contains( "/" ) )
    {
      throw new IllegalArgumentException( "Username must not be null or a path" + ma );
    }

    if ( root == null || root.length() == 0 )
    {
      throw new IllegalArgumentException( "Root for tmpDir must not be null" );
    }
    String foldersInTmpDir = FilenameUtils.concat( ma, folderPath );
    String mappedTmpDir = FilenameUtils.concat( root, foldersInTmpDir );
    if ( mappedTmpDir == null )
    {
      throw new IllegalArgumentException( "Cannot return a path null " );
    }
    return mappedTmpDir;
  }

  /**
   * Get the representation of the receipt from the data upload/download for the output
   */
  public static String modifyReceiptForOutput( Serializable receipt )
  {
    if ( receipt instanceof String )
    {
      int init = FTPServletConfiguration.getConfigurationEntry( TMP_DIR_UPLOAD, null ).length() + 11;
      if ( init > 0 && init < ( (String) receipt ).length() )
      {
        receipt = ( (String) receipt ).substring( init );
      }
    }
    return receipt.toString();
  }

  /**
   * Extract a request parameter from the HTTPRequest, if the parameter is required and not found an <code>FTPServiceException</code> is thrown.
   */
  public static String extractRequestParameter( HttpServletRequest request, String parameterName, boolean required ) throws IOException, FTPServiceException
  {
    String parameterValue = request.getParameter( parameterName );
    LOG.debug( parameterName + " set to " + parameterValue );
    if ( required && ( parameterValue == null || parameterValue.length() == 0 ) )
    {
      String msg = "No " + parameterName + " available: Cannot execute ftp request";
      LOG.warn( msg );
      throw new FTPServiceException( msg );
    }
    return parameterValue;

  }

  /**
   * Generate the error message from the information passed as parameter
   */
  public static String generateErrorMessage( Throwable e, String message )
  {
    String msg = FAILURE_CODE + " " + message + " " + ( e != null ? e.getMessage() : "" );
    LOG.error( msg );
    return msg;
  }

}
