package ch.ige.jipps.header;

import java.sql.Date;
import java.util.Locale;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.util.date.LanguageDateFormatter;

/**
 * Builds the special cases for the IP header in the report.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: IpHeaderBuilder.java,v 1.13 2008/01/10 17:18:38 lewis_lee Exp $
 */
interface IpHeaderBuilder
{
  /**
   * Gets the report's title
   * @param ip The IP containing all data
   * @param reportDate The report's date
   * @return The title of the report
   */
  public String getDescription( GenericIP ip, Date reportDate );

  /**
   * Gets the report's language.
   * @param ip The IP containing all data
   * @param language The language as requested by the user
   * @return The report's language
   */
  public Locale getReportLanguage( GenericIP ip, Locale language );

  /**
   * Gets the formatted IP number.
   * @param ip The IP containing all data
   * @return The formatted IP number
   */
  public String getFormattedIpNumber( GenericIP ip );

  /**
   * Gets the formatted IP number for an SPC's base patent.
   * @param ip The IP containing all data
   * @return The formatted IP number for an SPC's base patent
   */
  public String getFormattedBaseIpNumber( GenericIP ip );

  /**
   * Gets the formatted application date.
   * @param ip The IP containing all data
   * @param dateFormatter Formats the date
   * @return The formatted date of application
   */
  public String getApplicationDate( GenericIP ip, LanguageDateFormatter dateFormatter );

  /**
   * Gets the formatted date showing when the application was published.
   * @param ip The IP containing all data
   * @param dateFormatter Formats the date
   * @return The formatted application publication date
   */
  public String getApplicationPublicationDate( GenericIP ip, LanguageDateFormatter dateFormatter );

  /**
   * Gets the formatted application number.
   * @param ip The IP containing all data
   * @return The formatted application number
   */
  public String getFormattedApplicationNumber( GenericIP ip );

  /**
   * For an SPC, this gets the formatted date when the original patent was registered.
   * @param ip The IP containing all data
   * @param dateFormatter Formats the date
   * @return The formatted start of protection for an SPC's base patent
   */
  public String getBaseProtectionStart( GenericIP ip, LanguageDateFormatter dateFormatter );

  /**
   * Gets the formatted last date for opposition to a trademark.
   * After a trademark has been published, anyone can enter opposition with the IPI.
   * @param ip The IP containing all data
   * @param reportDate The date when the report is printed
   * @param dateFormatter Formats the date
   * @return The formatted last date for opposition to a trademark
   */
  public String getOppositionDeadline( GenericIP ip, Date reportDate, LanguageDateFormatter dateFormatter );

  /**
   * Gets the publication information.
   * @param ip The IP containing all data
   * @param dateFormatter Formats the date
   * @param reportDate The date when the report is printed
   * @param isFirstPublication If this is <code>true</code>, the first publication
   *        is fetched, otherwise the last publication
   * @return A <code>List</code> of publication information, see description above
   */
  public PublicationVO getPublication( GenericIP ip, LanguageDateFormatter dateFormatter, Date reportDate, boolean isFirstPublication );
}
