package ch.ige.jipps.header.test;

import org.custommonkey.xmlunit.XMLUnit;
import org.w3c.dom.Document;

import ch.ige.jipps.test.AbstractTestFacadeXml;
import ch.ige.jipps.vo.ReferenceXMLVO;

/**
 * Gets the reference XML documents and compares their general print and page header to what is generated on the fly.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version FacadeXmlPrintHeaderTest.java,v 1.10 2007/03/23 16:03:55 lewis_lee Exp
 */
public class FacadeXmlPrintHeaderTest extends AbstractTestFacadeXml
{
  /**
   * The default constructor
   * @param args Any argument
   */
  public FacadeXmlPrintHeaderTest( String args )
  {
    super( args );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.checkAllIpTypes = true;
  }

  /**
   * Gets the XML and tests part of it in a subclass. This is required because otherwise JUnit doesn't find any test.
   * @throws Exception if the test fails
   */
  public void testGetXml() throws Exception
  {
    super.testGetXml( false );
  }

  /**
   * Compares the expected header with the actual header.
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was returned
   * @param referenceXmlVo The VO containing the data for the test case
   */
  @Override
  protected void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    Document expectedDocument = XMLUnit.buildTestDocument( expectedXml );
    Document actualDocument = XMLUnit.buildTestDocument( actualXml );

    compareXPath( expectedDocument, actualDocument, "/auszug/igekopf" );

    compareXPath( expectedDocument, actualDocument, "/auszug/header/printDat" );
    compareXPath( expectedDocument, actualDocument, "/auszug/header/page" );

    compareXPath( expectedDocument, actualDocument, "/auszug/header/eintrNr" );
    compareXPath( expectedDocument, actualDocument, "/auszug/header/eintrNrLbl" );

    // To be 100% sure...
    checkNode( expectedXml, actualXml, "header", referenceXmlVo );
  }
}
