package ch.ige.jipps.history.description.test;

import java.text.ParseException;
import java.util.Locale;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.model.GenericType;
import ch.ige.ipobjects.design.Design;
import ch.ige.ipobjects.design.model.DesignEvent;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.history.description.PublicationText;
import ch.ige.util.date.DateConverter;
import ch.ige.util.enums.PublicationTarget;
import ch.ige.util.enums.report.ReportType;

/**
 * Tests the creation of the publication text for designs.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version $Id: DesignPublicationTextTest.java,v 1.13 2008/05/21 13:49:37 lewis_lee Exp $
 */
public class DesignPublicationTextTest extends BasePublicationTextTestTemplate
{

  /**
   * The only constructor
   * @param name The test's name
   */
  public DesignPublicationTextTest( String name )
  {
    super( name );
  }

  // See superclass
  @Override
  public void testDefinitiveAndRequiredPublication() throws Exception
  {
    Design design = createDesign( true, true, "15.02.2005", 33 );
    Event event = design.getEvents().get( 0 );

    // **********************************************
    // Register report
    // **********************************************
    String publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021400 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state before event", "Noch nicht publiziert.", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021500 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state on event date", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021600 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state after event date", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    // **********************************************
    // Dossier report
    // **********************************************
    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021400 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state before event", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021500 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state on event date", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021600 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state after event date", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    // No publication date
    design = createDesign( true, true, null, -1 );
    event = design.getEvents().get( 0 );

    // **********************************************
    // Register report
    // **********************************************
    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021400 ), event, Locale.GERMAN,
          ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail without publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021500 ), event, Locale.GERMAN,
          ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail without publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021600 ), event, Locale.GERMAN,
          ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail without publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    // **********************************************
    // Dossier report
    // **********************************************
    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021400 ), event, Locale.GERMAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail without publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021500 ), event, Locale.GERMAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail without publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021600 ), event, Locale.GERMAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail without publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }
  }

  // See superclass
  @Override
  public void testDefinitiveAndNonRequiredPublication() throws Exception
  {
    Design design = createDesign( true, false, "18.01.2006", 2 );
    Event event = design.getEvents().get( 0 );
    String publicationText = null;

    // **********************************************
    // Register report
    // **********************************************
    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
          ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011800 ), event, Locale.ITALIAN,
          ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011901 ), event, Locale.ITALIAN,
          ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    // **********************************************
    // Dossier report
    // **********************************************
    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011800 ), event, Locale.ITALIAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011901 ), event, Locale.ITALIAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    // No publication date
    design = createDesign( true, false, null, -1 );
    event = design.getEvents().get( 0 );

    // **********************************************
    // Register report
    // **********************************************
    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state before event", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011800 ), event, Locale.ITALIAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state on event date", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011901 ), event, Locale.ITALIAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state after event date", null, publicationText );

    // **********************************************
    // Dossier report
    // **********************************************
    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state before event", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011800 ), event, Locale.ITALIAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state on event date", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011901 ), event, Locale.ITALIAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state after event date", null, publicationText );
  }

  // See superclass
  @Override
  public void testPreliminaryAndRequiredPublication() throws Exception
  {
    Design design = createDesign( false, true, "15.02.2005", 33 );
    Event event = design.getEvents().get( 0 );

    // **********************************************
    // Register report
    // **********************************************
    String publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021400 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state before event", "Noch nicht publiziert.", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021500 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state on event date", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021600 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state after event date", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    // **********************************************
    // Dossier report
    // **********************************************
    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021400 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state before event", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021500 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state on event date", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021600 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state after event date", "Veröffentlicht im mod. dép. Nr. 33 des Jahres 2005", publicationText );

    // No publication date
    design = createDesign( false, true, null, -1 );
    event = design.getEvents().get( 0 );

    // **********************************************
    // Register report
    // **********************************************
    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021400 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state before event", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021500 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state on event date", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021600 ), event, Locale.GERMAN,
        ReportType.REGISTER_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state after event date", null, publicationText );

    // **********************************************
    // Dossier report
    // **********************************************
    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021400 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state before event", "Noch nicht publiziert.", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021500 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state on event date", "Noch nicht publiziert.", publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2005021600 ), event, Locale.GERMAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state after event date", "Noch nicht publiziert.", publicationText );
  }

  // See superclass
  @Override
  public void testPreliminaryAndNonRequiredPublication() throws Exception
  {
    Design design = createDesign( false, false, "18.01.2006", 2 );
    Event event = design.getEvents().get( 0 );
    String publicationText = null;

    // **********************************************
    // Register report
    // **********************************************
    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
          ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011800 ), event, Locale.ITALIAN,
          ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011901 ), event, Locale.ITALIAN,
          ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    // **********************************************
    // Dossier report
    // **********************************************
    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011800 ), event, Locale.ITALIAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    try
    {
      publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011901 ), event, Locale.ITALIAN,
          ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
      fail( "Must fail with publication date" );
    }
    catch ( IllegalStateException ex )
    {
      // This is expected
    }

    // No publication date
    design = createDesign( false, false, null, -1 );
    event = design.getEvents().get( 0 );

    // **********************************************
    // Register report
    // **********************************************
    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state before event", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011800 ), event, Locale.ITALIAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state on event date", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011901 ), event, Locale.ITALIAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking state after event date", null, publicationText );

    // **********************************************
    // Dossier report
    // **********************************************
    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state before event", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011800 ), event, Locale.ITALIAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state on event date", null, publicationText );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011901 ), event, Locale.ITALIAN,
        ReportType.DOSSIER_REPORT_AFTER_APPLICATION, true );
    assertEquals( "Checking state after event date", null, publicationText );
  }

  // See superclass
  @Override
  public void testPublicationChange() throws Exception
  {
    // The last issue of the old publication
    Design design = createDesign( true, true, "18.01.2002", 12 );
    Event event = design.getEvents().get( 0 );

    String publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking before publication change", "Pubblicato nel FBDM N. 12 dell'anno 2002", publicationText );

    // The first issue of the new publication
    design = createDesign( true, true, "18.01.2002", 13 );
    event = design.getEvents().get( 0 );

    publicationText = PublicationText.getPublicationText( design, EreigIDat.from( 2006011799 ), event, Locale.ITALIAN,
        ReportType.SEARCH_REPORT_AFTER_REGISTRATION, true );
    assertEquals( "Checking before publication change", "Pubblicato nel mod. dép. no. 13 dell'anno 2002", publicationText );
  }

  /**
   * Creates a design.
   * @param isPublicationDefinitive Shows whether the publication of the event is definitive
   * @param isPublicationRequired Shows whether the publication of the event is required
   * @param publicationDate The publication date in the format <code>dd.MM.yyyy</code>
   * @param publicationIssue The publication issue; if it's <code>-1</code>, it's ignored
   * @return The whole design with one event containing the given parameters
   * @throws ParseException if the publication date can't be parsed
   */
  private Design createDesign( boolean isPublicationDefinitive, boolean isPublicationRequired, String publicationDate, int publicationIssue )
      throws ParseException
  {
    Design design = (Design) Types.DESIGN.createFacade( Types.DESIGN.newInstance() );
    design.set( GenericType.PROP_IPKEY, 99999 );
    design.set( GenericType.PROP_TYPE, Types.DESIGN.getBAGISType() );
    Integer issue = null;
    if ( publicationIssue != -1 )
    {
      issue = Integer.valueOf( publicationIssue );
    }
    Event event = DesignEvent.wrap( new MockEvent( isPublicationRequired, isPublicationDefinitive, DateConverter.stringToSqlDate( publicationDate ), issue,
        null, PublicationTarget.PAPER ) );
    setEvent( design, event );
    return design;
  }
}
