package ch.ige.jipps.history.detail;

import java.util.List;

import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.ipobjects.time.TimePeriod;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.util.date.DateFormatter;

/**
 * Contains the detail lines and maybe the report text number.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: Detail.java,v 1.10 2007/12/19 08:57:02 lewis_lee Exp $
 */
public class Detail
{
  /**
   * The object that created this
   */
  private BaseHistoryItemDetail<? extends TimePeriod> creator;

  /**
   * The number of the report text to load, which may be <code>null</code>
   */
  public Integer reportTextNumber;

  /**
   * A <code>List</code> of <code>String</code>s containining the lines for the text block
   */
  public List<String> textLines;

  /**
   * Contains the historized image (e.g. for a trademark).
   */
  public ImageVO image;

  /**
   * The formatted end date in the format <code>dd.MM.yyyy</code>
   */
  public String endDate;

  /**
   * The empty constructor is private because only {@link #Detail( TimePeriod, BaseHistoryItemDetail )} must be used.
   */
  @SuppressWarnings( "unused" )
  private Detail()
  {
    super();
  }

  /**
   * Constructs this with a time period, the only way to construct this.
   * @param timePeriod The time period; if this is not <code>null</code> and it has an end date, then {@link #endDate}
   * is set.
   * @param creator The object that creates this
   */
  public Detail( TimePeriod timePeriod, BaseHistoryItemDetail<? extends TimePeriod> creator )
  {
    this.creator = creator;
    if ( timePeriod != null && timePeriod.getPeriodStopTime() != 0 )
    {
      this.endDate = DateFormatter.shortFormat( EreigIDat.from( timePeriod.getPeriodStopTime() ).getDate() );
    }
  }

  /**
   * Gets the object that created this.
   * @return The object that created this
   */
  public BaseHistoryItemDetail<? extends TimePeriod> getCreator()
  {
    return this.creator;
  }
}
