package ch.ige.jipps.history.detail.trademark;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Locale;
import java.util.Map;

import ch.ige.ipobjects.common.dao.XxDomainDAO;
import ch.ige.ipobjects.common.vo.XxDomainVO;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.trademark.model.TrademarkKindHistory.TrademarkKindItem;
import ch.ige.ipobjects.trademark.vo.MarMarartVO;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.collection.MapConverter;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.constants.XxDomainConstants;

/**
 * Creates the detail lines for a trademark's kind.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: TrademarkKindDetail.java,v 1.8 2007/12/19 08:57:21 lewis_lee Exp $
 */
public class TrademarkKindDetail extends BaseHistoryItemDetail<TrademarkKindItem>
{
  /**
   * Creates this with its detail data.
   * @param trademarkKinds The trademark kinds to format
   * @param language The language to use for formatting
   */
  public TrademarkKindDetail( TimePeriodCollection<TrademarkKindItem> trademarkKinds, Locale language )
  {
    super( trademarkKinds, null, language );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<TrademarkKindItem, Detail> createBuilder()
  {
    return new TrademarkKindDetailBuilder();
  }

  /**
   * The class that actually formats the trademark kind.
   */
  public class TrademarkKindDetailBuilder extends AbstractHistoryItemDetailBuilder<TrademarkKindItem, Detail>
  {
    /**
     * Creates the builder.
     */
    public TrademarkKindDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<TrademarkKindItem> detailData )
    {
      try
      {
        final TrademarkKindItem trademarkKind = detailData.get( 0 );
        Detail formattedTrademarkKind = createFormattedTrademarkKind( trademarkKind );
        if ( formattedTrademarkKind != null )
        {
          return createListSingleDataItemSingleRepresentation( trademarkKind, formattedTrademarkKind );
        }
        else
        {
          return EmptyHistoryItemDetailList.getInstance();
        }
      }
      catch ( SQLException ex )
      {
        throw new RuntimeException( "Can't read trademark kind names", ex );
      }
    }

    /**
     * Creates the formatted trademark kinds.
     * @param trademarkKind The trademark kind format
     * @return The formatted trademark kind
     * @throws SQLException if the trademark kind name can't be fetched
     */
    private Detail createFormattedTrademarkKind( TrademarkKindItem trademarkKind ) throws SQLException
    {
      if ( trademarkKind.getKinds().isEmpty() )
      {
        return null;
      }

      Detail detail = new Detail( trademarkKind, TrademarkKindDetail.this );
      detail.reportTextNumber = ReportTextConstants.TRADEMARK_KIND_INTEGER;
      detail.textLines = new ArrayList<String>();

      Map<Integer, XxDomainVO> trademarkKindMap = MapConverter.toMap( new XxDomainDAO().findDomainValue( XxDomainConstants.TRADEMKARK_KIND,
          TrademarkKindDetail.this.getLanguage() ), Integer.class, "domainLnr" );

      for ( MarMarartVO numericTrademarkKind: trademarkKind.getKinds() )
      {
        XxDomainVO trademarkKindName = trademarkKindMap.get( numericTrademarkKind.getTrademarkKind().toDB() );
        if ( trademarkKindName == null )
        {
          throw new IllegalArgumentException( "Found no trademark kind (domain value) for " + numericTrademarkKind );
        }
        detail.textLines.add( trademarkKindName.getBez() );
      }
      return detail;
    }
  }
}
