package ch.ige.jipps.imageconv;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Map;

import org.apache.log4j.Logger;

import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.constants.ImageTypeConstants;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGDecodeParam;
import com.sun.image.codec.jpeg.JPEGImageDecoder;
import com.sun.media.jai.codec.MemoryCacheSeekableStream;

/**
 * Writes JPEG images.
 *
 * @author Johannes Lang
 * @version $Id: JPEGImageWriter.java,v 1.15 2009/05/05 15:47:28 lewis_lee Exp $
 */
public class JPEGImageWriter extends ImageWriter
{
  /**
   * The log.
   */
  private static final Logger LOG = Logger.getLogger( JPEGImageWriter.class );

  /**
   * Creates this.
   */
  public JPEGImageWriter()
  {
    super();
  }

  // @see ch.ige.jipps.imageconv.ImageWriter#processImage(java.lang.String, byte[])
  @Override
  public Map<String, Object> processImage( String pathWithoutFileType, byte[] image ) throws IOException
  {
    String destinationImageType = processImageData( image, pathWithoutFileType );
    LOG.debug( "destinationImageType returned from ImageConversion: " + destinationImageType );
    Map<String, Object> map = getDimensions();
    map.put( ImageProcessor.IMAGE_TYPE, destinationImageType );
    return map;
  }

  /**
   * Processes a JPEG image.
   * @param image The image (raw data)
   * @param path The path without the extension (file type)
   * @return The file extension of the converted file, usually {@link ImageTypeConstants#IMAGE_TYPE_JPEG}
   * @throws IOException If the file cannot be read
   */
  private String processImageData( byte[] image, String path ) throws IOException
  {
    extractSizeAndDensity( new MemoryCacheSeekableStream( new ByteArrayInputStream( image ) ) );
    String sourceFilePath = path + FileConstants.EXTENSION_JPG;
    writeToFile( image, sourceFilePath );
    return callServlet( sourceFilePath, ImageTypeConstants.IMAGE_TYPE_JPG );
  }

  /**
   * Reads the image size and density from a JPEG image.
   * @param memorySeekableStream Contains the image data
   * @throws IOException if the metadata can't be extracted from the image
   */
  private void extractSizeAndDensity( MemoryCacheSeekableStream memorySeekableStream ) throws IOException
  {
    JPEGImageDecoder jpgDecoder = JPEGCodec.createJPEGDecoder( memorySeekableStream );
    jpgDecoder.decodeAsBufferedImage();
    JPEGDecodeParam jpgParam = jpgDecoder.getJPEGDecodeParam();
    this.imageHeigthInPixel = jpgParam.getHeight();
    this.imageWidthInPixel = jpgParam.getWidth();
    try
    {
      if ( jpgParam.getDensityUnit() == JPEGDecodeParam.DENSITY_UNIT_DOTS_CM )
      {
        this.xDensity = (int) ( jpgParam.getXDensity() * ImageWriter.CONVERSION_DPCM_TO_DPI );
        this.yDensity = (int) ( jpgParam.getYDensity() * ImageWriter.CONVERSION_DPCM_TO_DPI );
      }
      else
      {
        this.xDensity = jpgParam.getXDensity();
        this.yDensity = jpgParam.getYDensity();
      }
    }
    catch ( Exception ex )
    {
      this.xDensity = ImageProcessor.DEFAULT_DENSITY;
      this.yDensity = ImageProcessor.DEFAULT_DENSITY;
    }
  }
}
