package ch.ige.jipps.test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.jms.Message;

import ch.ige.ipobjects.common.SwissregPublicationCode;
import ch.ige.ipobjects.common.vo.SwissregPublicationVO;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.PublicationConfirmationMDBean;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.jipps.PublicationConfirmationMDBean}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: PublicationConfirmationMDBeanTest.java,v 1.2 2008/05/22 13:54:28 lewis_lee Exp $
 */
public class PublicationConfirmationMDBeanTest extends PublicationConfirmationTest
{
  /**
   * The publications to print
   */
  private List<SwissregPublicationVO> publications;

  /**
   * The event numbers to print. This is normally configured in the DB,
   * but must be done manually for this test to run.
   */
  private static final List<Integer> EVENT_NUMBERS_TO_PRINT = Arrays.asList( new Integer[] { 48004 } );

  /**
   * The unit under test
   */
  private PublicationConfirmationMDBean bean = new PublicationConfirmationMDBean();

  /**
   * The only constructor
   * @param name The test's name
   */
  public PublicationConfirmationMDBeanTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    super.setupDatabase();

    this.publications = new ArrayList<SwissregPublicationVO>( TRADEMARKS.length );
    for ( int i = 0; i < TRADEMARKS.length; i++ )
    {
      SwissregPublicationVO publication = new SwissregPublicationVO( IPKeyVO.create( TRADEMARKS[i] ), EreigIDat.from( EVENT_DATES[i] ),
          SwissregPublicationCode.PUBLICATION_WITH_DATE, DateConverter.stringToSqlDate( PUBLICATION_DATES[i] ) );
      this.publications.add( publication );
    }
  }

  /**
   * Tests {@link ch.ige.jipps.PublicationConfirmationMDBean#printPublicationConfirmations(Message, List, List)}.
   * @throws Exception if the test fails
   */
  public void testPrintPublicationConfirmations() throws Exception
  {
    try
    {
      this.bean.printPublicationConfirmations( null, null, this.publications );
      fail( "Null event numbers must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      this.bean.printPublicationConfirmations( null, new ArrayList<Integer>(), this.publications );
      fail( "Empty event numbers must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      this.bean.printPublicationConfirmations( null, EVENT_NUMBERS_TO_PRINT, null );
      fail( "Null publication confirmations must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      this.bean.printPublicationConfirmations( null, EVENT_NUMBERS_TO_PRINT, new ArrayList<SwissregPublicationVO>() );
      fail( "Empty publication confirmations must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    List<AnswerVO> answers = this.bean.printPublicationConfirmations( null, EVENT_NUMBERS_TO_PRINT, this.publications );
    assertNotNull( "Checking for answers", answers );
    assertEquals( "Checking number of answers", TRADEMARKS.length, answers.size() );
    for ( int i = 0; i < answers.size(); i++ )
    {
      AnswerVO answer = answers.get( i );
      assertEquals( "Checking that the print job was created", AnswerVO.STATUS_OK, answer.printStatus );
      assertEquals( "Checking for print job IDs", 1, answer.listPrintId.size() );
      assertEquals( "Checking for form type", PrintJobVO.LETTER, answer.formTyp );
    }
  }
}
