package ch.ige.jipps.text.test;

import java.util.List;

import junit.framework.TestCase;

import org.apache.commons.lang.StringUtils;

import ch.ige.jipps.text.ConvertText;

/**
 * Tests {@link ch.ige.jipps.text.ConvertText}.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Priska Steiner
 * @author    Eric Lewis
 * @version   $Id: ConvertTextTest.java,v 1.11 2008/07/21 13:37:18 lewis_lee Exp $
 */
public class ConvertTextTest extends TestCase
{
  /**
   * @see junit.framework.TestCase#TestCase(String)
   */
  public ConvertTextTest( String args )
  {
    super( args );
  }

  /**
   * Tests the creation of the HTML tags
   */
  public void testGetHtmlTags()
  {
    // Better than no test at all  ;-)
    List<String> tags = ConvertText.getHtmlTags();
    assertEquals( "Checking number of HTML tags", 14, tags.size() );

    assertTrue( "Checking that <p> exists", tags.indexOf( "<p>" ) > -1 );
    assertTrue( "Checking that </p> exists", tags.indexOf( "</p>" ) > -1 );
    assertTrue( "Checking that <p/> exists", tags.indexOf( "<p/>" ) > -1 );

    assertTrue( "Checking that <b> exists", tags.indexOf( "<b>" ) > -1 );
    assertTrue( "Checking that </b> exists", tags.indexOf( "</b>" ) > -1 );
    assertTrue( "Checking that <b/> doesn't exist", tags.indexOf( "<b/>" ) == -1 );

    assertTrue( "Checking that <strong> exists", tags.indexOf( "<b>" ) > -1 );
    assertTrue( "Checking that </strong> exists", tags.indexOf( "</strong>" ) > -1 );
    assertTrue( "Checking that <strong/> doesn't exist", tags.indexOf( "<strong/>" ) == -1 );

    assertTrue( "Checking that <h1> exists", tags.indexOf( "<h1>" ) > -1 );
    assertTrue( "Checking that </h1> exists", tags.indexOf( "</h1>" ) > -1 );
    assertTrue( "Checking that <h1/> doesn't exist", tags.indexOf( "<h1/>" ) == -1 );
  }

  /**
   * Tests the creation of the list tags
   */
  public void testGetListTags()
  {
    // Better than no test at all  ;-)
    List<String> tags = ConvertText.getListTags();

    assertEquals( "Checking number of list tags", 224, tags.size() );

    assertTrue( "Checking that <p> exists", tags.indexOf( "<p>" ) > -1 );
    assertTrue( "Checking that </p> exists", tags.indexOf( "</p>" ) > -1 );
    assertTrue( "Checking that <p/> exists", tags.indexOf( "<p/>" ) > -1 );

    assertTrue( "Checking that <b> exists", tags.indexOf( "<b>" ) > -1 );
    assertTrue( "Checking that </b> exists", tags.indexOf( "</b>" ) > -1 );
    assertTrue( "Checking that <b/> doesn't exist", tags.indexOf( "<b/>" ) == -1 );

    assertTrue( "Checking that <strong> exists", tags.indexOf( "<b>" ) > -1 );
    assertTrue( "Checking that </strong> exists", tags.indexOf( "</strong>" ) > -1 );
    assertTrue( "Checking that <strong/> doesn't exist", tags.indexOf( "<strong/>" ) == -1 );

    assertTrue( "Checking that <h1> exists", tags.indexOf( "<h1>" ) > -1 );
    assertTrue( "Checking that </h1> exists", tags.indexOf( "</h1>" ) > -1 );
    assertTrue( "Checking that <h1/> doesn't exist", tags.indexOf( "<h1/>" ) == -1 );
  }

  /**
   * Tests splitting address lines.
   */
  public void testSplitLines()
  {
    assertTrue( "Checking empty String", ConvertText.splitLines( null ).isEmpty() );
    assertTrue( "Checking empty String", ConvertText.splitLines( "" ).isEmpty() );

    String[] address = { "Schweizerischer Skiverband", "Worbstrasse 52", "Postfach 478", "3074 Muri bei Bern" };
    List<String> result = ConvertText.splitLines( StringUtils.join( address, "\n" ) );

    for ( int i = 0; i < address.length; i++ )
    {
      assertEquals( "Checking line " + i, address[i], result.get( i ) );
    }
  }

  /**
   * Tests splitting comma-separated lists.
   */
  public void testSplitCommaSeparatedList()
  {
    String[] address = { "Schweizerischer Skiverband", "Worbstrasse 52", "Postfach 478", "3074 Muri bei Bern" };
    List<String> result = ConvertText.splitCommaSeparatedList( StringUtils.join( address, ", " ) );

    for ( int i = 0; i < address.length; i++ )
    {
      assertEquals( "Checking line " + i, address[i], result.get( i ) );
    }
  }

  /**
   * Tests converting newline character to HTML breaks.
   */
  public void testConvertNewlineToBreak()
  {
    String text = "Schweizerischer Skiverband\nWorbstrasse 52\nPostfach 478\n3074 Muri bei Bern";
    String result = ConvertText.convertNewlineToBreak( text );
    assertEquals( "Checking that newline characters are converted", "Schweizerischer Skiverband<br>Worbstrasse 52<br>Postfach 478<br>3074 Muri bei Bern",
        result );
  }

  /**
   * Tests replacing XML special characters for XSL formatting.
   */
  public void testPrepareXmlForXsl()
  {
    String text = "%%$$&lt;Schweizerischer<br> Skiverband<br><br>&gt;%%*$";
    String result = ConvertText.prepareXmlForXsl( text );
    assertEquals( "Checking for complete text", "&lt;<Schweizerischer<br/>Skiverband<p/>>&gt;", result );
  }

  /**
   * Tests replacing double escaped XML (or rather HTML) for XSL formatting.
   *
   */
  public void testPrepareDoubleEscapedXmlForXsl()
  {
    String text = "&lt;value&gt;Linus &amp;amp; Silvia Silvestri AG&amp;lt;br/&amp;gt;Rorschacherstrasse 126&amp;lt;br/&amp;gt;9450 Lüchingen&lt;/value&gt;";
    String expectedResult = "<value>Linus &amp; Silvia Silvestri AG&lt;br/&gt;Rorschacherstrasse 126&lt;br/&gt;9450 Lüchingen</value>";
    String result = ConvertText.prepareDoubleEscapedXmlForXsl( text );
    assertEquals( "Checking for complete text", expectedResult, result );
  }

  /**
   * Tests the replacement of address tags.
   */
  public void testReplaceAddressTags()
  {
    String text = "<b><Schweizerischer Skiverband> Gmbh</b>";
    String result = ConvertText.replaceHtmlTags( text );
    assertEquals( "Checking the replacement of address tags", "<b>%%$$Schweizerischer Skiverband%%*$ Gmbh</b>", result );
  }

  /**
   * Tests the replacement of list tags within XML.
   */
  public void testReplaceHtmlTags()
  {
    String text = null;
    assertNull( "Checking null String", ConvertText.replaceHtmlTags( text ) );

    text = "";
    assertEquals( "Checking empty String", "", ConvertText.replaceHtmlTags( text ) );

    text = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?><hello>world<br>and world<p>and world</p></hello>";
    String expectedResult = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>%%$$hello%%*$world<br>and world<p>and world</p>%%$$/hello%%*$";
    assertEquals( "Testing XML with ISO 8859-1", expectedResult, ConvertText.replaceHtmlTags( text ) );

    text = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><hello>world<br/>and world<p/>and <b>world</b></hello>\n<goodbye>cruel world</goodbye>";
    expectedResult = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>%%$$hello%%*$world<br/>and world<p/>and <b>world</b>%%$$/hello%%*$\n%%$$goodbye%%*$cruel world%%$$/goodbye%%*$";
    assertEquals( "Testing XML with UTF-8", expectedResult, ConvertText.replaceHtmlTags( text ) );
  }

  /**
   * Tests escaping the HTML tags within XML.
   */
  public void testEscapeHtmlTags()
  {
    String text = null;
    assertNull( "Checking null String", ConvertText.escapeHtmlTags( text ) );

    text = "";
    assertEquals( "Checking empty String", "", ConvertText.escapeHtmlTags( text ) );

    text = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?><hello>world<br>and world<p>and world</p></hello>";
    String expectedResult = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?><hello>world&lt;br&gt;and world&lt;p&gt;and world&lt;/p&gt;</hello>";
    assertEquals( "Testing XML with ISO 8859-1", expectedResult, ConvertText.escapeHtmlTags( text ) );

    text = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><hello>world<br/>and world<p/>and <b>world</b></hello>\n<goodbye>cruel world</goodbye>";
    expectedResult = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><hello>world&lt;br/&gt;and world&lt;p/&gt;and &lt;b&gt;world&lt;/b&gt;</hello>\n<goodbye>cruel world</goodbye>";
    assertEquals( "Testing XML with UTF-8", expectedResult, ConvertText.escapeHtmlTags( text ) );
  }

  /**
   * Tests the replacement of HTML tags within XML.
   */
  public void testReplaceListTags()
  {
    String text = null;
    assertNull( "Checking null String", ConvertText.replaceListTags( text ) );

    text = "";
    assertEquals( "Checking empty String", "", ConvertText.replaceListTags( text ) );

    text = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?><hello>world<br>and world<p>and world</p></hello><artprice_date>12.12.1200</artprice_date>";
    String expectedResult = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>%%$$hello%%*$world<br>and world<p>and world</p>%%$$/hello%%*$<artprice_date>12.12.1200</artprice_date>";
    assertEquals( "Testing XML with ISO 8859-1", expectedResult, ConvertText.replaceListTags( text ) );

    text = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><hello>world<br/>and world<p/>and <b>world</b></hello>\n<goodbye>cruel world</goodbye><artprice_date>12.12.1200</artprice_date>";
    expectedResult = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>%%$$hello%%*$world<br/>and world<p/>and <b>world</b>%%$$/hello%%*$\n%%$$goodbye%%*$cruel world%%$$/goodbye%%*$<artprice_date>12.12.1200</artprice_date>";
    assertEquals( "Testing XML with UTF-8", expectedResult, ConvertText.replaceListTags( text ) );
  }
}
