package ch.ige.jipps.vo;

import java.lang.reflect.Field;
import java.sql.Timestamp;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.dbutils.vo.IdentifiableVO;

/**
 * Represents one trademark renewal within a batch run.
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: AmvVO.java,v 1.15 2007/05/02 12:12:52 locher Exp $
 */
public class AmvVO extends AbstractVO implements IdentifiableVO<Integer>, Comparable<AmvVO>, Cloneable
{
  // @see ch.ige.dbutils.vo.IdentifiableVO#getIdentifier()
  public Integer getIdentifier()
  {
    // This is not the primary key, though...
    return this.amvLaufId;
  }

  // @see ch.ige.dbutils.vo.IdentifiableVO#setIdentifier(java.io.Serializable)
  public void setIdentifier( Integer amvLaufId )
  {
    // This is not the primary key, though...
    this.amvLaufId = amvLaufId;
  }

  /**
   * The unique ID of the trademark renewal batch run
   */
  public int amvLaufId = 0;

  /**
   * The unique ID of the address to which the renewal is sent
   */
  public int adrNr = 0;

  /**
   * The unique key of the trademark which was renewed
   */
  public int schutitKey = 0;

  /**
   * The unique ID of the invoice
   */
  public int bfaktId = 0;

  /**
   * The number of the IP
   */
  // TODO The IP number must be a number!
  public String schutitNr = "";

  /**
   * The status of the processing, see {@link ch.ige.jipps.vo.AmvLaufVO#verarbStat} for possible values.
   */
  public int verarbStat = AmvLaufVO.PROCESSING_STARTED;

  /**
   * The PDF file, which contains the letter confirming the trademark renewal
   */
  public byte[] printJobFile;

  /**
   * The creation date. Note that this is set automatically when this is saved in the database, so don't set it and try
   * to save it.
   */
  public Timestamp createdDat = null;

  /**
   * The date of the latest update. Note that this is set automatically when this is saved in the database, so don't set
   * it and try to save it.
   */
  public Timestamp changedDat = null;

  // @see java.lang.Object#clone()
  public Object clone()
  {
    try
    {
      return super.clone();
    }
    catch ( CloneNotSupportedException ex )
    {
      return null;
    }
  }

  @Override
  public boolean equals( Object obj )
  {
    // TODO according to findbugs(tm) this should be overwritten to comply with the compareTo method ie. not just
    // calling super here!
    return super.equals( obj );
  }

  // @see java.lang.Comparable#compareTo(java.lang.Object)
  public int compareTo( AmvVO amvVO )
  {
    if ( this.adrNr < amvVO.adrNr )
    {
      return -1;
    }
    else if ( this.adrNr == amvVO.adrNr )
    {
      return this.schutitNr.compareToIgnoreCase( amvVO.schutitNr );
    }
    else
    {
      return 1;
    }
  }

  // @see ch.ige.dbutils.vo.AbstractVO#toString()
  public String toString()
  {
    // Don't fill the log with binary data!
    return ( new ReflectionToStringBuilder( this )
    {
      protected boolean accept( Field field )
      {
        return super.accept( field ) && !field.getName().equals( "printJobFile" );
      }
    } ).toString();
  }
}
