package ch.ige.jipps.vo;

import java.sql.Date;
import java.util.Locale;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.language.Localizable;

/**
 * Represents the data from ReferenceXML.
 * <p/>
 * The data is used temporary during the migration from Swissreg to Swissreg II,
 * in order to compare the old excerpts (pdfs) with the new ones.
 * <p/>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * @author Esther Bandschapp
 * @version $Id: ReferenceXMLVO.java,v 1.19 2008/07/23 08:23:07 bandschapp Exp $
 */
public class ReferenceXMLVO extends AbstractVO implements Localizable
{
  /**
   * The unique ID of the reference XML.
   */
  private Integer refXmlId;

  /**
   * The unique key of the IP.
   */
  private Integer ipKey;

  /**
   * The type of IP.
   */
  private IPType ipType;

  /**
   * The language in which the reference XML was created.
   */
  private BAGISLanguageDomain ipLanguage;

  /**
   * The report type used to create the reference XML.
   */
  private ReportType reportType;

  /**
   * The report date for which the reference XML was created.
   */
  private Date reportDate;

  /**
   * The reference XML document.
   */
  private String xmlData;

  /**
   * The reference XSL-FO data, which is the step between {@link #xmlData} and {@link #pdfFile}.
   */
  private String xslFoData;

  /**
   * The reference PDF document.
   */
  private byte[] pdfFile;

  /**
   * Gets the unique ID of the reference XML.
   * @return the unique ID of the reference XML
   */
  public Integer getRefXmlId()
  {
    return this.refXmlId;
  }

  /**
   * Sets the unique ID of the reference XML.
   * @param refXmlId The unique ID of the reference XML
   */
  public void setRefXmlId( Integer refXmlId )
  {
    this.refXmlId = refXmlId;
  }

  /**
   * Gets the unique key of the IP.
   * @return the unique key of the IP
   */
  public Integer getIpKey()
  {
    return this.ipKey;
  }

  /**
   * Sets the unique key of the IP.
   * @param ipKey The unique key of the IP to set
   */
  public void setIpKey( Integer ipKey )
  {
    this.ipKey = ipKey;
  }

  /**
   * Gets the type of IP.
   * @return the type of IP
   */
  public IPType getIpType()
  {
    return this.ipType;
  }

  /**
   * Sets the type of IP.
   * @param ipType The type of IP to set
   */
  public void setIpType( IPType ipType )
  {
    this.ipType = ipType;
  }

  /**
   * Gets the language in which the reference XML was created.
   * @return the language in which the reference XML was created
   */
  public Locale getLocale()
  {
    return this.ipLanguage.toLocale();
  }

  /**
   * Sets the language in which the reference XML was created.
   * @param ipLanguage The language in which the reference XML was created
   */
  public void setLocale( Locale ipLanguage )
  {
    this.ipLanguage = BAGISLanguageDomain.parse( ipLanguage );
  }

  /**
   * Gets the report type used to create the reference XML.
   * @return the report type used to create the reference XML
   */
  public ReportType getReportType()
  {
    return this.reportType;
  }

  /**
   * Sets the report type used to create the reference XML.
   * @param reportType The report type used to create the reference XML
   */
  public void setReportType( ReportType reportType )
  {
    this.reportType = reportType;
  }

  /**
   * Gets the report date for which the reference XML was created.
   * @return the report date for which the reference XML was created
   */
  public Date getReportDate()
  {
    return this.reportDate;
  }

  /**
   * Sets the report date for which the reference XML was created.
   * @param reportDate The report date for which the reference XML was created
   */
  public void setReportDate( Date reportDate )
  {
    this.reportDate = reportDate;
  }

  /**
   * Gets the reference XML document.
   * @return the reference XML document
   */
  public String getXmlData()
  {
    return this.xmlData;
  }

  /**
   * Sets the reference XML document.
   * @param xmlData The reference XML document
   */
  public void setXmlData( String xmlData )
  {
    this.xmlData = xmlData;
  }

  /**
   * Gets the reference XSL-FO data, which is the step between {@link #xmlData} and {@link #pdfFile}.
   * @return the reference XSL-FO data, which is the step between {@link #xmlData} and {@link #pdfFile}
   */
  public String getXslFoData()
  {
    return this.xslFoData;
  }

  /**
   * Sets the reference XSL-FO data, which is the step between {@link #xmlData} and {@link #pdfFile}.
   * @param xslFoData The reference XSL-FO data, which is the step between {@link #xmlData} and {@link #pdfFile}
   */
  public void setXslFoData( String xslFoData )
  {
    this.xslFoData = xslFoData;
  }

  /**
   * Gets the reference PDF document.
   * @return the reference PDF document
   */
  public byte[] getPdfFile()
  {
    return this.pdfFile;
  }

  /**
   * Sets the reference PDF document.
   * @param pdfFile The reference PDF document to set
   */
  public void setPdfFile( byte[] pdfFile )
  {
    this.pdfFile = pdfFile;
  }

  /**
   * Checks whether this is the same as <code>obj</code>.
   * Note that <strong>only a subset of this</strong> is checked,
   * not including {@link #refXmlId}, {@link #xmlData} and {@link #pdfFile}.
   * @param obj The object to compare with
   * @return <code>true</code> if the subset of this matches with <code>obj</code>,
   *         otherwise <code>false</code>
   */
  public boolean equals( Object obj )
  {
    if ( obj == null )
    {
      return false;
    }
    if ( obj == this )
    {
      return true;
    }
    if ( !( obj instanceof ReferenceXMLVO ) )
    {
      return false;
    }
    ReferenceXMLVO other = (ReferenceXMLVO) obj;
    return ( this.ipKey == null ? other.ipKey == null : this.ipKey.equals( other.ipKey ) )
        && ( this.ipType == null ? other.ipType == null : this.ipType.equals( other.ipType ) )
        && ( this.ipLanguage == null ? other.ipLanguage == null : this.ipLanguage.equals( other.ipLanguage ) )
        && ( this.reportType == null ? other.reportType == null : this.reportType.equals( other.reportType ) )
        && ( this.reportDate == null ? other.reportDate == null : this.reportDate.equals( other.reportDate ) );
  }

  // See superclass
  public int hashCode()
  {
    int result = 17;
    result = 37 * result + this.ipKey.hashCode();
    result = 37 * result + this.ipType.hashCode();
    result = 37 * result + this.reportType.hashCode();
    result = 37 * result + this.reportDate.hashCode();
    return result;
  }
}
