package ch.ige.pctedi.client;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.util.validation.Check;

/**
 * Handling Validity Properties - Programmeigenschaften anhand Property Datei setzen
 * Pfad der Property Datei wird in der Batch-Prozedur gesetzt
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A.Rueegsegger
 * @version $Id: PctProperties.java,v 1.10 2008/07/01 07:08:10 kuenzli Exp $
 */
public class PctProperties
{
  private static final Logger LOG = Logger.getLogger( PctProperties.class );

  public static final String PROPERTIES = "pct.properties";

  public static final String LOGFILE_PATH = "logfile.pathname";

  public static final String CONFIG_PATH = "config.path";

  // Mail-Configuration
  public static final String MAIL_SMTP_HOST = "mail.smtp.host";

  public static final String MAIL_SEND_TO = "mail.send.to";

  public static final String MAIL_SEND_CC = "mail.send.cc";

  public static final String MAIL_SEND_FROM = "mail.send.from";

  public static final String MAIL_SEND_LISTEN_TO = "mail.send.listen.to";

  public static final String MAIL_SEND_LISTEN_CC = "mail.send.listen.cc";

  // Download
  public static final String DOWNLOAD_URL = "download.url";

  // Zipfile
  public static final String ZIPFILE_NAME = "zipfile.name";

  public static final String ZIPFILE_TARGET = "zipfile.target";

  public static final String UNZIP_DIRECTORY = "unzip.directory";

  public static final String GAZETTE_NAME = "gazette.name";

  public static final String WPA_NAME = "wpa.name";

  public static String[] keys = { PROPERTIES, LOGFILE_PATH, CONFIG_PATH, MAIL_SMTP_HOST, MAIL_SEND_TO, MAIL_SEND_CC, MAIL_SEND_FROM, MAIL_SEND_LISTEN_TO,
      MAIL_SEND_LISTEN_CC, DOWNLOAD_URL, ZIPFILE_NAME, ZIPFILE_TARGET, UNZIP_DIRECTORY, GAZETTE_NAME, WPA_NAME };

  private static PctProperties INSTANCE = null;

  private Properties pctProps = null;

  private PctProperties()
  {
  }

  /**
   * Einlesen der Programmeigenschaften aus der Property Datei<br>
   * Setzen der Programmeigenschaften als System-Properties
   * @throws Exception
   */
  private void loadProperties()
  {
    try
    {
      this.pctProps = readProperties();
    }
    catch ( Exception e )
    {
      throw new RuntimeException("Could not read properties",e);
    }
    /*
        for ( int i = 0; i < keys.length; i++ )
        {
          String value = props.getProperty( keys[i] );
          if ( value != null && !value.equalsIgnoreCase( "null" ) )
          {
            System.setProperty( keys[i], value );
          }
        }
    */
  }

  /**
   * Gets a property, either from the file configured in
   * {@link #CONFIG} or from a System property, which overrides the file property.
   * @param key The key to look for
   * @return The value for the property
   */
  public String getProperty( String key )
  {
    Check.notEmpty( key, "property key" );

    String valueToUse = null;
    String systemValue = System.getProperty( key );
    if ( StringUtils.isNotEmpty( systemValue ) && !systemValue.equalsIgnoreCase( "null" ) )
    {
      valueToUse = systemValue;
    }
    else
    {
      String fileValue = this.pctProps.getProperty( key );
      if ( StringUtils.isNotEmpty( fileValue ) && !fileValue.equalsIgnoreCase( "null" ) )
      {
        valueToUse = fileValue;
      }
      else
      {
        throw new IllegalArgumentException( "Unknown property key " + key );
      }
    }
    return valueToUse;
  }

  /**
   * Einlesen der Programmeigenschaften aus der Property Datei<br>
   * Setzen der Programmeigenschaften als System-Properties
   * 
   */
  private static Properties readProperties() throws MalformedURLException, IOException
  {
    // Property ipcloadvalidity.config wird in Batch-Prozedur gesetzt
    String fileUrl = System.getProperty( PROPERTIES );
    LOG.debug( "load properties from: " + fileUrl );

    URL propsFileURL = new URL( fileUrl );
    Properties props = new Properties();
    props.load( propsFileURL.openStream() );
    return props;
  }

  @Override
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    String key = null;
    String val = null;

    for ( int i = 0; i < keys.length; i++ )
    {
      key = keys[i];
      val = pctProps.getProperty( key );
      buf.append( "  " + key + " = " + val + "\n" );
    }

    return buf.toString();
  }

  /**
   * Get instance of this and initializes properties map from file.
   * @return this
   * @throws Exception 
   * @throws Exception
   */
  public static PctProperties getInstance()
  {
    if ( INSTANCE == null )
    {
      INSTANCE = new PctProperties();
      INSTANCE.loadProperties();
    }
    return INSTANCE;
  }
}
