package ch.ige.pctedi.client.batch;

import java.io.File;
import java.net.URL;
import java.util.Properties;

import org.apache.log4j.Logger;

import ch.ige.pctedi.client.PctProperties;
import ch.ige.util.mail.MailSender;

/**
 * Aufbereitung einer Nachricht zum Versenden via smtp
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */

public class MailMessage
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger( MailMessage.class );

  //
  // fields
  //
  private String smptServer = null;

  private String to = null;

  private String cc = null;

  private String toListen = null;

  private String ccListen = null;

  private String from = null;

  private String subject = null;

  private String body = null;

  private String attachment = null;

  public MailMessage( String subject ) throws Exception
  {
    this( subject, "", null );
  }

  public MailMessage( String subject, String body, String attachment )
  {
    this.subject = subject;
    this.body = body;
    this.attachment = attachment;

    // Programmeigenschaften einlesen
    loadProperties();
  }

  /**
   * Einlesen der Programmeigenschaften aus der Property Datei<br>
   * Setzen der Programmeigenschaften als System-Properties
   * Pfad der Property Datei wird in der Batch-Prozedur gesetzt
   */
  public void loadProperties()
  {
    try
    {
      // Property ipcloadvalidity.config wird in Batch-Prozedur gesetzt
      String fileUrl = PctProperties.getInstance().getProperty( PctProperties.PROPERTIES );
      URL propsFileURL = new URL( fileUrl );
      Properties props = new Properties();
      props.load( propsFileURL.openStream() );

      smptServer = props.getProperty( PctProperties.MAIL_SMTP_HOST );
      to = props.getProperty( PctProperties.MAIL_SEND_TO );
      cc = props.getProperty( PctProperties.MAIL_SEND_CC );
      toListen = props.getProperty( PctProperties.MAIL_SEND_LISTEN_TO );
      ccListen = props.getProperty( PctProperties.MAIL_SEND_LISTEN_CC );
      from = props.getProperty( PctProperties.MAIL_SEND_FROM );
    }
    catch ( Exception ex )
    {
      LOG.error( "MailMessage.loadProperties() - Ein unerwarteter Fehler ist aufgetreten. \n" + getSystemProperties() + "Exception : " + ex.getMessage() );
    }
  }

  /**
   * MailMessage senden
   */
  public void send() throws Exception
  {
    MailSender.sendMail( smptServer, from, to, cc, subject, body, new File(attachment) );
  }

  /**
   * MailMessage senden
   */
  public void sendLogfile() throws Exception
  {
    MailSender.sendMail( smptServer, from, toListen, ccListen, subject, body, new File(attachment) );
  }

  /**
   * Gibt die System Properties zurück
   * @return String System Properties
   */
  private String getSystemProperties()
  {
    StringBuffer buf = new StringBuffer();
    String key = null;
    String val = null;

    String[] SYS_PROPERTIES = { PctProperties.PROPERTIES, PctProperties.MAIL_SMTP_HOST, PctProperties.MAIL_SEND_FROM, PctProperties.MAIL_SEND_TO,
        PctProperties.MAIL_SEND_CC };

    for ( int i = 0; i < SYS_PROPERTIES.length; i++ )
    {
      key = SYS_PROPERTIES[i];
      val = PctProperties.getInstance().getProperty( key );
      buf.append( "  " + key + " = " + val + "\n" );
    }

    return buf.toString();
  }

  @Override
  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( "\n" );
    buf.append( "  subject      = <" + this.subject + ">\n" );
    buf.append( "  body         = <" + this.body + ">\n" );
    buf.append( "  attachment   = <" + this.attachment + ">\n" );

    buf.append( getSystemProperties() );

    return buf.toString();
  }
}
