package ch.ige.pctedi.client.batch;

import org.apache.log4j.Logger;

import ch.ige.pctedi.BuildInfo;
import ch.ige.pctedi.client.PctZipFile;

/**
 * 
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   ruegsegger
 * @version  $Id: Process.java,v 1.4 2008/03/11 14:55:12 ruegsegger Exp $
 */
public abstract class Process
{
  private static final Logger LOG = Logger.getLogger( "ch.ige.pctedi.client.wpa" );

  protected StringBuffer logMsg = new StringBuffer();

  protected boolean error = false;

  public PctZipFile zipFile = null;

  /**
   * Gibt an ob sich ein Fehler ereignet hat
   * @return boolean true fehler aufgetreten
   */
  public boolean isErrorOccured()
  {
    return this.error;
  }

  /**
   * Gibt die geloggten Debugmeldungen zurück
   * @return String
   */
  public String getLogMessage()
  {
    return this.logMsg.toString();
  }

  /**
   * Fügt übergebenen String der Fehlermeldung hinzu
   * @param message anzufügende Fehlermeldung
   */
  protected void logAndAppendMsg( String message )
  {
    LOG.error( message );
    logMsg.append( "\n exeption occured processing file: " + zipFile.getPath() + " " );
    logMsg.append( message );
  }

  public void start( PctZipFile pctZipFile ) throws Exception
  {
    this.zipFile = pctZipFile;
    process( pctZipFile );
    sendLogfile();
  }

  protected abstract void process( PctZipFile pctZipFile ) throws Exception;

  private void sendLogfile()
  {
    String attachement = getAttachement();

    if ( attachement != null )
    {
      MailMessage msg = new MailMessage( getSubject(), getBody(), attachement );
      try
      {
        LOG.debug( "try to send logfiles" );
        msg.sendLogfile();
      }
      catch ( Exception ex )
      {
        LOG.error( ex );
        LOG.info( "Message to send: " + msg.toString() );

        this.error = true;
        this.logAndAppendMsg( ex.toString() );
      }
    }
  }

  private String getBody()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( "CVS Tag = " + BuildInfo.CVS_TAG + "\n" );
    buf.append( "Build Id = " + BuildInfo.BUILD_ID + "\n" );

    return buf.toString();
  }

  protected String getSubject()
  {
    return null;
  }

  protected String getAttachement()
  {
    return null;
  }

}
