package ch.ige.pctedi.client.wpa;

import java.io.File;
import java.io.FileInputStream;
import java.util.List;

import org.apache.log4j.Logger;

import ch.ige.pctedi.client.XMLDoc;
import ch.ige.pctedi.server.PctDelegate;
import ch.ige.pctedi.vo.PatVO;
import ch.ige.pctedi.vo.PctVO;
import ch.ige.pctedi.vo.StatistikVO;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: WpaProcess.java,v 1.7 2008/03/11 14:11:01 ruegsegger Exp $
 */
public class WpaProcess
{
  private static final Logger LOG = Logger.getLogger( WpaProcess.class );

  private static List<String> listBland = null;

  private WpaLogFile logFile = WpaLogFile.getInstance();

  private StatistikVO statistikVO = new StatistikVO();

  private boolean error = false;

  private StringBuffer logMsg = new StringBuffer();

  private Schutit schutit = null;

  private Pat pat = null;

  private Ereignis ereignis = null;

  private PctAdr pctAdr = null;

  private PatIpc patIpc = null;

  private Prioritaet prioritaet = null;

  public WpaProcess() throws Exception
  {
    if ( listBland == null )
    {
      listBland = PctDelegate.getInstance().getAllBland();
    }

    schutit = new Schutit();
    pat = new Pat();
    ereignis = new Ereignis();
    pctAdr = new PctAdr( listBland );
    patIpc = new PatIpc();
    prioritaet = new Prioritaet( listBland );
  }

  public void process( File file )
  {
    try
    {
      PctVO pctVO = parseFile( file );
      savePctVO( pctVO );
    }
    catch ( Exception ex )
    {
      this.error = true;
      logAndAppendMsg( "process file : " + file.getPath() + " failed with Exception: " + ex );
    }
  }

  public StatistikVO getStatistik()
  {
    return statistikVO;
  }

  /**
   * Gibt an ob sich ein Fehler ereignet hat
   * @return boolean true fehler aufgetreten
   */
  public boolean isErrorOccured()
  {
    return this.error;
  }

  /**
   * Gibt die geloggten Debugmeldungen zurück
   * @return String
   */
  public String getLogMessage()
  {
    return this.logMsg.toString();
  }

  /**
   * Fügt übergebenen String der Fehlermeldung hinzu
   * @param message anzufügende Fehlermeldung
   */
  private void logAndAppendMsg( String message )
  {
    LOG.error( message );
    logMsg.append( "\n" );
    logMsg.append( message );
  }

  private void savePctVO( PctVO pctVO ) throws Exception
  {
    // Aufbereitetes pctVO (Value-Object) an Sessionbean übergeben
    // und in die DB schreiben, falls das "Pat-verfahren != 0 ist
    if ( pctVO.patVO.patVerfahr != 0 )
    {
      // Test ob Schutztitel bereits vorhanden ist.

      // existiert und A1 oder A2         ==> Fehler
      // existiert und A3 oder grösser    ==> OK  nix übernehmern
      //                                      Mutation eines PCT
      // exist nicht und A1 oder A2       ==> OK übernehmen
      // exist nicht und A3 oder grösser  ==> übernehmen und Log
      //                                      sollte eigentlich vorhanden sein

      int statSchutitFound = PctDelegate.getInstance().findPct( pctVO );
      // Mögliche Werte in statSchutitFound:
      //    0: kein Schutztitel vorhanden 
      //    1: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben das selbe Resultat. 
      //    2: Schutztitel vorhanden; nur Suche über schutitNr ergab ein Resultat. 
      //    3: Schutztitel vorhanden; nur Suche über GesuchNr ergab ein Resultat. 
      //    4: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben unterschiedliche Resultate.

      // Sofern ein Schutztitel gefunden wird.. 
      if ( statSchutitFound > 0 )
      {
        if ( pctVO.kind.equalsIgnoreCase( "A1" ) || pctVO.kind.equalsIgnoreCase( "A2" ) )
        {
          logFile.clearMessage();
          if ( statSchutitFound == 3 )
          {
            logFile.appendMessage( WpaLogFile.INFO, "PCT " + pctVO.schutitVO.schutitNr + "/" + pctVO.schutitVO.gesuchNr
                + " bereits vorhanden. Aber nur unter der Anmeldenummer gefunden" );
          }
          // Titel mittels SchutitNr gefunden
          else if ( statSchutitFound == 2 )
          {
            logFile.appendMessage( WpaLogFile.INFO, "PCT " + pctVO.schutitVO.schutitNr + "/" + pctVO.schutitVO.gesuchNr
                + " bereits vorhanden. Aber nur unter der Veröffentlichungsnummer gefunden" );
          }
          else if ( statSchutitFound == 4 )
          // Je ein Schutztitel mittels SchutitNr und einer mittels GesuchNr gefunden. 
          {
            logFile.appendMessage( WpaLogFile.INFO, "PCT " + pctVO.schutitVO.schutitNr + "/" + pctVO.schutitVO.gesuchNr
                + " bereits vorhanden. Es wurden bei der Suche nach Anmelde- und Veröffentlichungsnummer unterschiedliche Schutztitel gefunden" );

          }
          else
          {
            // Status ist 1. Hier muss keine Meldung mehr ausgegeben werden.
          }
          logFile.writeMessage();
        } // else ist OK da Mutation und daher nichts übernehmen.
      }
      else
      {
        if ( !pctVO.kind.equalsIgnoreCase( "A1" ) && !pctVO.kind.equalsIgnoreCase( "A2" ) )
        {
          logFile.appendMessage( WpaLogFile.INFO, "PCT " + pctVO.schutitVO.schutitNr + "/" + pctVO.schutitVO.gesuchNr + " sollte eigentlich vorhanden"
              + " sein. Wird korrigiert." );
        }
        //Statistik nachführen
        switch( pctVO.patVO.patVerfahr )
        {
          case PatVO.EURO_PCT:
            statistikVO.euroCnt++;
            break;
          case PatVO.CH_PCT:
            statistikVO.chCnt++;
            break;
          case PatVO.CH_EURO_PCT:
            statistikVO.chEuroCnt++;
            break;
        }

        // PCT schreiben
        PctDelegate.getInstance().createPct( pctVO );
        logFile.writeMessage();
      }
    }
    else
    {
      statistikVO.notChCnt++;
      logFile.clearMessage();
    }
  }

  /**
   * // Es wird je File nur ein PCT eingelesen und in ein ValueObject abfüllen.
   *
   * @param   fileName      Filename des zu bearbeitenden PCT
   */
  private PctVO parseFile( File file ) throws Exception
  {
    PctVO pctVO = new PctVO();
    XMLDoc xml;

    FileInputStream fi = new FileInputStream( file );
    xml = new XMLDoc( fi );

    pctVO.kind = xml.getValueOf( "publication-reference", "document-id/kind" );

    try
    {
      pctVO.schutitVO = schutit.extract( xml );
      pctVO.patVO = pat.extract( xml );
      pctVO.listEreignis = ereignis.extract( xml );
      pctVO.listPctAdr = pctAdr.extract( xml );
      pctVO.listPatIpc = patIpc.extract( xml );
      pctVO.listPrio = prioritaet.extract( xml );
    }
    catch ( Exception ex )
    {
      logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " aufgrund eines aufgetretenen Fehlers nicht verarbeitet" );
      logFile.writeMessage();
      throw ex;
    }

    return pctVO;
  }
}
