package ch.ige.pctedi.client.wpa.test;

import java.io.FileInputStream;
import java.util.List;

import ch.ige.pctedi.client.XMLDoc;
import ch.ige.pctedi.client.test.ClientTestCase;
import ch.ige.pctedi.client.wpa.PatIpc;
import ch.ige.pctedi.client.wpa.WpaLogFile;
import ch.ige.pctedi.vo.PatIpcVO;

/**
 * junit test of class PatIpc
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   ruegsegger
 * @version  $Id: TestPatIpc.java,v 1.12 2008/03/11 14:55:13 ruegsegger Exp $
 */
public class TestPatIpc extends ClientTestCase
{
  private XMLDoc xml;

  private String fileIpcr = "C:\\jp\\projects\\pctedi\\testfiles\\wpa_ipcr_classification.xml";

  private String fileIpc = "C:\\jp\\projects\\pctedi\\testfiles\\wpa_GB0403434_17022005.xml";

  public TestPatIpc( String s )
  {
    super( s );
  }

  private void loadIpcr()
  {
    try
    {
      FileInputStream fi = new FileInputStream( fileIpcr );
      xml = new XMLDoc( fi );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  private void loadIpc()
  {
    try
    {
      FileInputStream fi = new FileInputStream( fileIpc );
      xml = new XMLDoc( fi );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  public void test00()
  {
    PatIpc patIpc = new PatIpc();

    try
    {
      loadIpcr();
      List<PatIpcVO> al = patIpc.extract( xml );
      WpaLogFile.getInstance().writeMessage();

      // check default values
      PatIpcVO vo = al.get( 0 );
      assertEquals( "A01B", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "A", vo.classLevel );
      assertEquals( "I", vo.classValue );
      assertEquals( "B", vo.geschichte );
      assertEquals( "H", vo.source );
      assertEquals( "CH", vo.generatingOffice );
      assertEquals( sdf.parse( "20040809" ), vo.erfassDat );
      assertEquals( 199, vo.erfassMa );

      vo = al.get( 1 );
      assertEquals( "A01N002500", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 2, vo.patIpcLnr );
      assertEquals( "I", vo.classValue );
      assertEquals( sdf.parse( "20060101" ), vo.erfassDat );

      // check ungültig werte
      vo = al.get( 2 );
      assertEquals( "A01C", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 3, vo.patIpcLnr );
      assertEquals( "I", vo.classValue );
      assertEquals( sdf.parse( "20040809" ), vo.erfassDat );

      //check H, non-inventive
      vo = al.get( 3 );
      assertEquals( "C12R0001125", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 4, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      //check N, i.O
      vo = al.get( 4 );
      assertEquals( "A01B000102", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 3, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "I", vo.classValue );

      //check N, indexed
      vo = al.get( 5 );
      assertEquals( "C12R0001125", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 3, vo.patIpcCd );
      assertEquals( 2, vo.patIpcLnr );
      assertEquals( "I", vo.classValue );

      //check N, non-inventive, indexed
      vo = al.get( 6 );
      assertEquals( "A01D001100", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 4, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      //check N, non-inventive, indexed
      vo = al.get( 7 );
      assertEquals( "A01C", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 4, vo.patIpcCd );
      assertEquals( 2, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      //check N, non-inventive, not-indexed
      vo = al.get( 8 );
      assertEquals( "C12R0001125", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 5, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      //check N, non-inventive, not-indexed
      vo = al.get( 9 );
      assertEquals( "A01D010100", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 5, vo.patIpcCd );
      assertEquals( 2, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      assertTrue( al.size() == 10 );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  public void testIpc()
  {
    PatIpc patIpc = new PatIpc();

    try
    {
      loadIpc();
      List<PatIpcVO> al = patIpc.extract( xml );
      WpaLogFile.getInstance().writeMessage();

      // check default values
      PatIpcVO vo = al.get( 0 );
      assertEquals( "F03B001308", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "A", vo.classLevel );
      assertEquals( "I", vo.classValue );
      assertEquals( "B", vo.geschichte );
      assertEquals( "H", vo.source );
      assertEquals( "CH", vo.generatingOffice );
      assertEquals( sdf.parse( "20040809" ), vo.erfassDat );
      assertEquals( 199, vo.erfassMa );

      vo = al.get( 1 );
      assertEquals( "F03B000312", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 3, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );

      vo = al.get( 2 );
      assertEquals( "F03B000314", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 3, vo.patIpcCd );
      assertEquals( 2, vo.patIpcLnr );

      assertTrue( al.size() == 3 );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }
}
