package ch.ige.pctedi.client.wpa.test;

import java.io.FileInputStream;
import java.util.Date;
import java.util.List;

import ch.ige.pctedi.client.XMLDoc;
import ch.ige.pctedi.client.test.ClientTestCase;
import ch.ige.pctedi.client.wpa.Prioritaet;
import ch.ige.pctedi.client.wpa.WpaLogFile;
import ch.ige.pctedi.vo.PrioritaetVO;

/**
 * junit test of class Prioritaet
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   ruegsegger
 * @version  $Id: TestPrioritaet.java,v 1.14 2008/03/11 14:55:12 ruegsegger Exp $
 */
public class TestPrioritaet extends ClientTestCase
{
  private XMLDoc xml;

  private Prioritaet prio;

  private String filename = "C:\\jp\\projects\\pctedi\\testfiles\\wpa_priority_data.xml";

  public TestPrioritaet( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    try
    {
      FileInputStream fi = new FileInputStream( filename );
      xml = new XMLDoc( fi );
      prio = new Prioritaet();
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  @Override
  protected void tearDown()
  {
    WpaLogFile.getInstance().writeMessage();
  }

  public void test00()
  {
    try
    {
      List<PrioritaetVO> listPrioVO = prio.extract( xml );

      assertTrue( listPrioVO.size() == 5 );

      if ( listPrioVO.size() > 0 )
      {
        PrioritaetVO vo = listPrioVO.get( 0 );
        assertEquals( 1, vo.prioLnr );
        assertEquals( "DE", vo.blandCd );
        assertEquals( "10 2004 017 334.6", vo.prioNr );
        assertEquals( sdf.parse( "20040407" ), vo.prioDat );

        // nicht durch das Programm abgefüllte Attribute (Initialwerte)
        assertEquals( 0, vo.schutitKey );
        assertNull( vo.prioVonEreig );
        assertNull( vo.prioBisEreig );
        assertNull( vo.prioBem );
        assertEquals( 0, vo.regStat );
        assertNull( vo.sprache );
        assertEquals( 0, vo.version );

        // ungueltiges Priodat (2004011023)
        // prioDat darf nicht null sein, fillingdate wird eingefügt
        // ungueltiges Prioland (ZZ)
        // blandCd darf nicht nulls sein, CH wird eingefügt
        vo = listPrioVO.get( 1 );
        assertEquals( 2, vo.prioLnr );
        assertEquals( "CH", vo.blandCd );
        assertEquals( "10 2004 056 405.1", vo.prioNr );
        assertEquals( sdf.parse( "20050405" ), vo.prioDat );

        vo = listPrioVO.get( 2 );
        assertEquals( 3, vo.prioLnr );
        assertEquals( "CH", vo.blandCd );
        assertEquals( "", vo.prioNr );
        assertEquals( sdf.parse( "20051017" ), vo.prioDat );
      }
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  public void test01getMinPrioDat()
  {
    try
    {
      Date minPrioDat = prio.getMinPrioDat( xml );
      assertEquals( sdf.parse( "20030208" ), minPrioDat );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

}
