package ch.ige.pctedi.server;

import java.sql.*;
import javax.ejb.*;
import ch.ige.util.server.DBHelper;



/**
 * Vergibt den nächsten Primärschlüssel
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 *
 * @author  Trachsel, Iten
 * @version $Id: PctXxIdentNumberBean.java,v 1.9 2008/07/01 07:43:35 kuenzli Exp $
 */

/**
 * <!-- begin-xdoclet-definition -->
 * 
 * @ejb.bean name="PctXxIdentNumber" description="An EJB named PctXxIdentNumber" 
 * display-name="PctXxIdentNumber" 
 * view-type="local"
 * jndi-name="PctXxIdentNumber" 
 * type="Stateless" 
 * transaction-type="Container" 
 * @ejb.transaction type="Required"
 *  
 * <!-- end-xdoclet-definition -->
 */
public class PctXxIdentNumberBean implements SessionBean
{
  /**
   * Default id.
   */
  private static final long serialVersionUID = 1L;
  private final int MAX_RETRY = 50;
  SessionContext sessionContext;

  public void ejbCreate() throws CreateException
  {
  }

  public void ejbRemove()
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void setSessionContext( SessionContext sessionContext )
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Vergibt den nächsten Primärschlüssel
   * Diese Funktion muss in einer eigenen Transaktion ablaufen,
   * damit Tabelle nur möglichst kurze Zeit gelockt wird
   * @param name Name des Primärschlüssels. Konstanten aus XxIdentNumber.
   * @return nächster Primärschlüssel
   * 
   * @ejb.interface-method
   */
  public int getNextId( String name )
  {
    Connection con = null;
    PreparedStatement psSelect = null;
    PreparedStatement psUpdate = null;
    ResultSet rs = null;
    String query = null;

    int wert = 0;
    int version = 0;
    int counter = 0;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      do
      {
        psSelect = con.prepareStatement(
            "SELECT wert, version " +
            "FROM xx_ident_number " +
            "WHERE ident = 'bag' AND name = ? AND code = 0" );
        psSelect.setString( 1, name );
        rs = psSelect.executeQuery();

        if ( rs.next() )
        {
          wert = rs.getInt( "wert" );
          version = rs.getInt( "version" );
        }
        else
        {
          throw new EJBException( "Name = " + name + "in Tabelle XX_IDENT_NUMBER nicht gefunden" );
        }

        query = "UPDATE xx_ident_number " +
            "SET wert = wert + sequenz, " +
            "version = ? " +
            "WHERE ident = 'bag' AND name = ? AND code = 0";
        psUpdate = con.prepareStatement( query );
        psUpdate.setInt( 1, version + 1 );
        psUpdate.setString( 2, name );
      }
      while ( psUpdate.executeUpdate() != 1 && ++counter < MAX_RETRY );

      if ( counter < MAX_RETRY )
      {
        return wert;
      }
      else
      {
        throw new EJBException( "Probleme bei Update xx_ident_number Query: " +
                                query + " version = " + version + ", name = " + name );
      }
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
    finally
    {
      DBHelper.close( psSelect, psUpdate, con );
    }
  }

} // end class
