package ch.ige.pctedi.server.dao;

import java.sql.*;
import ch.ige.pctedi.vo.PatVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für PAT
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Martin Trachsel
 * @version $Id: PatDAO.java,v 1.7 2006/12/11 14:34:59 ruegsegger Exp $
 */

public class PatDAO
{

  /**
   * Fügt neue Daten in die Tabelle PAT ein.
   * @param patVO
   * @throws SQLException
   */
  public int insert( PatVO patVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "INSERT INTO pat(" +
          "schutit_key, " +
          "stammges_nr, " +
          "hauptpat_nr, " +
          "altpat_nr, " +
          "pat_verfahr, " +
          "ausstell_dat, " +
          "ausstell_name, " +
          "iks_dat, " +
          "iks_bez, " +
          "iks_nr, " +
          "spc_nr, " +
          "pct_anm_nr, " +
          "pct_anm_spra, " +
          "pct_publ_nr, " +
          "pct_publ_spra, " +
          "pct_publ_dat, " +
          "ep_anm_spra, " +
          "ep_apubl_spra, " +
          "ep_apubl_dat, " +
          "ep_bpubl_spra, " +
          "ep_bpubl_dat, " +
          "ep_stammpat_nr, " +
          "index_ipc, " +
          "version) " +
          "VALUES (" +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "? )";

      ps = con.prepareStatement( query );
      ps.setInt( 1, patVO.schutitKey );
      ps.setString( 2, patVO.stammgesNr );
      ps.setString( 3, patVO.hauptpatNr );
      ps.setString( 4, patVO.altpatNr );
      ps.setInt( 5, patVO.patVerfahr );
      if ( patVO.ausstellDat == null )
      {
        ps.setDate( 6, null );
      }
      else
      {
        ps.setDate( 6, new java.sql.Date( patVO.ausstellDat.getTime() ) );
      }
      ps.setString( 7, patVO.ausstellName );
      if ( patVO.iksDat == null )
      {
        ps.setDate( 8, null );
      }
      else
      {
        ps.setDate( 8, new java.sql.Date( patVO.iksDat.getTime() ) );
      }
      ps.setString( 9, patVO.iksBez );
      ps.setString( 10, patVO.iksNr );
      ps.setString( 11, patVO.spcNr );
      ps.setString( 12, patVO.pctAnmNr );
      ps.setString( 13, patVO.pctAnmSpra );
      ps.setString( 14, patVO.pctPublNr );
      ps.setString( 15, patVO.pctPublSpra );
      if ( patVO.pctPublDat == null )
      {
        ps.setDate( 16, null );
      }
      else
      {
        ps.setDate( 16, new java.sql.Date( patVO.pctPublDat.getTime() ) );
      }
      ps.setString( 17, patVO.epAnmSpra );
      ps.setString( 18, patVO.epApublSpra );
      if ( patVO.epApublDat == null )
      {
        ps.setDate( 19, null );
      }
      else
      {
        ps.setDate( 19, new java.sql.Date( patVO.epApublDat.getTime() ) );
      }
      ps.setString( 20, patVO.epBpublSpra );
      if ( patVO.epBpublDat == null )
      {
        ps.setDate( 21, null );
      }
      else
      {
        ps.setDate( 21, new java.sql.Date( patVO.epBpublDat.getTime() ) );
      }
      ps.setString( 22, patVO.epStammpatNr );
      ps.setString( 23, patVO.indexIpc );
      ps.setInt( 24, patVO.version );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, PatDAO.class, query + patVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}

