package ch.ige.pctedi.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import ch.ige.pctedi.vo.PrioritaetVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für Priorität
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Anita Rueegsegger, Ueli Ryser
 * @version $Id: PrioritaetDAO.java,v 1.8 2008/03/13 15:35:39 ruegsegger Exp $
 */

public class PrioritaetDAO
{
  /**
   * Fügt einen Datensatz in die Tabelle PRIORITAET ein
   * @param vo                  ValueObject PrioritaetVO
   *
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public int insert( PrioritaetVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO prioritaet(" + "schutit_key, " + "prio_lnr, " + "bland_cd, " + "prio_von_ereig, " + "prio_bis_ereig, " + "prio_nr, " + "prio_dat, "
          + "prio_bem, " + "reg_stat, " + "sprache, " + "version) " + "VALUES( ?,?,?,?,?,?,?,?,?,?,? ) ";
      ps = con.prepareStatement( query );
      ps.setInt( 1, vo.schutitKey );
      ps.setInt( 2, vo.prioLnr );
      ps.setString( 3, vo.blandCd );
      if ( vo.prioVonEreig == null )
      {
        ps.setNull( 4, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 4, vo.prioVonEreig.intValue() );
      }
      if ( vo.prioBisEreig == null )
      {
        ps.setNull( 5, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 5, vo.prioBisEreig.intValue() );
      }
      ps.setString( 6, vo.prioNr );
      if ( vo.prioDat == null )
      {
        ps.setDate( 7, null );
      }
      else
      {
        ps.setDate( 7, new java.sql.Date( vo.prioDat.getTime() ) );
      }
      ps.setString( 8, vo.prioBem );
      ps.setInt( 9, vo.regStat );
      if ( vo.sprache == null )
      {
        ps.setNull( 10, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 10, vo.sprache.intValue() );
      }
      ps.setInt( 11, vo.version );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, PrioritaetDAO.class, query + vo.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

}
