package ch.ige.pctedi.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;

import ch.ige.pctedi.vo.SchutitVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für Schutztitel
 * <p>
 * Beschreibung:  SchutitVO ist das VO der Tabelle Schutit.
 * Copyright:     Copyright (c) 2002, , Eidgenössisches Institut für Geistiges Eigentum
 * Organisation:  IGE
 * @version       $Id: SchutitDAO.java,v 1.12 2008/03/13 15:35:39 ruegsegger Exp $
 */

public class SchutitDAO
{
  /**
   * Fügt einen Datensatz in die Tabelle SCHUTIT ein
   * @param  vo      ValueObject
   *
   * @exception SQLException            Unerwarteter SQL Fehler
   */
  public int insert( SchutitVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO schutit(" + "schutit_key, " + "ma_nr, " + "schutit_typ, " + "sprache, " + "gesuch_nr, " + "schutit_nr, " + "register_kz, "
          + "schutz_beginn, " + "schutz_ende, " + "loesch_dat, " + "beschreibung, " + "upp_beschreibung, " + "ihr_zeichen, " + "dossier_beachten, "
          + "intern_bem, " + "kz_ueberwach, " + "erfass_ma, " + "erfass_dat, " + "mut_ma, " + "mut_dat, " + "erst_ereig_dat, " + "faellig_jahr, " + "version) "
          + "VALUES ( ?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,? ) ";

      ps = con.prepareStatement( query );
      ps.setInt( 1, vo.schutitKey );
      if ( vo.maNr == null )
      {
        ps.setNull( 2, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 2, vo.maNr.intValue() );
      }
      ps.setInt( 3, vo.schutitTyp );
      ps.setInt( 4, vo.sprache );
      ps.setString( 5, vo.gesuchNr );
      ps.setString( 6, vo.schutitNr );
      ps.setInt( 7, vo.registerKz );
      if ( vo.schutzBegin == null )
      {
        ps.setDate( 8, null );
      }
      else
      {
        ps.setDate( 8, new java.sql.Date( vo.schutzBegin.getTime() ) );
      }
      if ( vo.schutzEnde == null )
      {
        ps.setDate( 9, null );
      }
      else
      {
        ps.setDate( 9, new java.sql.Date( vo.schutzEnde.getTime() ) );
      }
      if ( vo.loeschDat == null )
      {
        ps.setDate( 10, null );
      }
      else
      {
        ps.setDate( 10, new java.sql.Date( vo.loeschDat.getTime() ) );
      }
      ps.setString( 11, vo.beschreibung );
      ps.setString( 12, vo.uppBeschreibung );
      ps.setString( 13, vo.ihrZeichen );
      ps.setInt( 14, vo.dossierBeachten );
      ps.setString( 15, vo.internBem );
      ps.setInt( 16, vo.kzUeberwach );
      ps.setInt( 17, vo.erfassMa );
      /** TODO erfassdat ersetzt durch sysdate java.sql.Date ist ohne Zeit*/
      ps.setDate( 18, new java.sql.Date( System.currentTimeMillis() ) );
      if ( vo.mutMa == null )
      {
        ps.setNull( 19, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 19, vo.mutMa.intValue() );
      }
      if ( vo.mutDat == null )
      {
        ps.setDate( 20, null );
      }
      else
      {
        ps.setDate( 20, new java.sql.Date( vo.mutDat.getTime() ) );
      }
      if ( vo.erstEreigDat == null )
      {
        ps.setDate( 21, null );
      }
      else
      {
        ps.setDate( 21, new java.sql.Date( vo.erstEreigDat.getTime() ) );
      }

      ps.setInt( 22, vo.faelligJahr );
      ps.setInt( 23, vo.version );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, SchutitDAO.class, query + vo.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Sucht den SchutitKey zu einem Schutztitel über SchutitNr
   * @param   schutitNr   String
   * @param   schutitTyp  int
   * @return  int schutitKey
   *
   * @exception SQLException            Unerwarteter SQL Fehler
   */
  public int selectSchutitKeyWithSchutitNr( String schutitNr, int schutitTyp ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    int schutitKey = 0;

    try
    {
      if ( schutitNr != null && !schutitNr.equals( "" ) ) // Gesuche haben nicht unbedigt eine Schutit_Nummer
      {
        con = DBHelper.getConnection( DBHelper.BAGIS );
        query = "SELECT schutit_key " + "FROM schutit " + "WHERE schutit_nr = ? " + "AND schutit_typ = ? ";

        ps = con.prepareStatement( query );
        ps.setString( 1, schutitNr );
        ps.setInt( 2, schutitTyp );
        rs = ps.executeQuery();

        if ( rs.next() )
        {
          schutitKey = rs.getInt( "schutit_key" );
        }
      }
      return schutitKey;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, SchutitDAO.class, query + " schutitNr: " + schutitNr + " schutitTyp: " + schutitTyp );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Sucht den SchutitKey zu einem Schutztitel über SchutitNr
   * @param   gesuchNr   String
   * @param   schutitTyp  int
   * @return  int schutitKey
   *
   * @exception SQLException            Unerwarteter SQL Fehler
   */
  public int selectSchutitKeyWithGesuchNr( String gesuchNr, int schutitTyp ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    int schutitKey = 0;

    try
    {
      if ( gesuchNr != null && !gesuchNr.equals( "" ) ) // Gesuche haben nicht unbedigt eine Schutit_Nummer
      {
        con = DBHelper.getConnection( DBHelper.BAGIS );
        query = "SELECT schutit_key " + "FROM schutit " + "WHERE gesuch_nr = ? " + "AND schutit_typ = ? ";

        ps = con.prepareStatement( query );
        ps.setString( 1, gesuchNr );
        ps.setInt( 2, schutitTyp );
        rs = ps.executeQuery();

        if ( rs.next() )
        {
          schutitKey = rs.getInt( "schutit_key" );
        }
      }
      return schutitKey;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, SchutitDAO.class, query + " gesuchNr: " + gesuchNr + " schutitTyp: " + schutitTyp );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Lösche eines EP, wobei bei Nichterfolg False zurückgegeben werden muss
   * @param schutitKey  des zu löschenden EP
   * @return true or false
   */
  public int deleteSchutit( int schutitKey ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      ps = con.prepareStatement( "DELETE FROM schutit WHERE schutit_key = ?" );
      ps.setInt( 1, schutitKey );
      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, SchutitDAO.class, query + schutitKey );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * returns minimal date of filling and prio date
   * @param schutitKey 
   * @return minimal date of filling and prio date
   * @throws SQLException
   */
  public Date selectMinimalDate( int schutitKey ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    Date minDatum = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      query = "SELECT MIN(datum) AS datum FROM ( SELECT MIN(schutz_beginn) AS datum FROM schutit WHERE schutit_key = ? UNION "
          + "SELECT MIN(prio_dat) AS datum FROM prioritaet WHERE schutit_key = ? )";
      ps = con.prepareStatement( query );
      ps.setInt( 1, schutitKey );
      ps.setInt( 2, schutitKey );

      rs = ps.executeQuery();

      while ( rs.next() )
      {
        minDatum = rs.getDate( "datum" );
      }
      return minDatum;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, PrioritaetDAO.class, query + "schutitKey:" + schutitKey );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
