package ch.ige.pctedi.server.dao.test;

import java.sql.SQLException;
import java.util.List;

import ch.ige.pctedi.server.dao.EreignisDAO;
import ch.ige.pctedi.server.dao.SchutitDAO;
import ch.ige.pctedi.vo.EreignisVO;
import ch.ige.pctedi.vo.SchutitVO;
import ch.ige.pctedi.vo.test.Testdata;

/**
 * Junit-Test der Klasse EreignisDAO
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: EreignisDAOTest.java,v 1.10 2008/03/13 15:35:39 ruegsegger Exp $
 */
public class EreignisDAOTest extends DAOTestCase
{
  private EreignisDAO ereignisDAO = new EreignisDAO();

  private EreignisVO ereignisVO = Testdata.createEreignisErsterfassung();

  private SchutitDAO schutitDAO = new SchutitDAO();

  private SchutitVO schutitVO;

  public EreignisDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    try
    {
      schutitVO = Testdata.createSchutit();
      schutitDAO.insert( schutitVO );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  @Override
  protected void tearDown()
  {
    try
    {
      schutitDAO.deleteSchutit( schutitVO.schutitKey );
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
    }
  }

  public void test00Insert()
  {
    try
    {
      int result = ereignisDAO.insert( ereignisVO );
      assertEquals( 1, result );
    }
    catch ( SQLException ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  public void test02EreigIdat()
  {
    try
    {
      // insert erforderlich, da in teardown cascade gelöscht wird
      ereignisDAO.insert( ereignisVO );

      int nextEreigIdat = ereignisDAO.getNextereigIdat( ereignisVO.schutitKey, ereignisVO.ereigIdat );
      assertEquals( ereignisVO.ereigIdat + 5, nextEreigIdat );
    }
    catch ( SQLException ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  public void test03SelectArtNr()
  {
    try
    {
      // insert erforderlich, da in teardown cascade gelöscht wird
      ereignisDAO.insert( ereignisVO );

      List<EreignisVO> ereignisList = ereignisDAO.selectArtNr( ereignisVO.schutitKey, ereignisVO.artNr );

      assertTrue( ereignisList.size() > 0 );
    }
    catch ( SQLException ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  public void test04ExistArtNr()
  {
    try
    {
      // insert erforderlich, da in teardown cascade gelöscht wird
      ereignisDAO.insert( ereignisVO );

      boolean exist = ereignisDAO.existArtNr( ereignisVO.schutitKey, ereignisVO.artNr );

      assertTrue( exist );
    }
    catch ( SQLException ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  public void test05DeleteArtNr()
  {
    try
    {
      EreignisVO ereignis30040 = new EreignisVO();

      // nun nächstes ereigidat bestimmen
      ereignis30040.ereigIdat = 2008010105;
      ereignis30040.schutitKey = ereignisVO.schutitKey;
      ereignis30040.artNr = EreignisVO.PRUEFUNG;

      int anz = ereignisDAO.insert( ereignis30040 );
      assertEquals( 1, anz );

      anz = ereignisDAO.delete( ereignisVO.schutitKey, EreignisVO.PRUEFUNG );
      assertEquals( 1, anz );
    }
    catch ( SQLException ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Ein Ereignis mit Abschlussdatum kann nicht mehr gelöscht werden,
   * deshalb wird dieser Test auskommentiert, da nicht mehrfach ausführbar
   */
  public void _test04Update()
  {
    try
    {
      // insert erforderlich, da in teardown cascade gelöscht wird
      ereignisDAO.insert( ereignisVO );

      List<EreignisVO> ereignisList = ereignisDAO.selectArtNr( ereignisVO.schutitKey, ereignisVO.artNr );

      EreignisVO vo = ereignisList.get( 0 );

      vo.ereigStatNr = 19;
      vo.abschlDat = new java.util.Date();
      vo.abschlmutDat = new java.util.Date();
      int anz = ereignisDAO.update( vo );

      assertTrue( anz == 1 );
    }
    catch ( SQLException ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }
}
