package ch.ige.pctedi.server.dao.test;

import java.text.SimpleDateFormat;

import ch.ige.pctedi.server.dao.IpcDAO;
import ch.ige.pctedi.vo.IpcVO;
import ch.ige.pctedi.vo.test.Testdata;

/**
 * Test der Klasse IpcDAO
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: IpcDAOTest.java,v 1.8 2008/03/11 14:11:01 ruegsegger Exp $
 */

public class IpcDAOTest extends DAOTestCase
{
  private IpcDAO ipcDAO = new IpcDAO();

  private IpcVO ipcVO = null;

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyy-MM-dd" );

  public IpcDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    try
    {
      ipcVO = Testdata.createIpc();
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle IPC
   */
  public void test01Select()
  {
    System.out.println( "Test SELECT " );
    IpcVO selectVO = null;

    try
    {
      selectVO = ipcDAO.select( ipcVO.ipcSymb, ipcVO.gueltigVon );
      if ( selectVO == null )
      {
        fail( "SELECT: Datensatz existiert nicht" );
      }
      else
      {
        assertEquals( ipcVO.ipcSymb, selectVO.ipcSymb );
        assertEquals( ipcVO.zusatz, selectVO.zusatz );
        assertEquals( sdf.format( ipcVO.gueltigVon ), sdf.format( selectVO.gueltigVon ) );

        String ipcVOGueltigBis = ipcVO.gueltigBis != null ? sdf.format( ipcVO.gueltigBis ) : null;
        String selectVOGueltigBis = selectVO.gueltigBis != null ? sdf.format( selectVO.gueltigBis ) : null;

        assertEquals( ipcVOGueltigBis, selectVOGueltigBis );
        assertEquals( ipcVO.classLevel, selectVO.classLevel );
        assertEquals( ipcVO.corePred, selectVO.corePred );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle IPC
   */
  public void test02Select()
  {
    System.out.println( "Test SELECT " );
    IpcVO selectVO = null;

    try
    {
      selectVO = ipcDAO.selectYoungest( ipcVO.ipcSymb );
      if ( selectVO == null )
      {
        fail( "SELECT: Datensatz existiert nicht" );
      }
      else
      {
        assertEquals( ipcVO.ipcSymb, selectVO.ipcSymb );
        assertEquals( ipcVO.zusatz, selectVO.zusatz );
        assertEquals( sdf.format( ipcVO.gueltigVon ), sdf.format( selectVO.gueltigVon ) );

        String ipcVOGueltigBis = ipcVO.gueltigBis != null ? sdf.format( ipcVO.gueltigBis ) : null;
        String selectVOGueltigBis = selectVO.gueltigBis != null ? sdf.format( selectVO.gueltigBis ) : null;

        assertEquals( ipcVOGueltigBis, selectVOGueltigBis );
        assertEquals( ipcVO.classLevel, selectVO.classLevel );
        assertEquals( ipcVO.corePred, selectVO.corePred );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
