package ch.ige.pctedi.server.dao.test;

import java.util.Date;

import ch.ige.pctedi.server.PctBO;
import ch.ige.pctedi.vo.PctVO;
import ch.ige.pctedi.vo.SchutitVO;

/**
 * Junit-Test der Klasse PctBO
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author M. Wild
 * @version $Id: PctBOTest.java,v 1.4 2008/06/19 12:17:06 ruegsegger Exp $
 */

public class PctBOTest extends DAOTestCase
{
  private PctVO pctVO;

  private PctBO pctBO;

  /**
   * Konstruktor
   * @param s
   */
  public PctBOTest( String s )
  {
    super( s );

  }

  @Override
  protected void setUp()
  {
    try
    {
      pctBO = new PctBO();
      pctVO = new PctVO();
      pctVO.schutitVO = new SchutitVO();
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Test für Methode findPct.
   * Suchen nach Schutztitel mit GesuchsNr und SchutitNr.
   * Es können folgende Fälle auftreten:
   *  0: kein Schutztitel vorhanden
   *  1: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben das selbe Resultat.
   *  2: Schutztitel vorhanden; nur Suche über schutitNr ergab ein Resultat.
   *  3: Schutztitel vorhanden; nur Suche über GesuchNr ergab ein Resultat.
   *  4: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben unterschiedliche Resultate.
   */

  /**
   * Prüft weder Gesuch noch Schutit_nr werden gefunden.
   */
  public void test01findPct()
  {
    pctVO.schutitVO.gesuchNr = "5400620065";
    pctVO.schutitVO.schutitNr = "5463609";

    int retval = pctBO.findPct( pctVO );

    assertEquals( 0, retval );

  }

  /**
   * Schutit gefunden, selber Titel nach GesuchNr und schutitNr.
   */
  public void test02findPct()
  {
    pctVO = new PctVO();
    pctVO.schutitVO = new SchutitVO();
    pctVO.schutitVO.gesuchNr = "IL2006001356";
    pctVO.schutitVO.schutitNr = "2007060666";

    int retval = pctBO.findPct( pctVO );

    assertEquals( 1, retval );
  }

  /**
   * Schutit gefunden aber nur über SchutitNr.
   */
  public void test03findPct()
  {
    pctVO.schutitVO.gesuchNr = "1236";
    pctVO.schutitVO.schutitNr = "2007060667";

    int retval = pctBO.findPct( pctVO );

    assertEquals( 2, retval );
  }

  /**
   * Schutit gefunden aber nur über GesuchNr.
   */
  public void test04findPct()
  {
    pctVO.schutitVO.gesuchNr = "IL2006001356";
    pctVO.schutitVO.schutitNr = "1963";

    int retval = pctBO.findPct( pctVO );

    assertEquals( 3, retval );
  }

  /**
   * Schutit gefunden; unterschiedliche über schutitNr und GesuchNr.
   */
  public void test05findPct()
  {
    pctVO.schutitVO.gesuchNr = "IL2006001360";
    pctVO.schutitVO.schutitNr = "2007060668";

    int retval = pctBO.findPct( pctVO );

    assertEquals( 4, retval );

  }

  /**
  * Dieser Test ist nicht wiederholbar
  * Um diese Methode zu testen, müssen erst geeignete Datensätze gefunden werden
  * Schutitkey angeben
  */
  public void _testcreateExtension()
  {
    /* Testfall mit 30010, 30062
     * 
     * Frist Ereignis 30010 neu berechnen und auf Frist setzen
     * Ereignis 30040 neu erstellen ohne Frist
     * Ereignis 30062 annulliren 
     */
    String message = pctBO.createExtension( 1201620985, new Date() );
    assertEquals( "Frist 30010 neu berechnet - Ereignis 30040 neu erstellt - Ereignis 30062 annulliert ( Schutitkey: 1201620985", message );

    /* Testfall mit 30010, 30040
     * 
     * Frist Ereignis 30010 neu berechnen und auf Frist setzen
     * Ereignis 30040 löschen und neu erstellen ohne Frist 
     */
    message = pctBO.createExtension( 1202158686, new Date() );
    assertEquals( "Frist 30010 neu berechnet - Ereignis 30040 neu erstellt ( Schutitkey: 1202158686)", message );

    /* Testfall mit 30010, 30040, 30062
     * 
     * Frist Ereignis 30010 neu berechnen und auf Frist setzen
     * Ereignis 30040 neu erstellen ohne Frist
     * Ereignis 30062 annulliren 
     */
    message = pctBO.createExtension( 1200935104, new Date() );
    assertEquals(
        "Frist 30010 neu berechnet - Ereignis 30040 bereits vorhanden - Ereignis 30040 neu erstellt - Ereignis 30062 annulliert ( Schutitkey: 1200935104)",
        message );
  }
}
