package ch.ige.rbbagisexport.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.rbbagisexport.common.RBBagisExportConfiguration;
import ch.ige.rbbagisexport.server.config.RBBagisExportConfigReader;
import ch.ige.rbbagisexport.server.util.EreignisseContainer;
import ch.ige.rbbagisexport.server.util.InvalidEreignisseList;
import ch.ige.rbbagisexport.server.vo.RBExportVO;

/**
 * DAO to select Multiple RBExportVO.
 * <p>
 * Copyright (c) 2007, Eidgenössisches Institut für Geistiges Eigentum
 * @author M. Künzli
 * @version $Id: RBMultipleExportDAO.java,v 1.23 2008/06/20 08:35:07 kuenzli Exp $
 */
public class RBMultipleExportDAO extends AbstractBagisDAO
{
	//----------------------------------------------------------------------------
  // Private Members
  //----------------------------------------------------------------------------
  private static final long serialVersionUID = 1L;
  // Logger
  private static final Logger LOG = Logger.getLogger( RBMultipleExportDAO.class );

  //----------------------------------------------------------------------------
  // Constructors
  //----------------------------------------------------------------------------
  public RBMultipleExportDAO() throws Exception
  {
  }

  //----------------------------------------------------------------------------
  // Public Methods
  //----------------------------------------------------------------------------
  /**
   * Fetches all the patent applications to be exported and puts the data into a
   * list of RBExportVO.
   * @return List or RBExportVO 
   * @throws Exception
   */
  public List<RBExportVO> selectExportData() throws Exception
  {
    List<Integer> schutitKeyList = selectSchutitData();
    ArrayList<RBExportVO> exportVOList = new ArrayList<RBExportVO>();
    RBExportDAO exportDAO = new RBExportDAO();
    for ( Integer schutitKey: schutitKeyList )
    {
      exportVOList.add( exportDAO.selectRBExportData( schutitKey ) );
    }

    return exportVOList;
  }

  //----------------------------------------------------------------------------
  // Private Methods
  //----------------------------------------------------------------------------
  @SuppressWarnings("unchecked")
  private List<Integer> selectSchutitData() throws SQLException, InvalidEreignisseList
  {
    // Handler Class
    class RSHandler implements ResultSetHandler
    {
      public Object handle( ResultSet rs ) throws SQLException
      {
        ArrayList<Integer> schutitList = new ArrayList<Integer>();
        Integer exportSchutitKey = null;
        while ( rs.next() )
        {
          exportSchutitKey = Integer.valueOf( rs.getInt( "schutit_key" ) );
          schutitList.add( exportSchutitKey );
        }
        return schutitList;
      }
    }
    RBBagisExportConfigReader confReader = RBBagisExportConfigReader.instance();
    RBBagisExportConfiguration conf = confReader.readConfigurationFromDb();
    String ereignisseSql = "";
    String ereignisse = null;
    try
    {
      ereignisse = conf.getExportEreignisListe();
      LOG.info( "selectSchutitData(): EreignisListe = " + ereignisse );
      ereignisseSql = new EreignisseContainer( ereignisse ).getSqlNotation();

    }
    catch ( InvalidEreignisseList e )
    {
      LOG.error( "selectSchutitData(): Ereignis List is invalid: " + ereignisse );
      throw e;
    }
    // Selection
    
    // Prepared Statement for regular use
    StringBuilder ps = new StringBuilder();
    ps.append( " SELECT s.schutit_key " )
      .append( " FROM schutit s " )
      .append( " WHERE s.schutit_typ = 1 " )
      .append( " AND s.loesch_dat IS null " )
      .append( " AND EXISTS ( SELECT e.schutit_key " )
      .append( "              FROM ereignis e, ereig_stat t" )
      .append( "              WHERE e.schutit_key = s.schutit_key " )
      .append( "              AND e.art_nr IN ( " + ereignisseSql + " ) " )
      .append( "              AND e.ereig_stat_nr = t.ereig_stat_nr ")
      .append( "              AND t.reg_faehig = 1 )")
      .append( " AND NOT EXISTS ( SELECT sr.schutit_key " )
      .append( "                  FROM searchreport sr " )
      .append( "                  WHERE sr.schutit_key = s.schutit_key " )
      .append( "                  AND sr.type = 1 " )
      .append( "                  AND sr.state != 0 ) " );

    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "selectSchutitData(): ereignisseSql = " + ereignisseSql );
    }

    return (ArrayList<Integer>) this.getQueryRunner().query( ps.toString(), new RSHandler() );
  }
  
  @SuppressWarnings( "unused" )
  private StringBuilder createPrepStatementForGrantedPatents(String ereignisseSql){
    // **** For File Test Reasons with Third Party ****
    // **** Select Granted Patents only ****
    StringBuilder psGrantedPatents = new StringBuilder();
    psGrantedPatents.append( " SELECT s.schutit_key " )
    .append( " FROM schutit s " )
    .append( " WHERE s.register_kz = 1 " )
    .append( " AND s.schutit_typ = 1 " )
    .append( " AND s.loesch_dat IS null " )
    .append( " s.erfass_dat > to_date('01-Jun-2002') " )
    .append( " AND EXISTS ( SELECT e.schutit_key " )
    .append( "              FROM ereignis e " )
    .append( "              WHERE e.schutit_key = s.schutit_key " )
    .append( "              AND e.art_nr IN ( " + ereignisseSql + " ) ) " )
    .append( " AND NOT EXISTS ( SELECT sr.schutit_key " )
    .append( "                  FROM searchreport sr " )
    .append( "                  WHERE sr.schutit_key = s.schutit_key " )
    .append( "                  AND sr.type = 1 " )
    .append( "                  AND sr.state != 0 ) " );
    return psGrantedPatents;
  }
}
