package ch.ige.rbbagisimport.client;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;

import org.apache.log4j.Logger;

import ch.ige.rbbagisimport.common.vo.SearchReportFileVO;
import ch.ige.rbbagisimport.common.vo.XxSysparamsVO;
import ch.ige.util.IOUtil;

/**
 * Handles the File-Access 
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: FileHelper.java,v 1.10 2008/06/10 13:52:17 wild Exp $
 */
public class FileHelper
{
  private static final Logger LOG = Logger.getLogger( FileHelper.class );

  /*  *******************************************/
  /*                PUBLIC-METHODS              */
  /*  *******************************************/

  private File[] draftFiles = null;

  private File[] finalFiles = null;

  /**
   * Returns SearchReportFile VO's containing File Information of Search Reports to be imported
   * @return ArrayList<SearchReportFiles> SearchReport Files
   * @throws FileNotFoundException
   * @throws IOException
   */
  public ArrayList<SearchReportFileVO> readXMLFiles() throws FileNotFoundException, IOException
  {
    String path = "//" + ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_SERVER )
        + ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_PATH );
    draftFiles = getFiles( path + "/" + ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_DRAFT_PARAM_ID ) );
    finalFiles = getFiles( path + "/" + ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_FINAL_PARAM_ID ) );
    return createVOs();
  }

  /**
   * Moves the processed Files to the "processed" folder and the faultily ones to the error folder 
   * @param reports ArrayList<SearchReportFile> SearchReports
   */
  public void cleanup( ArrayList<SearchReportFileVO> reports )
  {
    String newFolder = getProcessedFolderName();
    String basePathError = "//" + ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_SERVER )
        + ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_FOLDER_ERROR_PARAM_ID );
    String basePathProcessed = "//" + ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_SERVER )
        + ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_FOLDER_PROCESSED_PARAM_ID );
    String draft = ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_DRAFT_PARAM_ID );
    String fin = ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_FINAL_PARAM_ID );
    for ( SearchReportFileVO file: reports )
    {
      File myFile = new File( file.getFilePath() );
      if ( file.getProcessState() == SearchReportFileVO.PROCESS_STATE_SUCCEEDED )
      {
        if ( !new File( basePathProcessed + "/" + newFolder ).exists() )
        {
          if ( !new File( basePathProcessed + "/" + newFolder ).mkdir() )
          {
            LOG.error( "Unable create new folder: " + basePathProcessed + "/" + newFolder );
          }

          if ( !new File( basePathProcessed + "/" + newFolder + "/" + draft ).mkdir() )
          {
            LOG.error( "Unable create new folder: " + basePathProcessed + "/" + newFolder + "/" + draft );
          }
          if ( !new File( basePathProcessed + "/" + newFolder + "/" + fin ).mkdir() )
          {
            LOG.error( "Unable create new folder: " + basePathProcessed + "/" + newFolder + "/" + fin );
          }
        }
        System.gc(); // is necessary because renaming of files doesn't work reliably
        String newFileName = "";
        if ( file.getSrType() == SearchReportFileVO.SR_TYPE_DRAFT )
        {
          newFileName = basePathProcessed + "/" + newFolder + "/" + draft + "/" + myFile.getName();
        }
        else
        {
          newFileName = basePathProcessed + "/" + newFolder + "/" + fin + "/" + myFile.getName();
        }
        File newFile = new File( newFileName );
        if ( !myFile.renameTo( newFile ) )
        {
          LOG.error( "Unable to rename File from " + myFile.getPath() + " to " + newFileName );;
        }
        file.setFilePathProcessed( newFile.getAbsolutePath() );

      }
      else if ( file.getProcessState() == SearchReportFileVO.PROCESS_STATE_FAILED )
      {
        System.gc(); // is necessary because renaming of files doesn't work reliably
        // Das Rename funktioniert nicht, wenn bereits ein File vorhanden ist. 
        // Aus diesem Grund lösche ich es sichherheitshalber schon mal...
        String newFileName = basePathError + "/" + myFile.getName();
        File oldFile = new File( newFileName );
        if ( oldFile.delete() )
        {
          LOG.debug( "File deleted: " + newFileName );
        }

        File newFile = new File( newFileName );
        if ( !myFile.renameTo( newFile ) )
        {
          LOG.error( "Unable to rename File from " + myFile.getPath() + " to " + newFileName );;
        }
        file.setFilePathProcessed( newFile.getAbsolutePath() );
      }
    }
  }

  /*  *******************************************/
  /*                PRIVATE-METHODS              */
  /*  *******************************************/

  private File[] getFiles( String folder )
  {
    return new File( folder ).listFiles();
  } // end method

  private ArrayList<SearchReportFileVO> createVOs() throws FileNotFoundException, IOException
  {
    ArrayList<SearchReportFileVO> searchReports = new ArrayList<SearchReportFileVO>();
    if ( draftFiles != null )
    {
      for ( File file: draftFiles )
      {
        SearchReportFileVO srf = new SearchReportFileVO( getXMLString( file ), SearchReportFileVO.SR_TYPE_DRAFT,
            SearchReportFileVO.PROCESS_STATE_NOT_PROCESSED, file.getAbsolutePath() );
        searchReports.add( srf );
      }
    }
    if ( finalFiles != null )
    {
      for ( File file: finalFiles )
      {
        SearchReportFileVO srf = new SearchReportFileVO( getXMLString( file ), SearchReportFileVO.SR_TYPE_FINAL,
            SearchReportFileVO.PROCESS_STATE_NOT_PROCESSED, file.getAbsolutePath() );
        searchReports.add( srf );
      }
    }
    return searchReports;
  } // end method

  private String getXMLString( File file ) throws IOException
  {
    return IOUtil.toString( file, IOUtil.createStrictUTF8Decoder() );
  }

  private String getProcessedFolderName()
  {
    // determine Folder by Date
    StringBuffer buf = new StringBuffer();
    Calendar cal = Calendar.getInstance();

    buf.append( cal.get( Calendar.YEAR ) );
    buf.append( addZeros( cal.get( Calendar.MONTH ) + 1 ) );
    buf.append( addZeros( cal.get( Calendar.DAY_OF_MONTH ) ) );
    buf.append( "_" );
    buf.append( addZeros( cal.get( Calendar.HOUR_OF_DAY ) ) );
    buf.append( addZeros( cal.get( Calendar.MINUTE ) ) );

    return buf.toString();
  }

  private String addZeros( int orig )
  {
    String retval = "";
    if ( orig < 10 )
    {
      retval = "0" + orig;
    }
    else
    {
      retval = String.valueOf( orig );
    }
    return retval;
  }

} // end class