package ch.ige.st36xmlgenerator;

import java.sql.SQLException;
import java.util.List;

import org.apache.commons.configuration.Configuration;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.ipobjects.time.BAGISTime;
import ch.ige.st36xmlgenerator.exception.IllegalBatchArgumentException;
import ch.ige.util.enums.infrastructure.DBRealm;

public class ST36BiblGeneratorProperties extends AbstractVO
{

  private int limit;

  private int start_ereigidat;

  private int end_ereigidat;

  private String data_store_location;

  private String data_store_location_processed;

  private String data_store_location_tosend;

  private List<String> articles;

  private String smptServer;

  private String to;

  private String cc;

  private String from;

  private String toISO;

  public ST36BiblGeneratorProperties( int limit, int startEreigidat, int endEreigidat, String dataStoreLocation, String dataStoreLocationProcessed,
      String dataStoreLocationZip, List<String> articles, String smtpServer, String mailDestination, String mailCC, String mailSender, String mailDestinationISO )
  {
    this.limit = limit;
    this.start_ereigidat = startEreigidat;
    this.end_ereigidat = endEreigidat;
    this.data_store_location = dataStoreLocation;
    this.data_store_location_processed = dataStoreLocationProcessed;
    this.data_store_location_tosend = dataStoreLocationZip;
    this.articles = articles;
    this.smptServer = smtpServer;
    this.to = mailDestination;
    this.cc = mailCC;
    this.from = mailSender;
    this.toISO = mailDestinationISO;
  }

  /**
   * Paket Grösse an Schriften die gleichzeitig generiert werden
   */
  public int getLimit()
  {
    return limit;
  }

  /**
   * Angabe ab wann der nächste Lauf Daten generieren muss
   */
  public int getStartPoint()
  {
    return start_ereigidat;
  }

  /**
   * Angabe bis wann der nächste Lauf Daten generieren muss
   */
  public int getEndPoint()
  {
    return end_ereigidat;
  }

  /**
   * Pfad zur Ablage der generierten Daten
   */
  public String getDataStoreLocation()
  {
    return data_store_location;
  }

  /**
   * Pfad zur Ablage der zu sendenden Daten
   */
  public String getDataStoreLocationProcessed()
  {
    return data_store_location_processed;
  }

  /**
   * Pfad zur Ablage der gesandten Daten
   */
  public String getDataStoreLocationZip()
  {
    return data_store_location_tosend;
  }

  /**
   * Artikelnummern die die Generiereung der Daten auslösen
   */
  public List<String> getArtikels()
  {
    return articles;
  }
  
  /**
   * Mail Server
   */
  public String getSMTPServer()
  {
    return smptServer;
  }

  /**
   * Mail Adresse Empfänger
   */
  public String getMailDestination()
  {
    return to;
  }

  /**
   * Mail Adresse cc
   */
  public String getMailCC()
  {
    return cc;
  }

  /**
   * Mail Adresse Sender
   */
  public String getMailSender()
  {
    return from;
  }

  /**
   * Mail Adresse Empfänger ISO
   */
  public String getMailDestinationISO()
  {
    return toISO;
  }

  /**
   * Set the new start ereigidat
   * @param ereigIdat the new start ereigidat to set
   */
  public void setStartEreigidat( int ereigIdat )
  {
    this.start_ereigidat = ereigIdat;
  }

  /**
   * Set the new end ereigidat
   * @param ereigIdat the new end ereigidat to set
   */
  public void setEndEreigidat( int ereigIdat )
  {
    this.end_ereigidat = ereigIdat;
  }

  /**
   * Initialize from database (ST36XMLGENERATORCONFIG)
   */
  public static ST36BiblGeneratorProperties initializeFromDatabase() throws SQLException
  {
    Configuration configuration = DAOPool.getDummy( DBRealm.BAGIS ).getConfigurationTable( "ST36XMLGENERATORCONFIG", null, "KEY", "VALUE", "SCOPE" )
        .getFullConfiguration( true );
    return initializeFromConfiguration( configuration );
  }

  private static ST36BiblGeneratorProperties initializeFromConfiguration( Configuration configuration ) throws SQLException
  {
    ST36BiblGeneratorProperties props = new ST36BiblGeneratorProperties( configuration.getInt( "limit" ), configuration.getInt( "start_ereigidat" ),
        configuration.getInt( "end_ereigidat" ), configuration.getString( "data_store_location" ), configuration.getString( "data_store_location_processed" ),
        configuration.getString( "data_store_location_tosend" ), configuration.getList( "articles" ), configuration.getString( "smtpServer" ), configuration
            .getString( "mailDestination" ), configuration.getString( "mailCC" ), configuration.getString( "mailSender" ), configuration
            .getString( "mailDestinationISO" ) );
    props.validate();
    return props;
  }

  /**
   * Validate the consistency of the properties
   */
  public void validate()
  {
    if ( this.end_ereigidat != 0 && this.start_ereigidat >= this.end_ereigidat )
    {
      throw new IllegalBatchArgumentException( " start (" + this.start_ereigidat + ") and/or end ereigidat (" + this.end_ereigidat + ") are not valid" );
    }
    if ( this.limit == 0 )
    {
      throw new IllegalBatchArgumentException( "limit cannot e null" );
    }
    if ( this.data_store_location == null || this.data_store_location.length() == 0 )
    {
      throw new IllegalBatchArgumentException( " data store location cannot be null" );
    }
    if ( this.articles == null || this.articles.isEmpty() )
    {
      throw new IllegalBatchArgumentException( " Articles cannot be null" );
    }
  }

  @Override
  public String toString()
  {
    StringBuilder builder = new StringBuilder( super.toString() );
    builder.append( "[Limit=" + limit );
    builder.append( ", start_ereigidat=" + BAGISTime.formatForDebug( start_ereigidat ) );
    builder.append( ", end_ereigidat=" + BAGISTime.formatForDebug( end_ereigidat ) );
    builder.append( ", data_store_location=" + data_store_location );
    builder.append( ", data_store_location_tosend=" + data_store_location_tosend );
    builder.append( ", data_store_location_processed=" + data_store_location_processed );
    builder.append( ", [articles: " );
    for ( String article :  articles)
    {
      builder.append( article );
    }
    builder.append( "]" );
    builder.append( ", smptServer=" + smptServer );
    builder.append( ", mail_from=" + from );
    builder.append( ", mail_to=" + to );
    builder.append( ", mail_toISO=" + toISO );
    builder.append( ", mail_cc=" + cc );
    builder.append( "]" );
    return builder.toString();
  }



}
