package ch.ige.st36xmlgenerator.builder;

import java.io.IOException;

import javax.xml.bind.JAXBException;
import javax.xml.transform.TransformerException;

import ch.ige.xmlutil.bind.BindingType;
import ch.ige.xmlutil.bind.Bindings;
import ch.ige.xmlutil.doc.XMLDocument;

public abstract class AbstractFactory<X>
{

  protected BindingType bindType;

  protected Class<? extends X> type;

  // assert single threaded access
  protected X rootObject;

  abstract public void setRootAttributes( Object[] par );

  public AbstractFactory( BindingType binding, Class<? extends X> type )
  {

    this.bindType = binding;
    this.type = type;
    this.rootObject = binding.createTreeObject( type );
  }

  public AbstractFactory( BindingType binding, Class<? extends X> type, X rootObject )
  {
    this.bindType = binding;
    this.type = type;
    this.rootObject = rootObject;
  }

  public XMLDocument getMarshalledDocument() throws JAXBException, TransformerException, IOException, ClassNotFoundException
  {
    // does not work for ch-document_id country Bindings.validate(bindType, rootObject);
    return Bindings.marshalToXMLDocument( bindType, rootObject );
  }

  public String getPhysicalSchemaLocation()
  {
    return this.bindType.getPhysicalSchemaLocation();
  }

  public void reset()
  {
    this.rootObject = bindType.createTreeObject( type );
  }

}
