/**
 * 
 */
package ch.ige.st36xmlgenerator.builder;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import ch.ige.chpatdoc.ChPatentDocument;
import ch.ige.st36xmlgenerator.exception.ST36GenerationRuntimeException;
import ch.ige.st36xmlgenerator.format.XMLElementConverter;
import ch.ige.st36xmlgenerator.metadata.XPathXMLAccessor.AttributeDefinition;
import ch.ige.xmlutil.bind.Bindings;

/**
 * Factory of the ST36 Object Tree. 
 * <p>
 * The ST36 object tree is created with the help of the {@link ch.ige.xmlutil.bind.Bindings} class. 
 * @author fabrizioli
 * 
 */
// TODO Builder:  Parts are built and composed
// ContainerObjectBuilder
// TreeObjectBuilder
// LeafObjectBuilder
// AbstractObjectBuilder  : rekursion
public class Factory<X> extends AbstractFactory<X>
{

  private static final Logger LOG = Logger.getLogger( Factory.class );

  private static final Object[] NO_PARAMS = new Object[] {};

  Factory<Object> listFactory;

  private String identifier;


  public static class ST36Factory extends Factory<ChPatentDocument>
  {
    public ST36Factory()
    {
      super( ChPatentDocument.class, "ch-patent-document" );
    }

    public void setRootAttributes( String lang, Date publDate, Date prodDate, Integer schutitKey )
    {
      this.rootObject.setCountry( "CH" );
      this.rootObject.setLang( lang );
      String schemaloc = bindType.getPhysicalSchemaLocation();
      this.rootObject.setDtdVersion( schemaloc.substring( schemaloc.lastIndexOf( "/" ) + 1 ) );
      this.rootObject.setId( "ch_" + schutitKey );
      this.rootObject.setDateProduced( prodDate );
      this.rootObject.setDatePubl( publDate );
      // schema location is set in using the createMarshaller method in the Binding class of xmlutil
    }

    @Override
    public void setRootAttributes( Object[] attrs )
    {
      setRootAttributes( (String) attrs[0], (Date) attrs[1], (Date) attrs[2], (Integer) attrs[3] );
    }
  }

  protected Factory( Class<? extends X> type, String identifier )
  {
    super( Bindings.CHPATDOC, type );
    this.identifier = identifier;
  }

  protected Factory( X rootObject, Class<? extends X> type, String identifier )
  {
    super( Bindings.CHPATDOC, type, rootObject );
    this.identifier = identifier;
  }

  public Factory<Object> getListFactory()
  {
    return listFactory;
  }

  public String getIdentifier()
  {
    return identifier;
  }

  // annahme auf dem Pfad nur ein List element
  public boolean createPathAndSetValue( String xpathFromRoot, Object value, boolean newListElement, Map<String, List<AttributeDefinition>> attributeListMap )
      throws ClassNotFoundException
  {

    // get the root element of the xpath and compare it with the root object
    StringTokenizer tokenizer = new StringTokenizer( xpathFromRoot, "/" );
    String rootElement = tokenizer.nextElement().toString().trim();
    String nextSetterMethod = null; //XMLElementConverter; 
//			.toSetter(rootElement);
    String name = XMLElementConverter.toRootClassName( (String) rootElement );
    if ( rootElement == null || !rootObject.getClass().getSimpleName().startsWith( name ) || !this.identifier.trim().equals( rootElement.trim() ) )
    {
      throw new IllegalArgumentException( rootElement + " is not expected root " + identifier + " or class name  " + rootObject.getClass().getSimpleName()
          + " is not as expected " + name );
    }

    // initialize recursion
    Object navigateToElementObject = rootObject;
    String actualElement = rootElement;
    boolean valueSet = false;
    if ( tokenizer.hasMoreElements() )
    {
      do
      {
        // element in the xpath expression ,used to determine the classname 
        // of the JAXB-Object to create, the setter method to set the object and
        // the getter method to get the object, in case it already exists in the object tree.
        String nextElement = (String) tokenizer.nextElement();
        // matchnames for method 
        nextElement = matchElement( nextElement, actualElement, false );
        Object nextNode = null;
        String nextGetterMethod = XMLElementConverter.toGetter( nextElement );

        // try to get the next object from the actual object, 
        // 1. the next object is a container
        if ( isContainer( nextElement, actualElement ) )
        {
          if ( this.listFactory == null || !this.listFactory.getIdentifier().equals( actualElement ) )
          {
            this.listFactory = new Factory<Object>( navigateToElementObject, navigateToElementObject.getClass(), actualElement );
          }
          nextNode = getContainer( navigateToElementObject, nextGetterMethod );
          if ( nextNode != null )
          {
            actualElement = nextElement;
            navigateToElementObject = nextNode;
            //continue;
          }

        }
        // try to get the next object from the actual object, 
        // 2. the next object is not a container
        else
        {
          Method getter = getMethod( navigateToElementObject.getClass(), nextGetterMethod );

          assert ( getter != null );

          try
          {
            nextNode = getter.invoke( navigateToElementObject, new Object[] {} );
            if ( nextNode != null )
            {
              actualElement = nextElement;
              navigateToElementObject = nextNode;
              continue;
            }
          }
          catch ( IllegalAccessException e )
          {
            LOG.error( e );
          }
          catch ( InvocationTargetException e )
          {
            LOG.error( e );
          }
        }

        // the next object does not exist yet, we have to create it
        nextSetterMethod = XMLElementConverter.toSetter( nextElement );
        nextElement = matchElement( nextElement, actualElement, true );

        Class<?> nextClass = null;
        Object objectToSet = null;

        // if it is a "leaf" JAXB-object, it has a simple classname, 
        // matching exactly the name of the element, except when
        // the type of the "leaf"-object allows multiple occurrence (
        // that is a "choice" or "" type )
        if ( !tokenizer.hasMoreElements() && !multipleOccurrenceIn( nextElement ) )
        {
          nextClass = Class.forName( "ch.ige.chpatdoc." + XMLElementConverter.toSimpleClassName( nextElement ) );
          objectToSet = value;
          valueSet = true; // ?
        }
        else
        {
          // otherwise it may have a classname with "Type" at the and
          // or with "RefType" at the end
          try
          {
            nextClass = Class.forName( "ch.ige.chpatdoc." + XMLElementConverter.toClassName( nextElement ) );
          }
          catch ( Exception e )
          {
            nextClass = Class.forName( "ch.ige.chpatdoc." + XMLElementConverter.toRefClassName( nextElement ) );
          }
          objectToSet = bindType.createTreeObject( nextClass );
        }

        boolean isContainer = isContainer( nextElement, actualElement );

        objectToSet = setObject( nextElement, nextSetterMethod, navigateToElementObject, objectToSet, isContainer, newListElement, attributeListMap
            .get( nextElement ) );
        actualElement = nextElement;
        navigateToElementObject = objectToSet;
      } while ( tokenizer.hasMoreElements() );

      if ( multipleOccurrenceIn( actualElement ) && nextSetterMethod != null )
      {
        setObject( "Content", nextSetterMethod, navigateToElementObject, value, isContainer( "", actualElement ), false, attributeListMap.get( "Content" ) );
        valueSet = true;
      }
      if ( !valueSet )
      {
        throw new IllegalArgumentException( "value " + value + " was not set ." );
      }

      return true;

    }
    return false;
  }

  private void addAttributes( Object target, Class<?> type, List<AttributeDefinition> attributeList, Integer sequenceCounter )
  {
    for ( AttributeDefinition attributeDefinition: attributeList )
    {

      Method attributeSetterMethod = getMethod( type, XMLElementConverter.toSetter( attributeDefinition.getName() ) );
      Object attributeValue = attributeDefinition.getValue();
      if ( ( attributeValue == null || ( (String) attributeValue ).length() == 0 ) && attributeDefinition.getValueType().equals( "integersequence" ) )
      {
        attributeValue = sequenceCounter;
      }
      try
      {
        attributeSetterMethod.invoke( target, new Object[] { attributeValue } );

        LOG.debug( "attribute set on " + type + ":" + attributeDefinition.getName() + "=" + attributeValue );
      }
      catch ( IllegalAccessException e )
      {
        LOG.error( e );
      }
      catch ( InvocationTargetException e )
      {
        LOG.error( e );
      }
    }

  }

  // leaf object with multiple occurrence
  private boolean isSpecialCase( String nextElement )
  {
    if ( nextElement.equals( "invention-title" ) )
    {
      return true;
    }
    return false;
  }

  private Object setObject( String nextElement, String nextSetterMethod, Object navigateToElementObject, Object objectToSet, boolean isContainer,
      boolean newElement, List<AttributeDefinition> attributeList ) throws Error
  {
    if ( isContainer )
    {

      String getterMethod = XMLElementConverter.toGetter( nextElement );
      List<Object> container = null;
      if ( navigateToElementObject instanceof List )
      {
        container = (List) navigateToElementObject;
      }
      else
      {
        container = getContainer( navigateToElementObject, getterMethod );
      }
      if ( container.isEmpty() || newElement )
      {
        if ( attributeList != null )
        {
          addAttributes( objectToSet, objectToSet.getClass(), attributeList, Integer.valueOf( container.size() + 1 ) );
        }
        container.add( objectToSet );
        LOG.debug( "object set in list :" + objectToSet );

      }
      else
      {

        Iterator<Object> iter = container.iterator();
        Object lastElement = null;
        while ( iter.hasNext() )
        {
          lastElement = iter.next();
        }
        return ( lastElement );
      }

    }
    else
    {
      if ( attributeList != null )
      {
        addAttributes( objectToSet, objectToSet.getClass(), attributeList, new Integer( 1 ) );
      }
      Method setter = getMethod( navigateToElementObject.getClass(), nextSetterMethod );
      assert ( setter != null );
      try
      {
        setter.invoke( navigateToElementObject, new Object[] { objectToSet } );

        LOG.debug( "object set :" + objectToSet );
      }
      catch ( IllegalAccessException e )
      {
        LOG.error( e );
        throw new ST36GenerationRuntimeException( "Error in using method " + setter.getName(), e );
      }
      catch ( InvocationTargetException e )
      {
        LOG.error( e );
        throw new ST36GenerationRuntimeException( "Error in using method " + setter.getName(), e );
      }
    }
    return objectToSet;
  }

  private boolean multipleOccurrenceIn( String nextElement )
  {
    if ( nextElement.equals( "invention-title" ) || nextElement.equals( "ch-inventor-entitlement-waiver" ) )
    {
      return true;
    }
    return false;
  }

  private boolean isContainer( String nextElement, String actualElement )
  {
    if ( actualElement.equals( "invention-title" ) || actualElement.equals( "ch-inventor-entitlement-waiver" ) || nextElement.equals( "applicant" )
        || nextElement.equals( "inventor" ) || nextElement.equals( "assignee" ) || nextElement.equals( "agent" ) || nextElement.equals( "priority-claim" )
        || nextElement.equals( "classification-ipcr" ) )
    {
      return true;
    }
    return false;
  }

  private List<Object> getContainer( Object navigateToElementObject, String getterMethodName )
  {
    Method getter = getMethod( navigateToElementObject.getClass(), getterMethodName );
    List<Object> container = null;
    if ( getter != null )
    {
      try
      {
        container = (List) getter.invoke( navigateToElementObject, new Object[] {} );
      }
      catch ( IllegalAccessException e )
      {
        LOG.error( e );
      }
      catch ( InvocationTargetException e )
      {
        LOG.error( e );
      }
    }
    return container;
  }

  private String matchElement( String nextElement, String actualElement, boolean className )
  {
    if ( className && nextElement.equals( "document-id" ) && actualElement.endsWith( "publication-reference" ) )
    {
      return "ch-document-id";
    }
    else if ( className && nextElement.equals( "document-id" ) )
    {
      return "document-id-content";
    }
    else if ( className
        && ( nextElement.equals( "parent-doc" ) || nextElement.equals( "pct-or-regional-filing-data" ) || nextElement
            .equals( "pct-or-regional-publishing-data" ) ) )
    {
      return "document-id";
    }
    else if ( nextElement.equals( "class" ) )
    {
      return "clazz";
    }
    else if ( actualElement.equals( "invention-title" ) )
    {
      return "content";
    }
    else if ( className && nextElement.equals( "address" ) )
    {
      return "address-group";
    }
    return nextElement;
  }

  public Method getMethod( Class<?> providingClass, String methodName ) throws ST36GenerationRuntimeException
  {
    Method[] methods = providingClass.getDeclaredMethods();
    for ( int i = 0; i < methods.length; i++ )
    {
      if ( methods[i].getName().equals( methodName ) )
      {
        return methods[i];
      }
    }
    // look in methods of superclass
    methods = providingClass.getMethods();
    for ( int i = 0; i < methods.length; i++ )
    {
      if ( methods[i].getName().equals( methodName ) )
      {
        return methods[i];
      }
    }
    final String errorMsg = "method '" + methodName + "' not found in " + providingClass.getName();
    LOG.fatal( errorMsg );
    throw new ST36GenerationRuntimeException( errorMsg );
  }

  @Override
  public void reset()
  {
    super.reset();
    this.listFactory = null;
  }

  @Override
  public void setRootAttributes( Object[] par )
  {
    throw new UnsupportedOperationException();

  }

}
