package ch.ige.st36xmlgenerator.format;

import ch.ige.util.enums.IPType;

/**
 * Formatter for entries not ready for the ch-st36 XML when read with IPOBJECTS
 * @author fabrizioli
 *
 */
public class Formatter
{

  /**
   * Format the application number for the ch-st36 XML
   * @param applicationNumber the applicaöltion number as read from BAGIS
   * @param type the type of patent
   * @return the formatted application number
   */
  public static String formatApplicationNumber( String applicationNumber, IPType type )
  {
    if ( applicationNumber != null && type == IPType.PATENT_CH )
    {
      int length = applicationNumber.length();
      if ( length < 4 )
      {
        throw new IllegalArgumentException( "Application number with length > 3 expected: " + applicationNumber );
      }
      StringBuilder buffer = new StringBuilder( length );
      buffer.append( applicationNumber.substring( 0, length - 3 ) );
      buffer.append( '/' );
      buffer.append( applicationNumber.substring( length - 3, length - 1 ) );
      return buffer.toString();
    }
    else
    {
      throw new IllegalArgumentException( "Formatting only for CH Patents, application number: " + applicationNumber );
    }
  }

  /**
   * Format the patent number for the ch-st36 XML
   * @param patentNumber the patent number as read from BAGIS
   * @param type the type of patent
   * @return the formatted patent number
   */
  public static String formatPatentNumber( Long patentNumber, IPType type )
  {
    String patentNumberString = patentNumber.toString();
    if ( patentNumberString != null && type == IPType.PATENT_CH )
    {
      int length = patentNumberString.length();
      if ( length < 5 )
      {
        throw new IllegalArgumentException( "RelatedDocument number with length > 4 expected: " + patentNumberString );
      }
      StringBuilder buffer = new StringBuilder( length );
      buffer.append( patentNumberString.substring( 0, length ) );
      return buffer.toString();
    }
    else
    {
      throw new IllegalArgumentException( "Formatting only for CH Patents, relatedDocument number " + patentNumberString );
    }
  }

  /**
   * Format the PCT application number for the ch-st36 XML
   * @param pctApplicationNumber the PCT application number as read from BAGIS
   * @param type the type of patent
   * @return the formatted application number
   */
  public static String[] formatPCTApplicationNumber( String pctApplicationNumber, IPType type )
  {
    if ( pctApplicationNumber != null && type == IPType.PATENT_CH )
    {
      int length = pctApplicationNumber.length();
      if ( length < 11 )
      {
        throw new IllegalArgumentException( "PCT-Application number with length > 10 expected: " + pctApplicationNumber );
      }
      String[] result = new String[2];
      result[0] = pctApplicationNumber.substring( 0, 2 );

      StringBuilder buffer = new StringBuilder( length - 1 );
      buffer.append( pctApplicationNumber.substring( 2, length - 6 ) );
      buffer.append( '/' );
      buffer.append( pctApplicationNumber.substring( length - 6, length ) );
      result[1] = buffer.toString();
      return result;
    }
    else
    {
      throw new IllegalArgumentException( "Formatting only for CH Patents, pctApplication number " + pctApplicationNumber );
    }
  }

  /**
   * Format the PCT publication number for the ch-st36 XML
   * @param pctPublicationNumber the PCT publication number as read from BAGIS
   * @param type the type of patent
   * @return the formatted publication number
   */
  public static String formatPCTPublicationNumber( String pctPublicationNumber, IPType type )
  {
    if ( pctPublicationNumber != null && type == IPType.PATENT_CH )
    {
      int length = pctPublicationNumber.length();
      if ( length < 7 )
      {
        throw new IllegalArgumentException( "PCT-Publication number with length > 6 expected: " + pctPublicationNumber );
      }

      StringBuilder buffer = new StringBuilder( length + 1 );
      buffer.append( pctPublicationNumber.substring( 0, length - 6 ) );
      buffer.append( '/' );
      buffer.append( pctPublicationNumber.substring( length - 6, length ) );
      return buffer.toString();
    }
    else
    {
      throw new IllegalArgumentException( "Formatting only for CH Patents, pctPublication number " + pctPublicationNumber );
    }
  }
}
