package ch.ige.st36xmlgenerator.format;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import ch.ige.ipobjects.address.vo.SchutitRegadrWithWaiverTypeVO;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.time.visitors.TimePeriodFilter;
import ch.ige.util.enums.address.WaiverType;

/**
 * A filter for filtering the entries in inventors, that waived the publication, from those that are publicated 
 * @author fabrizioli
 *
 */
public class InventorAddressTypeFilter extends TimePeriodFilter<SchutitRegadrWithWaiverTypeVO>
{

  public static final TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> filterSingle( WaiverType waiverType,
      TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> addresses )
  {
    return filterMultiple( Collections.singleton( waiverType ), addresses );
  }

  public static final TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> filterMultiple( Set<WaiverType> waiverTypes,
      TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> addresses )
  {
    return new InventorAddressTypeFilter( waiverTypes ).filter( addresses );
  }

  public static final TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> filterAnyWaiverType( TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> addresses )
  {
    Set<WaiverType> allExceptNone = new HashSet<WaiverType>( WaiverType.ALL );
    allExceptNone.addAll( WaiverType.ONE );
    allExceptNone.addAll( WaiverType.THE_OTHER );
    allExceptNone.addAll( WaiverType.ALL_OTHERS );
    return new InventorAddressTypeFilter( allExceptNone ).filter( addresses );
  }

  private Set<WaiverType> waiverTypes;

  private InventorAddressTypeFilter( Set<WaiverType> waiverTypes )
  {
    super();
    this.waiverTypes = waiverTypes;
  }

  protected boolean shouldInclude( SchutitRegadrWithWaiverTypeVO period )
  {
    return waiverTypes.contains( period.getWaiverType() );
  }

}
