package ch.ige.st36xmlgenerator.metadata;

import java.util.Set;

import org.iso_relax.verifier.Schema;

/**
 * Extracts metadata information about a patent document. Metadata is commonly classified by INID numbers defined in
 * WIPO ST.9
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author fabrizioli
 * @version $Id: XMLGeneratorMetadataExtractor.java,v 1.4 2008/09/25 07:29:27 fabrizioli Exp $
 */
public interface XMLGeneratorMetadataExtractor
{

  /**
   * Gets the numeric identifier of the schema. Refers to the PATDOCSCHEMA table.
   * 
   * @return the identifier
   */
  public Integer getSchemaIdentifier();

  /**
   * Gets the String-Identifier of the schema.
   * 
   * @return the identifier
   */
  public String getStringSchemaIdentifier();

  /**
   *Gets the container of the accessors associated to an inid code and a kind code
   */
  public XMLAccessorContainer getXMLAccessors( Integer inidCode, String kindcode ) throws JAXBMetadataException;

  /**
   * Get the schema for the validation
   * @return
   */
  public Schema getVerifierSchema();

  /**
   * Get all inid codes mapped by accessors
   * @return
   */
  public Set<Integer> getAllInidCodes();

}
