package ch.ige.st36xmlgenerator.metadata;

import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import ch.ige.st36xmlgenerator.builder.Factory;

/**
 * An accessor corresponding to a path in the metadata file
 * <p>
 * immutable object
 * @author fabrizioli
 *
 */
public class XPathXMLAccessor implements XMLAccessor
{

  private static final Logger LOG = Logger.getLogger( XPathXMLAccessor.class );

  private String xpath;

  private List<String> listElements;

  private boolean newListElementStart;

  private Map<String, List<AttributeDefinition>> attributeListMap;

  public static XMLAccessor create( String xpath, List<String> listElements, boolean newListElementStart,
      Map<String, List<AttributeDefinition>> attributeListMap )
  {
    return new XPathXMLAccessor( xpath, listElements, newListElementStart, attributeListMap );
  }

  private XPathXMLAccessor( String xpath, List<String> listElements, boolean newListElementStart, Map<String, List<AttributeDefinition>> attributeListMap )
  {
    super();
    assert xpath != null;
    this.xpath = xpath;
    this.listElements = listElements;
    this.newListElementStart = newListElementStart;
    this.attributeListMap = attributeListMap;
  }


  public boolean createPathAndSetValue( Factory<?> factory, Object[] value ) throws ClassNotFoundException
  {
    return this.createPathAndSetValue( factory, value, (Map[]) null );
  }

  private String adaptPathToListElement( String xpath, String listElement )
  {
    return xpath.substring( xpath.indexOf( listElement ) );
  }

  public String toString()
  {
    return "Accessor(" + this.xpath + ")";
  }

  public boolean createPathAndSetValue( Factory<?> factory, Object value ) throws ClassNotFoundException
  {
    return createPathAndSetValue( factory, new Object[] { value } );
  }

  public void setAttributeMap( Map<String, List<AttributeDefinition>> attributeListMap )
  {
    this.attributeListMap = attributeListMap;
  }

  public static class AttributeDefinition
  {
    private String name;

    private String value;

    private String valueType;

    public AttributeDefinition( String name, String value, String valueType )
    {
      this.name = name;
      this.value = value;
      this.valueType = valueType;
    }

    public AttributeDefinition( String name, String value )
    {
      this.name = name;
      this.value = value;
      this.valueType = value;
    }

    public String getName()
    {
      return name;
    }

    public String getValue()
    {
      return value;
    }

    public String getValueType()
    {
      return valueType;
    }

    public void setValue( String value )
    {
      this.value = value;
    }
    
    public String toString()
    {
      return "[attribute: name = " + name + ", value = " + value + ", valueType = " + valueType + "]" ; 
    }

  }

  public boolean createPathAndSetValue( Factory<?> factory, Object[] values, Map<String, List<AttributeDefinition>>[] attributes ) throws ClassNotFoundException
  {
    assert ( values.length == 1 );
    if ( attributes != null && attributes.length > 0 && attributes[0] != null )
    {
      for ( Map.Entry<String, List<AttributeDefinition>> attributeDef: attributeListMap.entrySet() )
      {
        for ( Map.Entry<String, List<AttributeDefinition>> newAttributeValue: attributes[0].entrySet() )
        {

          if ( attributeDef.getKey().equals( newAttributeValue.getKey() ) )
          {
            attributeDef.setValue( newAttributeValue.getValue() );
          }
        }
      }
    }
    if ( listElements != null && !listElements.isEmpty() )
    {
      Factory<?> listfactory = factory.getListFactory();
      // only one listelement per path expected till now
      if ( listfactory != null && listfactory.getIdentifier().equals( listElements.get( 0 ) ) )
      {
        return listfactory.createPathAndSetValue( adaptPathToListElement( xpath, listElements.get( 0 ) ), values[0], newListElementStart, attributeListMap );
      }
    }
    return factory.createPathAndSetValue( xpath, values[0], newListElementStart, attributeListMap );

  }

  public boolean createPathAndSetValue( Factory<?> factory, Object value, Map<String, List<AttributeDefinition>> attributes ) throws ClassNotFoundException
  {
    return createPathAndSetValue( factory, new Object[] { value }, new Map[] { attributes } );
  }

}
