package ch.ige.st36xmlgenerator.test;

import java.io.File;
import java.io.IOException;
import java.sql.Date;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.common.DetailTerminatedException;
import ch.ige.ipobjects.event.vo.EreignisVO;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.patent.KindCode;
import ch.ige.ipobjects.patent.PatentCH;
import ch.ige.ipobjects.time.BAGISTime;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.st36xmlgenerator.InvalidPatentDocumentException;
import ch.ige.st36xmlgenerator.ST36BiblGenerator;
import ch.ige.st36xmlgenerator.ST36BiblGeneratorFactory;
import ch.ige.st36xmlgenerator.ST36BiblGeneratorImpl;
import ch.ige.st36xmlgenerator.ST36BiblGeneratorProperties;
import ch.ige.st36xmlgenerator.XMLDocumentWithMetadata;
import ch.ige.st36xmlgenerator.exception.ST36GenerationException;

public class ST36BiblTest extends AbstractXMLTestCase
{

  private static final Date DATE_PUBL = BAGISTime.convertIDatToDate( 2008050500 );

  public ST36BiblTest( String name )
  {
    super( name, "C:\\jp\\projects\\st36xmlgenerator\\xml\\test" );
  }

  private static final Integer SCHUTIT_WITH_ADDRESS1 = 1200949936;

  private static final Integer EVENT_SCHUTIT_WITH_ADDRESS1 = 2007113010; // not in Bagis

  private static final KindCode KIND_SCHUTIT_WITH_ADDRESS1 = KindCode.B1;

  private static final Integer SCHUTIT_WITH_ADDRESS2 = 1200642397;

  private static final Integer EVENT_SCHUTIT_WITH_ADDRESS2 = 2007051510; // not in Bagis

  private static final KindCode KIND_SCHUTIT_WITH_ADDRESS2 = KindCode.B1;

  private static final Integer SCHUTIT = 1200858062;

  private static final Integer EVENT_SCHUTIT = 2007091410; // not in Bagis

  private static final KindCode KIND_SCHUTIT = KindCode.B1;

  private static final Integer SCHUTIT_A1 = 1202332074;

  private static final Integer EVENT_SCHUTIT_A1 = 2008071805; // not in Bagis

  private static final Integer SCHUTIT_A2 = 1202318141;

  private static final Integer EVENT_SCHUTIT_A2 = 2008071805; // not in Bagis

  private static final Integer SCHUTIT_A3 = 1202318141;

  private static final Integer EVENT_SCHUTIT_A3 = 2008071810; // not in Bagis

  private static final Integer SCHUTIT_PRIO = 1200961117;

  private static final Integer EVENT_SCHUTIT_PRIO = 2007111510; // not in Bagis

  private static final KindCode KIND_SCHUTIT_PRIO = KindCode.B1;

  private static final Integer SCHUTIT_PCT_APPL = 1201244267;

  private static final Integer EVENT_SCHUTIT_PCT_APPL = 2007113010; // not in Bagis

  private static final KindCode KIND_SCHUTIT_PCT_APPL = KindCode.B1;

  private static final Integer SCHUTIT_PCT_PUBL = 1201436995;

  private static final Integer EVENT_SCHUTIT_PCT_PUBL = 2007111510; // not in Bagis

  private static final KindCode KIND_SCHUTIT_PCT_PUBL = KindCode.B1;

  private static final Integer SCHUTIT_WAIVER_ONED = 1201672882;

  private static final Integer EVENT_SCHUTIT_ONED = 2007083110; // not in Bagis

  private static final KindCode KIND_SCHUTIT_ONED = KindCode.B1;

  private static final Integer SCHUTIT_WAIVER_THEOTHERD = 1200841597;

  private static final Integer EVENT_SCHUTIT_THEOTHERD = 2006121510; // not in Bagis

  private static final KindCode KIND_SCHUTIT_THEOTHERD = KindCode.B1;

  private static final Integer SCHUTIT_WAIVER_ALLD = 1200729911;

  private static final Integer EVENT_SCHUTIT_ALLD = 2006073110; // not in Bagis

  private static final KindCode KIND_SCHUTIT_ALLD = KindCode.B1;

  private static final Integer SCHUTIT_WAIVER_ONEF = 1200923672;

  private static final Integer EVENT_SCHUTIT_ONEF = 2007062910; // not in Bagis

  private static final KindCode KIND_SCHUTIT_ONEF = KindCode.B1;

  private static final Integer SCHUTIT_WAIVER_ALLI = 1201844204;

  private static final Integer EVENT_SCHUTIT_ALLI = 2007091410; // not in Bagis

  private static final KindCode KIND_SCHUTIT_ALLI = KindCode.B1;

  private static final Integer SCHUTIT_DIVISION = 1201470561;

  private static final Integer EVENT_SCHUTIT_DIVISION = 2006063010; // not in Bagis

  private static final KindCode KIND_SCHUTIT_DIVISION = KindCode.B1;

  private static final Integer SCHUTIT_RELATEDDOC = 1200580523;

  private static final Integer EVENT_SCHUTIT_RELATEDDOC = 2000092910; // not in Bagis

  private static final KindCode KIND_SCHUTIT_RELATEDDOC = KindCode.B1;

//  private static final Integer SCHUTIT_INTEGRATION = 1201931207;
//
//  private static final Integer EVENT_SCHUTIT_INTEGRATION = 2008052105;
//
//  private static final Integer SCHUTIT_ABNAHME = 1201940365;
//
//  private static final Integer EVENT_SCHUTIT_ABNAHME = 2008061315;
  private static final Integer SCHUTIT_INTEGRATION = 1201931207;

  private static final Integer EVENT_SCHUTIT_INTEGRATION = 2007043015;

  private static final Integer SCHUTIT_ABNAHME = 1201940365;

  private static final Integer EVENT_SCHUTIT_ABNAHME = 2007053105;

  private static final KindCode KIND_SCHUTIT_ABNAHME = KindCode.B1;

  // Test data for EPA, regadr without PLZ!
  private static final Integer SCHUTIT_C2 = 69494;

  private static final Integer EVENT_SCHUTIT_C2 = 2004081305; // not in Bagis

  private static final KindCode KIND_SCHUTIT_C2 = KindCode.B1;

  private static final Integer SCHUTIT_EVENT_FAILURE = 1200774299;

  private static final Integer EVENT_SCHUTIT_EVENT_FAILURE = 2008022910; //in B AGIS

  private static final KindCode KIND_SCHUTIT_EVENT_FAILURE = KindCode.B1;

  private static final Integer SCHUTIT_FAILURE_ITEST_A1_1 = 1201947986;

  private static final Integer SCHUTIT_FAILURE_ITEST_A1_2 = 1201940365;

  private static final Integer SCHUTIT_ALL_WAIVER = 1202007886;

  private static final Integer EVENT_ALL_WAIVER = 2008071505;

  private static final Integer SCHUTIT_INTEG_TEST2_1 = 1200041106;

  private static final Integer EVENT_SCHUTIT_INTEG2_A1 = 2005063010;

  private static final Integer EVENT_SCHUTIT_INTEG2_B1 = 2007103110;

  private static final Integer SCHUTIT_INTEG_TEST2_2 = 1200807068;

  private static final Integer SCHUTIT_PROD_FEHLER = 1201726259;

  private static final Integer EVENT_FEHLER_PROD = 2008073110;

  // @see junit.framework.TestCase#setUp()
  protected void setUp() throws Exception
  {
    JAXBWellKnownSchemas.initializeMetadataExtractorFactoryFromWellKnownFiles();

  }

  protected void tearDown() throws Exception
  {

  }

  public void testB1() throws Exception
  {
    testAndValidate( SCHUTIT, KIND_SCHUTIT, EVENT_SCHUTIT );
  }

  private void testAndValidate( Integer schutit, KindCode kind, Integer event ) throws ST36GenerationException, DetailTerminatedException,
      InvalidPatentDocumentException, Exception, SAXException, IOException, ParserConfigurationException
  {
    ST36BiblGeneratorImpl generator = (ST36BiblGeneratorImpl) ST36BiblGeneratorFactory.getStandaloneForTest( DATE_PUBL );
    PatentCH patent = ServicesFactory.getStandaloneServices().findCHPatent( IPKeyVO.create( schutit ) );
    XMLDocumentWithMetadata doc = generator.createDocumentForEvent( patent, kind, EreigIDat.from( event ) );
    File folder = new File( this.folderPath + "generated", "CH" + schutit + "_" + kind );
    folder.mkdirs();
    doc.getDocument().saveToUTF8File( new File( folder, schutit + "_" + kind + ".xml" ) );
    generator.validate( doc.getDocument() );
    doc.getDocument().getDOM().normalizeDocument();
    compareWithStored( doc.getDocument().getDOM(), schutit, kind );

    folder.delete();
  }

  private void testAndValidateWithEvent( Integer schutit, KindCode kind, Integer event, boolean failure ) throws ST36GenerationException,
      DetailTerminatedException, InvalidPatentDocumentException, Exception, SAXException, IOException, ParserConfigurationException
  {
    ST36BiblGeneratorImpl generator = (ST36BiblGeneratorImpl) ST36BiblGeneratorFactory.getStandalone();
    PatentCH patent = ServicesFactory.getStandaloneServices().findCHPatent( IPKeyVO.create( schutit ) );
    List<EreignisVO> eventList = new ArrayList<EreignisVO>( 1 );

    eventList.add( (EreignisVO) patent.getEvents().get( EreigIDat.from( event ) ) );
    Map<Integer, List<EreignisVO>> map = new HashMap<Integer, List<EreignisVO>>();
    map.put( schutit, eventList );
    XMLDocumentWithMetadata doc = generator.generateBibliographicData( map, ST36BiblGeneratorProperties.initializeFromDatabase() ).get( schutit ).get( 0 );

    if ( failure )
    {
      assertNotNull( doc.getErrorMsg() );
      assertNull( doc.getDocument() );
    }
    else
    {
      compareWithStored( doc.getDocument().getDOM(), schutit, kind );
    }
  }

  public void testB1WithAddress1() throws Exception
  {
    testAndValidate( SCHUTIT_WITH_ADDRESS1, KIND_SCHUTIT_WITH_ADDRESS1, EVENT_SCHUTIT_WITH_ADDRESS1 );
  }

  public void testB1WithAddress2() throws Exception
  {
    testAndValidate( SCHUTIT_WITH_ADDRESS2, KIND_SCHUTIT_WITH_ADDRESS2, EVENT_SCHUTIT_WITH_ADDRESS2 );
  }

  public void testB1WithPrio() throws Exception
  {
    testAndValidate( SCHUTIT_PRIO, KIND_SCHUTIT_PRIO, EVENT_SCHUTIT_PRIO );
  }

  public void testB1WithPctApp() throws Exception
  {
    testAndValidate( SCHUTIT_PCT_APPL, KIND_SCHUTIT_PCT_APPL, EVENT_SCHUTIT_PCT_APPL );
  }

  public void testB1WithPctPub() throws Exception
  {
    testAndValidate( SCHUTIT_PCT_PUBL, KIND_SCHUTIT_PCT_PUBL, EVENT_SCHUTIT_PCT_PUBL );
  }

  public void testB1WithDivision() throws Exception
  {
    testAndValidate( SCHUTIT_DIVISION, KIND_SCHUTIT_DIVISION, EVENT_SCHUTIT_DIVISION );
  }

  public void testB1WithRelatedDoc() throws Exception
  {
    testAndValidate( SCHUTIT_RELATEDDOC, KIND_SCHUTIT_RELATEDDOC, EVENT_SCHUTIT_RELATEDDOC );
  }

  public void testB1WaiverOneD() throws Exception
  {
    testAndValidate( SCHUTIT_WAIVER_ONED, KIND_SCHUTIT_ONED, EVENT_SCHUTIT_ONED );
  }

  public void testB1WaiverOneF() throws Exception
  {
    testAndValidate( SCHUTIT_WAIVER_ONEF, KIND_SCHUTIT_ONEF, EVENT_SCHUTIT_ONEF );
  }

  public void testB1WaiverTheOtherD() throws Exception
  {
    testAndValidate( SCHUTIT_WAIVER_THEOTHERD, KIND_SCHUTIT_THEOTHERD, EVENT_SCHUTIT_THEOTHERD );
  }

  public void testB1WaiverAllD() throws Exception
  {
    testAndValidate( SCHUTIT_WAIVER_ALLD, KIND_SCHUTIT_ALLD, EVENT_SCHUTIT_ALLD );
  }

  public void testB1WaiverAllI() throws Exception
  {
    testAndValidate( SCHUTIT_WAIVER_ALLI, KIND_SCHUTIT_ALLI, EVENT_SCHUTIT_ALLI );
  }

  public void testA1() throws Exception
  {
    // TODO test data testAndValidate( SCHUTIT_A1, KindCode.A1, EVENT_SCHUTIT_A1 );
  }

  public void testA2() throws Exception
  {
    // TODO test data testAndValidate( SCHUTIT_A2, KindCode.A2, EVENT_SCHUTIT_A2 );
  }

  public void testA3() throws Exception
  {
    // TODO test datatestAndValidate( SCHUTIT_A3, KindCode.A3, EVENT_SCHUTIT_A3 );
  }

  public void testC2_B1() throws Exception
  {
    testAndValidate( SCHUTIT_C2, KindCode.B1, EVENT_SCHUTIT_C2 );
  }

  public void testAllWaiver_A1() throws Exception
  {
    // TODO test datatestAndValidate( SCHUTIT_ALL_WAIVER, KindCode.A1, EVENT_ALL_WAIVER );
  }

  public void testIntegTest2_1_B1() throws Exception
  {
    testAndValidate( SCHUTIT_INTEG_TEST2_1, KindCode.B1, EVENT_SCHUTIT_INTEG2_B1 );
  }

  public void testIntegTest2_2_B1() throws Exception
  {
    testAndValidate( SCHUTIT_INTEG_TEST2_2, KindCode.B1, EVENT_SCHUTIT_INTEG2_B1 );
  }

  public void testEventFailure_B1() throws Exception
  {
    testAndValidateWithEvent( SCHUTIT_EVENT_FAILURE, KindCode.B1, EVENT_SCHUTIT_EVENT_FAILURE, true );
  }

  public void testFailureITest1_A1() throws Exception
  {
    try
    {
      testAndValidate( SCHUTIT_FAILURE_ITEST_A1_1, KindCode.A1, 2007052130 );
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expecged type " + e, e instanceof ST36GenerationException );
    }
  }

  public void testFailureITest2_A1() throws Exception
  {
    try
    {
      testAndValidate( SCHUTIT_FAILURE_ITEST_A1_2, KindCode.A1, 2007053105 );
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expecged type " + e, e instanceof ST36GenerationException );
    }
  }

  public void testFailure() throws Exception
  {
    try
    {
      testAndValidate( SCHUTIT_ABNAHME, KindCode.A1, EVENT_SCHUTIT_ABNAHME );
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expecged type " + e, e instanceof ST36GenerationException );
    }
  }

  public void testEventInt_Erfinder_Abgschlossen() throws Exception
  {
    try
    {
      testAndValidate( SCHUTIT_INTEGRATION, KindCode.A1, EVENT_SCHUTIT_INTEGRATION );
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expecged type " + e, e instanceof ST36GenerationException );
    }
  }

  public void testSelectEvents() throws Exception
  {
    ST36BiblGenerator generator = ST36BiblGeneratorFactory.getStandalone();
    ST36BiblGeneratorProperties props = ST36BiblGeneratorProperties.initializeFromDatabase();
    Map<Integer, List<EreignisVO>> events = generator.getEventsToPublicate( props );
    assertTrue( "No events found", !events.isEmpty() );

  }

  // TODO test that properties contain st36 in path

}
