package ch.ige.ses.client.batch.test;

import java.sql.*;
import java.util.*;
import java.sql.Date;

import org.apache.log4j.BasicConfigurator;

import ch.ige.ses.client.batch.BatchBasis;
import ch.ige.ses.server.dao.ses.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.server.batchctrl.BatchDelegate;
import ch.ige.util.client.batch.AbstractBatch;
import ch.ige.util.server.DBHelper;
import junit.framework.*;

/**
* Testklasse für die Klasse SesReorganisation
* <p>
* Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum</p>
* @author T. Iten
* @version $Id: SesReorganisationTest.java,v 1.11 2008/03/04 07:10:52 iten Exp $
*/
public class SesReorganisationTest extends TestCase
{

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public SesReorganisationTest(String s)
  {
    super(s);
    
    // init log4j
    BasicConfigurator.configure();

    // init test as standalone client
    DBHelper.setRunOnServer(false);    
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(SesReorganisationTest.class);
  }

  protected void setUp() throws Exception
  {
    // lazy bone
  }

  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void _testJgbLauf()
  {
    try
    {
      JGBLaufDAO  dao = new JGBLaufDAO();
      JGBLaufVO   vo  = null;
      String      buf = null;

      if (dao.select("2004010199") == null)
        insertJgbLauf("2004010199", JGBLaufVO.SAP_STAT_FINISHED, 1);

      if (dao.select("2004020199") == null)
        insertJgbLauf("2004020199", JGBLaufVO.SAP_STAT_FINISHED, 2);

      if (dao.select("2004030199") == null)
        insertJgbLauf("2004030199", JGBLaufVO.SAP_STAT_FINISHED, 3);

      if (dao.select("2004040199") == null)
        insertJgbLauf("2004040199", JGBLaufVO.SAP_STAT_FINISHED, 4);

      if (dao.select("2004050199") == null)
        insertJgbLauf("2004050199", JGBLaufVO.SAP_STAT_FINISHED, 5);

      if (dao.select("2004060199") == null)
        insertJgbLauf("2004060199", JGBLaufVO.SAP_STAT_FINISHED, 6);

      if (dao.select("2003123199") == null)
        insertJgbLauf("2003123199", JGBLaufVO.SAP_STAT_STOPPED, 12);

      buf = BatchDelegate.getInstance().reorgDatabase("jgb_lauf",3);
      System.out.println(buf);

      vo = dao.select("2004010199");
      dao.delete(vo.jgbLaufId);
      vo = dao.select("2004020199");
      dao.delete(vo.jgbLaufId);
      vo = dao.select("2004030199");
      dao.delete(vo.jgbLaufId);
      vo = dao.select("2003123199");
      dao.delete(vo.jgbLaufId);

      assertTrue(true);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void _testAmvLauf()
  {
    try
    {
      AmvLaufDAO  dao = new AmvLaufDAO();
      AmvLaufVO   vo  = null;
      String      buf = null;


      if (dao.select("2004010199") == null)
        insertAmvLauf("2004010199", AmvLaufVO.LAUF_STAT_FINISHED, 1);

      if (dao.select("2004020199") == null)
        insertAmvLauf("2004020199", AmvLaufVO.LAUF_STAT_FINISHED, 2);

      if (dao.select("2004030199") == null)
      insertAmvLauf("2004030199", AmvLaufVO.LAUF_STAT_FINISHED, 3);

      if (dao.select("2004040199") == null)
        insertAmvLauf("2004040199", AmvLaufVO.LAUF_STAT_FINISHED, 4);

      if (dao.select("2004050199") == null)
        insertAmvLauf("2004050199", AmvLaufVO.LAUF_STAT_FINISHED, 5);

      if (dao.select("2004060199") == null)
        insertAmvLauf("2004060199", AmvLaufVO.LAUF_STAT_FINISHED, 6);

      if (dao.select("2003123199") == null)
        insertAmvLauf("2003123199", AmvLaufVO.LAUF_STAT_STOPPED, 12);


      buf = BatchDelegate.getInstance().reorgDatabase("amv_lauf",3);
      System.out.println(buf);

      vo = dao.select("2004010199");
      dao.delete(vo.amvLaufId);
      vo = dao.select("2004020199");
      dao.delete(vo.amvLaufId);
      vo = dao.select("2004030199");
      dao.delete(vo.amvLaufId);
      vo = dao.select("2003123199");
      dao.delete(vo.amvLaufId);

      assertTrue(true);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void testStakoKontrolle()
  {
    try
    {
      StakoKontrolleDAO dao       = new StakoKontrolleDAO();
      StakoKontrolleVO  voList[]  = new StakoKontrolleVO[4];
      String            buf       = null;


      insertStako(StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED, 1);
      voList[0] = dao.selectMaxKontrollId();

      insertStako(StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED, 2);
      voList[1] = dao.selectMaxKontrollId();

      insertStako(StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED, 3);
      voList[2] = dao.selectMaxKontrollId();

      insertStako(StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED, 4);
      insertStako(StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED, 5);
      insertStako(StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED, 6);

      insertStako(StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED_WITH_ERRORS, 12);
      voList[3] = dao.selectMaxKontrollId();


      buf = BatchDelegate.getInstance().reorgDatabase("stako_kontrolle",3);
      System.out.println(buf);

      dao.delete(voList[0].stakoKontrolleId);
      dao.delete(voList[1].stakoKontrolleId);
      dao.delete(voList[2].stakoKontrolleId);
      dao.delete(voList[3].stakoKontrolleId);

      assertTrue(true);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void testTrace()
  {
    try
    {
      TraceDAO  dao = new TraceDAO();
      TraceDAO  vo  = null;
      String    buf = null;

      insertTrace(1,1);
      insertTrace(2,2);
      insertTrace(3,3);
      insertTrace(4,4);
      insertTrace(5,5);
      insertTrace(6,6);

      buf = BatchDelegate.getInstance().reorgDatabase("trace",3);
      System.out.println(buf);

      buf = BatchDelegate.getInstance().reorgDatabase("trace",0);
      System.out.println(buf);

      assertTrue(true);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private void insertStako(int stakoStat, int month) throws Exception
  {
    StakoKontrolleVO   vo  = new StakoKontrolleVO();
    StakoKontrolleDAO  dao = new StakoKontrolleDAO();

    GregorianCalendar   today   = new GregorianCalendar();
    GregorianCalendar   created = new GregorianCalendar();

    // init
    created.set(Calendar.MONTH, today.get(Calendar.MONTH) - month);
    vo.stakoStat       = stakoStat;
    vo.verarbeitungDat = new java.sql.Date(created.getTimeInMillis());
    vo.vonDat          = new java.sql.Date(created.getTimeInMillis());
    vo.bisDat          = new java.sql.Date(created.getTimeInMillis());

    // create entry
    dao.insert(vo);
    vo = dao.selectMaxKontrollId();
    dao.updateCreatedDat(vo.stakoKontrolleId,created.getTime());
  }


  private void insertAmvLauf(String laufNr, int laufStat, int month) throws Exception
  {
    AmvLaufVO   vo  = new AmvLaufVO();
    AmvLaufDAO  dao = new AmvLaufDAO();

    GregorianCalendar   today   = new GregorianCalendar();
    GregorianCalendar   created = new GregorianCalendar();

    // init
    created.set(Calendar.MONTH, today.get(Calendar.MONTH) - month);
    vo.laufNr    = laufNr;
    vo.laufStat  = laufStat;

    // create entry
    dao.insert(vo);
    vo = dao.select(vo.laufNr);
    dao.updateCreatedDat(vo.amvLaufId,created.getTime());
  }


  private void insertJgbLauf(String laufNr, int sapStat, int month) throws Exception
  {
    JGBLaufVO   vo  = new JGBLaufVO();
    JGBLaufDAO  dao = new JGBLaufDAO();

    GregorianCalendar   today   = new GregorianCalendar();
    GregorianCalendar   created = new GregorianCalendar();

    // init
    created.set(Calendar.MONTH, today.get(Calendar.MONTH) - month);
    vo.laufNr    = laufNr;
    vo.sapStat   = sapStat;

    // create entry
    dao.insert(vo);
    vo = dao.select(vo.laufNr);
    dao.updateCreatedDat(vo.jgbLaufId,created.getTime());
  }

  private void insertTrace(int nr, int month) throws Exception
  {
    TraceVO   vo  = new TraceVO();
    TraceDAO  dao = new TraceDAO();

    GregorianCalendar   today   = new GregorianCalendar();
    GregorianCalendar   created = new GregorianCalendar();

    // init
    created.set(Calendar.MONTH, today.get(Calendar.MONTH) - month);
    vo.process   =  "SesReorgTest";
    vo.processNr = "99";
    vo.action    = "insert testrecord #" + nr;
    vo.data      = "some data ...";

    // create entry
    dao.insert(vo);
    vo.traceId = dao.selectMaxTraceId();
    dao.updateCreatedDat(vo.traceId,created.getTime());
  }

  //***************************************************************************
  // main
  //***************************************************************************

  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }


} // end class


