package ch.ige.ses.client.sapinbound;

import java.util.*;
import com.sap.mw.jco.*;
import ch.ige.ses.client.saputil.*;

/**
 * Basisklasse für alle SAP Remote Function Module Klassen.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: RFM.java,v 1.10 2006/12/11 12:43:32 iten Exp $
 */
public class RFM
{
  //
  // fields
  //
  protected String            fRfmName;            // sap rfm name
  protected IFunctionTemplate fJCoFuncTempl;       // jco function template
  protected JCO.Function      fJCoFunc;            // jco function


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public RFM(String rfmName, IRepository repository)
  {
    this.fRfmName      = rfmName;
    this.fJCoFuncTempl = repository.getFunctionTemplate(rfmName);
    this.fJCoFunc      = new JCO.Function(fJCoFuncTempl);
  }


  //---------------------------------------------------------------------------
  // methods for table return / type bapiret2
  //---------------------------------------------------------------------------

  public boolean isReturnOk()
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable("RETURN");
    String    buf;

    if (tbl.getNumRows() > 0)
    {
      do
      {
        buf = tbl.getString("TYPE");

        if ( ! checkReturnOkCode(buf) )
        {
          return false;
        }

      } while(tbl.nextRow());
    }

    return true;
  }


  public String toStringReturn()
  {
    JCO.Table    tbl = this.fJCoFunc.getTableParameterList().getTable("RETURN");
    StringBuffer buf = new StringBuffer();

    buf.append("TYPE ID NUMBER MESSAGE \n");
    if (tbl.getNumRows() > 0)
    {
      do
        buf.append ( Convert.toSapStr(tbl.getString("TYPE"),5) +
                     Convert.toSapStr(tbl.getString("ID"),3) +
                     Convert.toSapStr(tbl.getString("NUMBER"),7) +
                     tbl.getString("MESSAGE") + "\n" );
      while(tbl.nextRow());

    }
    else
    {
      buf.append("no results found");
    }

    return buf.toString();
  }


  public String toStringReturnNOK()
  {
    JCO.Table    tbl = this.fJCoFunc.getTableParameterList().getTable("RETURN");
    StringBuffer buf = new StringBuffer();
    String       type;

    buf.append("TYPE ID NUMBER MESSAGE \n");
    if (tbl.getNumRows() > 0)
    {
      do
      {
        type = tbl.getString("TYPE");

        if ( ! checkReturnOkCode(type) )
        {
          buf.append ( Convert.toSapStr(tbl.getString("TYPE"),5) +
                       Convert.toSapStr(tbl.getString("ID"),3) +
                       Convert.toSapStr(tbl.getString("NUMBER"),7) +
                       tbl.getString("MESSAGE") + "\n" );
        }

      } while(tbl.nextRow());

    }
    else
    {
      buf.append("no results found");
    }

    return buf.toString();
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private boolean checkReturnOkCode (String code)
  {
    String          buf = SapInbound.getProperties().getProperty("sap.rfm.returncode.ok");
    StringTokenizer st  = new StringTokenizer(buf, ",");
    String          tag = null;

    while( st.hasMoreElements() )
    {
      // get singel tag's to check
      tag = st.nextElement().toString();
      if ( tag.equals("BLANK") )
      {
        tag = "";
      }

      // do check
      if ( code.equals(tag) )
      {
        return true;
      }
    }

    return false;
  }

} // end class


