package ch.ige.ses.client.sapinbound;

import java.util.*;
import org.apache.log4j.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.util.vo.*;
import com.sap.mw.jco.*;

/**
 * SAP Remote Function Module BAPI_CUSTOMER_CHECK
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author Thomas Iten
 * @version $Id: RFM_Z_BAPI_CUSTOMER_CHECK.java,v 1.6 2006/12/11 12:43:32 iten Exp $
 */
public class RFM_Z_BAPI_CUSTOMER_CHECK extends RFM
{
  //
  // const
  //
  private static final Logger  LOG = Logger.getLogger(RFM_BAPI_CUSTOMER_CHECKEXISTENCE.class);

  //
  // fields
  //
  public  String  fNextStep;          // next step in process (trace info)

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public RFM_Z_BAPI_CUSTOMER_CHECK (IRepository repository)
  {
    super("Z_BAPI_CUSTOMER_CHECK",repository);
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void addVersadrList(ArrayList list)
  {
    for (Iterator iter = list.iterator(); iter.hasNext(); )
    {
      addVersadr( (VersadrCheckVO)iter.next() );
    }
  }


  public void addVersadr(VersadrCheckVO vo)
  {
    try
    {
      setCustomer(vo);
    }
    catch (Exception ex)
    {
      String buf = "addVersadr - exception: " + ex.toString() + "\n" +
                   "hint (adrNr): " + vo.adrNr + "\n" +
                   " hint (next step): " + fNextStep + "\n";
      LOG.error(buf);
      throw new RfmException(buf);
    }
  }


  public void checkVersadrList(ArrayList list)
  {
    try
    {
      checkCustomerList(list);
    }
    catch (Exception ex)
    {
      String buf = "checkVersadrList - exception: " + ex.toString();
      LOG.error(buf);
      throw new RfmException(buf);
    }
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------


  private void setCustomer(VersadrCheckVO vo)
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable("CUSTOMER");

    tbl.appendRow();

    fNextStep = "setCustomer - CUSTOMER";
    tbl.setValue(Convert.toSapNr(vo.adrNr,10),"CUSTOMER");

    fNextStep = "setCustomer - COMP_CODE";
    tbl.setValue(vo.compCode,"COMP_CODE");

    fNextStep = "setCustomer - SALESORG";
    tbl.setValue(vo.salesOrg,"SALESORG");

    fNextStep = "setCustomer - DISTR_CHAN";
    tbl.setValue(vo.distrChan,"DISTR_CHAN");

    fNextStep = "setCustomer - DIVISION";
    tbl.setValue(vo.distrChan,"DIVISION");
  }


  private void checkCustomerList(ArrayList al) throws Exception
  {
    JCO.Table       tbl = this.fJCoFunc.getTableParameterList().getTable("CUSTOMER");
    VersadrCheckVO  vo  = null;

    if ( (tbl.getNumRows() > 0) &&                  // valid return rows
         (tbl.getNumRows() == al.size()) )          // equal output/input count
    {
      al.clear();

      do
      {
        // init vo
        vo = new VersadrCheckVO();

        // get data
        vo.adrNr = tbl.getInt("CUSTOMER");

        if ( tbl.getString("CHECK_OK").equals("X") )
        {
          vo.openOrder = new Boolean(false);
          vo.openOrderMsg = VersadrCheckVO.OPEN_ORDER_FALSE_MSG;
        }
        else
        {
          vo.openOrder = new Boolean(true);
          vo.openOrderMsg = getOpenOrderMessage(vo.adrNr);
        }

        // add to result list
        al.add(vo);

      } while(tbl.nextRow());
    }
    else
    {
      throw new Exception
       ( "checkCustomerList - Input list size (" + al.size() + ") " +
         "is not eaqual to output list size (" + tbl.getNumRows() + ") " +
         "or output list size is empty" );
    }
  }


  private String getOpenOrderMessage(int adrNr)
  {
    StringBuffer buf = new StringBuffer();

    try
    {
      buf.append(VersadrCheckVO.OPEN_ORDER_TRUE_MSG);

      JCO.Table    tbl = this.fJCoFunc.getTableParameterList().getTable("RETURN");

      if (tbl.getNumRows() > 0)
      {
        do
          if ( tbl.getString("TYPE").equals("I") &&
               tbl.getString("PARAMETER").equals(Convert.toSapNr(adrNr,10)) )
          {
            buf.append(tbl.getString("MESSAGE"));
          }
        while(tbl.nextRow());
      }
    }
    catch (Exception ex)
    {
      LOG.warn("getOpenOrderMessage - Unerwarteter Fehler, Exception: " + ex.getMessage());
    }

    return buf.toString();
  }

} // end class


