package ch.ige.ses.client.sapinbound;

import java.math.BigDecimal;
import java.util.Iterator;
import java.util.Properties;

import org.apache.log4j.Logger;

import ch.ige.ses.client.saputil.Convert;
import ch.ige.ses.client.saputil.RfmException;
import ch.ige.ses.client.saputil.SapProperties;
import ch.ige.ses.client.saputil.SapText;
import ch.ige.ses.client.saputil.SapTextVO;
import ch.ige.ses.vo.bagis.SAPJGBParamVO;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.ses.vo.bagis.SAPRechnungsPositionVO;

import com.sap.mw.jco.IRepository;
import com.sap.mw.jco.JCO;

/**
 * SAP Remote Function Module Z_BAPI_ZVL_CREATE_BILLING_REQ
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ.java,v 1.46 2008/01/16 10:49:06 iten Exp $
 */
public class RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ extends RFM
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger( RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ.class );

  public static final int DEF_PRINTER_SETTING = 0; // printer settings:

  public static final String DEF_PRINTER_NAME = "BHRLP"; //   0 = do not set printername

  //   1 = use printername from bagis
  //   2 = use default printername
  //
  // fields
  //
  private boolean fShowArtText = false; // set by constructor

  private boolean fShowArtZusatzText = true;

  private int fPrinterSetting = DEF_PRINTER_SETTING;

  private String fPrinterDefName = DEF_PRINTER_NAME;

  private boolean fImmediatePrint = true;

  private String fEnvName = "";

  private boolean fImportParamIsSet = false; // set by setImportParam

  public String fNextStep = ""; // next step in process (trace info)

  private StringBuffer fLogDetail = null; // log details

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ( IRepository repository )
  {
    super( "Z_BAPI_ZVL_CREATE_BILLING_REQ", repository );

    // get property options
    SapProperties sp = new SapProperties();
    Properties prop = SapInbound.getProperties();

    if ( prop.getProperty( "sap.rechnung.pos.showarttext" ).equals( "1" ) )
      this.fShowArtText = true;
    else
      this.fShowArtText = false;

    if ( prop.getProperty( "sap.rechnung.pos.showartzusatztext" ).equals( "1" ) )
      this.fShowArtZusatzText = true;
    else
      this.fShowArtZusatzText = false;

    // printer settings
    this.fPrinterSetting = sp.getInt( prop, "sap.rechnung.printer.setting", this.DEF_PRINTER_SETTING );
    if ( this.fPrinterSetting == 2 )
      fPrinterDefName = prop.getProperty( "sap.rechnung.printer.defaultname", this.DEF_PRINTER_NAME );

    // immediate print behavior
    if ( prop.getProperty( "sap.rechnung.immediate_print" ).equals( "1" ) )
      this.fImmediatePrint = true;
    else
      this.fImmediatePrint = false;

    // environment name
    fEnvName = prop.getProperty( "env.name", "" );
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void addRechnung( SAPRechnungVO re )
  {
    try
    {
      // init
      fLogDetail = new StringBuffer(); // log details
      fLogDetail.append( "Details: bfakt_id=" + re.bfakt_id + " " );

      // set vaules
      if ( !this.fImportParamIsSet ) // set only once per call
        this.setImportParam( re );

      this.setHeader( re );
      this.setItem( re );
      this.setPartner( re );
      this.setConditions( re );
      this.setText( re );

      // log details
      LOG.debug( fLogDetail.toString() );

    }
    catch ( Exception ex )
    {
      LOG.error( "exception: " + ex.toString() );
      LOG.error( "hint (next step): " + fNextStep );
      throw new RfmException( ex.toString() );
    }
  }

  public String toStringRechnung()
  {
    String[] tn = { "HEADER", "ITEM", "PARTNER", "CONDITIONS", "TEXT" };
    JCO.ParameterList in = this.fJCoFunc.getImportParameterList();
    StringBuffer buf = new StringBuffer();

    // import params
    buf.append( in.toString() );

    // tables
    for ( int i = 0; i < tn.length; i++ )
    {
      JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable( tn[i] );
      buf.append( tbl.toString() );
    }

    return buf.toString();
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------
  private void setImportParam( SAPRechnungVO re )
  {
    JCO.ParameterList input = this.fJCoFunc.getImportParameterList();

    fNextStep = "setImportParam - DOC_TYPE";
    input.setValue( "ZVL", "DOC_TYPE" );

    fNextStep = "setImportParam - SALES_ORG";
    input.setValue( "1000", "SALES_ORG" );

    fNextStep = "setImportParam - DISTR_CHAN";
    input.setValue( "HO", "DISTR_CHAN" );

    fNextStep = "setImportParam - DIVISION";
    input.setValue( "00", "DIVISION" );

    fNextStep = "setImportParam - CREATED_BY / re.employeeId";
    if ( re.employeeId != null )
      input.setValue( re.employeeId, "CREATED_BY" );

    fNextStep = "setImportParam - TESTRUN";
    input.setValue( " ", "TESTRUN" );

    fNextStep = "setImportParam - COMMIT";
    input.setValue( "X", "COMMIT" ); // enable db commit

    this.fImportParamIsSet = true;
  }

  private void setHeader( SAPRechnungVO re )
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable( "HEADER" );

    tbl.appendRow();

    fNextStep = "setHeader - DOC_NUMBER / re.bfakt_id";
    tbl.setValue( Convert.toSapDocNumber( re.bfakt_id ), "DOC_NUMBER" );

    fNextStep = "setHeader - BILL_DATE / re.bfakt_dat";
    tbl.setValue( Convert.toSapBillDate( re.bfakt_dat ), "BILL_DATE" );

    fNextStep = "setHeader - BILL_BLOCK / re.regulierer";
    tbl.setValue( Convert.toSapBillBlock( re ), "BILL_BLOCK" );

    fNextStep = "setHeader - PROT_TITLE / re.schutit_nr";
    if ( re.schutit_nr != null )
      tbl.setValue( Convert.toSapStr( re.schutit_nr, 20 ), "PROT_TITLE" );

    fNextStep = "setHeader - PURCHASER / re.kontaktPerson";
    if ( re.kontaktPerson != null )
      tbl.setValue( re.kontaktPerson, "PURCHASER" );

    fNextStep = "setHeader - DEPARTMENT / re.abteilung & env.name(e,i,a)";
    if ( SapInbound.isProduction() )
    {
      if ( re.abteilung != null )
        tbl.setValue( re.abteilung, "DEPARTMENT" );
    }
    else
    {
      if ( re.abteilung != null )
      {
        tbl.setValue( re.abteilung + " <B>" + fEnvName.toUpperCase() + "</>", "DEPARTMENT" );
      }
      else
      {
        tbl.setValue( "<B>" + fEnvName.toUpperCase() + "</>", "DEPARTMENT" );
      }
    }

    fNextStep = "setHeader - OUR_REF / re.unsere_ref";
    if ( re.unsere_ref != null )
      tbl.setValue( re.unsere_ref, "OUR_REF" );

    fNextStep = "setHeader - TELEPHONE / re.tel_nr";
    if ( re.tel_nr != null )
      tbl.setValue( re.tel_nr, "TELEPHONE" );

    fNextStep = "setHeader - REG_MAIL / re.kz_einschr";
    tbl.setValue( Convert.toSapRegMail( re.kz_einschr ), "REG_MAIL" );

    fNextStep = "setHeader - DUE_DATE / re.verfall_dat";
    if ( re.verfall_dat != null )
      tbl.setValue( re.verfall_dat, "DUE_DATE" );

    fNextStep = "setHeader - PROT_HOLDER / re.inhaber";
    if ( re.inhaber != null )
      tbl.setValue( re.inhaber, "PROT_HOLDER" );

    fNextStep = "setHeader - PROT_TYPE / re.schutit_typ";
    tbl.setValue( re.schutit_typ, "PROT_TYPE" );

    fNextStep = "setHeader - PROT_TYPE_D / re.schutit_typ_bez";
    if ( re.schutit_typ_bez != null )
      tbl.setValue( re.schutit_typ_bez, "PROT_TYPE_D" );

    fNextStep = "setHeader - YOUR_REF / re.ihr_zeichen";
    if ( re.ihr_zeichen != null )
      tbl.setValue( re.ihr_zeichen, "YOUR_REF" );

    fNextStep = "setHeader - VAR_TEXT / re.var_text";
    if ( re.var_text != null )
      tbl.setValue( re.var_text, "VAR_TEXT" );

    if ( re.isJGB() )
    {
      fNextStep = "setHeader - ANN_FEE_SEQ_NO / re.jgbLaufNr";
      tbl.setValue( re.jgbLaufNr, "ANN_FEE_SEQ_NO" );
    }

    if ( re.isAMV() )
    {
      fNextStep = "setHeader - TM_EXT_SEQ_NO / re.amvLaufNr";
      tbl.setValue( re.amvLaufNr, "TM_EXT_SEQ_NO" );
    }
  }

  private void setItem( SAPRechnungVO re )
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable( "ITEM" );
    Iterator it = re.posList.iterator();
    SAPRechnungsPositionVO pos = null;
    SapText st = null;

    fLogDetail.append( "art_nr=" );

    if ( !it.hasNext() )
    {
      throw new RfmException( "setItem - re.posList enthält keine Elemente !" );
    }

    while ( it.hasNext() )
    {
      pos = (SAPRechnungsPositionVO) it.next();
      fLogDetail.append( pos.art_nr + "," );

      tbl.appendRow();

      fNextStep = "setItem - DOC_NUMBER / re.bfakt_id";
      tbl.setValue( Convert.toSapDocNumber( re.bfakt_id ), "DOC_NUMBER" );

      fNextStep = "setItem - ITM_NUMBER / pos.bfakt_posnr";
      tbl.setValue( Convert.toSapItmNumber( pos.bfakt_posnr ), "ITM_NUMBER" );

      fNextStep = "setItem - MATERIAL / pos.art_nr";
      tbl.setValue( Convert.toSapMaterial( pos.art_nr ), "MATERIAL" );

      fNextStep = "setItem - SHORT_TEXT / getArtikelPosText,getSplitTextHeader";
      st = new SapText( getArtikelPosText( pos ) ); // important: SapText
      if ( st.getSplitTextHeader() != null ) // must be new object !
      {
        tbl.setValue( st.getSplitTextHeader(), "SHORT_TEXT" );
      }

      fNextStep = "setItem - TARGET_QTY / pos.anzahl";
      tbl.setValue( pos.anzahl, "TARGET_QTY" );

      fNextStep = "setItem - TARGET_QU / pos.lageinh";
      tbl.setValue( Convert.toSapTargetQu( pos.lageinh ), "TARGET_QU" );
    }

    fLogDetail.deleteCharAt( fLogDetail.length() - 1 );
    fLogDetail.append( " " );
  }

  private void setPartner( SAPRechnungVO re )
  {
    Integer zre = null;
    SAPJGBParamVO jp = null;

    //
    // AG - Auftraggeber
    //
    fNextStep = "setPartner - AG / re.auftraggeber";
    if ( re.auftraggeber > 0 )
    {
      setPartnerEntry( re, "AG", re.auftraggeber, null );
    }
    else
    {
      // Auftraggeber darf nicht 0 sein, falls nicht vorhanden
      // wird der hauptRechnEmpf als Auftraggeber gesetzt
      setPartnerEntry( re, "AG", re.hauptRechEmpf, null );
    }

    //
    // RE - Hauptrechnungsempfänger
    //
    fNextStep = "setPartner - RE / re.hauptRechEmpf";
    setPartnerEntry( re, "RE", re.hauptRechEmpf, re.jgbHauptRechEmpfParam );
    setPartnerEntryCopy( re, "RE", re.hauptRechEmpf, re.jgbHauptRechEmpfParam );

    //
    // RG - Regulierer
    //
    fNextStep = "setPartner - RG / re.regulierer oder re.hauptRechEmpf";
    if ( re.regulierer > 0 )
    {
      setPartnerEntry( re, "RG", re.regulierer, null );
    }
    else
    {
      // Regulierer darf nicht 0 sein, falls nicht vorhanden
      // wird der hauptRechnEmpf als Regulierer gesetzt
      setPartnerEntry( re, "RG", re.hauptRechEmpf, null );
    }

    //
    // R0..R9 & Z0..Z9 - Zusatzrechnungsempfänger
    //
    if ( re.zusatzRechEmpf != null )
    {
      fNextStep = "setPartner - R0..R9 & Z0..Z9 / re.zusatzRechEmpf";

      // check maximal allowed number of partners
      if ( re.zusatzRechEmpf.size() > SAPRechnungVO.MAX_RECH_EMPF )
      {
        throw new RfmException( "setPartner - re.zusatzRechEmpf überschreitet max. Anzahl zulässige Elemente !" );
      }

      // set partner
      for ( int i = 0; i < re.zusatzRechEmpf.size(); i++ )
      {
        zre = (Integer) re.zusatzRechEmpf.get( i );
        if ( re.isJGB() )
          jp = (SAPJGBParamVO) re.jgbZusatzRechEmpfParam.get( i );

        setPartnerEntry( re, Convert.toSapPartnerRole( i ), zre.intValue(), jp );
        setPartnerEntryCopy( re, Convert.toSapPartnerRole( i ), zre.intValue(), jp );
      }
    } // end if
  }

  private void setPartnerEntry( SAPRechnungVO re, String partnerRole, int partnerNumb, SAPJGBParamVO jp )
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable( "PARTNER" );
    tbl.appendRow();
    fLogDetail.append( partnerRole + "=" + partnerNumb + " " );

    //
    // Mandantory Fields
    //
    fNextStep = "setPartnerEntry - DOC_NUMBER / re.bfakt_id";
    tbl.setValue( Convert.toSapDocNumber( re.bfakt_id ), "DOC_NUMBER" );

    fNextStep = "setPartnerEntry - ITM_NUMBER";
    tbl.setValue( Convert.toSapItmNumber( 0 ), "ITM_NUMBER" );

    fNextStep = "setPartnerEntry - PARTN_ROLE / " + partnerRole;
    tbl.setValue( partnerRole, "PARTN_ROLE" );

    fNextStep = "setPartnerEntry - PARTN_NUMB";
    tbl.setValue( Convert.toSapNr( partnerNumb, 10 ), "PARTN_NUMB" );

    // Nachrichten (weitere Felder) nicht setzen für AG & RG
    // da kein Ausdruck der Rechnung
    if ( partnerRole.equals( "AG" ) || partnerRole.equals( "RG" ) )
    {
      return;
    }

    //
    // Optional Fields
    //
    fNextStep = "setPartnerEntry - MESSAGE_TYPE";
    tbl.setValue( Convert.toSapMessageType( re, jp ), "MESSAGE_TYPE" );

    fNextStep = "setPartnerEntry - LANGU";
    tbl.setValue( Convert.toSapLangu( re.sprach_cd ), "LANGU" );

    fNextStep = "setPartnerEntry - MEDIUM";
    tbl.setValue( "1", "MEDIUM" );

    fNextStep = "setPartnerEntry - COPIES";
    tbl.setValue( 1, "COPIES" );

    fNextStep = "setPartnerEntry - DISPATCH_TIME";
    if ( re.druck_art != null )
      tbl.setValue( Convert.toSapDispatchTime( re.druck_art ), "DISPATCH_TIME" );

    fNextStep = "setPartnerEntry - PRINTER";
    switch( fPrinterSetting )
    {
      case 0: // do not set printername
        tbl.setValue( "", "PRINTER" );
        break;
      case 2: // use default printername
        tbl.setValue( this.fPrinterDefName, "PRINTER" );
        break;
    }

    fNextStep = "setPartnerEntry - IMMEDIATE_PRINT";
    if ( this.fImmediatePrint ) // standart behavior
    {
      if ( re.isRechnung() )
      {
        tbl.setValue( "X", "IMMEDIATE_PRINT" );
      }
      else
      {
        tbl.setValue( " ", "IMMEDIATE_PRINT" );
      }
    }
    else
    // switch off printing
    { // (for test purpose)
      tbl.setValue( " ", "IMMEDIATE_PRINT" );
    }

    // set jgb params
    if ( re.isJGB() && ( jp != null ) )
    {
      fNextStep = "setPartnerEntry - PRINT_GROUP / jp.druckperi";
      if ( jp.druckperi != null )
        tbl.setValue( jp.druckperi, "PRINT_GROUP" );

      fNextStep = "setPartnerEntry - BOX_NUMBER / jp.fach_nr";
      tbl.setValue( jp.fach_nr, "BOX_NUMBER" );

      fNextStep = "setPartnerEntry - PRINT_MODE / jp.druck_art";
      if ( jp.druck_art != null )
        tbl.setValue( jp.druck_art, "PRINT_MODE" );

      fNextStep = "setPartnerEntry - PROT_TYPE / re.schutit_typ";
      tbl.setValue( re.schutit_typ, "PROT_TYPE" );

      fNextStep = "setPartnerEntry - ADDRESS_VTR / jp.adr_nr_vtr";
      tbl.setValue( Convert.toSapNr( jp.adr_nr_vtr, 10 ), "ADDRESS_VTR" );

      fNextStep = "setPartnerEntry - SORT_TEXT / jp.sort_text";
      if ( jp.sort_text != null )
        tbl.setValue( jp.sort_text, "SORT_TEXT" );

      fNextStep = "setPartnerEntry - LIST_MODE / jp.listenart";
      tbl.setValue( Convert.toSapListMode( jp.listenart ), "LIST_MODE" );
    }

    // set amv params
    if ( re.isAMV() )
    {
      fNextStep = "setPartnerEntry - PROT_TITLE / re.schutit_nr";
      tbl.setValue( re.schutit_nr, "PROT_TITLE" );
    }

  }

  private void setPartnerEntryCopy( SAPRechnungVO re, String partnerRole, int partnerNumb, SAPJGBParamVO jp )
  {
    String msgType = Convert.toSapMessageType( re, jp );

    if ( msgType.equals( Convert.SAP_MESSAGE_TYPE_ESR ) || // add message types for
        msgType.equals( Convert.SAP_MESSAGE_TYPE_DEPKTO ) ) // copies only if needed
    {
      JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable( "PARTNER" );
      tbl.appendRow();
      fLogDetail.append( partnerRole + "=" + partnerNumb + "(copy) " );

      fNextStep = "setPartnerEntryCopy - DOC_NUMBER / re.bfakt_id";
      tbl.setValue( Convert.toSapDocNumber( re.bfakt_id ), "DOC_NUMBER" );

      fNextStep = "setPartnerEntryCopy - ITM_NUMBER";
      tbl.setValue( Convert.toSapItmNumber( 0 ), "ITM_NUMBER" );

      fNextStep = "setPartnerEntryCopy - PARTN_ROLE / " + partnerRole;
      tbl.setValue( partnerRole, "PARTN_ROLE" );

      fNextStep = "setPartnerEntryCopy - PARTN_NUMB";
      tbl.setValue( Convert.toSapNr( partnerNumb, 10 ), "PARTN_NUMB" );

      // Nachrichten (Kopie) setzen
      fNextStep = "setPartnerEntryCopy - MESSAGE_TYPE";
      tbl.setValue( Convert.toSapMessageTypeCopy( re.regulierer ), "MESSAGE_TYPE" );

      fNextStep = "setPartnerEntryCopy - LANGU";
      tbl.setValue( Convert.toSapLangu( re.sprach_cd ), "LANGU" );

      fNextStep = "setPartnerEntryCopy - MEDIUM";
      tbl.setValue( "1", "MEDIUM" );

      fNextStep = "setPartnerEntryCopy - COPIES";
      tbl.setValue( 1, "COPIES" );

      fNextStep = "setPartnerEntryCopy - DISPATCH_TIME";
      if ( re.druck_art != null )
        tbl.setValue( Convert.toSapDispatchTime( re.druck_art ), "DISPATCH_TIME" );

      fNextStep = "setPartnerEntryCopy - PRINTER";
      switch( fPrinterSetting )
      {
        case 0: // do not set printername
          tbl.setValue( "", "PRINTER" );
          break;
        case 2: // use default printername
          tbl.setValue( this.fPrinterDefName, "PRINTER" );
          break;
      }

      fNextStep = "setPartnerEntryCopy - IMMEDIATE_PRINT";
      if ( this.fImmediatePrint ) // standart behavior
      {
        tbl.setValue( "X", "IMMEDIATE_PRINT" );
      }
      else
      // switch off printing
      { // (for test purpose)
        tbl.setValue( " ", "IMMEDIATE_PRINT" );
      }
    } // end if
  }

  private void setConditions( SAPRechnungVO re )
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable( "CONDITIONS" );
    Iterator it = re.posList.iterator();
    SAPRechnungsPositionVO pos;

    if ( !it.hasNext() )
    {
      throw new RfmException( "setConditions - re.posList enthält keine Elemente !" );
    }

    while ( it.hasNext() )
    {
      pos = (SAPRechnungsPositionVO) it.next();

      tbl.appendRow();

      fNextStep = "setConditions - DOC_NUMBER / re.bfakt_id";
      tbl.setValue( Convert.toSapDocNumber( re.bfakt_id ), "DOC_NUMBER" );

      fNextStep = "setConditions - ITM_NUMBER / pos.bfakt_posnr";
      tbl.setValue( Convert.toSapItmNumber( pos.bfakt_posnr ), "ITM_NUMBER" );

      fNextStep = "setConditions - COND_TYPE";
      tbl.setValue( "PR00", "COND_TYPE" );

      fNextStep = "setConditions - COND_VALUE / pos.preis";
      if ( ( pos.preis != null ) && ( pos.preis.compareTo( new BigDecimal( 0 ) ) != 0 ) )
      {
        tbl.setValue( pos.preis, "COND_VALUE" );
        // LOG.debug( fNextStep + "=" + pos.preis );
      }

      fNextStep = "setConditions - CURRENCY / re.waehrung";
      if ( re.waehrung != null )
        tbl.setValue( re.waehrung, "CURRENCY" );
    }
  }

  private void setText( SAPRechnungVO re )
  {
    Iterator it = re.posList.iterator();
    SAPRechnungsPositionVO pos;

    //
    // set header text's
    //
    fNextStep = "setText - re.kopfText";
    setTextEntry( Convert.toSapDocNumber( re.bfakt_id ), // DOC_NUMBER
        Convert.toSapItmNumber( 0 ), // ITM_NUMBER
        "Z003", // TEXT_ID
        Convert.toSapLangu( re.sprach_cd ), // LANGU
        re.kopfText, // bagis text
        true ); // full text option

    fNextStep = "setText - re.beilagenText";
    setTextEntry( Convert.toSapDocNumber( re.bfakt_id ), // DOC_NUMBER
        Convert.toSapItmNumber( 0 ), // ITM_NUMBER
        "Z004", // TEXT_ID
        Convert.toSapLangu( re.sprach_cd ), // LANGU
        re.beilagenText, // bagis text
        true ); // full text option

    fNextStep = "setText - re.belehrungText";
    setTextEntry( Convert.toSapDocNumber( re.bfakt_id ), // DOC_NUMBER
        Convert.toSapItmNumber( 0 ), // ITM_NUMBER
        "Z005", // TEXT_ID
        Convert.toSapLangu( re.sprach_cd ), // LANGU
        re.belehrungText, // bagis text
        true ); // full text option

    //
    // set position text's
    //
    while ( it.hasNext() )
    {
      pos = (SAPRechnungsPositionVO) it.next();

      fNextStep = "setText - position text for art nr " + pos.art_nr;
      setTextEntry( Convert.toSapDocNumber( re.bfakt_id ), // DOC_NUMBER
          Convert.toSapItmNumber( pos.bfakt_posnr ), // ITM_NUMBER
          "0001", // TEXT_ID
          Convert.toSapLangu( re.sprach_cd ), // LANGU
          getArtikelPosText( pos ), // bagis text
          false ); // full text option
    }
  }

  private void setTextEntry( String sapDocNumber, String sapItmNumber, String sapTextID, String sapLangu, String bagisText, boolean fullBagisText )
  {
    if ( bagisText == null )
      bagisText = "";

    if ( bagisText.length() > 0 )
    {
      JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable( "TEXT" );
      SapText st = new SapText( bagisText );
      Iterator it;
      SapTextVO vo;

      //
      // set full or split text
      //
      if ( fullBagisText )
      {
        it = st.getTextList().iterator(); // sap text line 1..n
      }
      else
      {
        it = st.getSplitTextList().iterator(); // sap text line 2..n
      }

      //
      // add sap text lines to table
      //
      while ( it.hasNext() )
      {
        vo = (SapTextVO) it.next();

        tbl.appendRow();

        tbl.setValue( sapDocNumber, "DOC_NUMBER" );
        tbl.setValue( sapItmNumber, "ITM_NUMBER" );
        tbl.setValue( sapTextID, "TEXT_ID" );
        tbl.setValue( sapLangu, "LANGU" );
        tbl.setValue( vo.format, "FORMAT_COL" );
        tbl.setValue( vo.text, "TEXT_LINE" );
      } // end while

    } // end if
  } // end method

  private String getArtikelPosText( SAPRechnungsPositionVO pos )
  {
    String text = "";

    if ( pos.artText == null )
      pos.artText = "";
    if ( pos.artZusatzText == null )
      pos.artZusatzText = "";

    if ( this.fShowArtText && this.fShowArtZusatzText )
    {
      if ( pos.artText.length() > 0 )
        text = pos.artText + SapText.BAGIS_BREAK + pos.artZusatzText;
      else
        text = pos.artZusatzText;
    }
    else if ( this.fShowArtText )
    {
      text = pos.artText;
    }
    else if ( this.fShowArtZusatzText )
    {
      text = pos.artZusatzText;
    }

    return text;
  }

} // end class

