package ch.ige.ses.client.sapinbound;

import java.util.*;
import java.io.*;
import java.math.BigDecimal;

import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.util.vo.VersadrVO;

/**
 * Generiert Testdaten für den manuellen Modus.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: TestData.java,v 1.58 2008/01/07 15:02:38 iten Exp $
 */
public class TestData
{
  public static final String[] JGB_DRUCKPERI  = {"0","1","2","3","5","8","9"};
  public static final String[] JGB_LISTENART  = {"0","2","2","2","0","6","0"};
  public static final String[] JGB_DRUCK_ART  = {"3","4","3","3","4","4","3"};
  public static final int[]    JGB_FACH_NR    = { 1 , 2 , 3 , 4 , 5 , 6 , 7 };
  public static final int[]    JGB_ADR_NR_VTR = { 0 , 4711};
  public static final String[] JGB_SORT_TXT   = {"A","B","C"};

  public static int jgbIdx1 = 0;    // 0..6 - JGB_DRUCKPERI, JGB_LISTENART, JGB_DRUCK_ART
  public static int jgbIdx2 = 0;    // 0..6 - JGB_FACH_NR
  public static int jgbIdx3 = 0;    // 0..1 - JGB_ADR_NR_VTR
  public static int jgbIdx4 = 0;    // 0..2 - JGB_SORT_TXT

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public TestData()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // methods for SAPRechnungVO & SAPRechnungsPositionVO
  //---------------------------------------------------------------------------

  public static SAPRechnungVO createRechnung (int bfakt_id)
  {
    return createRechnung(bfakt_id,0,1);
  } // end method


  public static SAPRechnungVO createAMV ( int amvLaufId, String amvLaufNr,
                                          int amvRechnungCd, int bfakt_id )
  {
    return createAMV ( amvLaufId, amvLaufNr, amvRechnungCd,
                       bfakt_id, 0, 1);
  }

  public static SAPRechnungVO createAMV
      ( int amvLaufId, String amvLaufNr, int amvRechnungCd,
        int bfakt_id, int anzZusatzRechEmpf, int anzPosList )
  {
    SAPRechnungVO vo = createRechnung(bfakt_id,anzZusatzRechEmpf,anzPosList);

    vo.amvLaufId = amvLaufId;
    vo.amvLaufNr = amvLaufNr;
    vo.amvRechnungCd = amvRechnungCd;

    vo.druck_art  = "2";        // druckart amv
    vo.employeeId = "1487";     // amv user

    return vo;
  } // end method


  public static SAPRechnungVO createJGB (int jgbLaufId, String jgbLaufNr,
                                         int bfakt_id, int anzZusatzRechEmpf, int anzPosList)
  {
    SAPRechnungVO  vo = null;

    // rechnung stuff
    vo = createRechnung(bfakt_id,anzZusatzRechEmpf,anzPosList);

    vo.regulierer = 0;                        // immer 0 bei jgb
    vo.druck_art  = vo.DRUCK_ART_TYPES[3];    // druck patent jgb rechnungen

    // jgb stuff
    vo.jgbLaufId = jgbLaufId;
    vo.jgbLaufNr = jgbLaufNr;

    vo.jgbHauptRechEmpfParam = createJGBParam(bfakt_id,vo.hauptRechEmpf);

    for (Iterator i = vo.zusatzRechEmpf.iterator(); i.hasNext(); )
    {
      int zre =  ( (Integer)i.next() ).intValue();
      vo.jgbZusatzRechEmpfParam.add ( createJGBParam(bfakt_id,zre) );
    }

    // return vo
    return vo;
  }



  public static SAPRechnungVO createRechnung (int bfakt_id,
                                              int anzZusatzRechEmpf, int anzPosList)
  {
    SAPRechnungVO     vo  = new SAPRechnungVO();

    // Kopfdaten
    vo.bfakt_id      = bfakt_id;
    //vo.bfakt_nr    = 5200700;
    vo.bfakt_dat     = new Date();
    vo.schutit_nr    = "schutit_nr";

    // Mandant 500=3000000    Mandant 520=1029677
    vo.auftraggeber  = 6019070;

    // DepKto Ablauf:                 500=0           520=0
    // Annulierung,Frist,EKKB Test:   500=3000000     520=1029677
    vo.regulierer  = 0;

    // Mandant 500=3000001    Mandant 520=1029677
    vo.hauptRechEmpf = 6019070;

    for (int i = 0; i < anzZusatzRechEmpf; i++)
    {
      // Mandant 500=3000002    Mandant 520=1029677 oder 1029378...1029388
      vo.zusatzRechEmpf.add(new Integer(1029378 + i));
      vo.zusatzRechEmpfNichtVerarbeitet.add(new Integer(112233 + i));
      //vo.zusatzRechEmpf.add(new Integer(3000002));
    }

    vo.kontaktPerson    = "Kontaktperson";
    vo.employeeId       = "1187";
    vo.sprach_cd        = vo.SPRACH_CD_TYPES[0];              // D
    vo.druck_art        = vo.DRUCK_ART_TYPES[0];              // sofort
    vo.waehrung         = vo.WAEHRUNG_TYPES[0];               // CHF
    vo.unsere_ref       = "Unsere Referenz";
    vo.tel_nr           = "+41 031 3252525";
    vo.verfall_dat      = new Date();
    vo.inhaber          = "Inhaber";
    vo.kz_einschr       = vo.BOOLEAN_TYPES[0];
    vo.schutit_typ      = 1234;
    vo.schutit_typ_bez  = "1234";
    vo.ihr_zeichen      = "Ihr Zeichen";
    vo.var_text         = "Var Text";
    vo.abteilung        = "Abteilung XY";

    vo.kopfText         = "<b>Kopf Text 123456789012345678901234567890<br>12345678901234567890123456789012345678901234567890</b>";
    vo.beilagenText     = "BeilagenText 12345678901234567890";
    vo.belehrungText    = "Belehrungstext 12345678901234567890";

    // Rechnungspositionen
    for (int i = 0; i < anzPosList; i++)
    {
      vo.posList.add(createRechnungsPosition(bfakt_id,i+1));
    }



    // return
    return vo;

  } // end method


  public static SAPRechnungsPositionVO createRechnungsPosition (int bfakt_id,
                                                                int bfakt_posnr)
  {
    SAPRechnungsPositionVO vo = new SAPRechnungsPositionVO();

    // Rechnungsposition-Daten
    vo.bfakt_id      = bfakt_id;
    vo.bfakt_posnr   = bfakt_posnr;
    vo.art_nr        = "65500";
    vo.lageinh       = vo.LAGEINH_TYPES[2];                  // STK
    vo.anzahl        = 1;
    vo.preis         = new BigDecimal(12.5); 
    vo.artKurztext   = "Artikel Kurztext " + bfakt_posnr ;
    vo.artText       = "Artikel Text 45678901234567890123456789012345678901234567890 " + bfakt_posnr;
    vo.artZusatzText = "Artikel Zusatztext<br>01234567890123456789012345678901234567890 " + bfakt_posnr;
    //vo.artZusatzText = "";
    //vo.artZusatzText = "Artikel Zusatztext :-)";
    //vo.artZusatzText = "Artikel Zusatztext :-))<br>";
    //vo.artZusatzText = "Für mehrere gleichzeitig beantragte Änderungen ist nur die einfache Gebühr zu entrichten (Art. 33 MSchV).<br><br><b>Markenübertragung</b><br>FIFA Marketing AG<br>Ms. Susan Launchbury<br>Grafenauweg 2<br>PO Box 4250<br>6304 Zug<br><br><b>Firmaänderung:</b><br>Fédération Internationale de Football Association<br>(FIFA567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890)<br>Hitzigweg 11<br>8032 Zürich<br><br><b>Vertreteränderung (neue/r Vertreter/in):</b><br>Schweizer Dubler Day<br>Rechtsanwälte<br>Am Guggenberg 20<br>Postfach 89<br>8053 Zürich";
    //vo.artZusatzText = "Classes <b>soumis</b>es à la taxation<br>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b>Ist ja nur ein Test der neuen <b>Funktionalität</b><br>Atelier Bieri & Noth<br>Gesellschaftsstrasse 22<br>3012 Bern<br>jsdakfjöasdjkfasdöfjsdaöfdjasfjasdfjasdfjasdjfasdjklfjasdlöfjasdfjasdjflasdjklfasdklfjasdklöfjasdjfklasdklfasdjklfjasdfklasdfjlöasdjklfjklasdöfjöasdfjklasdklfasdklöfjklasödfjklasdfklöasdkl<br>";

    return vo;
  } // end method


  public static SAPJGBParamVO createJGBParam (int bfakt_id, int adr_nr)
  {
    SAPJGBParamVO vo = new SAPJGBParamVO();

    // jgbIdx1 0..6 - JGB_DRUCKPERI, JGB_LISTENART, JGB_DRUCK_ART
    // jgbIdx2 0..6 - JGB_FACH_NR
    // jgbIdx3 0..1 - JGB_ADR_NR_VTR
    // jgbIdx4 0..2 - JGB_SORT_TXT

    vo.bfakt_id        = bfakt_id;
    vo.adr_nr          = adr_nr;
    vo.druckperi       = JGB_DRUCKPERI[jgbIdx1];
    vo.listenart       = JGB_LISTENART[jgbIdx1];
    vo.fach_nr         = JGB_FACH_NR[jgbIdx2];
    vo.druck_art       = JGB_DRUCK_ART[jgbIdx1];
    vo.adr_nr_vtr      = JGB_ADR_NR_VTR[jgbIdx3];
    vo.sort_text       = JGB_SORT_TXT[jgbIdx4];

    jgbIdx4++;
    if(jgbIdx4 > 2)
    {
      jgbIdx4 = 0;

      jgbIdx3++;
      if (jgbIdx3 > 1)
      {
        jgbIdx3 = 0;

        jgbIdx2++;
        if (jgbIdx2 > 6)
        {
          jgbIdx2 = 0;

          jgbIdx1++;
          if (jgbIdx1 >6)
          {
            jgbIdx1 = 0;
          }
        }
      }
    } // jgbIdx1

    return vo;
  } // end method


  public static void modifyRechnung(SAPRechnungVO vo,String datafile) throws Exception
  {
    StringBuffer            buf = new StringBuffer();
    Iterator                it  = null;
    SAPRechnungsPositionVO  pos = null;
    int                     cnt = 0;

    Properties              prop= new Properties();
    int                     id  = 0;


    //
    // init
    //
    prop.load( new FileInputStream(datafile) );

    buf.append(vo.belehrungText);
    buf.append("<br>### Originalwerte ###<br>");


    //
    // bfakt_id
    //
    if (prop.getProperty("rechnung.bfakt_id.modify").equals("1"))
    {
      // id erhoehen und wiedr speichern
      id = Integer.parseInt(prop.getProperty("rechnung.bfakt_id"));
      id++;

      prop.setProperty("rechnung.bfakt_id",(new Integer(id)).toString());
      prop.store( new FileOutputStream(datafile), null );

      // originalwert sichern
      buf.append("  bfakt_id      = " + vo.bfakt_id      + "<br>");

      // neuer wert setzten
      vo.bfakt_id = id;
    }
    else
    {
      buf.append("  bfakt_id wird nicht ersetzt <br>");
    }


    //
    // adressen
    //
    if (prop.getProperty("rechnung.adressen.modify").equals("1"))
    {
      // originalwerte sichern
      buf.append("  auftraggeber  = " + vo.auftraggeber  + "<br>");
      buf.append("  regulierer    = " + vo.regulierer    + "<br>");
      buf.append("  hauptRechEmpf = " + vo.hauptRechEmpf + "<br>");
      for (int i = 0; i < vo.zusatzRechEmpf.size(); i++)
      {
        buf.append ( "  zusatzRechEmpf " + i + " = " +
                     (Integer) vo.zusatzRechEmpf.get(i) + "<br>" );
      }

      // neue werte setzen
      vo.auftraggeber  = Integer.parseInt ( prop.getProperty("rechnung.auftraggeber") );
      vo.hauptRechEmpf = Integer.parseInt ( prop.getProperty("rechnung.hauptRechEmpf") );
      if (vo.regulierer != 0)
        vo.regulierer  = Integer.parseInt ( prop.getProperty("rechnung.regulierer") );
      for (int i = 0; i < vo.zusatzRechEmpf.size(); i++)
      {
        vo.zusatzRechEmpf.set(i, new Integer(prop.getProperty("rechnung.zusatzRechEmpf")) );
      }

    }
    else
    {
      buf.append("  adressen werden nicht ersetzt <br>");
    }


    //
    // artikel positionen
    //
    if (prop.getProperty("position.art_nr.modify").equals("1"))
    {
      // originalwerte sichern
      it = vo.posList.iterator();
      while ( it.hasNext() )
      {
        cnt++;
        pos = (SAPRechnungsPositionVO) it.next();
        buf.append("  art_nr " + cnt + " = " + pos.art_nr + "<br>" );
      }

      // neue werte setzen
      it = vo.posList.iterator();
      while ( it.hasNext() )
      {
        pos = (SAPRechnungsPositionVO) it.next();
        pos.art_nr = prop.getProperty("position.art_nr");
      }

    }
    else
    {
      buf.append("  art_nr 1..n werden nicht ersetzt <br>");
    }


    //
    // belehrungText
    //
    vo.belehrungText = buf.toString();

  } // end method

  public static VersadrVO createAdresse (int adr_nr)
  {
    VersadrVO vo = new VersadrVO();

    vo.setAdrNr            (adr_nr);
    vo.setName             ("Name!");
    vo.setZeile1           ("Zeile1");
    vo.setZeile2           ("Zeile2");
    vo.setZeile3           ("Zeile3");
    vo.setZeile4           ("Zeile4");
    vo.setSuchbegr         ("Suchbegriff");
    vo.setPlz              ("PLZ");
    vo.setOrt              ("Ort");
    vo.setLandCd           ("FR");                  // Frankreich, wird direkt übernommen
    vo.setSprachCd         ("F");
    vo.setTelefon          ("Telefon");
    vo.setFax              ("Fax");
    vo.setEmailNameDomain  ("email@email.com");
    vo.setCustGroup        ("01");                  // 01=Deponentenkonto, 02=ohne Deponentenkonto
    vo.setTaxClass         ("1");
    vo.setCustGrp1         ("001");                 // alles ausser 001 wird zu <SPACE> konvertiert
    vo.setAdresseSapVersion(1);
    vo.setIsNew            (false);
    vo.setEmployeeId       ("1314");

    return vo;

  } // end method


  //---------------------------------------------------------------------------
  // methods for STAKO
  //---------------------------------------------------------------------------

  public static StakoVO createStako()
  {
    StakoVO vo     = new StakoVO();
    vo.anzahl      = new BigDecimal(1000);
    vo.artNr       = "110411";
    vo.bfaktId     = 0;
    vo.bfaktPosNr  = 0;
//    vo.datum       = null;
    vo.datum         = new java.sql.Date(new Date().getTime());
    vo.employeeId  = "1048";
    vo.herkunft    = "2";
    vo.kontrollId  = 1234567890;
    vo.schutitNr   = "1200583557";

    return vo;
  } // end method


  public static SAPStakoJobVO createStakoJob()
  {
    SAPStakoJobVO vo     = new SAPStakoJobVO();
    vo.modus       = SAPStakoJobVO.STAKO_MODUS_START;
    vo.bisDatum    = new java.sql.Date(new Date().getTime());

    return vo;
  } // end method


  //---------------------------------------------------------------------------
  // methods for SAPRechnungAnnullationVO
  //---------------------------------------------------------------------------

  public static SAPRechnungAnnullationVO createAnnullation(int bfaktId)
  {
    SAPRechnungAnnullationVO  vo = new SAPRechnungAnnullationVO();

    vo.bfakt_id = bfaktId;
    vo.absageGrund = "A";

    return vo;

  } // end method


  //---------------------------------------------------------------------------
  // methods for SAPJGBJobVO
  //---------------------------------------------------------------------------

  public static SAPJGBJobVO createJGBJob(int jgbLaufId)
  {
    SAPJGBJobVO  vo = new SAPJGBJobVO();

    vo.jgbLaufId    = jgbLaufId;
    vo.bfaktIdFrom  = 100000;
    vo.bfaktIdTo    = 100100;
    vo.lastJob = false;

    return vo;

  } // end method


  //---------------------------------------------------------------------------
  // methods for SAPRechnungStatusListVO
  //---------------------------------------------------------------------------

  public static SAPRechnungStatusListVO createStatusListe(int anzahl)
  {
    SAPRechnungStatusListVO vo = new SAPRechnungStatusListVO();

    // Rechnungspositionen
    for (int i = 0; i < anzahl; i++)
    {
      vo.statusList.add(createRechnungsStatus(123456 + i));
    }

    return vo;

  } // end method


  public static SAPRechnungStatusVO createRechnungsStatus (int bfaktId)
  {
    SAPRechnungStatusVO vo = new SAPRechnungStatusVO();

    // Rechnungsstatus-Daten
    vo.bfakt_id = bfaktId;
    vo.jgbLaufNr = "512";
    vo.status_druck = SAPRechnungStatusVO.STATUS_DRUCK_GEDRUCKT;
    vo.bfakt_dat = new Date();
    vo.status_zahlg = "4";
    vo.status_datum = new Date();
    vo.fakt_betrag = new BigDecimal(200.00);
    vo.waehr_cd = "CHF";
    vo.kz_kalk = "??";

    return vo;
  } // end method


  //---------------------------------------------------------------------------
  // methods for SAPAdrStatusVO
  //---------------------------------------------------------------------------

  public static SAPAdrStatusVO createAdrStatus(int adrNr)
  {
    SAPAdrStatusVO  vo = new SAPAdrStatusVO();

    vo.adrNr    = adrNr;
    vo.compCode = "1000";
    vo.salesOrg = "1000";
    vo.distrChan = "00";
    vo.division = "00";
    vo.modus = false;
    vo.action = "";
    vo.employeeId = "1314";

    return vo;

  } // end method

} // end class

