package ch.ige.ses.client.sapinbound.test;

import ch.ige.ses.client.saputil.StopWatch;

/**
 * Load Test für Sap Adressen anlegen im Request-Reply Modus
 * Generiert oder updatet Adressen mit verschiedenen Threads
 * <p>
 * Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Stefan Nydegger
 * @version $Id: SapAdrLoadTestSuite.java,v 1.5 2006/12/13 14:54:56 iten Exp $
 */
public class SapAdrLoadTestSuite
{

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public SapAdrLoadTestSuite()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  //  methods
  //---------------------------------------------------------------------------

  /**
   * Methode testRFMThreads generiert die angegebene Anzahl Threads und verteilt
   * die zu generierende Anzahl Adressen auf die Threads. Das Total der anzulegenden
   * Adressen ergibt sich aus (threads x repeats)
   *
   * @param threads   Anzahl zu generierender Threads
   * @param delay     Pause zwischen dem Anlegen einer einzelnen Adresse
   * @param repeats   Anzahl Wiederhol
   * @param startAdr  Beginn der Adressrange
   * @return  true wenn Adressen erfolgreich angelegt wurden
   */
  public boolean testRFMThreads(int threads, int delay, int repeats, int startAdr)
  {
    SapAdrLoadTestThread[] rfmThread = new SapAdrLoadTestThread[threads];
    StopWatch   sw        = new StopWatch();
    int         cntTotal  = repeats * threads;
    int         cntError  = 0;

    // init
    System.out.println("Es werden " + cntTotal + " Adressen angelegt");
    System.out.println("Startzeit: " + sw.start());

    // start threads
    for (int i = 0; i < threads; i++)
    {
      rfmThread[i] = new SapAdrLoadTestThread(repeats, delay, startAdr + i * repeats);
      rfmThread[i].start();
      System.out.println("Thread " + rfmThread[i].getName() + " erfolgreich gestartet");
    }

    //warten, bis alle Threads beendet sind
    System.out.println("Warten bis alle threads beendet sind ...");
    try
    {
      for (int i = 0; i < threads; i++)
      {
        rfmThread[i].join();
        cntError = cntError + rfmThread[i].errorCount;
        System.out.println("Thread " + rfmThread[i].getName() + " hatte " + rfmThread[i].errorCount + " Fehler");
      }
    }
    catch (InterruptedException ex)
    {
      System.out.println("InterruptedException " + ex.getMessage());
      ex.printStackTrace();
    }
    sw.stop();


    // statistik
    System.out.println("");
    System.out.println("### statistics - adress client in request/reply mode:");
    System.out.println("number of threads       = " + threads);
    System.out.println("number of repeats       = " + repeats);
    System.out.println("delay between request's = " + delay + " ms");
    System.out.println("adress number range     = " + startAdr + " to " + (startAdr + cntTotal -1));
    System.out.println("records ok              = " + (cntTotal - cntError) );
    System.out.println("records nok             = " + cntError );
    System.out.println("records total           = " + cntTotal );
    System.out.println("time start processing   = " + sw.toStringStartTime() );
    System.out.println("time end processing     = " + sw.toStringStopTime() );
    System.out.println("time elapsed            = " + sw.toStringDeltaTimeShort() );

   return true;
  }

} // end class

