package ch.ige.ses.client.saprequest;

import ch.ige.ses.exception.SapCallException;
import ch.ige.ses.util.Jms;
import ch.ige.ses.client.sapinbound.RFC;

/**
 * <p>SapDepKto:</p>
 * <p>Diese Klasse handelt die Requests zur Saldoabfrage eine Deponentenkontis</p>
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *    !                           ACHTUNG                           !
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *    !                                                             !
 *    !  Diese Klasse verwendet J2EE Restrictions und darf nur      !
 *    !  von Java Client Programmen oder Servlets verwendet werden. !
 *    !                                                             !
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *
 * <p>Copyright: Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Patrick Bolzli
 * @version $Id: SapDepKto.java,v 1.7 2006/12/11 12:43:36 iten Exp $
 */

public class SapDepKto extends SapRequest
{
  // constructors

  /**
   * Die Aufrufe des SAP Systems via SapDepKto verlaufen im Reply / Request
   * Modus. Das heisst, der Aufrufer wartet auf die Antwort des SAP Systems.
   */
  public SapDepKto()
  {
    super();
  }

  //-------------------------------------------------------
  // public methods
  //-------------------------------------------------------

  /**
   * Fragt via JMS auf SAP den Saldo eines Deponentenkontos ab
   * @param ktoNr entspricht der adr_nr
   * @return saldo
   * @throws SapCallException
   */
  public double getSaldo(int ktoNr) throws SapCallException
  {
    Double balance;
    SapReturn reply;
    Integer iKtoNr = new Integer(ktoNr);

    try
    {
      connect();

      reply = request( iKtoNr,                    // ktoNr als Integer da serialisierbar
                       Jms.MSG_TYPE_SAP_BALANCE,
                       Jms.MSG_DESC_SAP_BALANCE);

      if (reply.rfcCode != RFC.RET_CODE_SUCCESS)
      {
        throw new SapCallException( "Fehler beim Abfragen des Kontosaldos auf dem SAP System",
                                    reply.rfcMessage.toString());
      }
    }
    finally
    {
      disconnect();
    }
    balance = (Double)reply.sapData;
    return balance.doubleValue();

  }

} // end class

