package ch.ige.ses.client.saputil;

import java.util.Date;
import java.lang.*;

/**
 * Stopuhr für Zeitmessungen
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: StopWatch.java,v 1.8 2006/12/13 14:54:56 iten Exp $
 */
public class StopWatch
{
  //
  // fields
  //
  private Date fStartTime = null;                 // start & Stop time
  private Date fStopTime  = null;

  private int  fDay;                              // calculated fields
  private int  fHour;
  private int  fMin;
  private int  fSec;
  private int  fMs;

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public StopWatch()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
    * start watch and set start-time to the current time.
    * @return - start time as string
    */
  public String start()
  {
    this.fStartTime = new Date();
    this.fStopTime  = null;
    return this.toStringStartTime();
  }

  /**
    * stop watch and set stop-time to the current time.
    * @return - stop time as string
    */
  public String stop()
  {
    this.fStopTime = new Date();
    return this.toStringStopTime();
  }


  /**
    * deliver start time as string
    * @return start time
    */
  public String toStringStartTime()
  {
    return this.fStartTime.toString();
  }


  /**
    * deliver stop time as string
    * @return stop time
    */
  public String toStringStopTime()
  {
    return this.fStopTime.toString();
  }


  /**
    * deliver delta time as string in the long form <br>
    * (? days ? hours ? min ? sec ? ms)
    * @return delta time
    */
  public String toStringDeltaTimeLong()
  {
    StringBuffer buf = new StringBuffer();

    this.calcDelta();
    buf.append(fDay  + " days ");
    buf.append(fHour + " hours ");
    buf.append(fMin  + " min ");
    buf.append(fSec  + " sec ");
    buf.append(fMs   + " ms ");

    return buf.toString();
  }


  /**
    * deliver delta time as string in the short form <br>
    * (hh:mm:ss.xxx)
    * @return delta time
    */
  public String toStringDeltaTimeShort()
  {
    StringBuffer buf = new StringBuffer();
    int          hr;

    this.calcDelta();
    hr = this.fDay * 24 + this.fHour;

    if (hr > 9)
      buf.append(hr + ":");
    else
      buf.append("0" + hr + ":");

    if (fMin > 9)
      buf.append(fMin + ":");
    else
      buf.append("0" + fMin + ":");

    if (fSec > 9)
      buf.append(fSec + "." + fMs);
    else
      buf.append("0" + fSec + "." + fMs);

    return buf.toString();
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
    * calculate delta time and set intern fields.
    */
  private void calcDelta()
  {
    long delta = this.fStopTime.getTime() - this.fStartTime.getTime();
    long rest  = 0;

    this.fDay  = (int) (delta / (1000 * 60 * 60 *24));
    rest = this.fDay * 24 * 60 * 60 * 1000;

    this.fHour = (int) ((delta - rest)  / (1000 * 60 * 60));
    rest = rest + this.fHour * 60 * 60 * 1000;

    this.fMin  = (int) ((delta - rest ) / (1000 * 60));
    rest = rest + this.fMin * 60 * 1000;

    this.fSec  = (int) ((delta - rest ) / 1000);
    rest = rest + this.fSec * 1000;

    this.fMs   = (int) (delta - rest);
  }


} // end class


