package ch.ige.ses.server.bagisoutbound;

import java.util.*;

import ch.ige.ses.vo.bagis.*;
import ch.ige.util.*;
import ch.ige.util.server.*;

/**
 * BusinessDelegate Bagis Outbound
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Pirmin Suter
 * @version  $Id: SESDelegate.java,v 1.56 2006/12/13 14:48:40 iten Exp $
 */
public class SESDelegate
{
  //
  // fields
  //
  private static SESDelegate  SESDelegate;
  private        SESFacade    facade;


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  private SESDelegate() throws SESException
  {
    try
    {
      SESFacadeHome home = (SESFacadeHome)
      ServiceLocator.getInstance().getHome(SESFacadeHome.JNDI_NAME, SESFacadeHome.class);
      facade = home.create();
    }
    catch(Exception ex)
    {
      throw new SESException
        ( "SES (Konstruktur SESDelegate) - Kann nicht aufgerufen werden." +
          " Exception: " + ex.getMessage() );
    }
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public static SESDelegate getInstance() throws SESException
  {
    if (SESDelegate == null)
    {
      SESDelegate = new SESDelegate();
    }
    return SESDelegate;
  }

  //---------------------------------------------------------------------------
  //  public methods SESRechnungBean
  //---------------------------------------------------------------------------

  public boolean NewRechnung(final int bfakt_id) throws SESException
  {
    try
    {
      facade.createRechnung(bfakt_id);
      return true;                          // compatibility to older versions
    }
    catch (Exception ex)
    {
      throw new SESException ( SESException.UNMASK, ex.getMessage(),
                               "SES (NewRechnung) - " );
    }
  }


  public boolean ChangeRechnung(final int bfakt_id) throws SESException
  {
     try
     {
        facade.changeRechnung(bfakt_id);
        return true;                        // compatibility to older versions
     }
     catch (Exception ex)
     {
       throw new SESException ( SESException.UNMASK, ex.getMessage(),
                                "SES (ChangeRechnung) - " );
     }
   }


  public SAPRechnungVO getDruckvorschau(final int bfakt_id) throws SESException
  {
    try
    {
      return facade.previewRechnung(bfakt_id);
    }
    catch (Exception ex)
    {
      throw new SESException ( SESException.UNMASK, ex.getMessage(),
                               "SES (getDruckvorschau) - " );
    }
  }


  //---------------------------------------------------------------------------
  // public methods SESRechnungAnnullationBean
  //---------------------------------------------------------------------------

  public boolean AnnullationRechnung(final int bfakt_id, final String herkunft) throws SESException
  {
    try
    {
      return facade.AnnullationRechnung(bfakt_id, herkunft);
    }
    catch (Exception ex)
    {
      throw new SESException("SES (AnnullationRechnung) - Konnte BFAKT_ID nicht verarbeiten.");
    }
  }


  public boolean AnnullationRechnungJGB (final java.util.ArrayList alSAPRechnungAnnullationList) throws SESException
  {
    try
    {
      return facade.AnnullationRechnungJGB(alSAPRechnungAnnullationList);
    }
    catch (Exception ex)
    {
      throw new SESException("SES (AnnullationRechnungJGB) - konnte BFAKT_ID's nicht verarbeiten.");
    }
  }


  //---------------------------------------------------------------------------
  // public methods SESJGBStartBean
  //---------------------------------------------------------------------------

   public boolean JGBStartSendToJMS(String modus,int bfaktIdVon,int bfaktIdBis)
       throws SESException
   {
     try
     {
        return facade.JGBStartSendToJMS(modus, bfaktIdVon, bfaktIdBis);
     }
     catch (Exception ex)
     {
       throw new SESException ( "SES (JGBStartSendToJMS) - Konnte JGBStart nicht in JMS-Queue eintragen." +
                                " Exception: " + ex.getMessage() );
     }
   }

} // end class


