package ch.ige.ses.server.bagisoutbound;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.jms.JMSException;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.util.Jms;
import ch.ige.ses.util.JmsQueue;
import ch.ige.ses.util.SESConstants;

/**
 * SES: Bean zum Eintragen des JGB-Starts in die entsprechende JMS-Queue
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Pirmin Suter
 * @version  $Id: SESJGBStartBean.java,v 1.28 2007/12/24 10:13:23 iten Exp $
 * 
 * @ejb.bean 
 *  name="SESJGBStart"  
 *  display-name="SESJGBStart"
 *  view-type="local"
 *  jndi-name="SESJGBStart"
 *  local-jndi-name="SESJGBStartLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 */
public class SESJGBStartBean implements SessionBean
{
  //TODO maintenance (optional) - jms handling umbauen

  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(SESJGBStartBean.class);
  private static final boolean  JMS_SESSION_TRANSACTED        = false;
  private static final int      JMS_SESSION_ACKNOWLEDGE_MODE  = Session.AUTO_ACKNOWLEDGE;

  //
  // fields
  //
  SessionContext  sessionContext;

  private JmsQueue        fJmsQueue = null;         // set by ejbCreate()

  private QueueSession    fSession  = null;         // set by connect()
  private QueueSender     fSender   = null;

  
  //---------------------------------------------------------------------------
  //  bean methods
  //---------------------------------------------------------------------------

  /**
   * Initialisierung der JMS-Queue für das spätere Senden der Messages
   * @throws CreateException 
   */
  public void ejbCreate() throws CreateException 
  {
      fJmsQueue = new JmsQueue ( 
          JmsQueue.RUN_ON_APPSRV,
          Jms.JNDI_JMS_QUEUE_FACTORY_NAME,
          Jms.JMS_JGB_START_QUEUE_NAME,
          JMS_SESSION_TRANSACTED,
          JMS_SESSION_ACKNOWLEDGE_MODE );
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }
  
  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  //  methods
  //---------------------------------------------------------------------------
  
  /**
   * Methode zum Senden der Message an das SESJGBStartMDBBean (JGB-Verarbeitung) 
   * via die Queue SES.jgbStartQueue
   * 
   * @param   modus      PROD od. TEST
   * @param   minBFaktId Kleinste BFAKT_ID
   * @param   maxBFaktId Grösste BFAKT_ID
   * @return  boolean
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public boolean JGBStartSendToJMS(String modus, int minBFaktId, int maxBFaktId) throws Exception {
    
    javax.jms.TextMessage tm = null;

    try
    {
      connect();
      
      if (modus.equals(SESConstants.MODUS_PROD))
      {
         tm = fSession.createTextMessage(SESConstants.TM_JGB_START);
      }
      else if (modus.equals(SESConstants.MODUS_TEST))
      {
         tm = fSession.createTextMessage(SESConstants.TM_JGB_TEST);
         tm.setIntProperty("minBFaktId", minBFaktId);
         tm.setIntProperty("maxBFaktId", maxBFaktId);
      }
      else if (modus.equals(SESConstants.MODUS_STOP))
      {
        tm = fSession.createTextMessage(SESConstants.TM_JGB_STOP);
      }
      else if (modus.equals(SESConstants.MODUS_RESTART))
      {
        tm = fSession.createTextMessage(SESConstants.TM_JGB_RESTART);
        tm.setIntProperty("minBFaktId", minBFaktId);
        tm.setIntProperty("maxBFaktId", maxBFaktId);
      }
      else
      {
        LOG.error("Kein gültiger Modus (" + SESConstants.MODUS_PROD + ", "+ SESConstants.MODUS_TEST + ", " + SESConstants.MODUS_STOP + " od. " + SESConstants.MODUS_RESTART + ") an Methode JGBStartSendToJMS übergeben");
      }
      fSender.send(tm);
      return true;
    }
    catch (JMSException ex)
    {
      LOG.error("sendRechnung - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }
  
  private void connect() throws NamingException, JMSException 
  {
    fJmsQueue.connectSender();

    fSession = fJmsQueue.getSession();
    fSender  = fJmsQueue.getSender();
  }
  
  private void disconnect()
  {
    fJmsQueue.disconnect();
    fSession = null;        
    fSender  = null;
  }
  
  
} // end of class

