package ch.ige.ses.server.bagisoutbound;

import java.util.*;

import org.apache.log4j.*;
import ch.ige.util.*;
import ch.ige.ses.server.dao.bagis.*;
import ch.ige.ses.server.dao.ses.*;
import ch.ige.ses.server.sapctrl.*;
import ch.ige.ses.util.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.vo.bagis.*;

/**
 * Business Object für das Einlesen der Jahresgebühren von Bagis
 * und für das Schreiben in die SES-Tabellen.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: SESJgbBagisBO.java,v 1.87 2007/12/24 10:13:23 iten Exp $
 */
public class SESJgbBagisBO
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(SESJgbBagisBO.class);

  private static final String   SOURCE_NAME    = "SESJgbBagisBO";   // error vo source name
  private static final int      DEF_BLOCK_SIZE = 1000;              // set in case of error

  //
  // fields
  //
  private SESJgbBagisReadLocal  ejbBagis = null;    // set by constructor
  private SESJgbWorkLocal       ejbSes   = null;

  private	int           blockSize     = 0;          // set by constructor

  private	int           minBfaktID    = 0;          // set by start & restart
  private	int           maxBfaktID    = 0;
  private	int           numberOfRuns  = 0;

  private JGBLaufVO     jgbLaufVO     = null;       // init by start & restart and
                                                    // changed by processData

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  /**
   * Konstruktor
   *
   * @param sesJgbBagisReadLocal Local Remote-Interface des Beans SESJgbBagisRead
   * @param sesJgbWorkLocal      Local Remote-Interface des Beans SESJgbWork
   */
  public SESJgbBagisBO ( SESJgbBagisReadLocal sesJgbBagisReadLocal,
                         SESJgbWorkLocal      sesJgbWorkLocal )
  {
    this.ejbBagis = sesJgbBagisReadLocal;
    this.ejbSes   = sesJgbWorkLocal;

    try
    {
      this.blockSize = Integer.parseInt
                        ( ejbSes.propertySel("bagisoutbound.numberOfRows") );
    }
    catch (Exception ex)
    {
      LOG.warn ( "Constructor - Fehler beim einlesen der Blockgrösse " +
                 "Exception : " + ex.getMessage() );

      this.blockSize = DEF_BLOCK_SIZE;
    }
  }

  //---------------------------------------------------------------------------
  // public methods - jgb lauf
  //---------------------------------------------------------------------------

  public void start(int minBfaktID, int maxBfaktID)
  {

    try
    {
      LOG.info("start(min,max) - minBfaktID=" + minBfaktID + ",maxBfaktID=" + maxBfaktID);

      // BfaktId Range setzen und Anzahl Durchläufe bestimmen
      this.minBfaktID = minBfaktID;
      this.maxBfaktID = maxBfaktID;
      numberOfRuns = (maxBfaktID - minBfaktID) / blockSize + 1;
      LOG.info("start(min,max) - numberOfRuns=" + numberOfRuns);

      // Neuer Eintrag in JGB_LAUF erzeugen
      jgbLaufVO = new JGBLaufVO();
      jgbLaufVO.laufNr                  = generateLaufNr();
      jgbLaufVO.anzahlBfakturGelesen    = 0;
      jgbLaufVO.countBfakturToSap       = 0;
      jgbLaufVO.anzahlBfaktjgbGeneriert = 0;
      jgbLaufVO.minBfakturToSap         = minBfaktID - 1;
      ejbSes.jgbLaufIns(jgbLaufVO);

      // Generierte LaufID lesen
      jgbLaufVO = ejbSes.jgbLaufSel(jgbLaufVO.laufNr);

      // Aufbereitung und Uebertragung Daten von BAGIS nach SES
      processData();

    }
    catch (Exception ex)
    {
      String msg = "start(minBfaktID,maxBfaktID) - Ein unerwarteter Fehler ist aufgetreten " +
                   "Exception : " + ex.getMessage();
      LOG.fatal(msg);
      ejbSes.errorSave(SOURCE_NAME,msg);
    }
  }


  public void start()
  {
    String    buf        = null;
    HashMap   hashMap    = null;
    int       count      = 0;
    int       minBfaktID = 0;
    int       maxBfaktID = 0;

    try
    {
      LOG.info("start() - SelectBFaktIdRangeForJGB");
      hashMap = new BfakturDAO().selectBfaktIdRangeForJGB();

      if (hashMap != null)
      {
        count = ((Integer)hashMap.get("count")).intValue();

        if (count > 0)
        {
          LOG.debug("start() - BfaktIdRangeForJGB selektiert");
          minBfaktID = ((Integer)hashMap.get("minValue")).intValue();
          maxBfaktID = ((Integer)hashMap.get("maxValue")).intValue();

          start(minBfaktID,maxBfaktID);
        }
        else
        {
          buf = "BfaktIdRangeForJGB - Es wurden keine Einträge gefunden !";
          LOG.debug("start() - " + buf);
          throw new Exception(buf);
        }
      }
      else
      {
        buf = "BfaktIdRangeForJGB - Es wurde keine Row zurückgeliefert !";
        LOG.debug("start() - " + buf);
        throw new Exception(buf);
      }

    }
    catch (Exception ex)
    {
      String msg = "start() - Ein unerwarteter Fehler ist aufgetreten " +
                   "Exception : " + ex.getMessage();
      LOG.fatal(msg);
      ejbSes.errorSave(SOURCE_NAME,msg);
    }
  }


  public void restart(int minBfaktID, int maxBfaktID)
  {
    try
    {
      LOG.info("restart(min,max) - minBfaktID=" + minBfaktID + ",maxBfaktID=" + maxBfaktID);

      // BfaktId Range setzen und Anzahl Durchläufe bestimmen
      this.minBfaktID = minBfaktID;
      this.maxBfaktID = maxBfaktID;
      numberOfRuns = (maxBfaktID - minBfaktID) / blockSize + 1;
      LOG.info("restart(min,max) - numberOfRuns=" + numberOfRuns);

      // Letzter Eintrag von JGB_LAUF holen
      jgbLaufVO = new JGBLaufDAO().selectMaxLaufID();

      // Ueberprüfung Startkriterien
      if ( jgbLaufVO.sesStat != JGBLaufVO.SES_STAT_STOPPED )
      {
        throw new SESException
                    ("restart(min,max) - Nur erlaubt, falls SesStat = SES_STAT_STOPPED");
      }

      // Update SES_STAT und Restart
      if ( ejbSes.jgbLaufUpdSesStat ( jgbLaufVO.jgbLaufId,
                                      jgbLaufVO.sesStat,
                                      jgbLaufVO.SES_STAT_STARTED ) == 0 )
      {
        throw new SESException
                    ("restart(min,max) - Verarbeitung bereits gestartet oder beendet");
      }
      else
      {
        jgbLaufVO.sesStat = JGBLaufVO.SES_STAT_STARTED;
      }

      // Aufbereitung und Uebertragung Daten von BAGIS nach SES
      processData();

    }
    catch (Exception ex)
    {
      String msg = "restart(min,max) - Ein unerwarteter Fehler ist aufgetreten " +
                   "Exception : " + ex.getMessage();
      LOG.fatal(msg);
      ejbSes.errorSave(SOURCE_NAME,msg);
    }
  }


  public void restart()
  {
    HashMap hashMap    = null;
    int     minBfaktID = 0;
    int     maxBfaktID = 0;

    try
    {
      LOG.info("restart() - SelectBFaktIdRangeForJGB");

      // Letzter Eintrag von JGB_LAUF holen
      jgbLaufVO = new JGBLaufDAO().selectMaxLaufID();

      // Bestimmung minBfaktID aus Tabelle BFAKTUR_SES für diesem Lauf
      minBfaktID = new BfakturSESDAO().selectMaxBfaktID(jgbLaufVO.jgbLaufId);

      // Bestimmung maxBfaktID aus Tabelle BFAKTUR
      hashMap = new BfakturDAO().selectBfaktIdRangeForJGB();
      maxBfaktID = ((Integer)hashMap.get("maxValue")).intValue();

      // Anpassung minBfaktID (falls notwendig)
      // Wenn keine bfaktur in diesem Lauf vorhanden ist wird 0 zurückgegeben
      // Dann wird als minbfaktId die erste JGB-relevante bfakt_id genommen
      if(minBfaktID == 0)
      {
        minBfaktID = ((Integer)hashMap.get("minValue")).intValue();
      }
      else
      {
        minBfaktID++;
      }

      // Restart
      restart(minBfaktID,maxBfaktID);

    }
    catch (Exception ex)
    {
      String msg = "restart() - Ein unerwarteter Fehler ist aufgetreten " +
                   "Exception : " + ex.getMessage();
      LOG.fatal(msg);
      ejbSes.errorSave(SOURCE_NAME,msg);
    }
  }


  public void stop()
  {
    try
    {
      ejbSes.requestStop();
    }
    catch (SESException ex)
    {
      LOG.fatal(ex.getMessage());
      ejbSes.errorSave(SOURCE_NAME,ex.getMessage());
    }
    catch (Exception ex)
    {
      String msg = "stop() - Ein unerwarteter Fehler ist aufgetreten " +
                   "Exception : " + ex.getMessage();
      LOG.fatal(msg);
      ejbSes.errorSave(SOURCE_NAME,msg);
    }
  }

  //---------------------------------------------------------------------------
  // private methods - jgb lauf
  //---------------------------------------------------------------------------

  /**
   * Aufbereitung und Uebertragung Daten von BAGIS nach SES
   *
   * @throws SESException Unerwarteter Fehler
   */
  private void processData() throws SESException
  {
    int         runStart    = 0;
    int         runStop     = 0;
    int         runNr       = 0;
    ArrayList   list        = null;


    //
    // loop for numberOfRuns
    //
    LOG.info("processData - Total numberOfRuns=" + numberOfRuns);

    for ( runNr = 1;  runNr <= numberOfRuns &&
                      jgbLaufVO.sesStat == jgbLaufVO.SES_STAT_STARTED; runNr++ )
    {
      // set start / stop of run
      LOG.info("processData - runNr=" + runNr);

      runStart = jgbLaufVO.minBfakturToSap + 1;
      runStop  = (runNr < numberOfRuns ? runStart + blockSize - 1 : maxBfaktID);


      // read data from bagis
      LOG.info("processData - readJGBFromBagis (" + runStart + "," + runStop + ")");
      list = ejbBagis.readJGBFromBagis(runStart,runStop);

      jgbLaufVO.anzahlBfakturGelesen = jgbLaufVO.anzahlBfakturGelesen +
                                       list.size();
      LOG.info("processData - readJGBFromBagis, list.size()=" + list.size());


      // save data to ses
      // and update status_druck auf bagis
      if (list.size() > 0)
      {                                               // update counts:
        LOG.info("processData - save data to ses");   // jgbLaufVO.countBfakturToSap
        saveJGBToSES(list);                           // jgbLaufVO.anzahlBfaktjgbGeneriert
      }


      // send job to sap inbound service
      SAPJGBJobVO job = new SAPJGBJobVO();

      try
      {
        job.jgbLaufId    = jgbLaufVO.jgbLaufId;
        job.bfaktIdFrom  = runStart;
        job.bfaktIdTo    = runStop;
        job.lastJob      = (runNr < numberOfRuns ? false : true);

        LOG.info ( "processData - sendJGBJob :" + " jgbLaufId=" + job.jgbLaufId +
                   ", bfaktIdFrom=" + job.bfaktIdFrom + ", bfaktIdTo=" + job.bfaktIdTo +
                   ", lastJob=" + job.lastJob );

        new SapInboundCtrlLookup().getLocalInterface().sendJGBJob(job);
        
        jgbLaufVO.minBfakturToSap = runStop;
      }
      catch (Exception ex)
      {
        throw new SESException
          ( "processData - Send SAPJGBJobVO zum SapInbound Service ist fehlgeschlagen \n" +
            "Die Verarbeitung der Jahresgebühren wird abgebrochen \n" +
            "SAPJGBJobVO : \n" + job.toString() + "\n" +
            "Exception : \n" + ex.getMessage() );
      }


      // Aktualisierung Counts und aktuellen SES_STAT lesen
      ejbSes.jgbLaufUpdSesCount(jgbLaufVO);
      jgbLaufVO = ejbSes.jgbLaufSel(jgbLaufVO.laufNr);

    } // end for


    //
    // update SES_STAT
    //
    if ( runNr < numberOfRuns )
    {
      LOG.info("processData - jgbLaufUpdSesStat to SES_STAT_STOPPED");

      ejbSes.jgbLaufUpdSesStat ( jgbLaufVO.jgbLaufId,
                                 JGBLaufVO.SES_STAT_STOPPED );
    }
    else
    {
      LOG.info("processData - jgbLaufUpdSesStat to SES_STAT_FINISHED");

      ejbSes.jgbLaufUpdSesStat ( jgbLaufVO.jgbLaufId,
                                 JGBLaufVO.SES_STAT_FINISHED );
    }

  } // end processData


  /**
   * Speichert Rechnungen auf Tabelle SES Tabellen und erhöht SES Counts,
   * falls das abspeichern erfolgreich war.
   * Im Fehlerfall werden die daten in der Error Tabelle gespeichert.
   *
   * @param list  ArrayList mit SAPRechnungVO
   */
  private void saveJGBToSES(ArrayList list)
  {
    SAPRechnungVO  vo  = null;
    ErrorVO        err = null;

    for (int i = 0; i < list.size(); i++)
    {
      vo  = (SAPRechnungVO) list.get(i);

      vo.jgbLaufId = jgbLaufVO.jgbLaufId;       // lauf_id und lauf_nr in
      vo.jgbLaufNr = jgbLaufVO.laufNr;          // SAPRechnungVO speichern

      // LOG.info("saveJGBToSES - insert SAPRechnungVO with bfakt_id=" + vo.bfakt_id);
      err = ejbSes.saveSingleVO(vo);

      if (err == null)
      {
        // LOG.info("saveJGBToSES - insert SAPRechnungVO with bfakt_id=" + vo.bfakt_id + " successfully done");

        // update counts
        jgbLaufVO.countBfakturToSap++;
        jgbLaufVO.anzahlBfaktjgbGeneriert = jgbLaufVO.anzahlBfaktjgbGeneriert +
                                            vo.jgbZusatzRechEmpfParam.size()  + 1;

        // update status druck auf bagis
        err = ejbBagis.updateDruckstatus(vo);
        if (err != null)
        {
          LOG.info("saveJGBToSES - update Druckstatus BAGIS with bfakt_id=" + vo.bfakt_id + " failed, save message to error table");
          ejbSes.errorSave(err);
        }

      }
      else
      {
        LOG.info("saveJGBToSES - insert SAPRechnungVO with bfakt_id=" + vo.bfakt_id + " failed, save message to error table");
        ejbSes.errorSave(err);
      }
    } // end for
  }


  /**
   * Diese Methode generiert die LaufNr im Format "YYYYMMDDNN" wobei NN
   * einer aufsteigende Nummer von 00..99 ist.
   * Im Produktivmodus ist nur ein lauf pro Tag erlaubt. dass heisst die
   * Laufnummer hat immer das Format "YYYYMMDD00"
   *
   * @return laufNr
   * @throws SESException  falls mehrer Läufe pro Tag (im Produktiv Modus)
   * @throws Exception     oder im Falle einer unerwarteten Fehlermeldung
   */
  private String generateLaufNr() throws SESException, Exception
  {
    JGBLaufVO     vo        = null;   // Letzter Eintrag in der JGB_LAUF Tabelle
    String        laufCnt   = null;   // Lauf Zähler im Format "nn"
    String        laufDate  = null;   // Lauf datum im Format "JJJJMMTT"
    String        modus     = null;   // JGB Lauf Modus [TEST oder PROD]


    //
    // Letzter Eintrag von JGB_LAUF holen
    //
    LOG.debug("generateLaufNr - letzten Laufeintrag holen");
    vo = new JGBLaufDAO().selectMaxLaufID();

    //
    // Kein Eintrag in der Tabelle
    //
    if (vo == null)
    {
      LOG.debug("generateLaufNr - Noch kein Eintrag in der Tabelle JGB_Lauf vorhanden");
      laufDate = LaufNr.getTodayDate();
      laufCnt  = "00";
      LOG.debug("generateLaufNr - Return : " + laufDate + laufCnt);
      return (laufDate + laufCnt);
    }

    //
    // Es sind bereits Einträge vorhanden
    //
    LOG.debug("generateLaufNr - Letzter Eintrag in der Tabelle JGB_Lauf : " + vo.laufNr);
    laufDate = vo.laufNr.substring(0,8);
    laufCnt  = vo.laufNr.substring(8,10);

    try
    {
      LOG.debug("generateLaufNr - Selektieren der Property jgb.modus");
      modus = ejbSes.propertySel("jgb.modus");
    }
    catch (Exception ex)
    {
      LOG.error("generateLaufNr - Fehler beim Selektieren der Property jgb.modus " + ex.getMessage());
      modus = SESConstants.MODUS_TEST;
    }

    if ( modus.equals(SESConstants.MODUS_TEST) )          // Test - Modus
    {
      LOG.debug("generateLaufNr - Modus = TEST");
      if ( laufDate.equals(LaufNr.getTodayDate()) )
      {
        int nr = Integer.parseInt(laufCnt);
        nr++;
        laufCnt = (nr < 10 ? "0" + String.valueOf(nr) : String.valueOf(nr));
      }
      else
      {
        laufDate = LaufNr.getTodayDate();
        laufCnt  = "00";
      }
    }
    else                                                  // Produktiv - Modus
    {
      LOG.debug("generateLaufNr - Modus = PROD");
      if ( laufDate.equals(LaufNr.getTodayDate()) )
      {
        throw new SESException
                   ( "generateLaufNr - Im Produktivmodus ist nur ein JGB Lauf pro Tag gestattet \n" +
                     "Die Verarbeitung der Jahresgebühren wird abgebrochen" );
      }
      else
      {
        laufDate = LaufNr.getTodayDate();
        laufCnt  = "00";
      }
    }

    LOG.debug("generateLaufNr - Return : " + laufDate + laufCnt);
    return (laufDate + laufCnt);
  }



  //---------------------------------------------------------------------------
  // public methods - misc
  //---------------------------------------------------------------------------

  /**
   * Aktualisiert bei der JGB Verarbeitung den Status Druck auf BAGIS von
   * 3 (bereit zum Druck JGB) auf 1 (bereit zum Druck).
   * Die Methode wird vom SesMonitor verwendet, falls im JGB Ablauf der ent-
   * sprechende Aufruf nicht geht.
   *
   * @param  vo         Rechnungsdaten
   * @throws Exception  Es ist ein unerwarteter fehler aufgetreten
   */
  public void setBfakturStatusDruck (SAPRechnungVO vo) throws Exception
  {
    ErrorVO err = null;

    err = ejbBagis.updateDruckstatus(vo);
    if (err != null)
    {
      LOG.info("setBfakturStatusDruck - " + err.description);
      throw new Exception(err.description);
    }
  } // end setBfakturStatusDruck


  /**
   * Selektiert Daten von JGB_LAUF und BFAKTUR_SES und erstellt eine Liste
   * aller fehlerhaften Jahresgebühren eines Laufes.
   *
   * @param jgbLaufId Jahresgebühren Lauf
   * @return Fehlerliste als String
   * @throws SESException
   */
  public String getProtokoll(int jgbLaufId) throws SESException
  {
    ArrayList     al      = null;
    String        laufNr;
    int           anzahl;
    StringBuffer  sb      = new StringBuffer();
    try
    {
      // JGB-LaufNr selektieren
      laufNr    = ejbSes.getJgbLauf(jgbLaufId).laufNr;
      // fehlerhafte BFAKTURen selektieren
      al        = ejbSes.getJgbErrors(jgbLaufId);
      anzahl    = al.size();  // Anzahl der fehlerhaften BFAKTURen ermitteln

      // Kopfdaten aufbereiten
      sb.append("Fehlerliste JGB Lauf Nr: " + laufNr + " (JGB Lauf Id: " + jgbLaufId +")\n \n");
      sb.append("BfaktId\tHauptempf.\tFehlerquelle\tFehlerbeschreibung\n");
      sb.append("================================================================================ \n\n");

      // Fheler aufbereiten
      SAPRechnungVO vo = null;
      StringBuffer error_description;

      for (int i = 0; i < anzahl; i++)  //Loop über Fehler - ArrayList
      {
        vo = (SAPRechnungVO)al.get(i);

        if (vo.error_description != null)
        {
          error_description = new StringBuffer(vo.error_description);

          // In der error_description muss nach jedem Newline oder Carriage Return der Text wieder eingeschoben werden :-|
          for (int k = error_description.length()-1; k>=0 ; k--)
          {
            if (error_description.substring( k, k+1).equals("\n"))
            {
              error_description.replace(k, k+1, "\n\t\t\t\t");
            }
          }

          for (int k = error_description.length()-1; k>=0 ; k--)
          {
            if (error_description.substring( k, k+1).equals("\r"))
            {
              error_description.replace(k, k+1, "\r\t\t\t\t");
            }
          }
        }
        else
        {
          error_description = new StringBuffer("null");
        }

        sb.append(vo.bfakt_id + "\t");                    // bfakt_id
        sb.append(vo.hauptRechEmpf + "\t");               // Hauptrechnungsempfänger
        sb.append(vo.error_source + "\t");                // Fehlerquelle
        sb.append(error_description.toString() + "\n");   // Fehlerbeschreibung
      }

      // Anzahl Fehler ausgeben
      sb.append("\n");
      sb.append("Anzahl Fehler: " + anzahl);

      return sb.toString();
    }
    catch (Exception ex)
    {
      String msg = "getProtokoll - Ein unerwarteter Fehler ist aufgetreten " +
                   "Exception : " + ex.getMessage();
      LOG.fatal(msg);
      throw new SESException(msg);
    }
  } // end getProtokoll


  /**
   * Selektiert Daten von der SES - DB und der BAGIS - DB und erstellt daraus
   * eine Statistik des JGB-Laufes analog der JGB - Statistiken von SAP
   *
   * @param   jgbLaufId     ID des gewünschten JGB Lauf
   * @return                Statistik oder Fehlermeldung als String
   */
  public String getStatistik(int jgbLaufId)
  {
    StatistikCalcVO   vo          = new StatistikCalcVO();  // Statistik Daten
    ArrayList         empfList    = null;                   // Verarbeitete RE's
    HashMap           adrLandMap  = new HashMap();          // AdrNr=Ländercode Map

    int               prevBfaktId = 0;
    boolean           bagisStatus = false;

    try
    {
      //---------------------------------------------------------------------
      // SES Daten aufbereiten
      //---------------------------------------------------------------------

      vo.setLauf( ejbSes.getJgbLauf(jgbLaufId) );

      vo.sesFakturen     = ejbSes.countBfaktur(jgbLaufId);
      vo.sesRe           = ejbSes.countEmpfaenger(jgbLaufId);
      vo.sesDruckperi    = ejbSes.getAnzPerioden(jgbLaufId);

      vo.sesMinBfaktId   = ejbSes.selectMinBfaktID(jgbLaufId);
      vo.sesMaxBfaktId   = ejbSes.selectMaxBfaktID(jgbLaufId);

      vo.sesReAnSapOK    = ejbSes.countByStatus(jgbLaufId, "1") +
                           ejbSes.countByStatus(jgbLaufId, "9");
      vo.sesReAnSapNOK   = ejbSes.countByStatus(jgbLaufId, "E");

      //---------------------------------------------------------------------
      // Statistik Vergleich SES <> SAP aufbereiten
      //---------------------------------------------------------------------

      // Liste mit verarbeiteten JGB Rechnungsempfängern laden
      empfList = ejbSes.getStatistikCalcReVOList(jgbLaufId);

      // Map mit Adress Nummern und dazugehörigem Ländercode aufbereiten
      for (Iterator iter = empfList.iterator(); iter.hasNext(); )
      {
        StatistikCalcReVO item = (StatistikCalcReVO)iter.next();
        // Jede (gleiche) Adresse wird nur einmal in der Map aufgenommen
        adrLandMap.put(new Integer(item.adrNr),null);
      }

      adrLandMap = ejbBagis.getLandCd(adrLandMap);

      // Liste aller Rechnungsempfänger durchsuchen
      for (Iterator iter = empfList.iterator(); iter.hasNext(); )
      {
        StatistikCalcReVO item = (StatistikCalcReVO)iter.next();
        String            land = (String) adrLandMap.get(new Integer (item.adrNr));

        // Anzahl Fakturaanforderungen
        if (item.bfaktId != prevBfaktId)
        {
          vo.vglFakturen++;
          prevBfaktId = item.bfaktId;
          bagisStatus = false;
        }

        // Anzahl Bagis Status
        // (Messung pro Fakturaanforderung und nicht Rechnungsempfänger)
        if ( ( ! item.listenArt.equals("7")             ) &&
             ( land.equals("CH ") || land.equals("LI ") ) )
        {
          if (! bagisStatus)
          {
            vo.vglBagisStatus++;
          }

          bagisStatus = true;
        }

        // Anzahl Rechnungsempfänger (FA)
        vo.vglReFa++;                                 // Anzahl Rechnungsempfänger (FA)

        if (land.equals("CH ") || land.equals("LI "))
          vo.vglInland++;                             // - davon Inland (CH/LI)
        else
          vo.vglAusland++;                            // - davon Ausland

        if (item.listenArt.equals("7"))
          vo.vglLA7++;                                // - davon mit Listenart 7
      } // for


      // Anzahl Rechnungsempfänger (JGB)
      vo.vglReJgb = vo.vglInland - vo.vglLA7;

      //---------------------------------------------------------------------
      // Bagis Daten aufbereiten
      //---------------------------------------------------------------------

      vo.bagFakturen = ejbBagis.getCntFakturen(vo.sesMinBfaktId, vo.sesMaxBfaktId);
      vo.bagRe       = ejbBagis.getCntRechEmpf(vo.sesMinBfaktId, vo.sesMaxBfaktId);

      //---------------------------------------------------------------------
      // return statistik as string
      //---------------------------------------------------------------------
      return vo.toString();

    }
    catch (Exception ex)
    {
      String msg = "getStatistik - Ein unerwarteter Fehler ist aufgetreten " +
                   "Exception : " + ex.getMessage();

      LOG.error(msg);

      //---------------------------------------------------------------------
      // return error meaasge as string
      //---------------------------------------------------------------------
      return msg;
    }

  } // end getStatistik


} // end class


