package ch.ige.ses.server.bagisoutbound;

import java.util.ArrayList;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.bagis.BfakturDAO;
import ch.ige.ses.server.sapctrl.SapDelegateInbound;
import ch.ige.ses.server.sapctrl.SapInboundCtrlLookup;
import ch.ige.ses.util.SESConstants;
import ch.ige.ses.vo.bagis.SAPRechnungAnnullationVO;

/**
 * SES: Bean zur Verarbeitung Annullationen von BAGIS-Rechnungen
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Pirmin Suter
 * @version  $Id: SESRechnungAnnullationBean.java,v 1.36 2007/12/24 10:13:23 iten Exp $
 * 
 * @ejb.bean 
 *  name="SESRechnungAnnullation"  
 *  display-name="SESRechnungAnnullation"
 *  view-type="local"
 *  jndi-name="SESRechnungAnnullation"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SapInboundCtrl"
 *   view-type="local"
 */
public class SESRechnungAnnullationBean implements SessionBean
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(SESRechnungAnnullationBean.class);

  //
  // fields
  //
  SessionContext  sessionContext;

  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   */
  public void ejbCreate()
  {
    // lazy bone
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  // methods
  //---------------------------------------------------------------------------

  /**
   * Die Methode AnnullationRechnung wird einerseits von der Komponente BAEX
   * aufgerufen (bei der ONLINE-Annullierung einer Rechnung) und andererseits
   * von einem Batch-Programm, welches die Fristen der BAGIS-Rechnung
   * überwacht
   *
   * @param bfakt_id    Identifikation einer BFaktur
   * @param herkunft    Verarbeitungsart [ONLINE, BATCH, HALBAUTO]
   * @return boolean    true faös erfolgreich, sonst false
   * 
   * @ejb.interface-method
   */
  public boolean AnnullationRechnung(int bfakt_id, String herkunft)
  {
    SAPRechnungAnnullationVO  vo = new SAPRechnungAnnullationVO();

    // init vo
    LOG.debug("AnnullationRechnung - bfakt_id = " + bfakt_id);
    vo.bfakt_id = bfakt_id;

    if ( herkunft.equals(SESConstants.HERKUNFT_ONLINE) )
    {
      vo.absageGrund = SAPRechnungAnnullationVO.ABSAGE_GRUND[0];    // bagis: annuliert
    }
    else if( herkunft.equals(SESConstants.HERKUNFT_BATCH) )
    {
      vo.absageGrund = SAPRechnungAnnullationVO.ABSAGE_GRUND[1];    // bagis: frist abgelaufen
    }
    else if(herkunft.equals(SESConstants.HERKUNFT_HALBAUTO))
    {
      vo.absageGrund = SAPRechnungAnnullationVO.ABSAGE_GRUND[1];    // bagis: frist abgelaufen
    }

    try
    {
      if ( herkunft.equals(SESConstants.HERKUNFT_HALBAUTO) )
      {
        LOG.debug("AnnullationRechnung - UpdateJGBBFakturFristablaufInArbeit (HALBAUTO)");
        new BfakturDAO().updateBfakturFristablaufInArbeitJGB(vo.bfakt_id);
      }
      else
      {
        LOG.debug("AnnullationRechnung - UpdateBFakturAnnullationInArbeit (ONLINE & BATCH)");
        new BfakturDAO().updateBfakturAnnullationInArbeit(vo.bfakt_id);
      }

      LOG.debug("AnnullationRechnung - Send Annualation to SAPInbound");
      new SapInboundCtrlLookup().getLocalInterface().sendAnnulation(vo);

      LOG.debug("AnnullationRechnung - Alle Methoden durchgeführt!");
      return true;
    }
    catch (Exception ex)
    {
      LOG.error("AnnullationRechnung - Exception occured: " + ex.toString());
      throw new EJBException(ex);
    }
  }


  /**
   * Die Methode dient zur Übergabe einer Liste von JGB-Rechnungen,
   * deren Frist auf BAGIS abgelaufen ist.
   *
   * @param  alSAPRechnungAnnullationList  Liste mit den abgelaufenden JGB-Rechnungen
   * @return                               true falls annualtion erfolgreich
   * 
   * @ejb.interface-method
   */
  public boolean AnnullationRechnungJGB(java.util.ArrayList alSAPRechnungAnnullationList)
  {
    SAPRechnungAnnullationVO  vo     = null;
    ArrayList                 voList = new ArrayList();

    try
    {
      LOG.debug("AnnullationRechnungJGB - Liste mit " + alSAPRechnungAnnullationList.size() + "erhalten.");

      for (int i = 0; i < alSAPRechnungAnnullationList.size(); i++)
      {
        vo = new SAPRechnungAnnullationVO();
        vo.bfakt_id    = new Integer(alSAPRechnungAnnullationList.get(i).toString()).intValue();
        vo.absageGrund = SAPRechnungAnnullationVO.ABSAGE_GRUND[1];    // bagis: frist abgelaufen

        LOG.debug("AnnullationRechnungJGB - UpdateJGBBFakturFristablaufInArbeit");
        new BfakturDAO().updateBfakturFristablaufInArbeitJGB(vo.bfakt_id);
        voList.add(vo);
      }

      LOG.debug("AnnullationRechnungJGB - Send Annualtion List to SAPInbound");
      new SapInboundCtrlLookup().getLocalInterface().sendAnnulationList(voList);

      LOG.debug("AnnullationRechnungJGB - Alle Methoden durchgeführt!");
      return true;
    }
    catch (Exception ex)
    {
      throw new EJBException(ex);
    }
  }

} // end class


