package ch.ige.ses.server.dao.bagis;

import java.sql.*;
import java.util.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.util.server.*;

/**
 * Kapselung der DB-Zugriffe auf die Tabelle BFAKTUR
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Pirmin Suter, Thomas Iten
 * @version $Id: BfakturDAO.java,v 1.15 2006/12/11 12:43:32 iten Exp $
 */

public class BfakturDAO extends BagisDAO
{

  //---------------------------------------------------------------------------
  // methods - select
  //---------------------------------------------------------------------------

  /**
   * Rechnungen im angegebenen Datumsbereich lesen
   *
   * @param   fromDate      start date range
   * @param   toDate        end date range
   * @throws  SQLException  unexpected error
   * @return  List          list with bfakt_id's (as Integer)
   */
  public List selectBfaktId(String fromDate, String toDate) throws SQLException
  {
    Connection  con   = null;
    Statement   stmt  = null;
    ResultSet   rs    = null;
    ArrayList   res   = new ArrayList();

    try
    {
      con = getConnection();

      resetQuery();

      query.append("SELECT bfakt_id ");
      query.append("FROM bfaktur ");
      query.append("WHERE bfakt_id > 0 ");
      query.append("AND bfakt_dat between to_date('" + fromDate + "','dd.mm.yyyy') and to_date('" + toDate + " 23:59:59','dd.mm.yyyy hh24:mi:ss') ");
      query.append("AND fakt_betrag > 0 ");
      query.append("AND formart_cd in ('115','117','121','122', '140','150',");
      query.append("'151','190','191','192','193', '198','200',");
      query.append("'201', '240', '290', '291','405','425',");
      query.append("'428', '430', '448','450','451','460','495','620','633',");
      query.append("'635','640','650','651','652','664','810')");

      stmt = con.createStatement();
      rs = stmt.executeQuery(getQuery());

      while (rs.next())
      {
        res.add( new Integer(rs.getInt("bfakt_id")) );
      }
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
    return res;
  }


  /**
   * Methode zum Holen der bfakt_id Range der für die Jahresgebühren
   * benötigten Rechnungen.
   *
   * @throws SQLException   Unerwarteter Fehler
   * @return HashMap        map mit bfakt_id range
   */
  public HashMap selectBfaktIdRangeForJGB() throws SQLException
  {
    Connection        con     = null;
    PreparedStatement ps      = null;
    ResultSet         rs      = null;
    HashMap           hashmap = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("SELECT MIN(bfakt_id) as minBfaktID, MAX(bfakt_id) as maxBfaktID, COUNT(*) as cnt ");
      query.append("FROM  bfaktur ");
      query.append("WHERE bfakt_id > " + BFAKTID_BASE + " ");
      query.append("AND   (druck_art = '3' OR druck_art = '4') ");
      query.append("AND   status_druck = '3' ");
      query.append("AND   status_zahlg = '1' ");

      ps = con.prepareStatement(getQuery());
      rs = ps.executeQuery();

      if ( rs.next() )
      {
        hashmap = new HashMap(3);

        hashmap.put ( "minValue",
                      rs.getObject("minBfaktID") == null ? null : new Integer(rs.getInt("minBfaktID")) );

        hashmap.put ( "maxValue",
                      rs.getObject("maxBfaktID") == null ? null : new Integer(rs.getInt("maxBfaktID")) );

        hashmap.put ( "count", new Integer(rs.getInt("cnt")) );
      }

      return hashmap;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Methode zum Holen der Rechnungen (bfakt_id's), deren Frist abgelaufen ist.
   * Beachte:
   *   - IR Marken (Schutztiteltyp = 5) werden nicht berücksichtigt.
   *
   * @throws SQLException   Unerwarteter Fehler
   * @return List           Liste mit bfakt_id's (Integer)
   */
  public List selectBfaktIdForFristAblauf() throws SQLException
  {
    Connection  con   = null;
    Statement   stmt  = null;
    ResultSet   rs    = null;
    List        res   = new ArrayList();

    try
    {
      con = getConnection();

      resetQuery();
      query.append("SELECT bfakt_id ");
      query.append("FROM   bfaktur ");
      query.append("WHERE  bfakt_id > " + BFAKTID_BASE + " ");
      query.append("AND    verfall_dat is not null ");
      query.append("AND    status_zahlg = '1' ");
      query.append("AND    ( trunc(verfall_dat) < trunc(sysdate-15) and bfakt_typ in ('1','2','3') ) ");
      query.append("AND    druck_art <> '3' ");
      query.append("AND    druck_art <> '4' ");
      query.append("AND    ( schutit_typ <> 5 OR schutit_typ is null ) ");
      query.append("AND    formart_cd not in ('652','664') ");

      stmt = con.createStatement();
      rs = stmt.executeQuery(getQuery());

      while (rs.next())
      {
        res.add( new Integer(rs.getInt("bfakt_id")) );
      }

      return res;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  /**
   * Methode zum Holen der JGB Rechnungen (bfakt_id's), deren Frist abgelaufen ist.
   * Beachte:
   *  - IR Marken (Schutztiteltyp = 5) werden nicht berücksichtigt.
   *  - STATUS_ZAHLG = 'A' wird durch ee_selection auf BAGIS gesetzt
   *
   * @throws SQLException   Unerwarteter Fehler
   * @return List           Liste mit bfakt_id's (Integer)
   */
  public List selectBfaktIdForFristAblaufJGB() throws SQLException
  {
    Connection  con   = null;
    Statement   stmt  = null;
    ResultSet   rs    = null;
    List        res   = new ArrayList();

    try
    {
      con = getConnection();

      resetQuery();
      query.append("SELECT bfakt_id ");
      query.append("FROM   bfaktur ");
      query.append("WHERE  bfakt_id > " + BFAKTID_BASE + " ");
      query.append("AND    verfall_dat is not null ");
      query.append("AND    status_zahlg = 'A' ");
      query.append("AND    bfakt_typ = '1' ");
      query.append("AND    trunc(verfall_dat) < trunc(sysdate-15) ");
      query.append("AND    druck_art in ('3','4')  ");
      query.append("AND    kz_frist_abgelaufen = 1 ");

      stmt = con.createStatement();
      rs = stmt.executeQuery(getQuery());

      while (rs.next())
      {
        res.add( new Integer(rs.getInt("bfakt_id")));
      }

      return res;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  /**
   * Selektiert die Erinnerungen 1 und 2 (Unterscheidung via statusDruck)
   * für die automatische Markenverlängerung (AMV).
   *
   * @param   statusDruck   Status Druck für Erinnerung 1 oder 2
   * @throws  SQLException  Unerwarteter Fehler
   * @return  ArrayList     Liste mit Rechnungsnummern bfakt_id's (Integer)
   *                        oder leere Liste falls keine Einträge vorhanden
   */
  public ArrayList selectBfaktIdForAMV (String statusDruck) throws SQLException
  {
    Connection    con     = null;
    Statement     stmt    = null;
    ResultSet     rs      = null;
    ArrayList     result  = new ArrayList();

    try
    {
      con = getConnection();

      resetQuery();
      query.append("SELECT bfakt_id ");
      query.append("FROM   bfaktur ");
      query.append("WHERE  bfakt_id > " + BFAKTID_BASE + " ");
      query.append("AND    bfakt_typ in ('1','2','3') ");
      query.append("AND    druck_art = '2' ");
      query.append("AND    status_druck = '" + statusDruck + "' ");
      query.append("AND    status_zahlg = '1' ");

      stmt = con.createStatement();
      rs = stmt.executeQuery(getQuery());

      while (rs.next())
      {
        result.add( new Integer(rs.getInt("bfakt_id")) );
      }

      return result;

    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  /**
   * get field kz_frist_abgelaufen for a given bfakt_id (pk)
   *
   * @param   bfakt_id      Identifikation der BFAKTUR
   * @throws  SQLException  Unerwarteter Fehler
   * @return                value of field kz_frist_abgelaufen
   */
  public int selectKzFristAbgelaufen(int bfakt_id) throws SQLException
  {
    Connection  con   = null;
    Statement   stmt  = null;
    ResultSet   rs    = null;
    int         kz    = 0;

    try
    {
      con = getConnection();

      setQuery("SELECT kz_frist_abgelaufen FROM bfaktur WHERE bfakt_id=" + bfakt_id);

      stmt = con.createStatement();
      rs = stmt.executeQuery(getQuery());

      if (rs.next())
        kz = rs.getInt("kz_frist_abgelaufen");

      return kz;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  /**
   * Testmethode zum selektieren von Rechnungen via Session Bean.
   *
   * @throws  SQLException  Unerwarteter Fehler
   * @return  SAPRechnungStatusListVO
   */
  public SAPRechnungStatusListVO selectBfaktIdForTestStatusBezahlt() throws SQLException
  {
    Connection  con   = null;
    Statement   stmt  = null;
    ResultSet   rs    = null;
    SAPRechnungStatusListVO ListVO = new SAPRechnungStatusListVO();

    try
    {
      con = getConnection();

      resetQuery();
      query.append("SELECT bfakt_id, status_zahlg, version ");
      query.append("FROM   bfaktur ");
      query.append("WHERE  bfakt_id between 3000000 and 3500000 ");
      query.append("AND    status_druck = '9' ");
      query.append("AND    status_zahlg = '1' ");

      stmt  = con.createStatement();
      rs    = stmt.executeQuery(getQuery());

      while (rs.next())
      {
        SAPRechnungStatusVO statusVO = new SAPRechnungStatusVO();

        statusVO.bfakt_id       = rs.getInt("bfakt_id");
        statusVO.status_zahlg   = rs.getString("status_zahlg");
        statusVO.version        = rs.getInt("version");

        ListVO.statusList.add(statusVO);
      }
      return ListVO;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  /**
   * Methode zum Holen von Rechnungen zum Testen des entsprechenden
   * SES-SessionBeans
   *
   * @param   bfakt_id        Rechnungsnummer oder 0. Im zweiten Fall wird ein Bereich zurückgegeben.
   * @throws  SQLException    Unerwarteter Fehler
   * @return  SAPRechnungStatusListVO
   */
  public SAPRechnungStatusListVO selectBfaktIdForTestStatusDruck(int bfakt_id) throws SQLException
  {
    Connection              con     = null;
    Statement               stmt    = null;
    ResultSet               rs      = null;
    SAPRechnungStatusListVO ListVO  = new SAPRechnungStatusListVO();

    try
    {
      con = getConnection();
      resetQuery();

      if(bfakt_id > 0)
      {
        query.append("SELECT bfakt_id, bfakt_dat, status_druck, version ");
        query.append("FROM   bfaktur ");
        query.append("WHERE  bfakt_id = " + bfakt_id);
      }
      else
      {
        query.append("SELECT bfakt_id, bfakt_dat, status_druck, version ");
        query.append("FROM   bfaktur ");
        query.append("WHERE  bfakt_id between 5000000 and 5000100 ");
        query.append("AND    status_druck = '1' ");
      }

      stmt = con.createStatement();
      rs = stmt.executeQuery(getQuery());

      while (rs.next())
      {
        SAPRechnungStatusVO statusVO = new SAPRechnungStatusVO();

        statusVO.bfakt_id = rs.getInt("bfakt_id");
        statusVO.bfakt_dat = rs.getDate("bfakt_dat");
        statusVO.status_druck = rs.getString("status_druck");
        statusVO.version = rs.getInt("version");

        ListVO.statusList.add(statusVO);
      }

      return ListVO;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  //---------------------------------------------------------------------------
  // methods - update
  //---------------------------------------------------------------------------

  /**
   * Aktualisierung der Rechnung (Zahlungsstatus) auf "Frist abgelaufen".
   * Es dürfen nur Rechnungen aktualisiert werden, welche auf dem Zwischen-
   * status 'B' (SES, Annulation in Arbeit) sind.
   *
   * @param   bfakt_id      Rechnugsnummer
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturFristAbgelaufen(int bfakt_id) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    status_zahlg='A', version=version+1 ");
      query.append("WHERE  bfakt_id=? ");
      query.append("AND    status_zahlg='B' ");

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, bfakt_id);

      return ps.executeUpdate();

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisierung der Rechnung (Zahlungsstatus) auf den Status 'B'
   * "SES Prozess Fristablauf ist dabei die Zahlung auf SAP zu annulieren".
   *
   * @param   bfakt_id      Rechnugsnummer
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturAnnullationInArbeit(int bfakt_id) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    status_zahlg='B', version=version+1 ");
      query.append("WHERE  bfakt_id=? ");

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, bfakt_id);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisierung der Rechnung (kz_frist_abgelaufen) auf den Status 2
   * "SES Prozess Fristablauf ist dabei die Zahlung auf SAP zu annulieren".
   *
   * @param   bfakt_id      Rechnugsnummer
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturFristablaufInArbeitJGB(int bfakt_id) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    kz_frist_abgelaufen=2, version=version+1 ");
      query.append("WHERE  bfakt_id=? ");

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, bfakt_id);

      return ps.executeUpdate();

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert eine Rechnung auf den Status bezahlt. Neben dem Zahlungs-
   * status werden dabei alle Zahlungsrelevanten Attribute auf die von SAP
   * gelieferten Werte aktualisiert.
   *
   * @param   vo            SAPRechnungStatusVO
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturBezahlt(SAPRechnungStatusVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    status_zahlg=?, fakt_betrag=?, kz_kalk=?, waehr_cd=?, ");
      query.append("       zahl_dat=?, buch_dat=?, version=version+1 ");
      query.append("WHERE  bfakt_id=? ");

      ps = con.prepareStatement(getQuery());

      ps.setString    (1, vo.status_zahlg);
      ps.setBigDecimal(2, vo.fakt_betrag);
      ps.setString    (3, vo.kz_kalk);
      ps.setString    (4, vo.waehr_cd);
      ps.setDate      (5, toSqlDate(vo.status_datum));
      ps.setDate      (6, toSqlDate(vo.buch_datum));
      ps.setInt       (7, vo.bfakt_id);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Annuliert eine Rechnung (Zahlungsstatus wird auf 9 gesetzt).
   * Es dürfen nur Rechnungen aktualisiert werden, welche auf dem Zwischen-
   * status 'B' (SES, Annulation in Arbeit) sind.
   *
   * @param   vo            SAPRechnungStatusVO
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturAnnuliert(SAPRechnungStatusVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    status_zahlg=?, version=version+1 ");
      query.append("WHERE  bfakt_id=? ");
      query.append("AND    status_zahlg='B' ");

      ps = con.prepareStatement(getQuery());

      ps.setString(1, vo.status_zahlg);
      ps.setInt   (2, vo.bfakt_id);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Annuliert eine JGB Rechnung (kz_frist_abgelaufen wird auf 3 gesetzt).
   * Es dürfen nur Rechnungen aktualisiert werden, welche auf dem Zwischen-
   * status kz_frist_abgelaufen=2 sind.
   *
   * @param   vo            SAPRechnungStatusVO
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturAnnuliertJGB(SAPRechnungStatusVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    kz_frist_abgelaufen=3, version=version+1 ");
      query.append("WHERE  bfakt_id=? ");
      query.append("AND    kz_frist_abgelaufen=2 ");

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, vo.bfakt_id);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Methode zum Aktualisierung des Attributes STATUS_ZAHLG und der dazugehörigen
   * Attribute nach der durchgeführten Stornierung einer bereits bezahlten
   * Rechnung in SAP.
   *
   * @param   vo            SAPRechnungStatusVO
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturStorniert(SAPRechnungStatusVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    status_zahlg=?, storno_datum=?, version=version+1 ");
      query.append("WHERE  bfakt_id=? ");
      query.append("AND    status_zahlg='4' ");

      ps = con.prepareStatement(getQuery());
      ps.setString(1, vo.status_zahlg);
      ps.setDate  (2, toSqlDate(vo.status_datum));
      ps.setInt   (3, vo.bfakt_id);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisierung des Druckstatus und Datum einer Einzelrechnung.
   *
   * @param   vo            SAPRechnungStatusVO
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturGedruckt(SAPRechnungStatusVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    status_druck=?, bfakt_dat=?, version=version+1 ");
      query.append("WHERE  bfakt_id=? ");
      query.append("AND    status_druck != ? ");

      ps = con.prepareStatement(getQuery());
      ps.setString(1, vo.status_druck);
      ps.setDate  (2, toSqlDate(vo.bfakt_dat));
      ps.setInt   (3, vo.bfakt_id);
      ps.setString(4, vo.status_druck);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisierung des Druckstatus einer JGB Einzelrechnung. Im Unterschied
   * zur Einzelrechnung wird nur der Status aktualisiert und nicht das Druckdatum !
   *
   * @param   vo            SAPRechnungStatusVO
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturGedrucktJGB(SAPRechnungStatusVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    status_druck=?, version=version+1 ");
      query.append("WHERE  bfakt_id=? ");
      query.append("AND    status_druck != ? ");

      ps = con.prepareStatement(getQuery());
      ps.setString(1, vo.status_druck);
      ps.setInt   (2, vo.bfakt_id);
      ps.setString(3, vo.status_druck);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisierung des Druckstatus einer Rechnung.
   *
   * @param   bfaktId       Rechnungsnummer
   * @param   statusDruck   Druckstatus der Rechnung
   * @throws  SQLException  Unerwarteter Fehler
   * @return  int           Anzahl aktualisierte Rechnungen (row count)
   */
  public int updateBfakturDruckstatus(int bfaktId, String statusDruck) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bfaktur ");
      query.append("SET    status_druck=?, version=version+1 ");
      query.append("WHERE  bfakt_id=? ");

      ps = con.prepareStatement(getQuery());
      ps.setString(1, statusDruck);
      ps.setInt   (2, bfaktId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

} // end class
