package ch.ige.ses.server.dao.bagis;

import java.sql.*;
import ch.ige.util.server.*;

/**
 * DataAccessObject für Jahresgebühren - Statistik
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Patrick Bolzli
 * @version $Id: JoinJgbStatistikDAO.java,v 1.7 2006/12/11 12:43:32 iten Exp $
 */

public class JoinJgbStatistikDAO  extends BagisDAO
{
  /**
   * Gibt die Anzahl Fakturaanforderungen zurück.
   *
   * @param   minBfaktId    minimale Rechnungsnummer des Laufs
   * @param   maxBfaktId    maximale Rechnungsnummer des Laufs
   * @throws  SQLException  unerwarteter Fehler
   * @return  int           Anzahl Fakturaanforderungen
   */
  public int getCntFakturen(int minBfaktId, int maxBfaktId) throws SQLException
  {
    Connection  con   = null;
    Statement   stmt  = null;
    ResultSet   rs    = null;
    int         cnt   = 0;

    try
    {
      con = getConnection();

      stmt = con.createStatement();

      setQuery (
          "select count(distinct new_bfakt_id) as cnt " +
          "from " +
          "( " +
          "select  pp.new_bfakt_id " +
          "from    proc_pend pp, schutit_empfadr se, bfaktur b " +
          "where   pp.schutit_key > 0 " +
          "and     pp.process_nr = 10 " +
          "and     pp.new_bfakt_id between " + minBfaktId + " and "  + maxBfaktId + " " +
          "and     pp.schutit_key = se.schutit_key  " +
          "and     se.empf_typ in (1,4)  " +
          "and     pp.new_bfakt_id = b.bfakt_id " +
          "and     b.druck_art in ('3', '4')  " +
          "union " +
          "select  pp.new_bfakt_id " +
          "from    proc_pend pp, bfaktur b " +
          "where   pp.schutit_key > 0  " +
          "and     pp.process_nr = 10 " +
          "and     pp.new_bfakt_id between " + minBfaktId + " and "  + maxBfaktId + " " +
          "and     pp.new_bfakt_id = b.bfakt_id " +
          "and     b.druck_art in ('3','4') " +
          "and     b.adr_nr = 500304 " +
          ")" );

      rs = stmt.executeQuery(getQuery());

      if (rs.next())
      {
        cnt = rs.getInt("cnt");
      }

      return cnt;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  } // end method


  /**
   * Gibt die Anzahl Rechnungsempfänger zurück.
   *
   * @param   minBfaktId    minimale Rechnungsnummer des Laufs
   * @param   maxBfaktId    maximale Rechnungsnummer des Laufs
   * @throws  SQLException  unerwarteter Fehler
   * @return  int           Anzahl Rechnungsempfänger
   */
  public int getCntRechEmpf(int minBfaktId, int maxBfaktId) throws SQLException
  {
    Connection  con   = null;
    Statement   stmt  = null;
    ResultSet   rs    = null;
    int         cnt   = 0;

    try
    {
      con = getConnection();

      stmt = con.createStatement();

      setQuery (
          "select count(new_bfakt_id) as cnt  " +
          "from " +
          "( " +
          "select  distinct new_bfakt_id,adr_nr  " +
          "from  " +
          "( " +
          "select   pp.new_bfakt_id, se.adr_nr " +
          "from    proc_pend pp, schutit_empfadr se, bfaktur b " +
          "where   pp.schutit_key > 0 " +
          "and     pp.process_nr = 10 " +
          "and     pp.new_bfakt_id between " + minBfaktId + " and "  + maxBfaktId + " " +
          "and     pp.schutit_key = se.schutit_key  " +
          "and     se.empf_typ in (1,4)  " +
          "and     pp.new_bfakt_id = b.bfakt_id " +
          "and     b.druck_art in ('3', '4')  " +
          "union " +
          "select  pp.new_bfakt_id, b.adr_nr " +
          "from    proc_pend pp, bfaktur b " +
          "where   pp.schutit_key > 0  " +
          "and     pp.process_nr = 10 " +
          "and     pp.new_bfakt_id between " + minBfaktId + " and "  + maxBfaktId + " " +
          "and     pp.new_bfakt_id = b.bfakt_id " +
          "and     b.druck_art in ('3','4') " +
          "and     b.adr_nr = 500304 " +
          ") " +
          ")" );

      rs    = stmt.executeQuery(getQuery());

      if (rs.next())
      {
        cnt = rs.getInt("cnt");
      }
      return cnt;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  } // end method


  /**
   * Holt den Ländercode zu der angegebenen Adresse.
   * Beachte:
   * Der Ländercode ist vom Typ CHAR(3) und wird mit "trailling blanks"
   * zurückgegeben. Beispiel Schweiz = "CH ".
   *
   * @param   adrNr           Adresse
   * @throws  SQLException    Unerwarteter Fehler
   * @return  String          Ländercode zu dieser Adresse
   */
  public String getLandCd(int adrNr) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    String            lc  = null;

    try
    {
      con = getConnection();

      ps = con.prepareStatement
        ( "select land_cd " +
          "from   adresse " +
          "where  adr_nr =? " );
      ps.setInt(1,adrNr);

      rs = ps.executeQuery();

      if ( rs.next() )
      {
        lc = rs.getString("land_cd");
      }

      return lc;

    }
    finally
    {
      DBHelper.close(ps, con);
    }

  } // end method


}  // end class

