package ch.ige.ses.server.dao.bagis;

import java.sql.*;


import java.util.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.util.server.*;

/**
 * Datenbankzugriffe für die Beschaffung der Rechnungsdaten BAGIS -> SAP
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Pirmin Suter
 * @version $Id: JoinSAPRechnungDAO.java,v 1.23 2007/12/28 08:51:17 iten Exp $
 */
public class JoinSAPRechnungDAO extends BagisDAO
{
  /**
   * Zwischenspeicher für einen Teil des Beilagetextes. Der endgültige Text
   * wird aus mehreren DB Feldern zusammengesetzt!
   */
  private String fBeilageText = "";

  //---------------------------------------------------------------------------
  // Einzelrechnung
  //---------------------------------------------------------------------------

  /**
   * Selektiert Rechnungsdaten von Bagis (ausgehende von der Tabelle bfaktur).
   *
   * @param   bfaktId         Rechnungsnummer
   * @throws  Exception       Unerwarteter Fehler
   * @return  SAPRechnungVO   Rechnungsdaten oder Null
   */
  public SAPRechnungVO readRechnung (int bfaktId) throws Exception
  {
    SAPRechnungVO vo = null;


    vo = selectRechnungBasis(bfaktId);        // Basisdaten

    if (vo != null)
    {
      selectRechnungTexte(vo);                // Formulartexte
      selectRechnungAuftraggeber(vo);         // Auftraggeber
      selectRechnungZusatzEmpf(vo);           // Zusätzliche Empfänger eruieren
      selectRechnungPosition(vo);             // Rechnungspositionen eruieren
    }

    return vo;

  } // end selectRechnung


  //---------------------------------------------------------------------------

  private SAPRechnungVO selectRechnungBasis(int bfaktId) throws Exception
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    SAPRechnungVO     vo  = null;

    try
    {
      con = getConnection();

      setBfaktStmtRechnung();

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, bfaktId);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        vo = fillSapRechnungVO(rs);
      }

      return vo;

    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(ps, con);
    }
  }


  private void selectRechnungTexte(SAPRechnungVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;

    String            textart = null;
    String            text    = null;

    try
    {
      // init
      con = getConnection();

      resetQuery();
      query.append("SELECT  bft.textart as textart, ");
      query.append("        bft.text as text ");
      query.append("FROM    bformtxt_druck bft, ");
      query.append("        adresse adr, ");
      query.append("        bfaktur b ");
      query.append("WHERE   b.bfakt_id=? ");
      query.append("AND     b.formart_cd=bft.formart_cd ");
      query.append("AND     b.sprach_cd=bft.sprach_cd ");
      query.append("AND     b.adr_nr=adr.adr_nr ");
      query.append("AND     (bft.textart between '0' and '4' OR bft.textart='8')");
      query.append("ORDER BY 1");

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, vo.bfakt_id);

      // get data
      rs = ps.executeQuery();

      while (rs.next())
      {
        textart = rs.getString("textart");
        text    = rs.getString("text");

        // replace null with "" to avoid "null" text within the following
        // concatinations. note: NVL(bft.text,'') does not work, since oracle
        // replaces the '' with null!
        if (text == null)
        {
          text = "";
        }


        if (textart.equals("0"))
        {
          vo.abteilung = text;
        }
        else if (textart.equals("1"))
        {
          vo.kopfText = vo.kopfText + text;
        }
        else if (textart.equals("2"))
        {
          if(vo.kopfText.equals(""))
          {
            vo.kopfText = "<br>" + text;
          }
          else
          {
            vo.kopfText = vo.kopfText + "<br><br>" + text;
          }
        }
        else if (textart.equals("3"))
        {
          if(vo.kto_nr.equals(""))
          {
            vo.beilagenText = vo.beilagenText + text;
          }
        }
        else if (textart.equals("4"))
        {
          if ( ! vo.kto_nr.equals("") )
          {
            vo.beilagenText = vo.beilagenText + text;
          }
        }
        else if (textart.equals("8"))
        {
            vo.belehrungText = text;
        }
      }

      // Freier Zusatztext zu Beilagentext hinzufügen
      if ( ! fBeilageText.equals("") )
      {
        vo.beilagenText = vo.beilagenText + "<br><br>" + fBeilageText;
      }

      if(vo.abteilung.equals(""))
      {
        vo.abteilung = "Keine TEXTART = 0 in BFORMTXT_DRUCK für Formular gefunden!";
      }
      if(vo.kopfText.equals(""))
      {
        vo.kopfText = "Keine TEXTART 1 od. 2 in BFORMTXT_DRUCK für Formular gefunden!";
      }

    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(ps,con);
      // enable faster clean up from garbage collector
      textart = null;
      text    = null;
    }
  }


  /**
   * Auftraggeberrolle eruieren, wobei die Adressrolle AUFTRAGGEBER
   * wie folgt gefüllt wird:
   *
   *  1. Prio - ADR_NR zu Registeradresse mit Rolle X
   *            (X-Inhaber; hat immer eine Versandadresse)
   *  2. Prio - Falls 1.Prio nicht vorhanden,
   *            erste ADR_NR eines Inhabers (I-Inhaber)
   *  3. Prio - Falls 2.Prio nicht vorhanden,
   *            ADR_NR des Rechnungsempfängers
   *
   * @param   vo            Rechnungsdaten
   * @throws  SQLException  Unerwarteter Fehler
   */
  private void selectRechnungAuftraggeber(SAPRechnungVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;

    try
    {
      // init
      con = getConnection();

      setAuftraggeberStmt();
      ps = con.prepareStatement(getQuery());
      ps.setInt(1, vo.bfakt_id);

      // get data
      rs = ps.executeQuery();

      while(rs.next())
      {
        if (rs.getInt("adr_rolle_cd") == 1)
        {
          vo.auftraggeber = rs.getInt("adr_nr");
          break;
        }
        else if (rs.getInt("adr_rolle_cd") == 2)
        {
          vo.auftraggeber = rs.getInt("adr_nr");
          break;
        }
        else
        {
          vo.auftraggeber = vo.hauptRechEmpf;
          break;
        }
      }

      if (vo.auftraggeber <= 0)
      {
        vo.auftraggeber = vo.hauptRechEmpf;
      }

    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(ps,con);
    }
  }


  private void selectRechnungZusatzEmpf(SAPRechnungVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;

    try
    {
      // init
      con = getConnection();

      resetQuery();
      query.append("SELECT  adr_nr ");
      query.append("FROM    pot_zahl ");
      query.append("WHERE   bfakt_id=? ");
      query.append("ORDER BY 1");

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, vo.bfakt_id);

      // get data
      rs = ps.executeQuery();
      int i = 0;

      while(rs.next())
      {
        if (i <  SAPRechnungVO.MAX_RECH_EMPF)
        {
          vo.zusatzRechEmpf.add( new Integer(rs.getInt("adr_nr")) );
        }
        else
        {
          vo.zusatzRechEmpfNichtVerarbeitet.add( new Integer(rs.getInt("adr_nr")) );
        }

        i++;
      } // end while
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(ps,con);
    }
  }


  private void selectRechnungPosition(SAPRechnungVO vo) throws Exception
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;

    try
    {
      // init
      if (vo.posList == null)
      {
        vo.posList = new ArrayList();
      }

      // get data
      con = getConnection();

      setBfaktPosStmtRechnung();

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, vo.bfakt_id);

      rs = ps.executeQuery();

      while (rs.next())
      {
        SAPRechnungsPositionVO posVO = fillSapRechnungsPositionsVO(rs);
        posVO.bfakt_id = vo.bfakt_id;
        vo.posList.add(posVO);
      }

    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(ps,con);
    }
  }

  //---------------------------------------------------------------------------
  // JGB Rechnung
  //---------------------------------------------------------------------------

  /**
   * Selektiert Rechnungsdaten von Bagis (ausgehende von der Tabelle bfaktur)
   * für die Jahresgebührverarbeitung
   *
   * @param   bfaktIdVon      Beginn der zu selektierenden Range
   * @param   bfaktIdBis      Ende der zu selektierenden Range
   * @throws  Exception       Unerwarteter Fehler
   * @return  ArrayList       List of SAPRechnungVO's
   */
  public ArrayList readJGB(int bfaktIdVon, int bfaktIdBis) throws Exception
  {
    Connection  con                   = null;

    PreparedStatement ps              = null;
    PreparedStatement psAuftraggeber  = null;
    PreparedStatement psEmpf          = null;
    PreparedStatement psBfaktPos      = null;

    ResultSet rs                      = null;
    ResultSet rsAuftraggeber          = null;
    ResultSet rsEmpf                  = null;
    ResultSet rsBfaktPos              = null;

    SAPRechnungVO sapRechnungVO       = null;
    ArrayList     alSAPJgbParamVO     = new ArrayList();
    ArrayList     al                  = new ArrayList();


    try
    {
      //-----------------------------------------------------------------------
      // init
      //-----------------------------------------------------------------------

      con = getConnection();


      //-----------------------------------------------------------------------
      // bfaktur
      //-----------------------------------------------------------------------

      setBfaktStmtJGB();                // store sql statement into query

      ps = con.prepareStatement(getQuery());
      ps.setInt(1, bfaktIdVon);
      ps.setInt(2, bfaktIdBis);

      rs = ps.executeQuery();

      while(rs.next())
      {
        SAPRechnungVO rechnungVO = new SAPRechnungVO();
        rechnungVO = fillSapRechnungVO(rs);
        al.add(rechnungVO);
      }

      DBHelper.close(rs);
      DBHelper.close(ps);

      //-----------------------------------------------------------------------
      // formulartexte
      //-----------------------------------------------------------------------

      HashMap           hash              = new HashMap();
      ArrayList         alFormtxt         = new ArrayList();
      BformtxtDruckDAO  bFormtxtDruckDAO  = new BformtxtDruckDAO();
      BformtxtDruckVO   bFormtxtDruckVO   = null;

      // select
      alFormtxt = bFormtxtDruckDAO.selectForJGB();

      // Textarten in HashMap einlesen
      // (key = formart_cd + "&" + sprach_cd  + "&" + textart)
      for (int i = 0; i < alFormtxt.size(); i++)
      {
        bFormtxtDruckVO = (BformtxtDruckVO)alFormtxt.get(i);
        hash.put(bFormtxtDruckVO.formartCd + "&" + bFormtxtDruckVO.sprachCd + "&" + bFormtxtDruckVO.textart, bFormtxtDruckVO);
      }

      // formtexte
      for (int i = 0; i < al.size(); i++)
      {
        sapRechnungVO = (SAPRechnungVO)al.get(i);

        if(hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "0") != null)
        {
          ((SAPRechnungVO)al.get(i)).abteilung = ((BformtxtDruckVO)hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "0")).text;
        }
        if(hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "1") != null)
        {
          ((SAPRechnungVO)al.get(i)).kopfText = ((BformtxtDruckVO)hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "1")).text;
        }
        if(hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "2") != null)
        {
          if(sapRechnungVO.kopfText != "")
          {
            ((SAPRechnungVO)al.get(i)).kopfText = sapRechnungVO.kopfText + "<br><br>" + ((BformtxtDruckVO)hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "2")).text;
          }
          else
          {
            ((SAPRechnungVO)al.get(i)).kopfText = sapRechnungVO.kopfText + "<br>" + ((BformtxtDruckVO)hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "2")).text;
          }
        }
        if(hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "3") != null)
        {
          if(sapRechnungVO.kto_nr.equals(""))
          {
            ((SAPRechnungVO)al.get(i)).beilagenText = sapRechnungVO.beilagenText + ((BformtxtDruckVO)hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "3")).text;
          }
        }
        if(hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "4") != null)
        {
          if(!sapRechnungVO.kto_nr.equals(""))
          {
            ((SAPRechnungVO)al.get(i)).beilagenText = sapRechnungVO.beilagenText + ((BformtxtDruckVO)hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "4")).text;
          }
        }
        if(hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "8") != null)
        {
          if(((BformtxtDruckVO)hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "8")).text != null)
          {
            ((SAPRechnungVO)al.get(i)).belehrungText = ((BformtxtDruckVO)hash.get(sapRechnungVO.formart_cd + "&" + sapRechnungVO.sprach_cd + "&" + "8")).text;
          }
        }

        //
        // Freier Zusatztext zu Beilagentext hinzufügen
        //
        if (fBeilageText != "")
        {
          ((SAPRechnungVO)al.get(i)).beilagenText = ((SAPRechnungVO)al.get(i)).beilagenText + "<br><br>" + fBeilageText;
        }
        if(((SAPRechnungVO)al.get(i)).abteilung.equals(""))
        {
          ((SAPRechnungVO)al.get(i)).abteilung = "Keine TEXTART = 0 in BFORMTXT_DRUCK für Formular gefunden!";
        }
        if(((SAPRechnungVO)al.get(i)).kopfText.equals(""))
        {
          ((SAPRechnungVO)al.get(i)).kopfText = "Keine TEXTART 1 od. 2 in BFORMTXT_DRUCK für Formular gefunden!";
        }

      } // end for formtexte


      //-----------------------------------------------------------------------
      // Auftraggeberrolle eruieren
      // Die Adressrolle AUFTRAGGEBER wird wie folgt gefüllt:
      //  1. Prio - ADR_NR zu Registeradresse mit Rolle X
      //            (X-Inhaber; hat immer eine Versandadresse)
      //  2. Prio - Falls 1.Prio nicht vorhanden,
      //            erste ADR_NR eines Inhabers (I-Inhaber)
      //  3. Prio - Falls 2.Prio nicht vorhanden,
      //            ADR_NR des Rechnungsempfängers
      //-----------------------------------------------------------------------

      for (int i = 0; i < al.size(); i++)
      {
        setAuftraggeberStmt();
        psAuftraggeber = con.prepareStatement(getQuery());
        psAuftraggeber.setInt(1, ((SAPRechnungVO)al.get(i)).bfakt_id);
        rsAuftraggeber = psAuftraggeber.executeQuery();

        while(rsAuftraggeber.next())
        {
          if (rsAuftraggeber.getInt("adr_rolle_cd") == 2)
          {
            ((SAPRechnungVO)al.get(i)).auftraggeber = rsAuftraggeber.getInt("adr_nr");
            break;
          }
          else if (rsAuftraggeber.getInt("adr_rolle_cd") == 1)
          {
            ((SAPRechnungVO)al.get(i)).auftraggeber = rsAuftraggeber.getInt("adr_nr");
            break;
          }
          else
          {
            ((SAPRechnungVO)al.get(i)).auftraggeber = ((SAPRechnungVO)al.get(i)).hauptRechEmpf;
            break;
          }
        }

        if(((SAPRechnungVO)al.get(i)).auftraggeber == 0)
        {
         ((SAPRechnungVO)al.get(i)).auftraggeber = ((SAPRechnungVO)al.get(i)).hauptRechEmpf;
        }

        DBHelper.close(rsAuftraggeber);
        DBHelper.close(psAuftraggeber);

      } // end for

      //-----------------------------------------------------------------------
      // Haupt- und Zusatzempfänger eruieren
      //-----------------------------------------------------------------------

      for (int i = 0; i < al.size(); i++)
      {
        resetQuery();

        query.append("SELECT ");
        query.append("b.adr_nr AS adr_nr, ");
        query.append("NVL(c.druckperi,'0') AS druckperi, ");
        query.append("NVL(c.listenart,'0') AS listenart, ");
        query.append("c.fach_nr, ");
        query.append("NVL(c.sort_cd,'0') AS sort_cd, ");
        query.append("b.druck_art AS druck_art ");
        query.append("FROM   bfaktur b LEFT JOIN faktpar c ON b.adr_nr = c.adr_nr ");
        query.append("WHERE  b.bfakt_id = ? ");

        query.append("UNION ALL ");

        query.append("SELECT ");
        query.append("pz.adr_nr AS adr_nr, ");
        query.append("NVL(c.druckperi,'0') AS druckperi, ");
        query.append("NVL(c.listenart,'0') AS listenart, ");
        query.append("c.fach_nr,  ");
        query.append("NVL(c.sort_cd,'0') AS sort_cd,  ");
        query.append("b.druck_art AS druck_art  ");
        query.append("FROM  bfaktur b, ");
        query.append("      pot_zahl pz LEFT JOIN faktpar c ON pz.adr_nr = c.adr_nr ");
        query.append("WHERE b.bfakt_id = ? ");
        query.append("AND   b.bfakt_id = pz.bfakt_id ");

        psEmpf = con.prepareStatement(getQuery());
        psEmpf.setInt(1, ((SAPRechnungVO)al.get(i)).bfakt_id);
        psEmpf.setInt(2, ((SAPRechnungVO)al.get(i)).bfakt_id);

        rsEmpf = psEmpf.executeQuery();
        alSAPJgbParamVO.clear();

        int j = 0;
        while(rsEmpf.next())
        {
          SAPJGBParamVO sapJgbParamVO = new SAPJGBParamVO();
          sapJgbParamVO.bfakt_id      = ((SAPRechnungVO)al.get(i)).bfakt_id;
          sapJgbParamVO.adr_nr        = rsEmpf.getInt("adr_nr");
          sapJgbParamVO.druckperi     = rsEmpf.getString("druckperi");
          sapJgbParamVO.listenart     = rsEmpf.getString("listenart");
          sapJgbParamVO.fach_nr       = rsEmpf.getInt("fach_nr");
          sapJgbParamVO.druck_art     = rsEmpf.getString("druck_art");

          if(! sapJgbParamVO.druckperi.equals("9"))
          {
            sapJgbParamVO.fach_nr = 0;
          }
          // Attribut adr_nr_vtr
          if(rsEmpf.getString("sort_cd").equals("0") || rsEmpf.getString("sort_cd").equals("3"))
          {
            sapJgbParamVO.adr_nr_vtr = 0;
          }
          if(rsEmpf.getString("sort_cd").equals("1") || rsEmpf.getString("sort_cd").equals("2")
             || rsEmpf.getString("sort_cd").equals("4"))
          {
            sapJgbParamVO.adr_nr_vtr = ((SAPRechnungVO)al.get(i)).hauptRechEmpf;
          }
          // Attribut sort_text
          if(rsEmpf.getString("sort_cd").equals("0") || rsEmpf.getString("sort_cd").equals("1"))
          {
            sapJgbParamVO.sort_text = ((SAPRechnungVO)al.get(i)).ihr_zeichen;
          }
          if(rsEmpf.getString("sort_cd").equals("2"))
          {
            sapJgbParamVO.sort_text = ((SAPRechnungVO)al.get(i)).inhaber;
          }
          if(rsEmpf.getString("sort_cd").equals("3") || rsEmpf.getString("sort_cd").equals("4"))
          {
            sapJgbParamVO.sort_text = ((SAPRechnungVO)al.get(i)).schutit_nr;
          }

          // Falls mehr als MAX Zusatzempfänger sep. speichern und
          // dann in ses_error schreiben
          if(j <= SAPRechnungVO.MAX_RECH_EMPF)
          {
            //erster Datensatz ist der Hauptempfänger
            if(j == 0)
            {
              ((SAPRechnungVO)al.get(i)).jgbHauptRechEmpfParam = sapJgbParamVO;
            }
            else
            {
              ((SAPRechnungVO)al.get(i)).zusatzRechEmpf.add(new Integer(sapJgbParamVO.adr_nr));
              ((SAPRechnungVO)al.get(i)).jgbZusatzRechEmpfParam.add(sapJgbParamVO);
            }
          }
          else
          {
            ((SAPRechnungVO)al.get(i)).zusatzRechEmpfNichtVerarbeitet.add(new Integer(sapJgbParamVO.adr_nr));
          }
          j++;
        }

        DBHelper.close(rsEmpf);
        DBHelper.close(psEmpf);

      } // end for

      //-----------------------------------------------------------------------
      // Rechnungspositionen eruieren
      //-----------------------------------------------------------------------

      for (int i = 0; i < al.size(); i++)
      {
        ArrayList alBfaktpos = new ArrayList();

        setBfaktPosStmtJGB();
        psBfaktPos = con.prepareStatement(getQuery());
        psBfaktPos.setInt(1, ((SAPRechnungVO)al.get(i)).bfakt_id);
        rsBfaktPos = psBfaktPos.executeQuery();

        while (rsBfaktPos.next())
        {
          SAPRechnungsPositionVO positionVO = new SAPRechnungsPositionVO();
          positionVO = fillSapRechnungsPositionsVO(rsBfaktPos);
          positionVO.bfakt_id = ((SAPRechnungVO)al.get(i)).bfakt_id;
          alBfaktpos.add(positionVO);
        }
        ((SAPRechnungVO)al.get(i)).posList = alBfaktpos;

        DBHelper.close(rsBfaktPos);
        DBHelper.close(psBfaktPos);

      }	// end for


    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(rsAuftraggeber);
      DBHelper.close(rsEmpf);
      DBHelper.close(rsBfaktPos);

      DBHelper.close(ps, con);
      DBHelper.close(psAuftraggeber);
      DBHelper.close(psEmpf);
      DBHelper.close(psBfaktPos);
    }

    //
    // return list with rechnung data
    //
    return al;
  }

  //---------------------------------------------------------------------------
  // private methods - fill vo's
  //---------------------------------------------------------------------------

  /**
   * Füllt SAPRechnungsPositionVO Rechnungspositionsdaten
   *
   * @param  rs                     ResultSet
   * @return SAPRechnungsPositionVO ValueObject mit Rechnungspositionen
   * @throws Exception              Unerwarteter Fehler
   */
  private SAPRechnungsPositionVO fillSapRechnungsPositionsVO(ResultSet rs) throws Exception
  {
    SAPRechnungsPositionVO vo = new SAPRechnungsPositionVO();


    vo.bfakt_posnr = rs.getInt("bfakt_posnr");

    if(rs.getInt("anzahl") > 0)
    {
      vo.anzahl = rs.getInt("anzahl");
    }

    if(rs.getString("art_nr") != null)
    {
      vo.art_nr = rs.getString("art_nr");
    }

    if(rs.getString("bez_art") != null)
    {
      vo.artKurztext = rs.getString("bez_art");
    }

    vo.artZusatzText = fillBfaktText( rs.getString("Zusatztext"),
                                      getClobAsString(rs,"ZusatztextLong") );

    if(rs.getInt("schutit_key") > 0)
    {
      vo.schutit_key = rs.getInt("schutit_key");
    }

    if(rs.getString("lageinh") != null)
    {
      vo.lageinh = rs.getString("lageinh");
    }

    if(rs.getBigDecimal("preis") != null)
    {
      vo.preis = rs.getBigDecimal("preis");
    }

    if(rs.getBigDecimal("wert") != null)
    {
      vo.wert = rs.getBigDecimal("wert");
    }

    if (rs.getDate("dat_ereignis") != null)
    {
      vo.dat_ereignis = rs.getDate("dat_ereignis");
    }

    if (rs.getDate("frist") != null)
    {
      vo.frist = rs.getDate("frist");
    }

    if(rs.getString("schutit_nr") != null)
    {
      vo.schutit_nr = rs.getString("schutit_nr");
    }

    return vo;
  }


  /**
   * Füllt SAPRechnungVO mit Rechnungsdaten
   *
   * @param   rs            RecordSet
   * @throws  Exception     Unerwarteter Fehler
   * @return  SAPRechnungVO Bfaktur
   */
  private SAPRechnungVO fillSapRechnungVO(ResultSet rs) throws Exception
  {
    SAPRechnungVO vo = new SAPRechnungVO();

    vo.bfakt_id = rs.getInt("bfakt_id");

    if(rs.getString("schutit_nr") != null)
    {
      vo.schutit_nr = rs.getString("schutit_nr");
    }

    if(rs.getString("unt_name") != null)
    {
      vo.unt_name = rs.getString("unt_name");
    }

    if(rs.getDate("verfall_dat") != null)
    {
      vo.verfall_dat = rs.getDate("verfall_dat");
    }

    if(rs.getString("inhaber") != null)
    {
      vo.inhaber = rs.getString("inhaber");
    }

    if(rs.getDate("bfakt_dat") != null)
    {
      vo.bfakt_dat = rs.getDate("bfakt_dat");
    }

    if(rs.getInt("adr_nr") > 0)
    {
      vo.hauptRechEmpf = rs.getInt("adr_nr");
    }

    if(rs.getString("beschreibung") != null)
    {
      vo.titel = rs.getString("beschreibung");
    }

    vo.kontaktPerson = fillBfaktText ( rs.getString("kontakt_person"),
                                       getClobAsString(rs,"kontakt_person_long")  );

    vo.diversText = fillBfaktText ( rs.getString("divers_text"),
                                    getClobAsString(rs,"divers_text_long") );

    if(rs.getString("kz_einschr") != null)
    {
      vo.kz_einschr = rs.getString("kz_einschr");
    }

    if(rs.getString("var_text") != null)
    {
      vo.var_text = rs.getString("var_text");
    }

    fBeilageText = fillBfaktText ( rs.getString("beilage_text"),
                                   getClobAsString(rs,"beilage_text_long") );

    if(rs.getString("kto_nr") != null)
    {
      vo.kto_nr = rs.getString("kto_nr");
    }

    if(rs.getObject("employee_id") != null)
    {
      Integer i = new Integer(rs.getInt("employee_id"));
      vo.employeeId = i.toString();
    }

    if(rs.getString("sprach_cd") != null)
    {
      vo.sprach_cd = rs.getString("sprach_cd");
    }

    if(rs.getString("druck_art") != null)
    {
      vo.druck_art = rs.getString("druck_art");
    }

    if(rs.getString("unsere_ref") != null)
    {
      vo.unsere_ref = rs.getString("unsere_ref");
    }

    if(rs.getString("tel_nr") != null)
    {
      vo.tel_nr = rs.getString("tel_nr");
    }

    if(rs.getDate("faellig_dat") != null)
    {
      vo.faellig_dat = rs.getDate("faellig_dat");
    }

    if(rs.getString("regulierer") != null)
    {
      vo.regulierer = new Integer(rs.getString("regulierer")).intValue();
    }

    if(rs.getInt("schutit_typ") > 0)
    {
      vo.schutit_typ = rs.getInt("schutit_typ");
    }

    if(rs.getString("schutit_typ_bez") != null)
    {
      vo.schutit_typ_bez = rs.getString("schutit_typ_bez");
    }

    if(rs.getString("ihr_zeichen") != null)
    {
      vo.ihr_zeichen = rs.getString("ihr_zeichen");
    }

    if(rs.getString("formart_cd") != null)
    {
      vo.formart_cd = rs.getString("formart_cd");
    }

    if(rs.getDate("schutz_beginn") != null)
    {
      vo.schutz_beginn = rs.getDate("schutz_beginn");
    }

    return vo;
  }


  /**
   * Die Tabelle bfaktxt hat die beiden Felder text und text_long. Je nachdem
   * wie gross der Rechnungstext ist, befindet er sich in einem der beiden
   * Felder. Diese Methode ermittelt den korrekten Text.
   *
   * Folgende Scenarien sind denkbar:
   *
   *    text          text_long     return Wert (Anzeige auf Rechnung)
   *    ---------------------------------------------------------
   *    null          null          default ("")
   *    null          xyz...        xyz...
   *
   *    $LONGVARCHAR  null          default ("")
   *    $LONGVARCHAR  xyz...        xyz...
   *
   *    abc...	      null          abc...
   *    abc...	      xyz...        abc...
   *
   * @param   text      Text normal
   * @param   textLong  Text gross
   * @return  String    Ermittelter Wert zur Anzeige auf der Rechnung
   */
  private String fillBfaktText (String text, String textLong)
  {
    String bfaktText = "";                            // default value

    if (text == null)
    {
      if (textLong != null)
      {
        bfaktText = textLong;
      }
    }
    else if (text.equalsIgnoreCase("$LONGVARCHAR"))
    {
      if (textLong != null)
      {
        bfaktText = textLong;
      }
    }
    else
    {
      bfaktText = text;
    }

    return bfaktText;
  }

  //---------------------------------------------------------------------------
  // private methods - get sql statement - auftraggeber
  //---------------------------------------------------------------------------

  /**
   * Liefert SQL-Statement für die Auftraggeber zu einer Rechnung
   */
  private void setAuftraggeberStmt()
  {
    resetQuery();

    query.append("SELECT sr.adr_rolle_cd as adr_rolle_cd, ");   // number, not null
    query.append("       a.adr_nr as adr_nr ");                 // number, not null

    query.append("FROM   bfaktur b, ");
    query.append("       schutit_regadr sr, ");
    query.append("       regadr r, ");
    query.append("       adresse a ");

    query.append("WHERE  b.schutit_key=sr.schutit_key ");
    query.append("AND    sr.regadr_nr=r.regadr_nr ");
    query.append("AND    r.adr_nr = a.adr_nr ");
    query.append("AND    (a.gesperrt=0 OR a.gesperrt IS NULL) ");
    query.append("AND    a.suchbegr != '**********' ");
    query.append("AND    b.bfakt_id=? ");
    query.append("AND    sr.adr_rolle_cd IN (1,2) ");
    query.append("AND    sr.bis_ereig_idat is null ");

    query.append("ORDER BY 1,2");
  }

  //---------------------------------------------------------------------------
  // private methods - set sql statement - bfaktur
  //---------------------------------------------------------------------------

  /**
   * Setzt die SQL Abfrage (select und from) für Einzel- oder JGB - Rechnung.
   *
   * Bemerkung SCHUTIT_KEY:
   *  Bei schutztitelabhängigen Rechnung ist das Attribut SCHUTIT_KEY leer,
   *  deshalb muss mit einem LEFT JOIN auf die Tabelle SCHUTIT zugegriffen
   *  werden!
   */
  private void setBfaktStmt ()
  {
    //
    // init
    //
    resetQuery();

    //
    // select
    //
    query.append("SELECT ");
    query.append("bfakt_id, ");                                   // number, NOT NULL
    query.append("trim(b.schutit_nr) as schutit_nr, ");           // char(12), nullable
    query.append("trim(b.unt_name) as unt_name, ");               // char(30), nullable
    query.append("b.verfall_dat, ");                              // date, nullable
    query.append("b.inhaber, ");                                  // varchar2, nullable
    query.append("b.bfakt_dat, ");                                // date, nullable
    query.append("b.adr_nr, ");                                   // number, nullable
    query.append("s.beschreibung, ");                             // varchar2, nullable
    // TODO remove after R1/2008 (bag_drucker)
    // query.append("trim(m.bag_drucker) as bag_drucker, ");      // char(8), nullable
    query.append(" m.employee_id, ");                             // number, nullable

    query.append("bx.text as beilage_text, ");                    // varchar2, nullable
    query.append("bx.text_long as beilage_text_long, ");          // clob, nullable
    query.append("bxk.text as kontakt_person, ");                 // varchar2, nullable
    query.append("bxk.text_long as kontakt_person_long, ");       // clob, nullable
    query.append("bxk2.text as divers_text, ");                   // varchar2, nullable
    query.append("bxk2.text_long as divers_text_long, ");         // clob, nullable

    query.append("trim(b.kz_einschr) as kz_einschr, ");           // char(1), nullable
    query.append("b.var_text, ");                                 // varchar2, nullable
    query.append("b.kto_nr, ");                                   // number, nullable
    query.append("trim(b.sprach_cd) as sprach_cd, ");             // char(3), nullable
    query.append("trim(b.druck_art) as druck_art, ");             // char(1), nullable
    query.append("trim(b.unsere_ref) as unsere_ref, ");           // char(25), nullable
    query.append("trim(b.tel_nr) as tel_nr, ");                   // char(20), nullable
    query.append("b.faellig_dat, ");                              // date, nullable
    query.append("b.kto_nr as regulierer, ");                     // number, nullable
    query.append("b.schutit_typ, ");                              // number, nullable
    query.append("trim(b.schutit_typ_bez) as schutit_typ_bez, "); // char(4), nullable, neu mit trim !
    query.append("trim(b.ihr_zeichen) as ihr_zeichen, ");         // char(30), nullable, neu mit trim !
    query.append("trim(b.formart_cd) as formart_cd, ");           // char(3), nullable, neu mit trim !
    query.append("s.schutz_beginn ");                             // date, nullable

    //
    // from
    //
    query.append("FROM bfaktur b LEFT JOIN bfaktxt bx   on b.txt_beilage = bx.text_nr ");
    query.append("               LEFT JOIN bfaktxt bxk  on b.txt_adr_zusatz = bxk.text_nr ");
    query.append("               LEFT JOIN bfaktxt bxk2 on b.txt_schluss = bxk2.text_nr ");
    query.append("               LEFT JOIN schutit s    on b.schutit_key = s.schutit_key, ");
    query.append("     ma m ");
  }


  /**
   * Liefert SQL Abfrage für Einzelrechnung, ausgeführt als PreparedStatement !
   *
   * @param   bfaktId   Rechnungs Nummer
   */
  private void setBfaktStmtRechnung()
  {
    // select & from
    setBfaktStmt();

    // where clause
    query.append("WHERE b.bfakt_id = ? ");
    query.append("AND   b.ma_nr=m.ma_nr");
  }


  /**
   * Liefert SQL Abfrage JGB - Rechnung, ausgeführt als PreparedStatement !
   */
  private void setBfaktStmtJGB()
  {
    // select & from
    setBfaktStmt();

    // where clause
    query.append("WHERE b.bfakt_id BETWEEN ? AND ? ");
    query.append("AND   b.ma_nr = m.ma_nr ");
    query.append("AND   (b.druck_art = '3' OR b.druck_art = '4') ");
    query.append("AND   b.status_druck = '3' ");
    query.append("AND   b.status_zahlg = '1' ");
    query.append("ORDER BY b.bfakt_id");
  }

  //---------------------------------------------------------------------------
  // private methods - set sql statement - bfaktpos
  //---------------------------------------------------------------------------

  /**
   * Liefert SQL Abfrage (select und from) für Einzel- oder
   * JGB - Rechnungspositionen.
   */
  private void setBfaktPosStmt()
  {
    //
    // init
    //
    resetQuery();

    //
    // select
    //
    query.append("SELECT ");
    query.append("b.bfakt_posnr, ");                              // number, not null
    query.append("b.anzahl, ");                                   // number, nullable
    query.append("trim(b.art_nr) as art_nr, ");                   // char(18), nullable
    query.append("c.bez_art, ");                                  // varchar2, nullable
    query.append("bx.text as Zusatztext, ");                      // varchar2
    query.append("bx.text_long as ZusatztextLong, ");             // clob, nullable
    query.append("b.schutit_key, ");                              // number, nullable
    query.append("trim(e.lageinh) AS lageinh, ");                 // char(3), nullable
    query.append("b.preis, ");                                    // number, nullable
    query.append("b.wert, ");                                     // number, nullable
    query.append("b.dat_ereignis, ");                             // date, nullable
    query.append("b.frist, ");                                    // date, nullable
    query.append("trim(b.schutit_nr) as schutit_nr, ");           // char(12), nullable
    query.append("trim(b.arttextart_cd) as arttextart_cd, ");     // char(3), nullable
    query.append("trim(f.sprach_cd) as sprach_cd ");              // char(3), nullable

    //
    // from
    //
    query.append("FROM ");
    query.append("bfaktpos b LEFT JOIN bfaktxt bx ON b.text_nr=bx.text_nr, ");
    query.append("artbez   c, ");
    query.append("artikel  e, ");
    query.append("bfaktur  f, ");
    query.append("adresse  g  ");
  }


  /**
   * Liefert SQL Abfrage für Einzelrechnung, ausgeführt als PreparedStatement !
   */
  private void setBfaktPosStmtRechnung()
  {
    // select & from
    setBfaktPosStmt();

    // where clause
    query.append("WHERE b.bfakt_id = ? ");
    query.append("AND   b.art_nr = c.art_nr ");
    query.append("AND   b.bfakt_id = f.bfakt_id ");
    query.append("AND   f.adr_nr = g.adr_nr ");
    query.append("AND   b.art_nr = e.art_nr ");
  }


  /**
   * Liefert SQL Abfrage für JGB, ausgeführt als PreparedStatement !
   */
  private void setBfaktPosStmtJGB()
  {
    // select & from
    setBfaktPosStmt();

    // where clause
    query.append("WHERE b.bfakt_id = ? ");
    query.append("AND   b.art_nr   = c.art_nr ");
    query.append("AND   b.bfakt_id = f.bfakt_id ");
    query.append("AND   f.adr_nr   = g.adr_nr ");
    query.append("AND   b.art_nr   = e.art_nr ");
  }

} // end class
