package ch.ige.ses.server.dao.ses;

import java.sql.*;
import java.util.ArrayList;

import ch.ige.ses.vo.ses.StakoKontrolleVO;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt für die Tabelle STAKO_KONTROLLE
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Stefan Nydegger
 * @version $Id: StakoKontrolleDAO.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */

public class StakoKontrolleDAO
{

  public StakoKontrolleDAO()
  {
    // lazy bone
  }

  /**
   * Selektierte alle Einträge der Tablle STAKO_KONTROLLE
   *
   * @return ArrayList      ArrayList mit Value Objects StakoKontrolleVO
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public ArrayList selectAll() throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    ArrayList al = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "SELECT stako_kontrolle_id, verarbeitung_dat, von_dat, bis_dat, " +
          "anzahl_ereignisma_gelesen, anzahl_bfaktur_gelesen, " +
          "anzahl_ereignisma_an_sap, anzahl_bfaktur_an_sap, " +
          "anzahl_ereignisma_an_sap_ok, anzahl_bfaktur_an_sap_ok, " +
          "stako_stat, herkunft_stat, created_dat, changed_dat " +
          "FROM SES.STAKO_KONTROLLE " +
          "ORDER BY stako_kontrolle_id DESC");

      rs = ps.executeQuery();

      while(rs.next())
      {
        StakoKontrolleVO  stakoKontrolleVO = new StakoKontrolleVO();

        stakoKontrolleVO.stakoKontrolleId         = rs.getInt("stako_kontrolle_id");
        stakoKontrolleVO.verarbeitungDat         = rs.getDate("verarbeitung_dat");
        stakoKontrolleVO.vonDat                  = rs.getDate("von_dat");
        stakoKontrolleVO.bisDat                  = rs.getDate("bis_dat");
        stakoKontrolleVO.anzahlEreignisMaGelesen = rs.getInt("anzahl_ereignisma_gelesen");
        stakoKontrolleVO.anzahlBfakturGelesen    = rs.getInt("anzahl_bfaktur_gelesen");
        stakoKontrolleVO.anzahlEreignisMaAnSap   = rs.getInt("anzahl_ereignisma_an_sap");
        stakoKontrolleVO.anzahlBfakturAnSap      = rs.getInt("anzahl_bfaktur_an_sap");
        stakoKontrolleVO.anzahlEreignisMaAnSapOK = rs.getInt("anzahl_ereignisma_an_sap_ok");
        stakoKontrolleVO.anzahlBfakturAnSapOK    = rs.getInt("anzahl_bfaktur_an_sap_ok");
        stakoKontrolleVO.stakoStat               = rs.getInt("stako_stat");
        stakoKontrolleVO.herkunftStat            = rs.getInt("herkunft_stat");
        stakoKontrolleVO.createdDat              = rs.getTimestamp("created_dat") == null ?
                                                   null : new Date(rs.getTimestamp("created_dat").getTime());
        stakoKontrolleVO.changedDat              = rs.getTimestamp("changed_dat") == null ?
                                                   null : new Date(rs.getTimestamp("changed_dat").getTime());
        al.add(stakoKontrolleVO);
      }
      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektierte alle Einträge der Tabelle STAKO_KONTROLLE die älter sind als
   * das angegebene Erstellungsdatum.
   *
   * @param  created        Erstellungsdatum
   * @return ArrayList      ArrayList mit Value Objects StakoKontrolleVO
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public ArrayList select(java.util.Date created) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    ArrayList al = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "SELECT stako_kontrolle_id, verarbeitung_dat, von_dat, bis_dat, " +
          "anzahl_ereignisma_gelesen, anzahl_bfaktur_gelesen, " +
          "anzahl_ereignisma_an_sap, anzahl_bfaktur_an_sap, " +
          "anzahl_ereignisma_an_sap_ok, anzahl_bfaktur_an_sap_ok, " +
          "stako_stat, herkunft_stat, created_dat, changed_dat " +
          "FROM SES.STAKO_KONTROLLE " +
          "WHERE CREATED_DAT < ? " +
          "ORDER BY stako_kontrolle_id DESC");

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );
      rs = ps.executeQuery();

      while(rs.next())
      {
        StakoKontrolleVO  stakoKontrolleVO = new StakoKontrolleVO();

        stakoKontrolleVO.stakoKontrolleId         = rs.getInt("stako_kontrolle_id");
        stakoKontrolleVO.verarbeitungDat         = rs.getDate("verarbeitung_dat");
        stakoKontrolleVO.vonDat                  = rs.getDate("von_dat");
        stakoKontrolleVO.bisDat                  = rs.getDate("bis_dat");
        stakoKontrolleVO.anzahlEreignisMaGelesen = rs.getInt("anzahl_ereignisma_gelesen");
        stakoKontrolleVO.anzahlBfakturGelesen    = rs.getInt("anzahl_bfaktur_gelesen");
        stakoKontrolleVO.anzahlEreignisMaAnSap   = rs.getInt("anzahl_ereignisma_an_sap");
        stakoKontrolleVO.anzahlBfakturAnSap      = rs.getInt("anzahl_bfaktur_an_sap");
        stakoKontrolleVO.anzahlEreignisMaAnSapOK = rs.getInt("anzahl_ereignisma_an_sap_ok");
        stakoKontrolleVO.anzahlBfakturAnSapOK    = rs.getInt("anzahl_bfaktur_an_sap_ok");
        stakoKontrolleVO.stakoStat               = rs.getInt("stako_stat");
        stakoKontrolleVO.herkunftStat            = rs.getInt("herkunft_stat");
        stakoKontrolleVO.createdDat              = rs.getTimestamp("created_dat") == null ?
                                                   null : new Date(rs.getTimestamp("created_dat").getTime());
        stakoKontrolleVO.changedDat              = rs.getTimestamp("changed_dat") == null ?
                                                   null : new Date(rs.getTimestamp("changed_dat").getTime());
        al.add(stakoKontrolleVO);
      }
      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  public StakoKontrolleVO selectMaxKontrollId() throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "SELECT stako_kontrolle_id, verarbeitung_dat, von_dat, bis_dat, " +
          "anzahl_ereignisma_gelesen, anzahl_bfaktur_gelesen, " +
          "anzahl_ereignisma_an_sap, anzahl_bfaktur_an_sap, " +
          "anzahl_ereignisma_an_sap_ok, anzahl_bfaktur_an_sap_ok, " +
          "stako_stat, herkunft_stat, created_dat, changed_dat " +
          "FROM SES.STAKO_KONTROLLE " +
          "WHERE stako_kontrolle_id in (SELECT MAX(stako_kontrolle_id) FROM SES.STAKO_KONTROLLE)");
      rs = ps.executeQuery();
      if( !rs.next())
      {
        return null;
      }

      StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();

      stakoKontrolleVO.stakoKontrolleId         = rs.getInt("stako_kontrolle_id");
      stakoKontrolleVO.verarbeitungDat         = rs.getDate("verarbeitung_dat");
      stakoKontrolleVO.vonDat                  = rs.getDate("von_dat");
      stakoKontrolleVO.bisDat                  = rs.getDate("bis_dat");
      stakoKontrolleVO.anzahlEreignisMaGelesen = rs.getInt("anzahl_ereignisma_gelesen");
      stakoKontrolleVO.anzahlBfakturGelesen    = rs.getInt("anzahl_bfaktur_gelesen");
      stakoKontrolleVO.anzahlEreignisMaAnSap   = rs.getInt("anzahl_ereignisma_an_sap");
      stakoKontrolleVO.anzahlBfakturAnSap      = rs.getInt("anzahl_bfaktur_an_sap");
      stakoKontrolleVO.anzahlEreignisMaAnSapOK = rs.getInt("anzahl_ereignisma_an_sap_ok");
      stakoKontrolleVO.anzahlBfakturAnSapOK    = rs.getInt("anzahl_bfaktur_an_sap_ok");
      stakoKontrolleVO.stakoStat               = rs.getInt("stako_stat");
      stakoKontrolleVO.herkunftStat            = rs.getInt("herkunft_stat");
      stakoKontrolleVO.createdDat              = rs.getTimestamp("created_dat") == null ?
                                                 null : new Date(rs.getTimestamp("created_dat").getTime());
      stakoKontrolleVO.changedDat              = rs.getTimestamp("changed_dat") == null ?
                                                 null : new Date(rs.getTimestamp("changed_dat").getTime());
      return stakoKontrolleVO;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Selektierte eine Row der Tablle STAKO_KONTROLLE via kontroll_id
   *
   * @param  kontrollId        Primary Key
   * @return StakoKontrolleVO  Value Object StakoKontrolleVO
   * @throws SQLException      Unerwarteter SQL-Fehler
   */
  public StakoKontrolleVO select(int kontrollId) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "SELECT stako_kontrolle_id, verarbeitung_dat, von_dat, bis_dat, " +
          "anzahl_ereignisma_gelesen, anzahl_bfaktur_gelesen, " +
          "anzahl_ereignisma_an_sap, anzahl_bfaktur_an_sap, " +
          "anzahl_ereignisma_an_sap_ok, anzahl_bfaktur_an_sap_ok, " +
          "stako_stat, herkunft_stat, created_dat, changed_dat " +
          "FROM SES.STAKO_KONTROLLE " +
          "WHERE stako_kontrolle_id = ? ");

      ps.setInt(1, kontrollId);

      rs = ps.executeQuery();
      if( !rs.next())
      {
        return null;
      }

      StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();

      stakoKontrolleVO.stakoKontrolleId         = rs.getInt("stako_kontrolle_id");
      stakoKontrolleVO.verarbeitungDat         = rs.getDate("verarbeitung_dat");
      stakoKontrolleVO.vonDat                  = rs.getDate("von_dat");
      stakoKontrolleVO.bisDat                  = rs.getDate("bis_dat");
      stakoKontrolleVO.anzahlEreignisMaGelesen = rs.getInt("anzahl_ereignisma_gelesen");
      stakoKontrolleVO.anzahlBfakturGelesen    = rs.getInt("anzahl_bfaktur_gelesen");
      stakoKontrolleVO.anzahlEreignisMaAnSap   = rs.getInt("anzahl_ereignisma_an_sap");
      stakoKontrolleVO.anzahlBfakturAnSap      = rs.getInt("anzahl_bfaktur_an_sap");
      stakoKontrolleVO.anzahlEreignisMaAnSapOK = rs.getInt("anzahl_ereignisma_an_sap_ok");
      stakoKontrolleVO.anzahlBfakturAnSapOK    = rs.getInt("anzahl_bfaktur_an_sap_ok");
      stakoKontrolleVO.stakoStat               = rs.getInt("stako_stat");
      stakoKontrolleVO.herkunftStat            = rs.getInt("herkunft_stat");
      stakoKontrolleVO.createdDat              = rs.getTimestamp("created_dat") == null ?
                                               null : new Date(rs.getTimestamp("created_dat").getTime());
      stakoKontrolleVO.changedDat              = rs.getTimestamp("changed_dat") == null ?
                                               null : new Date(rs.getTimestamp("changed_dat").getTime());

      return stakoKontrolleVO;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle STAKO_KONTROLLE ein
   *
   * @param     stakoKontrolleVO  ValueObject StakoKontrolleVO
   * @exception SQLException      Unerwarteter SQL Fehler
   */
  public void insert(StakoKontrolleVO stakoKontrolleVO) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "INSERT INTO ses.stako_kontrolle(" +
          "stako_kontrolle_id, " +
          "verarbeitung_dat, " +
          "von_dat, " +
          "bis_dat, " +
          "anzahl_ereignisma_gelesen, " +
          "anzahl_bfaktur_gelesen, " +
          "anzahl_ereignisma_an_sap, " +
          "anzahl_bfaktur_an_sap, " +
          "anzahl_ereignisma_an_sap_ok, " +
          "anzahl_bfaktur_an_sap_ok, " +
          "stako_stat, " +
          "herkunft_stat, " +
          "created_dat) " +
          "VALUES(SES.SEQ_STAKO_KONTROLLE_ID.nextval,sysdate,?,?,?,?,?,?,?,?,?,?,sysdate)"
          );

      if(stakoKontrolleVO.vonDat == null)
      {
        ps.setNull(1, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(1, new java.sql.Date(stakoKontrolleVO.vonDat.getTime()));
			}
      if(stakoKontrolleVO.bisDat == null)
      {
        ps.setNull(2, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(2, new java.sql.Date(stakoKontrolleVO.bisDat.getTime()));
			}
      ps.setInt(3, stakoKontrolleVO.anzahlEreignisMaGelesen);
      ps.setInt(4, stakoKontrolleVO.anzahlBfakturGelesen);
      ps.setInt(5, stakoKontrolleVO.anzahlEreignisMaAnSap);
      ps.setInt(6, stakoKontrolleVO.anzahlBfakturAnSap);
      ps.setInt(7, stakoKontrolleVO.anzahlEreignisMaAnSapOK);
      ps.setInt(8, stakoKontrolleVO.anzahlBfakturAnSapOK);
      ps.setInt(9, stakoKontrolleVO.stakoStat);
      ps.setInt(10, stakoKontrolleVO.herkunftStat);

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Aktualisiert einen Datensatz der Tabelle STAKO_KONTROLLE
   *
   * @param stakoKontrolleVO Value Object
   * @return Anzahl          aktualisierter Datensätze
   * @throws SQLException    Unerwarteter SQL Fehler
   */
  public int update(StakoKontrolleVO stakoKontrolleVO) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.stako_kontrolle SET " +
          "verarbeitung_dat = ? , " +
          "von_dat = ?, " +
          "bis_dat = ?, " +
          "anzahl_ereignisma_gelesen = ?, " +
          "anzahl_bfaktur_gelesen = ?, " +
          "anzahl_ereignisma_an_sap = ?, " +
          "anzahl_bfaktur_an_sap = ?, " +
          "anzahl_ereignisma_an_sap_ok = ?, " +
          "anzahl_bfaktur_an_sap_ok = ?, " +
          "stako_stat = ?, " +
          "herkunft_stat = ?, " +
          "changed_dat = sysdate " +
          "WHERE stako_kontrolle_id = ? "
          );

      if(stakoKontrolleVO.verarbeitungDat == null)
      {
        ps.setNull(1, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(1, new java.sql.Date(stakoKontrolleVO.verarbeitungDat.getTime()));
      }
      if(stakoKontrolleVO.vonDat == null)
      {
        ps.setNull(2, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(2, new java.sql.Date(stakoKontrolleVO.vonDat.getTime()));
      }
      if(stakoKontrolleVO.bisDat == null)
      {
        ps.setNull(3, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(3, new java.sql.Date(stakoKontrolleVO.bisDat.getTime()));
      }
      ps.setInt(4, stakoKontrolleVO.anzahlEreignisMaGelesen);
      ps.setInt(5, stakoKontrolleVO.anzahlBfakturGelesen);
      ps.setInt(6, stakoKontrolleVO.anzahlEreignisMaAnSap);
      ps.setInt(7, stakoKontrolleVO.anzahlBfakturAnSap);
      ps.setInt(8,  stakoKontrolleVO.anzahlEreignisMaAnSapOK);
      ps.setInt(9, stakoKontrolleVO.anzahlBfakturAnSapOK);
      ps.setInt(10, stakoKontrolleVO.stakoStat);
      ps.setInt(11, stakoKontrolleVO.herkunftStat);
      ps.setInt(12, stakoKontrolleVO.stakoKontrolleId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert das Feld created_dat. Die Funktion wird im Zusammenhang
   * mit der Datenbank Reorganisatin zu Testzwecken verwendet.
   *
   * @param   stakoKontrolleId    eindeutige Lauf Id
   * @param   created             neues created_dat
   * @throws SQLException         unerwartete Fehlermeldung
   * @return int                  Anzahl aktualisierte Datensätze
   */
  public int updateCreatedDat (int stakoKontrolleId, java.util.Date created) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps  = con.prepareStatement
              ( "update ses.stako_kontrolle " +
                "set    created_dat = ?, " +
                "       changed_dat = ? " +
                "WHERE  stako_kontrolle_id = ? "
              );

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );
      ps.setTimestamp( 2, new java.sql.Timestamp(created.getTime()) );
      ps.setInt(3, stakoKontrolleId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Löscht einen Datensatz aus der Tabelle STAKO_KONTROLLE
   *
   * @param     kontrollId    Primary Key
   * @return    int           Anzahl gelöschter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int delete(int kontrollId) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
      ( "DELETE FROM ses.stako_kontrolle where stako_kontrolle_id = ? ");
      ps.setInt(1, kontrollId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }
}


