package ch.ige.ses.server.dao.ses;
import java.sql.*;


import ch.ige.ses.vo.ses.*;
import ch.ige.util.server.*;


/**
 * Data Access Objekt für die Tabelle STATISTIK
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Anita Rüegsegger
 * @version $Id: StatistikDAO.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */
public class StatistikDAO
{

  public StatistikDAO()
  {
    // lazy bone
  }


  //----------------------------------------------------------------------------
  // methods
  //----------------------------------------------------------------------------

  /**
   * Fügt einen Eintrag in die Tabelle STATISTIK ein
   * Version wird berechnet
   * @param vo Statistikdaten
   * @throws Exception
   */
  public void insert(StatistikVO vo) throws Exception
  {
    Connection          con         = null;
    PreparedStatement   psVersion   = null;
    PreparedStatement   psInsert    = null;
    ResultSet           rs          = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      // Version berechnen
      psVersion = con.prepareStatement
        ( "SELECT MAX(version) AS version " +
          "FROM ses.statistik " +
          "WHERE jgb_lauf_id = ?" );
      psVersion.setInt(1, vo.jgbLaufId);

      rs = psVersion.executeQuery();

      if(rs.next())
      {
        vo.version = (rs.getInt("version")) + 1 ; // Version um 1 hochzählen
      }
      else
      {
        vo.version = 1;
      }

      // Daten einfügen
      psInsert = con.prepareStatement
        ( "INSERT INTO ses.statistik " +
          "(jgb_lauf_id, version, data) " +
          "VALUES(?,?,?)" );

      psInsert.setInt(1, vo.jgbLaufId);
      psInsert.setInt(2, vo.version);
      psInsert.setString(3, vo.data);

      psInsert.executeUpdate();
    }
    finally
    {
      DBHelper.close(psVersion, psInsert, con);
    }
  }

  /**
   * Selektiert den neusten Datensatz einer JGB-Verarbeitung - der mit der höchsten Version
   * Falls keiner gefunden wird wird null zurückgegeben
   * @param jgbLaufId Lauf-Id
   * @return StatistikVO mit Statistikdaten
   * @throws Exception
   */
  public StatistikVO select(int jgbLaufId) throws Exception
  {
    Connection    con  = null;
    Statement     stmt = null;
    ResultSet     rs   = null;
    StatistikVO   vo   = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      String subsql = "SELECT MAX(version) FROM ses.statistik WHERE jgb_lauf_id = " + jgbLaufId;

      String sql =  "SELECT jgb_lauf_id, version, data, created_dat " +
                    "FROM ses.statistik " +
                    "WHERE jgb_lauf_id = " + jgbLaufId + " and version = (" + subsql + ")";

      stmt =con.createStatement();
      rs = stmt.executeQuery(sql);

      while (rs.next())
      {
        vo = new StatistikVO();

        vo.jgbLaufId   = rs.getInt("jgb_lauf_id");
        vo.version     = rs.getInt("version");
        vo.data        = rs.getString("data");
        vo.createdDat  = rs.getTimestamp("created_dat") == null ?
                         null : new Date(rs.getTimestamp("created_dat").getTime());
      }

      return vo;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  }
}

