package ch.ige.ses.server.dao.ses;

import java.text.SimpleDateFormat;
import java.sql.*;
import java.sql.Date;
import java.util.*;


import ch.ige.ses.vo.ses.*;
import ch.ige.util.server.*;

/**
 * Data Access Objekt für die Tabelle TRACE
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: TraceDAO.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */
public class TraceDAO
{
  //
  // const
  //
  public static final int FLD_LEN_PROCESS     = 20;
  public static final int FLD_LEN_PROCESS_NR  = 4;
  public static final int FLD_LEN_ACTION      = 1000;
  public static final int FLD_LEN_DATA        = 4000;


  //----------------------------------------------------------------------------
  // constructors
  //----------------------------------------------------------------------------

  public TraceDAO()
  {
    // lazy bone
  }

  //----------------------------------------------------------------------------
  // methods
  //----------------------------------------------------------------------------

  public void insert(TraceVO vo) throws Exception
  {
    Connection          con  = null;
    PreparedStatement   ps   = null;

    try
    {
      // limit maximal field size
      if (vo.process != null)
      {
        if (vo.process.length() > FLD_LEN_PROCESS)
          vo.process = vo.process.substring(0,FLD_LEN_PROCESS-3) + "...";
      }

      if (vo.processNr != null)
      {
        if (vo.processNr.length() > FLD_LEN_PROCESS_NR)
          vo.processNr = vo.processNr.substring(0,FLD_LEN_PROCESS_NR);
      }

      if (vo.action != null)
      {
        if (vo.action.length() > FLD_LEN_ACTION)
          vo.action = vo.action.substring(0,FLD_LEN_ACTION-3) + "...";
      }

      if (vo.data != null)
      {
        if (vo.data.length() > FLD_LEN_DATA)
          vo.data = vo.data.substring(0,FLD_LEN_DATA-3) + "...";
      }

      // insert vo
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
        ( "INSERT INTO ses.trace " +
          "(trace_id, process, process_nr, action, data) " +
          "VALUES(SES.SEQ_TRACE_ID.nextval,?,?,?,?)" );

      ps.setString(1, vo.process);
      ps.setString(2, vo.processNr);
      ps.setString(3, vo.action);
      ps.setString(4, vo.data);

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Selektiert alle Attribute der Tabelle TRACE die den Selektionskriterien
   * entsprechen
   * @param traceSelKritVO enthält die Selektionskriterien
   * @return ArrayList von TraceVO die den Selektionskriteren entsprechen
   * absteigend sortiert nach trace_id
   * @throws Exception unerwartete Fehlersituation
   */
  public ArrayList select(TraceSelKritVO traceSelKritVO) throws Exception
  {
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;
    ArrayList al  = new ArrayList();

    StringBuffer select = new StringBuffer();
    StringBuffer where  = new StringBuffer();
    StringBuffer order  = new StringBuffer();
    String and = "";

    select.append("SELECT trace_id, " +
                          "process, " +
                          "process_nr, " +
                          "action, " +
                          "data, " +
                          "created_dat " +
                  "FROM ses.trace ");
    where.append("WHERE ");
    order.append("ORDER BY trace_id DESC ");

    // Process
    if (traceSelKritVO.process != null && !traceSelKritVO.process.equals(""))
    {
      where.append(and);
      where.append("process = '" + traceSelKritVO.process +"' ");
      and = "AND ";
    }

    // Action
    if (traceSelKritVO.action != null && !traceSelKritVO.action.equals(""))
    {
      where.append(and);
      where.append("LOWER (action) ");
      if (traceSelKritVO.action.indexOf('%') == -1)
      {
        where.append("= ");
      }
      else
      {
        where.append("LIKE ");
      }
      where.append("'");
      where.append(traceSelKritVO.action.toLowerCase());
      where.append("' ");
      and = "AND ";
    }

    // Data
    if (traceSelKritVO.data != null && !traceSelKritVO.data.equals(""))
    {
      where.append(and);
      where.append("LOWER (data) ");
      if (traceSelKritVO.data.indexOf('%') == -1)
      {
        where.append("= ");
      }
      else
      {
        where.append("LIKE ");
      }
      where.append("'");
      where.append(traceSelKritVO.data.toLowerCase());
      where.append("' ");
      and = "AND ";
    }

    // Created_dat
    if ( (traceSelKritVO.createdDatVon != null && !traceSelKritVO.createdDatVon.equals("") ) ||
         (traceSelKritVO.createdDatBis != null && !traceSelKritVO.createdDatBis.equals("") ))
    {
      where.append(and);

      // Selektionskriterien von bis gefüllt Suche between
      if ( traceSelKritVO.createdDatVon != null && !traceSelKritVO.createdDatVon.equals("") &&
          traceSelKritVO.createdDatBis != null && !traceSelKritVO.createdDatBis.equals(""))
      {
        where.append("TO_CHAR(created_dat, 'DD.MM.YYYY') BETWEEN " +
                     "TO_DATE('" + traceSelKritVO.createdDatVon + "', 'YYYY-MM-DD') AND " +
                     "TO_DATE('" + traceSelKritVO.createdDatBis + "', 'YYYY-MM-DD') ");
      }
      // Selektionskriterien von gefüllt Suche nach >=
      if (traceSelKritVO.createdDatVon != null && !traceSelKritVO.createdDatVon.equals("") &&
         ( traceSelKritVO.createdDatBis == null || traceSelKritVO.createdDatBis.equals("") ))
      {
        where.append("TO_CHAR(created_dat, 'DD.MM.YYYY') >= " +
                     "TO_DATE('" + traceSelKritVO.createdDatVon + "', 'YYYY-MM-DD') ");
      }
      // Selektionskriterien bis gefüllt Suche nach <=
      if ( ( traceSelKritVO.createdDatVon == null || traceSelKritVO.createdDatVon.equals("") ) &&
            traceSelKritVO.createdDatBis != null && !traceSelKritVO.createdDatBis.equals("") )
      {
        where.append("TO_CHAR(created_dat, 'DD.MM.YYYY') <= " +
                     "TO_DATE('" + traceSelKritVO.createdDatBis + "', 'YYYY-MM-DD') ");
      }

      and = "AND ";
    }

    // MaxRow
    if (traceSelKritVO.maxRow != 0)
    {
      where.append(and);
      where.append("ROWNUM <= " +  traceSelKritVO.maxRow + " ");
      and = "AND ";
    }

    String sql =  select.toString() +
                  ( where.toString().equals("WHERE ") ? "" : where.toString() ) +
                  order.toString();

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);

      stmt =con.createStatement();
      rs = stmt.executeQuery(sql.toString());

      while (rs.next())
      {
        TraceVO traceVO = new TraceVO();

        traceVO.traceId          = rs.getInt("trace_id");
        traceVO.process          = rs.getString("process");
        traceVO.processNr        = rs.getString("process_nr");
        traceVO.action           = rs.getString("action");
        traceVO.data             = rs.getString("data");
        traceVO.createdDat        = rs.getTimestamp("created_dat") == null ?
                                    null : new Date(rs.getTimestamp("created_dat").getTime());
        al.add(traceVO);
      }

      return al;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  }


  /**
   * Selektiert Anzahl Einträge sowie die maximale und minimale TRACE_ID aller
   * Einträge die älter sind als das angegebene Erstellungsdatum und gibt diese
   * in Form von Property Einträgen zurück.
   *
   * Beispiele Properties:
   *  a) leeres result set
   *        rs=false
   *  b) result set vorhanden, aber keine Einträge selektiert
   *        rs=true
   *        cnt=0
   *  c) result set vorhanden, Einträge gefunden
   *        rs=true
   *        cnt=10
   *        min=5320
   *        max=5329
   *
   * @param  created        Erstellungsdatum
   * @return Properties     Resultat der Abfrage
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public Properties select(java.util.Date created) throws SQLException
  {
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;
    Properties        prop  = new Properties();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( "select count(*) as CNT, max(trace_id) as MAX, min(trace_id) as MIN " +
              "from   trace " +
              "where  created_dat < ? " );

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );


      rs = ps.executeQuery();

      if (rs.next())
      {
        prop.setProperty("rs","true");

        prop.setProperty("cnt",rs.getString("CNT"));

        if (! prop.getProperty("cnt").equals("0"))
        {
          prop.setProperty("max",rs.getString("MAX"));
          prop.setProperty("min",rs.getString("MIN"));
        }
      }
      else
      {
        prop.setProperty("rs","false");
      }

      return prop;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Gibt die max. Trace Id zurück.
   * Wird kein Eintrag gefunden, so wird 0 zurückgegeben.
   *
   * @throws SQLException   unerwarteter Fehler
   * @return int            max. Trace Id oder 0
   */
  public int selectMaxTraceId() throws SQLException
  {
    Connection        con     = null;
    PreparedStatement ps      = null;
    ResultSet         rs      = null;
    int               traceId = 0;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement
            ( "select max(trace_id) as MAX " +
              "from   trace " );
      rs = ps.executeQuery();

      if (rs.next())
      {
        traceId = rs.getInt("MAX");
      }

      return traceId;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert das Feld created_dat. Die Funktion wird im Zusammenhang
   * mit der Datenbank Reorganisatin zu Testzwecken verwendet.
   *
   * @param   traceId       eindeutige Lauf Id
   * @param   created       neues created_dat
   * @throws SQLException   unerwartete Fehlermeldung
   * @return int            Anzahl aktualisierte Datensätze
   */
  public int updateCreatedDat (int traceId, java.util.Date created) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps  = con.prepareStatement
              ( "update ses.trace " +
                "set    created_dat = ? " +
                "where  trace_id = ? "
              );

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );
      ps.setInt(2, traceId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Löscht alle Einträge die älter sind als das angegebene Erstellungsdatum.
   *
   * @param   created       Erstellungsdatum
   * @throws  SQLException  unerwartete Fehlermeldung
   * @return  int           Anzahl gelöschte Einträge
   */
  public int delete(java.util.Date created) throws SQLException
  {
    Connection          con  = null;
    PreparedStatement   ps   = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
        ( "delete from ses.trace " +
          "where  created_dat < ?" );

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Alle Einträge die aelter als die übergebene Anzahl Tage sind
   * werden aus der Tabelle TRACE gelöscht
   *
   * @param days Anzahl Tage
   * @return Anzahl gelöschter Datensätze
   * @throws Exception Unerwartete Fehlersituation
   */
  public int deleteRows(int days) throws Exception
  {
    Connection          con  = null;
    PreparedStatement   ps   = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
        ( "DELETE FROM ses.trace WHERE created_dat < SYSDATE - ?");

      ps.setInt(1, days);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


} // end class


