package ch.ige.ses.server.dao.ses.test;

import java.util.*;
import junit.framework.*;

import ch.ige.ses.server.dao.ses.JGBLaufDAO;
import ch.ige.ses.vo.ses.JGBLaufVO;

/**
 * Test der Klasse JGBLaufDAO
 * Copyright (c) Eidgenössisches Institut für Geistiges Eigentum
 * <p>
 * @author A.Rueegsegger
 * @version $Id: JGBLaufDAOTest.java,v 1.9 2007/11/13 12:35:02 nydegger Exp $
 */
public class JGBLaufDAOTest extends SesTestCase
{
  //
  // const
  //
  private static final String LAUF_NR = "2003011000";

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public JGBLaufDAOTest(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(JGBLaufDAOTest.class);
  }

  protected void setUp() throws Exception
  {
    // lazy bone
  }

  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  /**
   *  Fügt einen Datensatz in die  Tabelle JGB_LAUF ein
   */
  public void testInsert()
  {
    System.out.println("JGB_LAUF: Test INSERT");
    JGBLaufDAO jgbLaufDAO   = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO     = new JGBLaufVO();
    jgbLaufVO.verarbEnde = null;
    jgbLaufVO.laufNr                  = LAUF_NR;
    jgbLaufVO.sesStat                 = 1;
    jgbLaufVO.anzahlBfakturGelesen    = 500;
    jgbLaufVO.anzahlBfaktjgbGeneriert = 700;
    jgbLaufVO.sapStat                 = 1;
    jgbLaufVO.anzahlBfakturAnSap      = 100;
    jgbLaufVO.anzahlBfakturAnSapOk    = 90;
    jgbLaufVO.anzahlBfakturAnSapNok   = 10;
    jgbLaufVO.anzahlBfaktjgbAnSap     = 150;

    try
    {
      jgbLaufDAO.insert(jgbLaufVO);
      // Testen ob eingefügt
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("VO wurde nicht eingefügt");
      }
      assertTrue("JGBLaufVO eingefügt ", jgbLaufVO != null);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   *  Selektiert die grösste jgb_lauf_id aus der Tabelle JGB_LAUF
   */
  public void testSelectMaxLaufID()
  {
    System.out.println("JGB_LAUF: Test SELECT MaxLaufID");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
		JGBLaufVO jgbLaufVO = new JGBLaufVO();
    try
    {
      jgbLaufVO = jgbLaufDAO.selectMaxLaufID();
      
      if (jgbLaufVO != null)
      {
        assertTrue(jgbLaufVO.jgbLaufId > 0);  
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

	/**
   *  Selektiert einen Datensatz aus der Tabelle JGB_LAUF
   */
  public void testSelect()
  {
    System.out.println("JGB_LAUF: Test SELECT");
    JGBLaufDAO  jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO   jgbLaufVO = new JGBLaufVO();
    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      assertEquals("sesStat: ", jgbLaufVO.sesStat, 1);
      assertEquals("sapStat: ", jgbLaufVO.sapStat, 1);
      assertEquals("anzahlBfakturGelesen: ", jgbLaufVO.anzahlBfakturGelesen, 500);
      assertEquals("anzahlBfaktjgbGeneriert: ", jgbLaufVO.anzahlBfaktjgbGeneriert, 700);
      assertEquals("anzahlBfakturAnSap: ", jgbLaufVO.anzahlBfakturAnSap, 100);
      assertEquals("anzahlBfakturAnSapOk: ", jgbLaufVO.anzahlBfakturAnSapOk, 90);
      assertEquals("anzahlBfakturAnSapNok: ", jgbLaufVO.anzahlBfakturAnSapNok, 10);
      assertEquals("anzahlBfaktjgbAnSap: ", jgbLaufVO.anzahlBfaktjgbAnSap, 150);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle JGB_LAUF
   */
  public void _testSelectLaufNR()
  {
    System.out.println("JGB_LAUF: Test SELECT LaufNr");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();
    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      jgbLaufVO = jgbLaufDAO.select(jgbLaufVO.jgbLaufId);
            assertEquals("sesStat: ", jgbLaufVO.sesStat, 1);
      assertEquals("sapStat: ", jgbLaufVO.sapStat, 1);
      assertEquals("anzahlBfakturGelesen: ", jgbLaufVO.anzahlBfakturGelesen, 500);
      assertEquals("anzahlBfaktjgbGeneriert: ", jgbLaufVO.anzahlBfaktjgbGeneriert, 700);
      assertEquals("anzahlBfakturAnSap: ", jgbLaufVO.anzahlBfakturAnSap, 100);
      assertEquals("anzahlBfakturAnSapOk: ", jgbLaufVO.anzahlBfakturAnSapOk, 90);
      assertEquals("anzahlBfakturAnSapNok: ", jgbLaufVO.anzahlBfakturAnSapNok, 10);
      assertEquals("anzahlBfaktjgbAnSap: ", jgbLaufVO.anzahlBfaktjgbAnSap, 150);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Selektiert alle Datensätze aus der Tabelle JGB_LAUF
   */
  public void testSelectAll()
  {
    System.out.println("JGB_LAUF: Test SELECT ALL ");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    ArrayList al = new ArrayList();
    try
    {
      al = jgbLaufDAO.selectAll();
      assertTrue("Select All: ", al.size() > 0);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

	/**
   *  Aktualisiert Attribut SES_STAT der Tabelle JGB_LAUF
   */
  public void testUpdateSesStat()
  {
    System.out.println("JGB_LAUF: Test UPDATE SES STATUS");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.updateSesStat(jgbLaufVO.jgbLaufId, 2);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("SES Status: ", jgbLaufVO.sesStat, 2);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Aktualisiert Attribut SES_STAT der Tabelle JGB_LAUF
   *  Das Attribut wird nur auf den Wert von sesStatNew aktualisiert,
   * wenn SES_STAT den Wert von sesStatOld hat
   */
  public void testUpdateSesStatWithParams()
  {
    System.out.println("JGB_LAUF: Test UPDATE SES STATUS mit Params");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.updateSesStat(jgbLaufVO.jgbLaufId, 2, 2);
        //Rückgabewert muss 1 sein
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("SES Status: ", jgbLaufVO.sesStat, 2);
        //Aufrufen ohne dass was geändert wird
        retVal = jgbLaufDAO.updateSesStat(jgbLaufVO.jgbLaufId, 1, 2);
        //Rückgabewert muss 0 sein
        assertEquals("Anzahl geänderter Datensätze = 0", retVal, 0);

      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Aktualisiert Attribut SAP_STAT der Tabelle JGB_LAUF
   */
  public void testUpdateSapStat()
  {
    System.out.println("JGB_LAUF: Test UPDATE SAP STATUS");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.updateSapStat(jgbLaufVO.jgbLaufId, 3);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("SAP Status: ", jgbLaufVO.sapStat, 3);

        retVal = jgbLaufDAO.updateSapStat(jgbLaufVO.jgbLaufId, JGBLaufVO.SAP_STAT_FINISHED);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("SAP Status & Ende Verarbeitung: ", jgbLaufVO.sapStat, JGBLaufVO.SAP_STAT_FINISHED);
        assertTrue("SAP Status & Ende Verarbeitung: ", jgbLaufVO.verarbEnde != null);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Aktualisiert Attribut ANZAHL_BFAKTUR_GELESEN der Tabelle JGB_LAUF
   */
  public void testUpdateAnzahlBfakturGelesen()
  {
    System.out.println("JGB_LAUF: Test UPDATE ANZAHL BFAKTUR GELSEN");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.updateAnzahlBfakturGelesen(jgbLaufVO.jgbLaufId, 1000);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("Anzahl Bfaktur gelesen: ", jgbLaufVO.anzahlBfakturGelesen, 1000);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   *  Aktualisiert Attribut ANZAHL_BFAKTJGB_GENERIERT der Tabelle JGB_LAUF
   */
  public void testUpdateAnzahlBfaktjgbGeneriert()
  {
    System.out.println("JGB_LAUF: Test UPDATE ANZAHL BFAKTJGB GENERIERT");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.updateAnzahlBfaktjgbGeneriert(jgbLaufVO.jgbLaufId, 1500);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("Anzahl Bfaktjgb generiert: ", jgbLaufVO.anzahlBfaktjgbGeneriert, 1500);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Aktualisiert Attribut ANZAHL_BFAKTUR_AN_SAP der Tabelle JGB_LAUF
   *  Aktualisiert Attribut ANZAHL_BFAKTJGB_AN_SAP der Tabelle JGB_LAUF
   */
  public void testUpdateAnzahlAnSap()
  {
    System.out.println("JGB_LAUF: Test testUpdateAnzahlAnSap");

    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO  jgbLaufVO  = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.incrementAnzahlAnSap(jgbLaufVO.jgbLaufId,1200,1000,200,1300);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("Anzahl Bfaktur an SAP :   ", jgbLaufVO.anzahlBfakturAnSap, 1300);
        assertEquals("Anzahl Bfaktur an SAP ok: ", jgbLaufVO.anzahlBfakturAnSapOk, 1090);
        assertEquals("Anzahl Bfaktur an SAP nok:", jgbLaufVO.anzahlBfakturAnSapNok, 210);
        assertEquals("Anzahl Bfaktjgb an SAP:   ", jgbLaufVO.anzahlBfaktjgbAnSap,1450);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   *  Aktualisiert Attribut MIN_BFAKTUR_TO_SAP der Tabelle JGB_LAUF
   */
  public void testUpdateMinBfakturToSap()
  {
    System.out.println("JGB_LAUF: Test UPDATE MIN BFAKTUR TO SAP");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.updateMinBfakturToSap(jgbLaufVO.jgbLaufId, 100);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("Min Bfaktur To Sap: ", jgbLaufVO.minBfakturToSap, 100);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Aktualisiert Attribut COUNT_BFAKTUR_TO_SAP der Tabelle JGB_LAUF
   */
  public void testUpdateCountBfakturToSap()
  {
    System.out.println("JGB_LAUF: Test UPDATE COUNT BFAKTUR TO SAP");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.updateCountBfakturToSap(jgbLaufVO.jgbLaufId, 200);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        //prüfen ob Änderung gespeichert
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals("Min Bfaktur To Sap: ", jgbLaufVO.countBfakturToSap, 200);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
  *  Aktualisiert einen Datensatz
  */
  public void _testUpdate()
  {
    System.out.println("Test UPDATE MIT VALUE OBJECT ");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        // Datensatz ändern
        jgbLaufVO.sapStat = 5;
        int retVal = jgbLaufDAO.update(jgbLaufVO);
        assertEquals("Anzahl geänderter Datensätze: ", retVal, 1);
        //Datensatz wieder selektieren und testen, ob Änderungen übernommen
        jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
        assertEquals(jgbLaufVO.sapStat, 5);
        //andere Felder müssen unverändert sein
        assertEquals("sesStat: ", jgbLaufVO.sesStat, 2);
        assertEquals("anzahlBfakturGelesen: ", jgbLaufVO.anzahlBfakturGelesen, 1000);
        assertEquals("anzahlBfaktjgbGeneriert: ", jgbLaufVO.anzahlBfaktjgbGeneriert, 1500);
        assertEquals("anzahlBfakturAnSap: ", jgbLaufVO.anzahlBfakturAnSap, 1300);
        assertEquals("anzahlBfaktjgbAnSap: ", jgbLaufVO.anzahlBfaktjgbAnSap, 1450);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   * Löscht einen Datensatz aus der Tabelle JGB_LAUF
   */
  public void testDelete()
  {
    System.out.println("JGB_LAUF: Test DELETE");
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();
    JGBLaufVO jgbLaufVO = new JGBLaufVO();

    try
    {
      jgbLaufVO = jgbLaufDAO.select(LAUF_NR);
      if (jgbLaufVO == null)
      {
        System.out.println("DELETE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = jgbLaufDAO.delete(jgbLaufVO.jgbLaufId);
        assertEquals("Anzahl gelöschte Datensätze = 1", retVal, 1);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }
} // end class

