package ch.ige.ses.server.sapctrl;

import ch.ige.ses.vo.bagis.*;
import java.lang.*;
import java.rmi.*;
import java.util.*;
import javax.ejb.*;

import ch.ige.util.ServiceLocator;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.ses.*;

/**
 * Delegate for SapInboundCtrlBean
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapDelegateInbound.java,v 1.26 2006/12/29 13:54:38 iten Exp $
 */
public class SapDelegateInbound
{
  //
  // fields
  //
  private static SapDelegateInbound   fDelegate       = null;
  private        SapInboundCtrl       fSapInboundCtrl = null;


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  private SapDelegateInbound() throws Exception
  {
    SapInboundCtrlHome  home;

    home = (SapInboundCtrlHome) ServiceLocator.getInstance().getHome
                                  (SapInboundCtrlHome.JNDI_NAME, SapInboundCtrlHome.class);

    fSapInboundCtrl = home.create();
  }

  //---------------------------------------------------------------------------
  // methods
  //---------------------------------------------------------------------------

  public static SapDelegateInbound getInstance() throws Exception
  {
    if (fDelegate == null)
    {
      fDelegate = new SapDelegateInbound();
    }
    return fDelegate;
  }

  
  public static void removeInstance() throws Exception
  {
    ServiceLocator.removeInstance();
    fDelegate = null;
  }
  

  //
  // send messages to sapinbound client
  //
  public void sendRechnung(SAPRechnungVO vo) throws Exception
  {
    fSapInboundCtrl.sendRechnung(vo);
  }


  public void sendAnnulation (SAPRechnungAnnullationVO vo) throws Exception
  {
    fSapInboundCtrl.sendAnnulation(vo);
  }


  public void sendAnnulationList (ArrayList list) throws Exception
  {
    fSapInboundCtrl.sendAnnulationList(list);
  }


  public void sendJGBJob (SAPJGBJobVO vo) throws Exception
  {
    fSapInboundCtrl.sendJGBJob(vo);
  }


  public void sendCtrlStop () throws Exception
  {
    fSapInboundCtrl.sendCtrlStop();
  }


  public void sendCtrlJgbFinish (int jgbLaudId) throws Exception
  {
    fSapInboundCtrl.sendCtrlJgbFinish(jgbLaudId);
  }

  public void sendCtrlAmvFinish (AmvLaufVO vo) throws Exception
  {
    fSapInboundCtrl.sendCtrlAmvFinish(vo);
  }

  public void sendStakoJob(SAPStakoJobVO sapStakoJobVO) throws Exception
  {
    fSapInboundCtrl.sendStakoJob(sapStakoJobVO);
  }

  public void sendStako(StakoVO stakoVO) throws Exception
  {
    fSapInboundCtrl.sendStako(stakoVO);
  }

  //
  // receive messages from sapinbound client
  //
  public void receiveError(ErrorVO err) throws Exception
  {
    fSapInboundCtrl.receiveError(err);
  }


  public void receiveTrace(TraceVO trace) throws Exception
  {
    fSapInboundCtrl.receiveTrace(trace);
  }


  //
  // process jgb request's from sapinbound client
  //
  public ArrayList requestJgbRechnungJob (SAPJGBJobVO job) throws Exception
  {
    return fSapInboundCtrl.requestJgbRechnungJob(job);
  }


  public void requestJgbRechnungUpdate (ArrayList rechnungList) throws Exception
  {
    fSapInboundCtrl.requestJgbRechnungUpdate(rechnungList);
  }


  public JGBLaufVO requestJgbLaufSelect (int jgbLaufId) throws Exception
  {
    return fSapInboundCtrl.requestJgbLaufSelect(jgbLaufId);
  }


  public void requestJgbLaufUpdate(int jgbLaufId, int sapStat) throws Exception
  {
    fSapInboundCtrl.requestJgbLaufUpdate(jgbLaufId,sapStat);
  }


  public boolean requestJgbEndCondition(int jgbLaufId) throws Exception
  {
    return fSapInboundCtrl.requestJgbEndCondition(jgbLaufId);
  }


  public boolean requestJgbStopCondition(int jgbLaufId) throws Exception
  {
    return fSapInboundCtrl.requestJgbStopCondition(jgbLaufId);
  }


  //
  // process amv request's from sapinbound client
  //
  public void requestAmvRechnungUpdate(AmvRechnungVO vo) throws Exception
  {
    fSapInboundCtrl.requestAmvRechnungUpdate(vo);
  }

  public void requestAmvLaufStatUpdate(int amvLaufId, int laufStat) throws Exception
  {
    fSapInboundCtrl.requestAmvLaufStatUpdate(amvLaufId,laufStat);
  }


  //
  // Stako Controlling methods to BAGIS
  //
  public boolean stakoUpdateArtikelStatus(ArrayList al) throws Exception
  {
    return fSapInboundCtrl.stakoUpdateArtikelStatus(al);
  }


  public ArrayList stakoSelectEreignisMA(Date von, Date bis, String exclMa) throws Exception
  {
    return fSapInboundCtrl.stakoSelectEreignisMA(von, bis, exclMa);
  }


  public ArrayList stakoSelectBfakturBfaktPos(Date von, Date bis, String exclMa) throws Exception
  {
    return fSapInboundCtrl.stakoSelectBfakturBfaktPos(von, bis, exclMa);
  }


  //
  // Stako Controlling methods to SES-DB
  //
  public void stakoCreateStakoKontrolle(StakoKontrolleVO stakoKontrolleVO) throws Exception
  {
    fSapInboundCtrl.stakoCreateStakoKontrolle(stakoKontrolleVO);
  }


  public void stakoSetStakoKontrolle(StakoKontrolleVO stakoKontrolleVO) throws Exception
  {
    fSapInboundCtrl.stakoSetStakoKontrolle(stakoKontrolleVO);
  }


  public StakoKontrolleVO stakoGetStakoKontrolle(int kontrollId) throws Exception
  {
    return fSapInboundCtrl.stakoGetStakoKontrolle(kontrollId);
  }


  public StakoKontrolleVO stakoGetLastStakoKontrolle() throws Exception
  {
    return fSapInboundCtrl.stakoGetLastStakoKontrolle();
  }


} // end class


