package ch.ige.ses.server.sapctrl;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.vo.bagis.SAPRechnungStatusListVO;
import ch.ige.ses.vo.ses.ErrorVO;
import ch.ige.ses.vo.ses.TraceVO;

/**
 * SES:Remote-Inteface für Bean SapOutboundCtrlBean
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Pirmin Suter
 * @version  $Id: SapOutboundCtrlBean.java,v 1.25 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="SapOutboundCtrl"  
 *  display-name="SapOutboundCtrl"
 *  view-type="remote"
 *  jndi-name="SapOutboundCtrl"
 *  local-jndi-name="SapOutboundCtrlLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SapReceive"
 *   view-type="local"
 */
public class SapOutboundCtrlBean implements SessionBean
{
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(SapOutboundCtrlBean.class);

  //
  // fields
  //
  SessionContext          sessionContext;

  SapReceiveLocalHome     sapReceiveLocalHome   = null;
  SapReceiveLocal         sapReceiveLocal       = null;


  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   */
  public void ejbCreate() 
  {
    Context   ctx;
    Object    ref;

    try
    {
      ctx = new InitialContext();

      ref                   = ctx.lookup(SapReceiveLocalHome.COMP_NAME);
      sapReceiveLocalHome   = (SapReceiveLocalHome)ref;
      sapReceiveLocal       = sapReceiveLocalHome.create();
    }
    catch (CreateException ex)
    {
      LOG.error("ejbCreate - CreateException occured: " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("ejbCreate - NamingException occured: " + ex.getMessage());
    }
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public String receivePing (String request)
  {
    return sapReceiveLocal.receivePing(request);
  }


  /**
   * @ejb.interface-method
   */
  public void receiveStatus(SAPRechnungStatusListVO list)
  {
    sapReceiveLocal.receiveStatus(list);
  }


  /**
   * @ejb.interface-method
   */
  public void receiveStatusJGB(SAPRechnungStatusListVO list)
  {
    sapReceiveLocal.receiveStatusJGB(list);
  }


  /**
   * @ejb.interface-method
   */
  public void receiveError(ErrorVO err)
  {
    sapReceiveLocal.receiveError(err);
  }


  /**
   * @ejb.interface-method
   */
  public void receiveTrace(TraceVO trace) throws Exception
  {
    sapReceiveLocal.receiveTrace(trace);
  }

} // end class


