package ch.ige.ses.server.sapctrl.test;

import java.util.*;

import org.apache.log4j.BasicConfigurator;

import junit.framework.*;

import ch.ige.ses.server.sapctrl.*;
import ch.ige.ses.vo.ses.*;

/**
 * Test des BusinessDelegate
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: SAPDelegateInboundTest.java,v 1.16 2007/01/10 08:57:41 iten Exp $
 */

public class SAPDelegateInboundTest extends TestCase
{
  private SapDelegateInbound sd = null;

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public SAPDelegateInboundTest(String s)
  {
    super (s);

    // init log4j
    BasicConfigurator.configure();
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
    return new TestSuite(SAPDelegateInboundTest.class);
  }

  protected void setUp() throws Exception
  {
    // lazy bone
  }

  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void testCreateStakoKontrolle()
  {
    System.out.println("TEST CREATE STAKOKONTROLLE");
    StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();

    stakoKontrolleVO.vonDat              = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
    stakoKontrolleVO.bisDat              = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
    stakoKontrolleVO.anzahlEreignisMaGelesen     = 1000;
    stakoKontrolleVO.anzahlBfakturGelesen        = 1000;
    stakoKontrolleVO.anzahlEreignisMaAnSap = 1000;
    stakoKontrolleVO.anzahlBfakturAnSap   = 1000;

    try
    {
      sd.getInstance().stakoCreateStakoKontrolle(stakoKontrolleVO);
      // Testen ob eingefügt
      stakoKontrolleVO = sd.getInstance().stakoGetLastStakoKontrolle();
      if (stakoKontrolleVO == null)
      {
        fail("VO wurde nicht eingefügt");
      }
      else
      {
        assertEquals("vonDatum: ", stakoKontrolleVO.vonDat, new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime()));
        assertEquals("bisDatum: ", stakoKontrolleVO.bisDat, new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime()));
        assertEquals("gelesenEreignisMa: ", stakoKontrolleVO.anzahlEreignisMaGelesen, 1000);
        assertEquals("gelesenBfaktur: ", stakoKontrolleVO.anzahlBfakturGelesen, 1000);
        assertEquals("uebermitteltEreigniMa: ", stakoKontrolleVO.anzahlEreignisMaAnSap, 1000);
        assertEquals("uebermitteltBfaktur: ", stakoKontrolleVO.anzahlBfakturAnSap, 1000);
        assertTrue("StakoKontrolleVO eingefügt ", stakoKontrolleVO != null);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testGetStakoKontrolle()
  {
    try
    {
      System.out.println("TEST GET STAKOKONTROLLE");
      StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();
      stakoKontrolleVO = sd.getInstance().stakoGetLastStakoKontrolle();
      int kontrollId = stakoKontrolleVO.stakoKontrolleId;
      stakoKontrolleVO = sd.getInstance().stakoGetStakoKontrolle(kontrollId);
      assertEquals("1. Test - kontrollId: ", stakoKontrolleVO.stakoKontrolleId, kontrollId);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _testGetLastStakoKontrolle()
  {
    try
    {
      System.out.println("TEST GET LAST STAKOKONTROLLE");
      StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();

      stakoKontrolleVO = sd.getInstance().stakoGetLastStakoKontrolle();
      assertTrue(stakoKontrolleVO.stakoKontrolleId > 23);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSetStakoKontrolle()
  {
    try
    {
      System.out.println("TEST SET STAKOKONTROLLE");
      StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();

      // Letzten Datensatz holen
      stakoKontrolleVO = sd.getInstance().stakoGetLastStakoKontrolle();
      // Datensatz ändern
      stakoKontrolleVO.verarbeitungDat       = new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime());
      stakoKontrolleVO.vonDat              = new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime());
      stakoKontrolleVO.bisDat              = new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime());
      stakoKontrolleVO.anzahlEreignisMaGelesen     = 3000;
      stakoKontrolleVO.anzahlBfakturGelesen        = 3000;
      stakoKontrolleVO.anzahlEreignisMaAnSap = 3000;
      stakoKontrolleVO.anzahlBfakturAnSap   = 3000;
      // Datensatz updaten
      sd.getInstance().stakoSetStakoKontrolle(stakoKontrolleVO);
      // Testen ob aktualisiert
      stakoKontrolleVO = sd.getInstance().stakoGetLastStakoKontrolle();
      assertEquals("verarbeitungDat: ", stakoKontrolleVO.verarbeitungDat, new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime()));
      assertEquals("vonDatum: ", stakoKontrolleVO.vonDat, new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime()));
      assertEquals("bisDatum: ", stakoKontrolleVO.bisDat, new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime()));
      assertEquals("gelesenEreignisMa: ", stakoKontrolleVO.anzahlEreignisMaGelesen, 3000);
      assertEquals("gelesenBfaktur: ", stakoKontrolleVO.anzahlBfakturGelesen, 3000);
      assertEquals("uebermitteltEreigniMa: ", stakoKontrolleVO.anzahlEreignisMaAnSap, 3000);
      assertEquals("uebermitteltBfaktur: ", stakoKontrolleVO.anzahlBfakturAnSap, 3000);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   * ACHTUNG!!!!!
   * Wird dieser Test aktiviert werden alle standpreis der BAGIS Tabelle artikel
   * auf 0 gesetzt und anschliessend die Artikel mit den übergebenen art_nr mit
   * standpreis = 1 aktualisiert!!!
   */
  public void _testUpdateArtikelStatus()
  {
    try
    {
      System.out.println("TEST UPDATE ArtikelStatus");
      ArrayList al = getArtikelNumbers();
      sd.getInstance().stakoUpdateArtikelStatus(al);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _testSelectBfakturBfaktPos()
  {
    ArrayList al = new ArrayList();
    try
    {
      System.out.println("TEST SELECT BFAKTURBFAKTPOS");
      al = sd.getInstance().stakoSelectBfakturBfaktPos(createDate(true), createDate(false), "");
      System.out.println("Anzahl VO = " + al.size());
      //Arraygrösse sollte grösser als 0 sein...
      assertTrue(al.size()>0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


 public void _testSelectEreignisMA()
 {
   ArrayList al = new ArrayList();
   try
   {
     System.out.println("TEST SELECT EREIGNISMA");
     al = sd.getInstance().stakoSelectEreignisMA(createDate(true), createDate(false), "");
     System.out.println("Anzahl VO = " + al.size());
     //Arraygrösse sollte grösser als 0 sein...
     assertTrue(al.size()>0);
   }
   catch (Exception e)
   {
     e.printStackTrace();
     fail(e.toString());
   }
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * erstellt von- und bis-Datum der Daterange. Zu Testzwecken ist die
   * Differenz 1 Tag
   *
   * @param   isVonDate   true, wenn Anfang Date Range<br>
   *                      false, wenn Ende Date Range
   * @return  Datum       Beginn oder Enddatum als Date
   */
  private Date createDate(boolean isVonDate)
  {
    GregorianCalendar gregD = new GregorianCalendar(2001, 6, isVonDate ? 1 : 2);
    //GregorianCalendar gregD = new GregorianCalendar(2003, 3, 13);
    return gregD.getTime();
  }

  /**
   * Liefert alle Artikelnummern, welche einen Standpreis > 0 haben<br>
   * Stand: 14.03.2003 auf e-bage
   * @return al ArrayList mit art_nr
   */
  private static ArrayList getArtikelNumbers()
  {
    ArrayList al = new ArrayList();
    String artNumbers = "10,100,10010,10050,10060,1010,1011,110,11001,110350,110365,"+
                        "110371,110372,110375,110395,110400,110410,110411,110412,110413,110425,110441,"+
                        "110443,110444,110445,110446,110447,110448,110449,110450,110451,110455,110456,"+
                        "110533,110535,110537,110539,110541,110543,110545,1130,11500,11510,11520,11525,"+
                        "11550,120,12025,12026,12032,12035,12037,12040,12050,12060,12070,12080,1210,12130,"+
                        "12160,12180,12187,12190,1220,12200,130,13010,13020,1310,13150,1320,13200,13201,140,"+
                        "14040,14041,14050,14071,14073,14081,14090,1410,14110,14115,14140,14170,14175,14180,"+
                        "14185,1420,14210,14260,14345,14450,14500,14600,14700,14710,150,1510,1520,1550,160,"+
                        "170,180,18010,18020,18030,18040,18045,18050,18060,18070,18080,18090,18095,181,18100,"+
                        "18110,18120,18125,18130,18140,18150,18160,18170,18190,182,18200,18210,18220,18230,"+
                        "18240,18245,18246,18250,18261,18265,18270,18280,18290,18330,185,186,18801,20,20020,"+
                        "20040,20060,20080,20100,20120,20170,20171,20198,20200,20209,20280,20290,20300,20310,"+
                        "20315,20320,20345,20430,20440,20500,20700,20710,27010,27030,27040,27045,27050,27060,"+
                        "27070,27080,27140,27250,27260,27300,27310,28010,28020,28040,28045,28050,28060,28070,"+
                        "28080,28095,28100,28110,28120,28125,28130,28140,28150,28160,28161,28170,28190,28200,"+
                        "28210,28220,28230,28240,28245,28246,28250,28261,28265,28270,28280,28290,28330,3,30,"+
                        "3000,40,40000,40012,40020,40025,40050,40500,40502,40506,40508,40510,40512,40516,40530,"+
                        "40532,40534,40540,40542,40544,40550,40556,40570,40572,40574,40575,42000,42008,42120,"+
                        "42540,42600,42700,42710,42720,4300,4310,43110,4312,43120,43135,4314,43140,43150,4316,"+
                        "43160,43170,43200,43210,43220,43230,43235,43240,43250,43260,43270,43280,43290,43410,"+
                        "43420,43500,43510,43680,43685,43700,43710,43720,43730,43760,43780,43785,43830,43900,"+
                        "44020,44022,44030,44120,44122,44130,44140,44142,44150,44152,44160,44205,44206,44207,"+
                        "44208,44210,44212,44220,44222,44230,44232,44240,44250,44252,44260,44270,44280,44510,"+
                        "44515,44520,44525,44565,44566,44570,46020,46023,46030,46035,46040,46042,46044,46060,"+
                        "46510,46520,48000,48001,48002,48003,48010,50,50010,52100,52110,52120,54000,54130,54135,"+
                        "54140,54150,54170,54220,54225,54230,54240,54250,54260,56030,56055,56510,57680,57780,6,"+
                        "60,60000,60300,60310,60320,60330,60400,60500,60800,60810,61500,61510,61520,61530,61540,"+
                        "61550,61560,61570,61580,61590,61600,61610,61630,61640,61650,61660,61670,61680,61690,"+
                        "61700,61710,61720,61730,61800,62001,62300,62310,62320,62330,62340,62350,62360,62370,"+
                        "62380,62390,62400,62410,62420,62430,62440,62450,62460,62470,62480,62490,62500,62510,"+
                        "62520,62530,62540,62550,62560,62570,62580,62590,62620,62800,62810,62820,62830,63500,"+
                        "63510,63520,63530,63540,63550,63560,63570,63580,63590,63600,63610,63620,63630,63640,"+
                        "65500,65510,65520,65530,65540,65550,65560,65570,65610,65620,65630,65640,65650,65660,"+
                        "65670,65710,65720,65730,65740,65750,65760,65770,65810,65820,65830,65840,65850,65860,"+
                        "65870,65900,65910,65920,65930,65940,65950,66100,66110,66120,66130,66140,66150,66160,"+
                        "66170,66180,68510,68520,68530,69500,69510,69520,69530,7,70,80,80000,81010,81020,81900,"+
                        "81910,88000,88010,88020,88030,88040,89000,90,90110,90115,90120,90125,90130,90131,90132,"+
                        "90133,90135,90140,90145,90150,90155,90159,90160,90161,90200,90201,90202,90203,90204,"+
                        "90205,90206,90207,90208,90209,90210,90211,90212,90213,90214,90215,90216,90221,90222,"+
                        "90223,90224,90225,90226,90227,91160,91202,92110,92120,92125,92130,92131,92132,92133,"+
                        "92175,92200,94000,94030,94040,94050,94080,94100,94105,94110,94111,94112,94113,94115,"+
                        "94120,94125,94128,94130,94135,94138,94140,94145,94150,94170,94171,94172,94173,94174,"+
                        "94176,94178,94180,94182,94184,94500,94510,94512,94520,94525,94530,94535,94540,94550,"+
                        "94551,94553,94950,97100,97110,97120,97130,97140,97150,97200,97210,97220,97230,97310,"+
                        "97320,98200,99020,99021,99022,99023,99024,99025,99026,99027,99029,99030,99032,99035,"+
                        "99060,99600,99601,99607,99608,99609,99610,99615,99616,99620,99622";
    StringTokenizer st = new StringTokenizer(artNumbers,",");
    while(st.hasMoreTokens())
    {
      al.add((String)st.nextToken());
    }
    System.out.println("al.size() = " + al.size());
    return al;
  }

} // end class

