package ch.ige.ses.vo.bagis;

import java.io.*;
import java.util.*;
import java.math.BigDecimal;

/**
 * SES Value Object für den Rechnungsstatus Druck und Zahlung
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author    Pirmin Suter, Thomas Iten
 * @version $Id: SAPRechnungStatusVO.java,v 1.6 2006/12/13 14:58:39 iten Exp $
 */

public class SAPRechnungStatusVO implements Serializable
{
  //
  // const
  //
  public  static final String   STATUS_DRUCK_GEDRUCKT = "9";
  public  static final String[] STATUS_ZAHLG_TYPES    = {"2","4","9","A"};
  public  static final String   LEGENDE_STATUS_DRUCK  = "(9=Gedruckt)";
  public  static final String   LEGENDE_STATUS_ZAHLG  = "(2=Storniert, 4=Bezahlt, 9=Annulliert, A=Fristablauf)";

  //
  // fields
  //
  public int        bfakt_id      = 0;
  public String     jgbLaufNr     = "";   // wird nur gesetzt bei JGB Rückmeldungen
  public String     amvLaufNr     = "";   // wird nur gesetzt bei AMV Rückmeldungen

  // status druck
  public String     status_druck  = "";
  public Date       bfakt_dat     = null;

  // status zahlung
  public String     status_zahlg   = "";
  public Date       status_datum   = null;                              // datum zahlung oder storno
  public Date       buch_datum     = null;                              // Buchungsdatum, wird für Postgrant benötigt
  public BigDecimal fakt_betrag    = new BigDecimal(0);
  public String     waehr_cd       = SAPRechnungVO.WAEHRUNG_TYPES[0];   // konstant CHF
  public String     kz_kalk        = "1";                               // konstant 1 , (not set by sap)

  // test felder
  public int        version        = 0;


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public SAPRechnungStatusVO()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // methods
  //---------------------------------------------------------------------------

  public String toString()
  {
    StringBuffer  buf = new StringBuffer();

    buf.append ("  bfakt_id         = "  + this.bfakt_id      + "\n");
    buf.append ("  jgbLaufNr        = "  + this.jgbLaufNr     + "\n");
    buf.append ("  amvLaufNr        = "  + this.amvLaufNr     + "\n");

    // status druck
    buf.append ("  status_druck     = "  + this.status_druck  + "\n");
    buf.append ("  bfakt_dat        = "  + this.bfakt_dat     + "\n");

    // status zahlung
    buf.append ("  status_zahlg     = "  + this.status_zahlg  + "\n");
    buf.append ("  status_datum     = "  + this.status_datum  + "\n");
    buf.append ("  buch_datum       = "  + this.buch_datum    + "\n");
    buf.append ("  fakt_betrag      = "  + this.fakt_betrag   + "\n");
    buf.append ("  waehr_cd         = "  + this.waehr_cd      + "\n");
    buf.append ("  kz_kalk          = "  + this.kz_kalk       + "\n");

    // test felder
    buf.append ("  version          = "  + this.version         + "\n");

    return buf.toString();
  }


  public String toStringValidDataOnly()
  {
    StringBuffer  buf = new StringBuffer();

    buf.append ("  bfakt_id         = "  + this.bfakt_id      + "\n");

    if ( ! jgbLaufNr.equals("") )
    {
      buf.append ("  jgbLaufNr        = "  + this.jgbLaufNr     + "\n");
    }

    if ( ! amvLaufNr.equals("") )
    {
      buf.append ("  amvLaufNr        = "  + this.amvLaufNr     + "\n");
    }


    if ( ! status_druck.equals("") )
    {
      buf.append ("  status_druck     = "  + this.status_druck  + " " + LEGENDE_STATUS_DRUCK + "\n");
      buf.append ("  bfakt_dat        = "  + this.bfakt_dat     + "\n");
    }

    if ( ! status_zahlg.equals("") )
    {
      buf.append ("  status_zahlg     = "  + this.status_zahlg  + " " + LEGENDE_STATUS_ZAHLG + "\n");
      buf.append ("  status_datum     = "  + this.status_datum  + "\n");
      buf.append ("  buch_datum       = "  + this.buch_datum    + "\n");
      buf.append ("  fakt_betrag      = "  + this.fakt_betrag   + "\n");
      buf.append ("  waehr_cd         = "  + this.waehr_cd      + "\n");
      buf.append ("  kz_kalk          = "  + this.kz_kalk       + "\n");
    }

    return buf.toString();
  }

} // end class

