package ch.ige.ses.vo.bagis;

import java.io.*;
import java.math.BigDecimal;
import java.util.Date;

/**
 * SES Value Object für eine Rechnungsposition.
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author    Pirmin Suter, Thomas Iten
 * @version   $Id: SAPRechnungsPositionVO.java,v 1.6 2006/12/13 14:58:39 iten Exp $
 */

public class SAPRechnungsPositionVO implements Serializable
{
  //
  // const
  //
  public static final String[] LAGEINH_TYPES = {"ANZ","STD","STK"};

  //
  // fields
  //

  // Rechnungsposition-Daten
  public int        bfakt_id      = 0;
  public int        bfakt_posnr   = 0;
  public String     art_nr        = "";
  public String     lageinh       = LAGEINH_TYPES[0];   // LAGEINH_TYPES
  public int        anzahl        = 0;
  public BigDecimal preis         = new BigDecimal(0);
  public String     artKurztext   = "";                 // Artikel Kurtztext
  public String     artText       = "";                 // Artikel Text
  public String     artZusatzText = "";                 // Artikel-Zusatztext
  public int        schutit_key   = 0;

  // Interne Felder
  public BigDecimal wert          = new BigDecimal(0);  //%%$$Wert       BFAKTPOS.WERT
  public Date       dat_ereignis  = null;               //%%$$Ereigdat   BFAKTPOS.DAT_EREGNIS
  public Date       frist         = null;               //%%$$Frist      BFAKTPOS.FRIST
  public String     schutit_nr    = "";                 //%%$$Schutit    BFAKTPOS.SCHUTIT_NR
  public Date       createdDat    = null;
  public Date       changedDat    = null;

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public SAPRechnungsPositionVO()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // methods
  //---------------------------------------------------------------------------
  public String toString()
  {
    StringBuffer  buf = new StringBuffer();

    buf.append ("  bfakt_id      = <"  + this.bfakt_id      + "> \n");
    buf.append ("  bfakt_posnr   = <"  + this.bfakt_posnr   + "> \n");
    buf.append ("  art_nr        = <"  + this.art_nr        + "> \n");
    buf.append ("  lageinh       = <"  + this.lageinh       + "> \n");
    buf.append ("  anzahl        = <"  + this.anzahl        + "> \n");
    buf.append ("  preis         = <"  + this.preis         + "> \n");
    buf.append ("  artKurztext   = <"  + this.artKurztext   + "> \n");
    buf.append ("  artText       = <"  + this.artText       + "> \n");
    buf.append ("  artZusatzText = <"  + this.artZusatzText + "> \n");
    buf.append ("  schutit_key   = <"  + this.schutit_key   + "> \n");

    buf.append ("  INTERNE FELDER \n");
    buf.append ("    wert        = <"  + this.wert          + "> \n");

    if (dat_ereignis == null)
      buf.append ("    dat_ereignis= null \n");
    else
      buf.append ("    dat_ereignis= <"  + this.dat_ereignis   + "> \n");

    if (frist == null)
      buf.append ("    frist       = null \n");
    else
      buf.append ("    frist       = <"  + this.frist   + "> \n");

    buf.append ("    schutit_nr  = <"  + this.schutit_nr    + "> \n");

    if (createdDat == null)
      buf.append ("    createdDat  = null \n");
    else
      buf.append ("    createdDat  = <"  + this.createdDat   + "> \n");

    if (changedDat == null)
      buf.append ("    changedDat  = null \n");
    else
      buf.append ("    changedDat  = <"  + this.changedDat   + "> \n");

    return buf.toString();
  } // end toString

} // end SAPRechnungsPositionVO

