package ch.ige.wdltool.dao;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

import ch.ige.dbutils.dao.AbstractWdlDAO;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.wdltool.vo.AnalyzedVO;

/**
 * Accesses the table <code>ANALYZED</code>.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Christian Heinzer
 * @version  $Id: AnalyzedDAO.java,v 1.3 2008/04/18 09:43:45 heinzer Exp $
 */
public class AnalyzedDAO extends AbstractWdlDAO<AnalyzedVO, Integer>
{

  /**
   * The only constructor.
   */
  public AnalyzedDAO()
  {
    super();
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<AnalyzedVO> getVOType()
  {
    return AnalyzedVO.class;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getTableName()
  @Override
  protected String getTableName()
  {
    return "analyzed";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getColumnNames()
  @Override
  protected String getColumnNames()
  {
    return "tm_schutit_key,analyzed_dat";
  }

  @Override
  protected Map<String, String> getAliases()
  {
    Map<String, String> aliases = new HashMap<String, String>();
    aliases.put( "ipKey", getColumnNames() );
    return aliases;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( AnalyzedVO analyzedVO )
  {
    Timestamp now = new Timestamp( System.currentTimeMillis() );
    return new Object[] { analyzedVO.getIPKey(), now };
  }

  /**
   * Checks for the existence of a IP Key.
   * @param ipKey The IP Key to check for
   * @return <code>true</code> if the key exists <code>false</code> otherwise
   * @throws SQLException if the query fails
   */
  public boolean contains( int ipKey ) throws SQLException
  {
    SQLStatement<BigDecimal> COUNT = SQL.select( "SELECT COUNT(*) FROM " + getTableName() + " WHERE TM_SCHUTIT_KEY = ?" );
    int count = selectInteger( getQueryRunner(), COUNT, new Object[] { ipKey } );
    return count > 0;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public Integer insert( final AnalyzedVO analyzed )
  {
    Integer id = super.insert( analyzed );
    return id;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#delete(java.lang.Integer)
  @Override
  public void delete( Integer id )
  {
    super.delete( id );
  }
}
