package ch.ige.wdltool.dao;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.util.enums.BAGISLanguage;
import ch.ige.wdltool.util.TermUtils;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.IpiTermCandidateVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.NegativeIpiTermVO;
import ch.ige.wdltool.vo.NiceTermVO;

/**
 * Tests {@link ch.ige.wdltool.dao.GeneralTermDAO} with term group stuff.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: GeneralTermDAOForTermGroupTest.java,v 1.9 2008/04/29 16:38:11 heinzer Exp $
 */
public class GeneralTermDAOForTermGroupTest extends BaseDAOTestTemplate<GeneralTermDAO, AbstractTermVO>
{
  /**
   * An existing term group ID
   */
  private static final int EXISTING_TERM_GROUP_ID = 8646; // This may change!

  /**
   * An existing German term which is translated in the term group ID {@link #EXISTING_TERM_GROUP_ID}.
   */
  private static final String EXISTING_GERMAN_TERM = "Gartenarbeiten"; // This may change!

  /**
   * An existing French term which is translated in the term group ID {@link #EXISTING_TERM_GROUP_ID}.
   */
  private static final String EXISTING_FRENCH_TERM = "jardinage"; // This may change!

  /**
   * An existing Italian term which is translated in the term group ID {@link #EXISTING_TERM_GROUP_ID}.
   */
  private static final String EXISTING_ITALIAN_TERM = "giardinaggio"; // This may change!

  /**
   * An unknown term group ID
   */
  private static final int UNKNOWN_TERM_GROUP_ID = 5; // This may change!

  /**
   * An existing term ID / German term
   */
  private static final Integer EXISTING_GERMAN_TERM_ID = 30940; // This may change!

  /**
   * An existing term ID / Italian term
   */
  private static final Integer EXISTING_FRENCH_TERM_ID = 39942; // This may change!

  /**
   * An existing term ID / French term
   */
  private static final Integer EXISTING_ITALIAN_TERM_ID = 48747; // This may change!

  /**
   * An unknown term ID
   */
  private static final int UNKNOWN_TERM_ID = 5; // This may change!

  /**
   * Some term groups
   */
  private static final int[] TERM_GROUP_IDS = { 8040, 7959 }; // This may change!

  /**
   * The number of terms in {@link #TERM_GROUP_IDS}
   */
  private static final int[] TERM_GROUP_SIZES = { 3, 2 }; // This may change!

  /**
   * The only constructor
   * @param name The test's name
   */
  public GeneralTermDAOForTermGroupTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.GeneralTermDAO#fillTermGroupTerms(List)}.
   * @throws Exception if the test fails
   */
  public void testFillTermGroupTerms() throws Exception
  {
    // One term
    NiceTermVO germanNiceTerm = new NiceTermVO();
    germanNiceTerm.setIdentifier( EXISTING_GERMAN_TERM_ID );
    germanNiceTerm.setTermGroupId( EXISTING_TERM_GROUP_ID );
    germanNiceTerm.setLanguage( Locale.GERMAN );

    List<AbstractTermVO> terms = createTermList( germanNiceTerm );
    getDAO().fillTermGroupTerms( terms );
    assertEquals( "Checking List size", 1, terms.size() );
    checkSimpleTranslation( germanNiceTerm.getTermGroupTerms(), germanNiceTerm.getLanguage(), EXISTING_TERM_GROUP_ID );

    // Two terms
    NiceTermVO frenchNiceTerm = new NiceTermVO();
    frenchNiceTerm.setIdentifier( EXISTING_FRENCH_TERM_ID );
    frenchNiceTerm.setTermGroupId( EXISTING_TERM_GROUP_ID );
    frenchNiceTerm.setLanguage( Locale.FRENCH );

    terms = createTermList( germanNiceTerm, frenchNiceTerm );
    getDAO().fillTermGroupTerms( terms );
    assertEquals( "Checking List size", 2, terms.size() );
    checkSimpleTranslation( germanNiceTerm.getTermGroupTerms(), germanNiceTerm.getLanguage(), EXISTING_TERM_GROUP_ID );
    checkSimpleTranslation( frenchNiceTerm.getTermGroupTerms(), frenchNiceTerm.getLanguage(), EXISTING_TERM_GROUP_ID );

    // Three terms
    NiceTermVO italianNiceTerm = new NiceTermVO();
    italianNiceTerm.setIdentifier( EXISTING_ITALIAN_TERM_ID );
    italianNiceTerm.setTermGroupId( EXISTING_TERM_GROUP_ID );
    italianNiceTerm.setLanguage( Locale.ITALIAN );

    terms = createTermList( italianNiceTerm, germanNiceTerm, frenchNiceTerm );
    getDAO().fillTermGroupTerms( terms );
    assertEquals( "Checking List size", 3, terms.size() );
    checkSimpleTranslation( italianNiceTerm.getTermGroupTerms(), italianNiceTerm.getLanguage(), EXISTING_TERM_GROUP_ID );
    checkSimpleTranslation( germanNiceTerm.getTermGroupTerms(), germanNiceTerm.getLanguage(), EXISTING_TERM_GROUP_ID );
    checkSimpleTranslation( frenchNiceTerm.getTermGroupTerms(), frenchNiceTerm.getLanguage(), EXISTING_TERM_GROUP_ID );

    // Check the biggest term groups. Note that the term IDs are set to a non-existing value, so the full term groups are
    // always fetched.
    terms.clear();
    for ( int i = 0; i < TERM_GROUP_IDS.length; i++ )
    {
      int termGroupId = TERM_GROUP_IDS[i];
      IpiTermVO ipiTerm = new IpiTermVO();
      ipiTerm.setIdentifier( UNKNOWN_TERM_ID );
      ipiTerm.setTermGroupId( termGroupId );
      ipiTerm.setLanguage( Locale.GERMAN );
      terms.add( ipiTerm );
    }

    getDAO().fillTermGroupTerms( terms );
    assertEquals( "Checking List size", TERM_GROUP_IDS.length, terms.size() );

    for ( int i = 0; i < TERM_GROUP_SIZES.length; i++ )
    {
      IpiTermVO ipiTerm = (IpiTermVO) terms.get( i );

      int expectedTotal = TERM_GROUP_SIZES[i];

      int totalTerms = 0;
      totalTerms += getTermGroupTermsSize( ipiTerm, Locale.GERMAN );
      totalTerms += getTermGroupTermsSize( ipiTerm, Locale.FRENCH );
      totalTerms += getTermGroupTermsSize( ipiTerm, Locale.ITALIAN );
      assertEquals( "Checking grand total of terms for term group ID " + ipiTerm.getTermGroupId(), expectedTotal, totalTerms );
    }
  }

  /**
   * Gets the size of term group terms for a specific language.
   * @param ipiTerm The IPI term containing the term group terms
   * @param language The language to use
   * @return The number of language-specific term group terms
   */
  private int getTermGroupTermsSize( IpiTermVO ipiTerm, Locale language )
  {
    Map<Locale, List<AbstractTermVO>> termGroupTerms = ipiTerm.getTermGroupTerms();
    if ( termGroupTerms == null || termGroupTerms.get( language ) == null )
    {
      return 0;
    }
    else
    {
      return termGroupTerms.get( language ).size();
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.GeneralTermDAO#fillTermGroupTerms(List)} with faults.
   * @throws Exception if the test fails
   */
  public void testFillTermGroupTermsWithFaults() throws Exception
  {
    try
    {
      getDAO().fillTermGroupTerms( null );
      fail( "Null terms must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    // Just an empty List - nothing changes
    List<AbstractTermVO> terms = new ArrayList<AbstractTermVO>();
    getDAO().fillTermGroupTerms( terms );
    assertTrue( "Checking result for empty List", terms.isEmpty() );

    // Terms which can't be in a term group - nothing changes
    NegativeIpiTermVO negativeIpiTerm = new NegativeIpiTermVO();
    IpiTermCandidateVO ipiTermCandidate = new IpiTermCandidateVO();
    terms = createTermList( negativeIpiTerm, ipiTermCandidate );
    getDAO().fillTermGroupTerms( terms );
    assertEquals( "Checking result for List of non-term group terms", 2, terms.size() );
    assertTrue( "Checking negative IPI term", terms.get( 0 ) == negativeIpiTerm );
    assertTrue( "Checking IPI term candidate", terms.get( 1 ) == ipiTermCandidate );

    try
    {
      NiceTermVO niceTerm = new NiceTermVO();
      niceTerm.setTermGroupId( UNKNOWN_TERM_GROUP_ID );
      getDAO().fillTermGroupTerms( createTermList( niceTerm ) );
      fail( "Unknown term group ID must fail" );
    }
    catch ( IllegalStateException e )
    {
      // This is expected
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.GeneralTermDAO#findAnyWordWithinActiveTerms(String)}.
   * @throws Exception if the test fails
   */
  public void testFindAnyWordWithinActiveTerms() throws Exception
  {
    List<AbstractTermVO> terms = getDAO().findAnyWordWithinActiveTerms( EXISTING_GERMAN_TERM );
    assertEquals( "Checking number of German terms", 1, terms.size() );
    NiceTermVO niceTerm = (NiceTermVO) terms.get( 0 );
    Map<Locale, List<AbstractTermVO>> translations = niceTerm.getTermGroupTerms();

    assertEquals( "Checking number of French terms", 1, translations.get( Locale.FRENCH ).size() );
    NiceTermVO frenchTranslation = (NiceTermVO) translations.get( Locale.FRENCH ).get( 0 );
    assertEquals( "Checking French translation", EXISTING_FRENCH_TERM, frenchTranslation.getTerm() );

    assertEquals( "Checking number of Italian terms", 1, translations.get( Locale.ITALIAN ).size() );
    NiceTermVO italianTranslation = (NiceTermVO) translations.get( Locale.ITALIAN ).get( 0 );
    assertEquals( "Checking Italian translation", EXISTING_ITALIAN_TERM, italianTranslation.getTerm() );

    // Now check for a big search
    terms = getDAO().findAnyWordWithinActiveTerms( "tal" ); // Part of German, Italian and French terms
    assertTrue( "Checking number of terms", terms.size() > 1900 );
    int totalTermGroupTerms = 0;
    for ( AbstractTermVO term: terms )
    {
      Integer termGroupId = TermUtils.getTermGroupId( term );
      if ( termGroupId != null )
      {
        Map<Locale, List<AbstractTermVO>> termGroupTerms = null;
        if ( term instanceof NiceTermVO )
        {
          termGroupTerms = ( (NiceTermVO) term ).getTermGroupTerms();
        }
        else if ( term instanceof IpiTermVO )
        {
          termGroupTerms = ( (IpiTermVO) term ).getTermGroupTerms();
        }
        if ( termGroupTerms != null && ( !termGroupTerms.isEmpty() ) )
        {
          for ( Locale language: termGroupTerms.keySet() )
          {
            List<AbstractTermVO> translatedTerms = termGroupTerms.get( language );
            if ( translatedTerms != null )
            {
              totalTermGroupTerms += translatedTerms.size();
            }
          }
        }
      }
    }
    assertTrue( "Checking number of term group terms", totalTermGroupTerms > 2100 );
  }

  /**
   * Checks for simple term translations: 2 other terms, 1 in each language.
   * @param termGroupTerms The term group terms to check
   * @param existingLanguage The existing language, which must not exist in <code>termGroupTerms</code>
   * @param termGroupId The existing term group ID
   */
  private void checkSimpleTranslation( Map<Locale, List<AbstractTermVO>> termGroupTerms, Locale existingLanguage, Integer termGroupId )
  {
    assertEquals( "Checking term group term languages", 3, termGroupTerms.size() );
    assertNotNull( "Checking for existing language", termGroupTerms.get( existingLanguage ) );
    assertTrue( "Checking for existing language", termGroupTerms.get( existingLanguage ).isEmpty() );
    for ( Locale language: BAGISLanguage.parse( existingLanguage ).getSwissLocalesExceptThis() )
    {
      List<AbstractTermVO> terms = termGroupTerms.get( language );
      assertNotNull( "Checking for language " + language, terms );
      assertEquals( "Checking for language " + language, 1, terms.size() );
      assertEquals( "Checking for term group ID", termGroupId, TermUtils.getTermGroupId( terms.get( 0 ) ) );
      for ( AbstractTermVO term: terms )
      {
        if ( TermUtils.getTermGroupId( term ) != null )
        {
          assertNotNull( "Checking for term group number", TermUtils.getTermGroupNumber( term ) );
        }
      }
    }
  }

  /**
   * Creates a <code>List</code> of terms.
   * @param terms The terms to convert to a <code>List</code> 
   * @return A <code>List</code> of terms
   */
  private List<AbstractTermVO> createTermList( AbstractTermVO... terms )
  {
    List<AbstractTermVO> termz = new ArrayList<AbstractTermVO>();
    for ( AbstractTermVO term: terms )
    {
      termz.add( term );
    }
    return termz;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<GeneralTermDAO> getDAOClass()
  {
    return GeneralTermDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected boolean assertVOEqual( AbstractTermVO originalVO, AbstractTermVO serializedVO )
  {
    return true;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected AbstractTermVO getSerializableVO()
  {
    return new NiceTermVO();
  }
}
