package ch.ige.wdltool.ejb;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.wdltool.analyzer.AnalysisResultVO;
import ch.ige.wdltool.ejb.impl.WdlServicesImpl;
import ch.ige.wdltool.exception.WrappedRuntimeException;

/**
 * The bean that offers the WDL services.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlBean.java,v 1.12 2008/04/18 13:33:47 heinzer Exp $
 *
 * @ejb.bean
 * name="Wdl"
 * 
 * 
 * description="An EJB offering WDL services"
 * display-name="WDL"
 * view-type="both"
 * type="Stateless"
 * transaction-type="Container"
 * remote-business-interface="ch.ige.wdltool.ejb.WdlServices"
 * local-business-interface="ch.ige.wdltool.ejb.WdlLocalServices"
 * @ejb.transaction type="RequiresNew"
 * 
 */
public class WdlBean extends WdlServicesImpl implements SessionBean
{
  /**
   * The session context
   */
  private SessionContext sessionContext;

  /**
   * Creates the bean.
   */
  public void ejbCreate()
  {
    ServicesFactory.getStandaloneServices(); //IP Object services in standalone mode
    WdlServicesFactory.setEJBWdlServices(); //Wdl services in JEE mode - makes sense in any case as we are already on the server
  }

  // @see javax.ejb.SessionBean#ejbActivate()
  public void ejbActivate() throws EJBException
  {
    // Nothing to do
  }

  // @see javax.ejb.SessionBean#ejbPassivate()
  public void ejbPassivate() throws EJBException
  {
    // Nothing to do
  }

  // @see javax.ejb.SessionBean#ejbRemove()
  public void ejbRemove() throws EJBException
  {
    // Nothing to do
  }

  // @see javax.ejb.SessionBean#setSessionContext(javax.ejb.SessionContext)
  public void setSessionContext( SessionContext sessionContext ) throws EJBException
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Single title analyze service is only offered internally (local view-type).
   * @see ch.ige.wdltool.ejb.WdlLocalServices#analyzeWdl(int)
   * @throws WrappedRuntimeException 
   * @ejb.interface-method view-type="local"
   * @ejb.transaction
   *   type="RequiresNew"
   */
  public AnalysisResultVO analyzeWdl( int ipKey ) throws WrappedRuntimeException
  {
    AnalysisResultVO analysisResult = super.analyzeWdl( ipKey );
    if ( analysisResult.hasSyntaxError() || analysisResult.hasGeneralError() )
    {
      this.sessionContext.setRollbackOnly(); //analyzer process failed - do not persist any changes (neither new Candidates nor the addition of the key to the ANALYZED Table) 
    }
    return analysisResult;
  }
}
