package ch.ige.wdltool.ejb;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.RandomStringUtils;

import ch.ige.wdltool.ParameterConstants;
import ch.ige.wdltool.dao.IpiTermDAO;
import ch.ige.wdltool.dao.IpiTermDAOTest;
import ch.ige.wdltool.vo.IpiTermVO;

/**
 * Tests {@link ch.ige.wdltool.ejb.WdlServices} for IPI terms. <p>Copyright (c) 2007, Swiss Federal Institute of
 * Intellectual Property</p>
 * @author Eric Lewis
 * @version $Id: WdlServicesForIpiTermTest.java,v 1.15 2009/02/02 13:45:10 bandschapp Exp $
 */
public class WdlServicesForIpiTermTest extends BaseServiceTestTemplate<IpiTermDAO, IpiTermVO, Integer, WdlServices>
{
  /**
   * The unit under test
   */
  private WdlServices wdlServices;

  /**
   * Used for utility stuff
   */
  private IpiTermDAOTest ipiTermDAOTest;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlServicesForIpiTermTest( String name )
  {
    super( name );
    this.wdlServices = WdlServicesFactory.getStandaloneWdlServices();
    this.ipiTermDAOTest = new IpiTermDAOTest( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findIpiTerm(Integer)}.
   * @throws Exception if the test fails
   */
  public void testFindIpiTerm() throws Exception
  {
    defaultTestRead( getExpectedVO(), MINUS_ONE, 999999999, getExpectedVO().getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#insertIpiTermAndRemoveIpiTermCandidate(IpiTermVO)}.
   * @throws Exception if the test fails
   */
  public void testInsertIpiTermAndRemoveIpiTermCandidate() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#insertIpiTermAndRemoveIpiTermCandidate(IpiTermVO)}
   * with existing stopword.
   * @throws Exception if the test fails
   */
  public void testInsertIpiTermAndRemoveIpiTermCandidateWithExistingStopword() throws Exception
  {
    defaultTestCreateTermWithExistingStopword( Locale.FRENCH, "une" );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#updateIpiTermAndRemoveIpiTermCandidate(IpiTermVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdateIpiTermAndRemoveIpiTermCandidate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#updateIpiTermAndRemoveIpiTermCandidate(IpiTermVO)}
   * with existing stopword.
   * @throws Exception if the test fails
   */
  public void testUpdateIpiTermAndRemoveIpiTermCandidatehExistingStopword() throws Exception
  {
    defaultTestUpdateTermWithExistingStopword( Locale.FRENCH, "pour" );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getInvalidPropertyValues(ch.ige.dbutils.vo.AbstractVO, boolean)
  @Override
  @SuppressWarnings( "unused" )
  public Map<String, Object[]> getInvalidPropertyValues( IpiTermVO valueObject, boolean isUpdate ) throws IllegalStateException
  {
    Map<String, Object[]> properties = new HashMap<String, Object[]>();

    if ( isUpdate )
    {
      properties.put( "identifier", new Integer[] { null, -1, 0 } );
    }
    else
    {
      properties.put( "identifier", new Integer[] { -1, 0 } );
    }

    properties.put( "classNumber", new Integer[] { null, 0, 46 } );

    properties.put( "termGroupId", new Integer[] { 0 } );

    properties.put( "language", new Locale[] { null, Locale.CHINESE } );

    properties.put( "term", new String[] { null, "", " ", " bla", "bla ", RandomStringUtils.randomAlphabetic( ParameterConstants.EXCEEDED_LIMIT_FIELD_TERM ),
        "´", "bl´a", "`", "bl`a", "^", "bl^a", "¨", "bl¨a", "bla" + '\u2014' + "bla" /* long dash, created automatically by Word */, "bla  bla" } );

    properties.put( "privateComment", new String[] { "", RandomStringUtils.randomAlphabetic( 501 ), "bla   bla" } );

    properties.put( "publicComment", new String[] { "", RandomStringUtils.randomAlphabetic( 501 ), "bla  bla" } );

    properties.put( "latestOccurrence", new Timestamp[] { new Timestamp( System.currentTimeMillis() + 50000 ) } );

    properties.put( "renewal", new Timestamp[] { new Timestamp( System.currentTimeMillis() + 50000 ) } );

    properties.put( "usageCount", new Integer[] { -1, 0 } );

    properties.put( "creationUser", new String[] { "", RandomStringUtils.randomAlphabetic( 26 ) } );

    properties.put( "changeUser", new String[] { null, "", RandomStringUtils.randomAlphabetic( 26 ) } );

    return properties;
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#deleteIpiTerm(Integer)}.
   * @throws Exception if the test fails
   */
  public void testDeleteIpiTerm() throws Exception
  {
    defaultTestDelete( MINUS_ONE, 999999999 );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getServices()
  @Override
  protected WdlServices getServices()
  {
    return this.wdlServices;
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getSerializableVO()
  @Override
  protected IpiTermVO getSerializableVO() throws SQLException
  {
    return this.ipiTermDAOTest.getSerializableVO();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  protected IpiTermVO getExpectedVO() throws ParseException
  {
    return this.ipiTermDAOTest.getExpectedVO();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOsAndValidateLoaded( IpiTermVO originalVO, IpiTermVO loadedVO )
  {
    this.ipiTermDAOTest.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.io.Serializable)
  @Override
  public IpiTermVO readWithService( Integer id ) throws Exception
  {
    return getServices().findIpiTerm( id );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( IpiTermVO vo )
  {
    vo.setIdentifier( null );
    vo.setTermGroupId( null ); // Otherwise it must exist
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#insertWithService(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public Integer insertWithService( IpiTermVO vo ) throws Exception
  {
    IpiTermVO newTerm = getServices().insertIpiTermAndRemoveIpiTermCandidate( vo ).getNewTerm();
    return newTerm.getIdentifier();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( IpiTermVO vo )
  {
    this.ipiTermDAOTest.modifyVOBeforeUpdate( vo );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#updateWithService(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void updateWithService( IpiTermVO vo ) throws Exception
  {
    getServices().updateIpiTermAndRemoveIpiTermCandidate( vo );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#deleteWithService(java.io.Serializable)
  @Override
  public void deleteWithService( Integer id ) throws Exception
  {
    getServices().deleteIpiTerm( id );
  }
}
