package ch.ige.wdltool.ejb;

import java.util.Collection;
import java.util.List;

import ch.ige.wdltool.dao.NiceTermGroupDAO;
import ch.ige.wdltool.dao.NiceTermGroupDAOTest;
import ch.ige.wdltool.vo.NiceTermGroupVO;

/**
 * Tests {@link ch.ige.wdltool.ejb.WdlServices} for Nice term groups.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlServicesForNiceTermGroupTest.java,v 1.6 2008/04/29 16:38:12 heinzer Exp $
 */
public class WdlServicesForNiceTermGroupTest extends BaseServiceTestTemplate<NiceTermGroupDAO, NiceTermGroupVO, Integer, WdlServices>
{
  /**
   * The unit under test
   */
  private WdlServices wdlServices;

  /**
   * Used for utility stuff
   */
  private NiceTermGroupDAOTest niceTermGroupDAOTest;

  /**
   * The number of an existing Nice term group
   * This may change!
   */
  private static final int NICE_TERM_GROUP_NUMBER = 70159;

  /**
   * The ID of {@link #NICE_TERM_GROUP_NUMBER}.
   * This may change!
   */
  private static final int NICE_TERM_GROUP_ID = 10180;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlServicesForNiceTermGroupTest( String name )
  {
    super( name );
    this.wdlServices = WdlServicesFactory.getStandaloneWdlServices();
    this.niceTermGroupDAOTest = new NiceTermGroupDAOTest( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findAllNiceTermGroups()}.
   * @throws Exception if the test fails
   */
  public void testFindAllNiceTermGroups() throws Exception
  {
    defaultTestReadCollection( getExpectedVO(), 4000, (Object[]) null );

    // There's not much to do, really. Just check the sorting.
    List<NiceTermGroupVO> termGroups = (List<NiceTermGroupVO>) readWithService( (Object[]) null );
    int previousTermGroupNumber = 0;
    for ( NiceTermGroupVO termGroup: termGroups )
    {
      assertTrue( "Check order", termGroup.getNiceTermNumber().intValue() > previousTermGroupNumber );
      previousTermGroupNumber = termGroup.getNiceTermNumber().intValue();
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findTermGroup(Integer)}.
   * @throws Exception if the test fails
   */
  public void testFindTermGroup() throws Exception
  {
    defaultTestRead( getExpectedVO(), -1, 808, NICE_TERM_GROUP_ID );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.io.Serializable)
  @Override
  public NiceTermGroupVO readWithService( Integer id ) throws Exception
  {
    return (NiceTermGroupVO) getServices().findTermGroup( id );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  public Collection<NiceTermGroupVO> readWithService( Object... searchParameters ) throws Exception
  {
    return getServices().findAllNiceTermGroups();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOsAndValidateLoaded( NiceTermGroupVO originalVO, NiceTermGroupVO loadedVO )
  {
    this.niceTermGroupDAOTest.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  protected NiceTermGroupVO getExpectedVO()
  {
    NiceTermGroupVO termGroup = new NiceTermGroupVO();
    termGroup.setIdentifier( NICE_TERM_GROUP_ID );
    termGroup.setNiceTermNumber( NICE_TERM_GROUP_NUMBER );
    return termGroup;
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getServices()
  @Override
  protected WdlServices getServices()
  {
    return this.wdlServices;
  }
}
