package ch.ige.wdltool.ejb;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.RandomStringUtils;

import ch.ige.wdltool.dao.StopwordDAO;
import ch.ige.wdltool.dao.StopwordDAOTest;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.vo.StopwordVO;

/**
 * Tests {@link ch.ige.wdltool.ejb.WdlServices} for stopwords.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlServicesForStopwordTest.java,v 1.14 2008/04/29 16:38:12 heinzer Exp $
 */
public class WdlServicesForStopwordTest extends BaseServiceTestTemplate<StopwordDAO, StopwordVO, Integer, WdlServices>
{
  /**
   * The unit under test
   */
  private WdlServices wdlServices;

  /**
   * Used for helper methods
   */
  private StopwordDAOTest stopwordDAOTest;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlServicesForStopwordTest( String name )
  {
    super( name );
    this.wdlServices = WdlServicesFactory.getStandaloneWdlServices();
    this.stopwordDAOTest = new StopwordDAOTest( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findStopword(Integer)}.
   * @throws Exception if the test fails
   */
  public void testFindStopword() throws Exception
  {
    defaultTestRead( getExpectedVO(), MINUS_ONE, Integer.MAX_VALUE, getExpectedVO().getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#insertStopword(StopwordVO)}.
   * @throws Exception if the test fails
   */
  public void testInsertStopword() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#updateStopword(StopwordVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdateStopword() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#deleteStopword(Integer)}.
   * @throws Exception if the test fails
   */
  public void testDeleteStopword() throws Exception
  {
    defaultTestDelete( MINUS_ONE, Integer.MAX_VALUE );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#deleteStopwords(java.util.Collection)}.
   * @throws Exception if the test fails
   */
  public void testDeleteStopwords() throws Exception
  {
    StopwordVO stopword1 = createAndInsert( true );
    StopwordVO stopword2 = createAndInsert( true );

    List<Integer> stopwordIds = new ArrayList<Integer>();
    stopwordIds.add( stopword1.getIdentifier() );
    stopwordIds.add( stopword2.getIdentifier() );

    this.wdlServices.deleteStopwords( stopwordIds );

    try
    {
      read( stopword1.getIdentifier() );
    }
    catch ( SQLException e )
    {
      WdlObjectNotFoundException causingException = checkCausingException( e, WdlObjectNotFoundException.class );
      assertEquals( "Checking ID", stopword1.getIdentifier(), causingException.getIdentifier() );
    }

    try
    {
      read( stopword2.getIdentifier() );
    }
    catch ( SQLException e )
    {
      WdlObjectNotFoundException causingException = checkCausingException( e, WdlObjectNotFoundException.class );
      assertEquals( "Checking ID", stopword2.getIdentifier(), causingException.getIdentifier() );
    }

    clearSavedId();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getInvalidPropertyValues(boolean)
  @Override
  @SuppressWarnings( "unused" )
  protected Map<String, Object[]> getInvalidPropertyValues( StopwordVO stopword, boolean isUpdate ) throws IllegalStateException
  {
    Map<String, Object[]> properties = new HashMap<String, Object[]>();

    if ( isUpdate )
    {
      properties.put( "identifier", new Integer[] { null, -1, 0 } );
    }
    else
    {
      properties.put( "identifier", new Integer[] { -1, 0 } );
    }

    properties.put( "language", new Locale[] { null, Locale.CHINESE } );

    properties.put( "word", new String[] { null, "", " ", "         ", RandomStringUtils.randomAlphabetic( 26 ) } );

    return properties;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  public StopwordVO readWithService( Integer id ) throws Exception
  {
    return getServices().findStopword( id );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( StopwordVO stopword )
  {
    stopword.setIdentifier( null );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#insertWithService(ch.ige.dbutils.vo.AbstractVO)
  public Integer insertWithService( StopwordVO stopword ) throws Exception
  {
    StopwordVO newStopword = getServices().insertStopword( stopword );
    return newStopword.getIdentifier();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( StopwordVO stopword )
  {
    this.stopwordDAOTest.modifyVOBeforeUpdate( stopword );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#updateWithService(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void updateWithService( StopwordVO stopword ) throws Exception
  {
    getServices().updateStopword( stopword );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  public void deleteWithService( Integer id ) throws Exception
  {
    getServices().deleteStopword( id );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  protected StopwordVO getExpectedVO()
  {
    return this.stopwordDAOTest.getExpectedVO();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOsAndValidateLoaded( StopwordVO originalVO, StopwordVO loadedVO )
  {
    this.stopwordDAOTest.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  protected StopwordVO getSerializableVO()
  {
    return this.stopwordDAOTest.getSerializableVO();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getServices()
  @Override
  protected WdlServices getServices()
  {
    return this.wdlServices;
  }
}
