package ch.ige.wdltool.ejb.server;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.naming.Context;
import javax.naming.InitialContext;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import ch.ige.wdltool.ejb.Wdl;
import ch.ige.wdltool.ejb.WdlHome;
import ch.ige.wdltool.exception.WdlUpdateException;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.CheckedClassTextGroupVO;
import ch.ige.wdltool.vo.ClassTextPartVO;
import ch.ige.wdltool.vo.NiceTermVO;
import ch.ige.wdltool.vo.WdlCheckResultVO;
import ch.ige.wdlutil.vo.ClassBlockVO;
import ch.ige.wdlutil.vo.ClassTextGroupVO;
import ch.ige.wdlutil.vo.WdlVO;

/**
 * Tests of Wdl Bean.
 * This is not intended to be run by build but works from developers machines with correct -D Parameters.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   heinzer
 * @version  $Id: TestWdlBean.java,v 1.10 2008/05/07 12:12:48 lewis_lee Exp $
 */
public class TestWdlBean extends TestCase
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( TestWdlBean.class );

  /**
   * The facade for WDL EJB
   */
  private Wdl wdlFacade;

  static
  {
    BasicConfigurator.configure();
  }

  /**
   * The only constructor.
   * @param name The test's name
   */
  public TestWdlBean( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    Context context = new InitialContext();
    this.wdlFacade = ( (WdlHome) context.lookup( WdlHome.JNDI_NAME ) ).create();
  }

  /**
   * Tests updating of a terms comment.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {

    List<AbstractTermVO> results = new ArrayList<AbstractTermVO>();
    try
    {
      results = this.wdlFacade.findAnyWordWithinActiveTerms( Locale.GERMAN, "tisch" );
    }
    catch ( RemoteException e )
    {
      LOG.error( "Error: " + e.getMessage(), e );
      fail( "RemoteException on findAnyWord: " + e.getMessage() );
    }
    AbstractTermVO singleResult = null;
    int resIndex = 0;
    while ( singleResult == null && resIndex < results.size() )
    {
      if ( results.get( resIndex ) instanceof NiceTermVO && results.get( resIndex ).getPrivateComment() != null
          && results.get( resIndex ).getPrivateComment().length() > 0 )
      {
        singleResult = results.get( resIndex );
      }
      resIndex++;
    }
    if ( singleResult != null )
    {
      String oldComment = singleResult.getPrivateComment();
      System.out.println( "comment old: " + oldComment );
      if ( oldComment != null && oldComment.length() > 1 )
      { //change first character random
        char firstChar = 'a';
        int adding = 1 + new java.util.Random().nextInt( 20 );
        for ( int i = 0; i < adding; i++ )
        {
          firstChar++;
        }
        oldComment = firstChar + oldComment.substring( 1, oldComment.length() );
      }
      singleResult.setPrivateComment( oldComment );
      System.out.println( "comment new: " + oldComment );
      singleResult.setPrivateComment( oldComment );

      try
      {
        this.wdlFacade.updateNiceTermAndRemoveIpiTermCandidate( (NiceTermVO) singleResult );
      }
      catch ( WdlUpdateException e )
      {
        LOG.error( "Error: " + e.getMessage(), e );
        fail( "WdlUpdateException: " + e.getMessage() );
      }
      catch ( RemoteException e )
      {
        LOG.error( "Error: " + e.getMessage(), e );
        fail( "RemoteException: " + e.getMessage() );
      }
      catch ( WdlValidationException e )
      {
        LOG.error( "Error: " + e.getMessage(), e );
        fail( "WdlValidationException: " + e.getMessage() );
      }
    }
  }

  /**
   * Tests checkWdl of {@link ch.ige.wdltool.ejb.Wdl}.
   */
  public void testCheck()
  {

    try
    {
      WdlCheckResultVO res = this.wdlFacade.checkWdl( 1201824844, false, null );
      LOG.debug( "checked: " + res.getName() );

      WdlVO wdlVO = res.getWdl();
      List<ClassBlockVO> checkedClassBlockVOs = wdlVO.getClassBlocks();

      for ( ClassBlockVO blockVO: checkedClassBlockVOs )
      {
        LOG.debug( blockVO.getClassNumber() + "\n" );

        LOG.debug( blockVO.getLimitation() ); //unchecked

        List<ClassTextGroupVO> checkedGroups = blockVO.getClassTextGroups(); //(List<CheckedClassTextGroupVO>)
        for ( ClassTextGroupVO groupVO: checkedGroups )
        {
          CheckedClassTextGroupVO checkedGroupVO = (CheckedClassTextGroupVO) groupVO;
          List<ClassTextPartVO> parts = checkedGroupVO.getClassTextParts();
          for ( ClassTextPartVO partVO: parts )
          {
            LOG.debug( "start_block_" + partVO.getCheckFlag() );
            LOG.debug( partVO.getClassTextPart() );
            LOG.debug( "stop_block_" + partVO.getCheckFlag() );
            LOG.debug( "\n" );
          }
          groupVO.getClassTextGroup();
        }
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( "exception occured in testFindTerm: " + e.getMessage() );
    }

  }

}
