package ch.ige.wdltool.enums;

import java.util.Map;

import ch.ige.util.enums.infrastructure.DBRealm;
import ch.ige.util.enums.mapping.DatabaseMappable;
import ch.ige.util.enums.mapping.DatabaseMapping;
import ch.ige.util.enums.mapping.Values;
import ch.ige.util.enums.mapping.DatabaseMapping.UnknownDatabaseValueException;

/**
 * Possible data sources.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: DataSource.java,v 1.3 2007/12/06 12:53:19 heinzer Exp $
 */
@Values( database = DBRealm.WDL, query = "SELECT DISTINCT data_source_id FROM data_source" )
public enum DataSource implements DatabaseMappable<Integer>
{
  /**
   * The Nice classification
   */
  NICE,

  /**
   * Created within IPI
   */
  IPI,

  /**
   * Comes from the WIPO
   */
  WIPO;

  // -------------------------------------------------------
  // DB Mapping

  /**
   * Maps to database values.
   */
  private static final Map<DataSource, Integer> DB_MAPPING = DatabaseMapping.<DataSource, Integer> decodeAndRegister( NICE, 1, IPI, 2, WIPO, 3 );

  /**
   * Parses the data source code and returns the corresponding data source.
   * @param dataSourceCode The data source code
   * @return The data source for <code>dataSourceCode</code>
   * @throws UnknownDatabaseValueException if the data source code is unknown
   */
  public static DataSource parse( Integer dataSourceCode ) throws UnknownDatabaseValueException
  {
    return DatabaseMapping.getRegisteredMapping( DataSource.class ).toEnum( dataSourceCode );
  }

  /**
   * Gets the value for the database.
   * @return The value for the database
   */
  public Integer toDB()
  {
    return DB_MAPPING.get( this );
  }
}
