package ch.ige.wdltool.enums;

import java.util.Map;

import ch.ige.util.enums.infrastructure.DBRealm;
import ch.ige.util.enums.mapping.DatabaseMappable;
import ch.ige.util.enums.mapping.DatabaseMapping;
import ch.ige.util.enums.mapping.Values;
import ch.ige.util.enums.mapping.DatabaseMapping.UnknownDatabaseValueException;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.IpiTermCandidateVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.NegativeIpiTermVO;
import ch.ige.wdltool.vo.NiceTermVO;

/**
 * Possible types for a term.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: TermType.java,v 1.9 2007/12/06 12:53:19 heinzer Exp $
 */
@Values( database = DBRealm.WDL, query = "SELECT DISTINCT term_type FROM term" )
public enum TermType implements DatabaseMappable<Integer>
{
  /**
   * A Nice term
   */
  NICE_TERM,

  /**
   * A term used within IPI
   */
  IPI_TERM,

  /**
   * A term which may be one day used within IPI
   */
  IPI_TERM_CANDIDATE,

  /**
   * A term which must not be used
   */
  NEGATIVE_IPI_TERM;

  // -------------------------------------------------------
  // Term class Mapping

  /**
   * Gets the class that corresponds to this term type.
   * @return The class for this term type
   */
  @SuppressWarnings( "unchecked" )
  public Class<? extends AbstractTermVO> getTermClass()
  {
    return CLASSES[toDB().intValue() - 1];
  }

  // -------------------------------------------------------
  // DB Mapping

  /**
   * Maps to database values.
   */
  private static final Map<TermType, Integer> DB_MAPPING = DatabaseMapping.<TermType, Integer> decodeAndRegister( NICE_TERM, 1, IPI_TERM, 2,
      IPI_TERM_CANDIDATE, 3, NEGATIVE_IPI_TERM, 4 );

  /**
   * The mapping to classes
   */
  private static final Class[] CLASSES = { NiceTermVO.class, IpiTermVO.class, IpiTermCandidateVO.class, NegativeIpiTermVO.class };

  /**
   * Parses the term type code and returns the corresponding term type.
   * @param termTypeCode The term type code
   * @return The term type for <code>termTypeCode</code>
   * @throws UnknownDatabaseValueException if the term type code is unknown
   */
  public static TermType parse( Integer termTypeCode ) throws UnknownDatabaseValueException
  {
    return DatabaseMapping.getRegisteredMapping( TermType.class ).toEnum( termTypeCode );
  }

  /**
   * Gets the value for the database.
   * @return The value for the database
   */
  public Integer toDB()
  {
    return DB_MAPPING.get( this );
  }
}
