package ch.ige.wdltool.exception;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import javax.ejb.RemoveException;

/**
 * Is thrown when an object should be deleted, but the operation fails.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlRemoveException.java,v 1.6 2008/01/17 17:41:31 lewis_lee Exp $
 */
public class WdlRemoveException extends RemoveException
{
  /**
   * The identifier(s) for the value object to remove/delete
   */
  private Collection<Serializable> identifiers = new ArrayList<Serializable>();

  /**
   * Creates this without a message.
   * @param cause The causing Exception
   * @param identifier The identifier of the object to remove/delete
   */
  public WdlRemoveException( Throwable cause, Serializable identifier )
  {
    this( null, cause, identifier );
  }

  /**
   * Creates this with a message.
   * @param message The error message
   * @param cause The causing Exception
   * @param identifier The identifier of the object to remove/delete
   */
  public WdlRemoveException( String message, Throwable cause, Serializable identifier )
  {
    super( message );
    initCause( cause );
    this.identifiers.add( identifier );
  }

  /**
   * The second only constructor.
   * @param cause The causing Exception
   * @param identifiers The identifiers of the object to remove/delete
   */
  public WdlRemoveException( Throwable cause, Collection<? extends Serializable> identifiers )
  {
    this( null, cause, identifiers );
  }

  /**
   * The second only constructor.
   * @param message The error message
   * @param cause The causing Exception
   * @param identifiers The identifiers of the object to remove/delete
   */
  public WdlRemoveException( String message, Throwable cause, Collection<? extends Serializable> identifiers )
  {
    super( message );
    initCause( cause );
    this.identifiers.addAll( identifiers );
  }

  /**
   * Gets the identifiers for the value objects to remove/delete.
   * @return the identifiers for the value objects to remove/delete
   */
  public Collection<Serializable> getIdentifiers()
  {
    return this.identifiers;
  }

  /**
   * Gets the identifier for the value object to remove/delete.
   * @return the identifier for the value object to remove/delete
   */
  public Serializable getIdentifier()
  {
    if ( this.identifiers.size() > 0 )
    {
      return (Serializable) ( this.identifiers.toArray()[0] );
    }
    else
    {
      return null;
    }
  }
}
