package ch.ige.wdltool.exception;

import java.util.List;

import org.hibernate.validator.InvalidValue;

import ch.ige.dbutils.vo.AbstractVO;

/**
 * Is thrown whenever an object is validated for correct data, but doesn't pass the test.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlValidationException.java,v 1.6 2008/05/07 14:08:16 lewis_lee Exp $
 */
public class WdlValidationException extends Exception
{
  /**
   * The value object that was validated
   */
  private AbstractVO valueObject;

  /**
   * The values that were considered invalid during validation
   */
  private List<InvalidValue> invalidValues;

  /**
   * Gets the value object that was validated.
   * @return the value object that was validated
   */
  public AbstractVO getValueObject()
  {
    return this.valueObject;
  }

  /**
   * Sets the value object that was validated.
   * @param valueObject The value object that was validated
   */
  public void setValueObject( AbstractVO valueObject )
  {
    this.valueObject = valueObject;
  }

  /**
   * Gets the values that were considered invalid during validation.
   * Note that {@link org.hibernate.validator.InvalidValue#getMessage()}
   * <strong>does not contain an error message</strong>. Since that's the task of the client,
   * the 'message' contains the same key as is used in the <code>ValidatorMessages.properties</code>, followed
   * by a <code>:</code> and a comma-separated list of configured values for the validation annotation.
   * <p>Examples:</p>
   * <ul>
   *  <li>The validation annotation <code>@NotNull</code> results in the 'message' <code>validator.notNull</code></li>
   *  <li>The validation annotation <code>@Min(value=1)</code> results in the 'message' <code>validator.min:1</code></li>
   *  <li>The validation annotation <code>@Length(min=1,max=50)</code> results in the 'message' <code>validator.length:1,50</code></li>
   * </ul>
   * @return the values that were considered invalid during validation
   */
  public List<InvalidValue> getInvalidValues()
  {
    return this.invalidValues;
  }

  /**
   * Sets the values that were considered invalid during validation.
   * @param invalidValues The values that were considered invalid during validation
   */
  public void setInvalidValues( List<InvalidValue> invalidValues )
  {
    this.invalidValues = invalidValues;
  }
}
