package ch.ige.wdltool.vo;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.util.enums.IPType;
import ch.ige.util.format.ReportApplicationNumberFormatter;
import ch.ige.util.format.ReportIpNumberFormatter;
import ch.ige.wdlutil.vo.WdlVO;

/**
 * Contains the results of a WDL check.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlCheckResultVO.java,v 1.11 2008/02/05 14:00:02 lewis_lee Exp $
 */
public class WdlCheckResultVO extends AbstractVO
{
  /**
   * The IP's unique key
   */
  private Integer ipKey;

  /**
   * The number of times the trademark was renewed
   */
  private Integer numberOfRenewals;

  /**
   * The trademark's IP number
   */
  private Long ipNumber;

  /**
   * The trademark's application number
   */
  private String applicationNumber;

  /**
   * The actual result of the check
   */
  private WdlVO wdl;

  /**
   * Creates this.
   * @param ipKey The IP's unique key
   * @param numberOfRenewals The number of times the trademark was renewed
   * @param ipNumber The trademark's IP number
   * @param applicationNumber The trademark's application number
   * @param wdlVO The
   */
  public WdlCheckResultVO( Integer ipKey, Integer numberOfRenewals, Long ipNumber, String applicationNumber, WdlVO wdlVO )
  {
    this.ipKey = ipKey;
    this.numberOfRenewals = numberOfRenewals;
    this.ipNumber = ipNumber;
    this.applicationNumber = applicationNumber;
    this.wdl = wdlVO;
  }

  /**
   * Gets the IP key for the trademark.
   * @return the trademark's IP key
   */
  public Integer getIpKey()
  {
    return this.ipKey;
  }

  /**
   * Gets the trademark's name.
   * @return The trademark's name
   */
  public String getName()
  {
    return this.wdl.getTrademarkName();
  }

  /**
   * Gets the trademark's formatted number, either
   * the IP number or the application number
   * @return The formatted IP number or application number
   */
  public String getFormattedNumber()
  {
    if ( this.ipNumber != null )
    {
      return ReportIpNumberFormatter.formatTrademark( this.ipNumber, IPType.TRADEMARK_CH, this.numberOfRenewals );
    }
    else
    {
      return ReportApplicationNumberFormatter.format( this.applicationNumber, IPType.TRADEMARK_CH );
    }
  }

  /**
   * Gets the WDL itself.
   * Note that the validated class text contains an XML-like structure (see below).
   * @return the wdl
   * @see ch.ige.wdltool.ejb.WdlServices#checkWdl(Integer, boolean, java.util.Map)
   */
  public WdlVO getWdl()
  {
    return this.wdl;
  }

}
